/*
FILE:     StatusReport.C
*/
/*
VERSION:  0.904-beta
*/
/*
DATE:     8/14/2003
*/
/*
  Comments and Questions to: sw-help@rcsb.rutgers.edu
*/
/*
COPYRIGHT 1999-2003 Rutgers - The State University of New Jersey

This software is provided WITHOUT WARRANTY OF MERCHANTABILITY OR
FITNESS FOR A PARTICULAR PURPOSE OR ANY OTHER WARRANTY, EXPRESS OR
IMPLIED.  RUTGERS MAKE NO REPRESENTATION OR WARRANTY THAT THE
SOFTWARE WILL NOT INFRINGE ANY PATENT, COPYRIGHT OR OTHER
PROPRIETARY RIGHT.

The user of this software shall indemnify, hold harmless and defend
Rutgers, its governors, trustees, officers, employees, students,
agents and the authors against any and all claims, suits,
losses, liabilities, damages, costs, fees, and expenses including
reasonable attorneys' fees resulting from or arising out of the
use of this software.  This indemnification shall include, but is
not limited to, any and all claims alleging products liability.
*/
/*
               PDB SOFTWARE LICENSE AGREEMENT

BY CLICKING THE ACCEPTANCE BUTTON OR INSTALLING OR USING 
THIS "SOFTWARE, THE INDIVIDUAL OR ENTITY LICENSING THE  
SOFTWARE ("LICENSEE") IS CONSENTING TO BE BOUND BY AND IS 
BECOMING A PARTY TO THIS AGREEMENT.  IF LICENSEE DOES NOT 
AGREE TO ALL OF THE TERMS OF THIS AGREEMENT
THE LICENSEE MUST NOT INSTALL OR USE THE SOFTWARE.

1. LICENSE AGREEMENT

This is a license between you ("Licensee") and the Protein Data Bank (PDB) 
at Rutgers, The State University of New Jersey (hereafter referred to 
as "RUTGERS").   The software is owned by RUTGERS and protected by 
copyright laws, and some elements are protected by laws governing 
trademarks, trade dress and trade secrets, and may be protected by 
patent laws. 

2. LICENSE GRANT

RUTGERS grants you, and you hereby accept, non-exclusive, royalty-free 
perpetual license to install, use, modify, prepare derivative works, 
incorporate into other computer software, and distribute in binary 
and source code format, or any derivative work thereof, together with 
any associated media, printed materials, and on-line or electronic 
documentation (if any) provided by RUTGERS (collectively, the "SOFTWARE"), 
subject to the following terms and conditions: (i) any distribution 
of the SOFTWARE shall bind the receiver to the terms and conditions 
of this Agreement; (ii) any distribution of the SOFTWARE in modified 
form shall clearly state that the SOFTWARE has been modified from 
the version originally obtained from RUTGERS.  

2. COPYRIGHT; RETENTION OF RIGHTS.  

The above license grant is conditioned on the following: (i) you must 
reproduce all copyright notices and other proprietary notices on any 
copies of the SOFTWARE and you must not remove such notices; (ii) in 
the event you compile the SOFTWARE, you will include the copyright 
notice with the binary in such a manner as to allow it to be easily 
viewable; (iii) if you incorporate the SOFTWARE into other code, you 
must provide notice that the code contains the SOFTWARE and include 
a copy of the copyright notices and other proprietary notices.  All 
copies of the SOFTWARE shall be subject to the terms of this Agreement.  

3. NO MAINTENANCE OR SUPPORT; TREATMENT OF ENHANCEMENTS 

RUTGERS is under no obligation whatsoever to: (i) provide maintenance 
or support for the SOFTWARE; or (ii) to notify you of bug fixes, patches, 
or upgrades to the features, functionality or performance of the 
SOFTWARE ("Enhancements") (if any), whether developed by RUTGERS 
or third parties.  If, in its sole discretion, RUTGERS makes an 
Enhancement available to you and RUTGERS does not separately enter 
into a written license agreement with you relating to such bug fix, 
patch or upgrade, then it shall be deemed incorporated into the SOFTWARE 
and subject to this Agreement. You are under no obligation whatsoever 
to provide any Enhancements to RUTGERS or the public that you may 
develop over time; however, if you choose to provide your Enhancements 
to RUTGERS, or if you choose to otherwise publish or distribute your 
Enhancements, in source code form without contemporaneously requiring 
end users or RUTGERS to enter into a separate written license agreement 
for such Enhancements, then you hereby grant RUTGERS a non-exclusive,
royalty-free perpetual license to install, use, modify, prepare
derivative works, incorporate into the SOFTWARE or other computer
software, distribute, and sublicense your Enhancements or derivative
works thereof, in binary and source code form.

4. FEES.  There is no license fee for the SOFTWARE.  If Licensee
wishes to receive the SOFTWARE on media, there may be a small charge
for the media and for shipping and handling.  Licensee is
responsible for any and all taxes.

5. TERMINATION.  Without prejudice to any other rights, Licensor
may terminate this Agreement if Licensee breaches any of its terms
and conditions.  Upon termination, Licensee shall destroy all
copies of the SOFTWARE.

6. PROPRIETARY RIGHTS.  Title, ownership rights, and intellectual
property rights in the Product shall remain with RUTGERS.  Licensee 
acknowledges such ownership and intellectual property rights and will 
not take any action to jeopardize, limit or interfere in any manner 
with RUTGERS' ownership of or rights with respect to the SOFTWARE.  
The SOFTWARE is protected by copyright and other intellectual 
property laws and by international treaties.  Title and related 
rights in the content accessed through the SOFTWARE is the property 
of the applicable content owner and is protected by applicable law.  
The license granted under this Agreement gives Licensee no rights to such
content.

7. DISCLAIMER OF WARRANTY.  THE SOFTWARE IS PROVIDED FREE OF 
CHARGE, AND, THEREFORE, ON AN "AS IS" BASIS, WITHOUT WARRANTY OF 
ANY KIND, INCLUDING WITHOUT LIMITATION THE WARRANTIES THAT IT 
IS FREE OF DEFECTS, MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE 
OR NON-INFRINGING.  THE ENTIRE RISK AS TO THE QUALITY AND 
PERFORMANCE OF THE SOFTWARE IS BORNE BY LICENSEE.  SHOULD THE 
SOFTWARE PROVE DEFECTIVE IN ANY RESPECT, THE LICENSEE AND NOT 
LICENSOR ASSUMES THE ENTIRE COST OF ANY SERVICE AND REPAIR.  
THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF 
THIS AGREEMENT.  NO USE OF THE PRODUCT IS AUTHORIZED HEREUNDER 
EXCEPT UNDER THIS DISCLAIMER.

8. LIMITATION OF LIABILITY.  TO THE MAXIMUM EXTENT PERMITTED BY
APPLICABLE LAW,  IN NO EVENT WILL LICENSOR BE LIABLE FOR ANY 
INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
OUT OF THE USE OF OR INABILITY TO USE THE SOFTWARE, INCLUDING, 
WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK 
STOPPAGE, COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL 
OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF ADVISED OF THE
POSSIBILITY THEREOF. 
*/
#include <iostream.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <malloc.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/wait.h>
#include <signal.h>
#include <time.h>

#include "CifFileObj.h"


void PendingFileToCIF(const char *iFile, const char *oFile, 
		      char **idList, int nList, int idOp);

void PendingFileToStatusmmCIF(const char *iFile, char **idList, int nList, int idOp);

void CIFToPendingFile(char *iFile, const char *oFile, const char *hFile,
		      char **idList, int nList, int idOp, int stripReleased,
		      int printHead);

void ReadCIFStatus(char *iFile);
void ReadCIFStatus1(char *iFile);
void ReadCIFStatusCIF(char *iFile);
void ReadCIFidsCIF(char *iFile);

static void ndb_set_string_to_lower1(char *string);
static void ndb_clean_string1(char *string);
static char **get_sid_list(const char *fname, int ibeg, int iend, int *numSid);
static int on_list(const char *target, char **list, int nList);

char *get_attribute_value_where(CifFileObj *fobj, const char *blockId, const char *category, 
				const char *attributeA, const char *attributeB, const char *valB);
char *set_attribute_value_where(CifFileObj *fobj, const char *blockId, const char *category, const char *setValA,
				const char *attributeA, const char *attributeB, const char *valB);
char *get_attribute_value(CifFileObj *fobj, const char *blockId, const char *category, const char *attribute);
char **get_attribute_values_where(CifFileObj *fobj, const char *blockId, const char *category, 
				  const char *attributeA, const char *attributeB, const char *valB,
				  int *nValues);
char **get_attribute_values(CifFileObj *fobj, const char *blockId, const char *category, const char *attribute, 
			    int *nValues);

static void dformat_1(char *date, char *odate);
static void dformat_2(char *date, char *odate);
static void dformat_3(char *date, char *odate);
static void dformat_4(const char *date, char *odate);
static void format_text(char *text, int width);
void CIFToBCPFile1(char *iFile, const char *oFile, const char *hFile, const char *sqlFile,
		  char **idList, int nList, int idOp, int stripReleased, const char *dbName);
void CIFToBCPFile2(char *iFile, const char *oFile, const char *hFile, const char *sqlFile,
		  char **idList, int nList, int idOp, int stripReleased, const char *dbName);


static char head1[]="\
                            PROTEIN DATA BANK\n\
                   Pending/Waiting List for %s\n\
**********************************************************************\n\
*                   Explanation of status line                       *\n\
*                                                                    *\n\
* INCOMPLETE - The PDB is waiting for materials from the depositor   *\n\
*              to make a complete deposition.                        *\n\
* PROCESSING - The PDB is checking and verifying this entry.         *\n\
* DEPOSITOR  - This entry is with the depositor for approval.        *\n\
* REVIEW     - This entry is undergoing a final review.              *\n\
* REL        - This entry is in the current release.                 *\n\
* HLD        - This entry is also on hold at the present time.       *\n\
* PUB        - This entry will be released on publication.           *\n\
*              It is the responsibility of the author to notify      *\n\
*              the PDB when the article is published.                *\n\
*                                                                    *\n\
* Current status is indicated by one of the above keywords appearing *\n\
* in all UPPER CASE and surrounded by **.  Other keywords appear in  *\n\
* lower case surrounded by -- to represent the entire processing     *\n\
* time line.                                                         *\n\
*                                                                    *\n\
* The PDB releases entries during the early morning hours each       *\n\
* Wednesday, New York time.  Entries that have been placed on hold   *\n\
* by their authors are made available on the first Wednesday         *\n\
* following their hold expiration date.                              *\n\
*                                                                    *\n\
**********************************************************************\n\
";

static char head2[]="\
                            PROTEIN DATA BANK\n\
                       On hold List for %s\n\
**********************************************************************\n\
*                   Explanation of status line                       *\n\
*                                                                    *\n\
* INCOMPLETE - The PDB is waiting for materials from the depositor   *\n\
*              to make a complete deposition.                        *\n\
* PROCESSING - The PDB is checking and verifying this entry.         *\n\
* DEPOSITOR  - This entry is with the depositor for approval.        *\n\
* REVIEW     - This entry is undergoing a final review.              *\n\
* HLD        - This entry is also on hold at the present time.       *\n\
* PUB        - This entry will be released on publication.           *\n\
*              It is the responsibility of the author to notify      *\n\
*              the PDB when the article is published.                *\n\
*                                                                    *\n\
* Current status is indicated by one of the above keywords appearing *\n\
* in all UPPER CASE and surrounded by **.  Other keywords appear in  *\n\
* lower case surrounded by -- to represent the entire processing     *\n\
* time line.                                                         *\n\
*                                                                    *\n\
* The PDB releases entries during the early morning hours each       *\n\
* Wednesday, New York time.  Entries that have been placed on hold   *\n\
* by their authors are made available on the first Wednesday         *\n\
* following their hold expiration date.                              *\n\
*                                                                    *\n\
**********************************************************************\n\
";




void main(int argc, char *argv[]) {

  char *pname=NULL;
  char *iFile=NULL, *oFile=NULL, *hFile=NULL, *idFile=NULL, *sqlFile=NULL, *dbName=NULL;
  int op=0, idOp=1;
  int nList=0;
  char **idList = NULL;
  int stripReleased=0, printHead=1;

  pname = argv[0];

  if (argc < 4) {
    cerr << pname << ":  usage = -f <inputfile> -o <outputfile> -hf <holdfile> -sql <sqlfile>" << endl 
	 << "            -readcif -bnl2cif -cif2bnl -getids -idlist <idfile> -idselect -idstrip" << endl
	 << "            -striprel  -nohead -bcp -bnl2statuscif -dbname <dbname>" << endl;
    exit(1);
  }
  

  for (int i=1; i < argc; i++) {
    if ( argv[i][0] == '-' ) {
      if (strcmp(argv[i],"-f") == 0 ) {
	i++;
	iFile = (char *) calloc(strlen(argv[i]) + 1, sizeof(char));
	strcpy(iFile,argv[i]);          
      }	else if (strcmp(argv[i],"-o") == 0 ) {
	i++;
	oFile = (char *) calloc(strlen(argv[i]) + 1, sizeof(char));
	strcpy(oFile,argv[i]);          

      }	else if (strcmp(argv[i],"-hf") == 0 ) {
	i++;
	hFile = (char *) calloc(strlen(argv[i]) + 1, sizeof(char));
	strcpy(hFile,argv[i]);          

      }	else if (strcmp(argv[i],"-sql") == 0 ) {
	i++;
	sqlFile = (char *) calloc(strlen(argv[i]) + 1, sizeof(char));
	strcpy(sqlFile,argv[i]);          

      }	else if (strcmp(argv[i],"-idlist") == 0 ) {
	i++;
	idFile = (char *) calloc(strlen(argv[i]) + 1, sizeof(char));
	strcpy(idFile,argv[i]);          
      }	else if (strcmp(argv[i],"-dbname") == 0 ) {
	i++;
	dbName = (char *) calloc(strlen(argv[i]) + 1, sizeof(char));
	strcpy(dbName,argv[i]);          
      }	else if (strcmp(argv[i],"-striprel") == 0 ) {
	stripReleased = 1;
      }	else if (strcmp(argv[i],"-nohead") == 0 ) {
	printHead = 0;
      }	else if (strcmp(argv[i],"-idstrip") == 0 ) {
	idOp = 1;
      }	else if (strcmp(argv[i],"-idselect") == 0 ) {
	idOp = 0;

      }	else if (strcmp(argv[i],"-bnl2cif") == 0 ) {
	op = 1;
      }	else if (strcmp(argv[i],"-cif2bnl") == 0 ) {
	op = 2;
      }	else if (strcmp(argv[i],"-readndb") == 0 ) {
	op = 3;
      }	else if (strcmp(argv[i],"-getids") == 0 ) {
	op = 4;
      }	else if (strcmp(argv[i],"-bcp") == 0 ) {
	op = 5;
      }	else if (strcmp(argv[i],"-bcpnew") == 0 ) {
	op = 8;
      }	else if (strcmp(argv[i],"-readcif") == 0 ) {
	op = 6;
      }	else if (strcmp(argv[i],"-bnl2statuscif") == 0 ) {
	op = 7;
      } else {
	cerr << pname << ":  usage = -f <inputfile> -o <outputfile> -hf <holdfile> " << endl
	     << "       -readcif -bnl2cif -cif2bnl -getids -idlist <idfile> -idselect -idstrip" << endl
	     << "            -striprel  -nohead -bcp -bnl2statuscif -dbname <dbname>" << endl;
	exit(1);
      }
    } else {
      cerr << pname << ":  usage = -f <inputfile> -o <outputfile> -hf <holdfile>" << endl 
           << "       -readcif -bnl2cif -cif2bnl -getids -idlist <idfile> -idselect -idstrip" << endl
	   << "            -striprel  -nohead -bcp -bnl2statuscif -dbname <dbname>" << endl;
      exit(1);
    }
  }


  if (idFile != NULL && strlen(idFile))  idList=get_sid_list(idFile, 0, 1000000, &nList);

  switch( op ) {
  case 1:
    PendingFileToCIF(iFile,oFile, idList, nList, idOp);
    break;

  case 2:
    CIFToPendingFile(iFile,oFile, hFile, idList, nList, idOp, stripReleased, printHead);
    break;

  case 3:  // Extract status from NDB-CIF file and produce a CIF data items... 
    ReadCIFStatus(iFile);
    break;

  case 4:  // Extract ids from mmCIF file and produce a CIF data items... 
    ReadCIFidsCIF(iFile);
    break;

  case 5:  // Extract ids from mmCIF file and produce a CIF data items... (old schema)
    CIFToBCPFile1(iFile,oFile, hFile, sqlFile, idList, nList, idOp, stripReleased,dbName);
    break;

  case 8:  // Extract ids from mmCIF file and produce a CIF data items...  (new schema)
    CIFToBCPFile2(iFile,oFile, hFile, sqlFile, idList, nList, idOp, stripReleased,dbName);
    break;

  case 6:  // Extract status from NDB-CIF file and produce a CIF data items... 
    ReadCIFStatusCIF(iFile);
    break;

  case 7:
    PendingFileToStatusmmCIF(iFile,idList, nList, idOp);
    break;

  }
}



void ReadCIFidsCIF(char *iFile)
{

  char *diags=NULL;
  CifFileObj * fobjR = NULL;
  char *blockId=NULL;

  char *sid=NULL;
  char *pdbID = NULL;
  char fnx[L_tmpnam+1];

  tmpnam(fnx);

  fobjR = new CifFileObj(fnx, WRITE_MODE, 80, "?", 1);
  fobjR->Read(iFile,diags);
  if (diags != NULL) printf("Diags: %s\n",diags);
  fobjR->CloseFile();
  if (fobjR) delete fobjR;

  fobjR = new CifFileObj(fnx, READ_MODE);
  blockId = fobjR->GetBlockName(0);

  sid  = get_attribute_value_where(fobjR, blockId, "database_2", "database_code", 
                                      "database_id", "NDB");

  pdbID  = get_attribute_value_where(fobjR, blockId, "database_2", "database_code", 
                                      "database_id", "PDB");
  fobjR->CloseFile();
  if (fobjR) delete fobjR;
  remove(fnx);
  if (sid) 
    printf("%s ",sid);
  else
    printf("NULL");

  if (pdbID) 
    printf("%s ",pdbID);
  else
    printf("NULL");
  printf("\n");
}


void ReadCIFStatusCIF(char *iFile)
{

  char *diags=NULL;
  CifFileObj * fobjR = NULL;
  char *blockId=NULL;

  char *sid=NULL;
  char *pdbID = NULL;
  char **authors = NULL;
  char **cauthors = NULL;
  char *title = NULL;

  char *depdate = NULL;
  char *hdate = NULL;
  char *pstatus = NULL;
  char *rstatus = NULL;
  char date1[30];
  char fnx[L_tmpnam+1];
  int i, nValues=0, nValues1=0, ilen=0;

  tmpnam(fnx);

  fobjR = new CifFileObj(fnx, WRITE_MODE, 80, "?", 1);
  fobjR->Read(iFile,diags);
  if (diags != NULL) fprintf(stderr, "Diags for file %s: %s\n",iFile, diags);
  fobjR->CloseFile();
  if (fobjR) delete fobjR;

  fobjR = new CifFileObj(fnx, READ_MODE);
  blockId = fobjR->GetBlockName(0);

  sid  = get_attribute_value_where(fobjR, blockId, "database_2", "database_code", 
                                      "database_id", "RCSB");

  if (!sid)   sid  = get_attribute_value_where(fobjR, blockId, "database_2", "database_code", 
					       "database_id", "NDB");

  pdbID  = get_attribute_value_where(fobjR, blockId, "database_2", "database_code", 
                                      "database_id", "PDB");

  authors = get_attribute_values(fobjR, blockId, "audit_author", "name", &nValues);
  cauthors = get_attribute_values_where(fobjR, blockId, "citation_author", "name", 
					"citation_id", "primary", &nValues1);

  title = get_attribute_value(fobjR, blockId, "struct", "title");
  depdate = get_attribute_value(fobjR, blockId, "ndb_database_status", "recvd_initial_deposition_date");
  hdate = get_attribute_value(fobjR, blockId,   "ndb_database_status", "date_hold_coordinates");

  pstatus = get_attribute_value(fobjR, blockId, "ndb_database_status", "status_code");
  rstatus = get_attribute_value(fobjR, blockId, "ndb_database_status", "author_release_status_code");

  //loop_
  //_bnl_status.pdb_id_code
  //_bnl_status.pdb_tid_code
  //_bnl_status.title
  //_bnl_status.author_list
  //_bnl_status.pstatus
  //_bnl_status.rstatus
  //_bnl_status.date_deposit
  //_bnl_status.date_accession
  //_bnl_status.date_hold

  printf("#\n");
  printf("# ENTRY ID = %s\n", sid);

  if (pdbID != NULL && strlen(pdbID) ) {
    printf("\'%s\'\n", pdbID);
  } else {
    printf("?\n");
  }

  printf("?\n");  // tracking no

  if (title != NULL && strlen(title) ) {
    //    printf(";\n%s\n;\n", title);

    format_text(title, 70);

  } else {
    printf("?\n");
  }

  if (authors != NULL && strlen(authors[0]) ) {
    
    for (i=0; i < nValues; i++) {
      if (authors[i]) ilen+=strlen(authors[i]);
    }
    char *buf = new char[ilen+ilen];
    strcpy(buf,"");
    for (i=0; i < nValues; i++) {
      if (authors[i]) {
	strcat(buf,authors[i]);
	if (i < nValues-1) strcat(buf,", ");
      }
    }
    format_text(buf, 70);
    if (buf) delete buf;
  } else if (cauthors != NULL && strlen(cauthors[0]) ) {
    
    for (i=0; i < nValues1; i++) {
      if (cauthors[i]) ilen+=strlen(cauthors[i]);
    }
    char *buf = new char[ilen+ilen];
    strcpy(buf,"");
    for (i=0; i < nValues1; i++) {
      if (cauthors[i]) {
	strcat(buf,cauthors[i]);
	if (i < nValues1-1) strcat(buf,", ");
      }
    }
    format_text(buf, 70);
    if (buf) delete buf;
  } else {
    printf("?\n");
  }

  if (pstatus != NULL && strlen(pstatus) ) {
    printf("\'%s\'\n", pstatus);
  } else {
    printf("?\n");
  }

  if (rstatus != NULL && strlen(rstatus) ) {
    if (!strcmp(rstatus,"HOLD")) 
      printf("\'HLD\'\n");
    else if (!strcmp(rstatus,"REL")) 
      printf("\'rel\'\n");
    else if (!strcmp(rstatus,"HPUB")) 
      printf("\'PUB\'\n");
    else
      printf("\'%s\'\n", rstatus);

  } else {
    printf("?\n");
  }

  if (depdate != NULL && strlen(depdate) > 1 ) {
    memset(date1,0,30);
    //    printf("\'%s\'\n", depdate);

    if (depdate[4] == '-' && depdate[7] == '-' && ( strlen(depdate) == 10 || strlen(depdate) == 16) ) {
      dformat_2(depdate,date1);
      printf("\'%s\'\n", date1);
      printf("\'%s\'\n", date1);
    } else if (depdate[2] == '-' && depdate[6] == '-' && strlen(depdate) == 9) {
      dformat_1(depdate,date1);
      printf("\'%s\'\n", date1);
      printf("\'%s\'\n", date1);
    } else {
      fprintf(stderr,"Deposition date format for %s is noncompliant: %s\n", sid, depdate);
      printf("?\n");
      printf("?\n");
    }
  } else {
    printf("?\n");
    printf("?\n");
  }

  if (hdate != NULL && strlen(hdate) > 1 ) {
    memset(date1,0,30);

    if (hdate[4] == '-' && hdate[7] == '-' && ( strlen(hdate) >= 10 )) {
      dformat_2(hdate,date1);
      printf("\'%s\'\n", date1);
    } else if (hdate[2] == '-' && hdate[6] == '-' && strlen(hdate) == 9) {
      dformat_1(hdate,date1);
      printf("\'%s\'\n", date1);
    } else {
      fprintf(stderr,"Hold date format for %s is noncompliant: %s\n", sid, hdate);
      printf("?\n");
    }
  } else {
    printf("?\n");
  }
  remove(fnx);
  fobjR->CloseFile();
  if (fobjR) delete fobjR;
}

//


void ReadCIFStatus1(char *iFile)
{

  char *diags=NULL;
  CifFileObj * fobjR = NULL;
  char *blockId=NULL;

  char *sid=NULL;
  char *pdbID = NULL;
  char *authors = NULL;
  char *title = NULL;

  char *depdate = NULL;
  char *hdate = NULL;
  char *pstatus = NULL;
  char *rstatus = NULL;
  char date1[30];
  char fnx[L_tmpnam+1];

  tmpnam(fnx);

  fobjR = new CifFileObj(fnx, WRITE_MODE, 80, "?", 1);
  fobjR->Read(iFile,diags);
  if (diags != NULL) printf("Diags: %s\n",diags);
  fobjR->CloseFile();
  if (fobjR) delete fobjR;

  fobjR = new CifFileObj(fnx, READ_MODE);
  blockId = fobjR->GetBlockName(0);

  sid   = get_attribute_value(fobjR, blockId, "NDB_HEADER", "Structure_ID");
  pdbID = get_attribute_value(fobjR, blockId, "NDB_PDBFIL", "PDB_ID");
  authors = get_attribute_value(fobjR, blockId, "NDB_AUTHOR", "Author1");
  title = get_attribute_value(fobjR, blockId, "NDB_TITLE", "Title");
  depdate = get_attribute_value(fobjR, blockId, "NDB_HEADER", "Initial_Deposition_Date");
  hdate = get_attribute_value(fobjR, blockId, "NDB_RELEAS", "Coordinate_On_Hold_Until_Date");
  pstatus = get_attribute_value(fobjR, blockId, "NDB_NDBFIL", "Status_Code");
  rstatus = get_attribute_value(fobjR, blockId, "NDB_RECVD2", "Author_Release_Status_Code");

  printf("#\n");
  printf("# ENTRY ID = %s\n", sid);

  if (pdbID != NULL && strlen(pdbID) ) {
    printf("\'%s\'\n", pdbID);
  } else {
    printf("?\n");
  }

  printf("?\n");  // tracking no

  if (title != NULL && strlen(title) ) {
    printf(";\n%s\n;\n", title);

    //    format_text(title, 70);

  } else {
    printf("?\n");
  }

  if (authors != NULL && strlen(authors) ) {
    printf(";\n%s\n;\n", authors);
    //    format_text(authors, 70);
  } else {
    printf("?\n");
  }

  if (pstatus != NULL && strlen(pstatus) ) {
    printf("\'%s\'\n", pstatus);
  } else {
    printf("?\n");
  }

  if (rstatus != NULL && strlen(rstatus) ) {
    if (!strcmp(rstatus,"HOLD")) 
      printf("\'HLD\'\n");
    else if (!strcmp(rstatus,"REL")) 
      printf("\'rel\'\n");
    else if (!strcmp(rstatus,"HPUB")) 
      printf("\'PUB\'\n");
    else
      printf("\'%s\'\n", rstatus);

  } else {
    printf("?\n");
  }

  if (depdate != NULL && strlen(depdate) > 1 ) {
    memset(date1,0,30);
    //    dformat_1(depdate,date1);
    
    printf("\'%s\'\n", depdate);
    printf("\'%s\'\n", depdate);
  } else {
    printf("?\n");
    printf("?\n");
  }

  if (hdate != NULL && strlen(hdate) > 1 ) {
    memset(date1,0,30);
    //    dformat_2(hdate,date1);
    printf("\'%s\'\n", hdate);
  } else {
    printf("?\n");
  }
  remove(fnx);
  fobjR->CloseFile();
  if (fobjR) delete fobjR;
}


void ReadCIFStatus(char *iFile)
{

  char *diags=NULL;
  CifFileObj * fobjR = NULL;
  char *blockId=NULL;

  char *sid=NULL;
  char *pdbID = NULL;
  char *authors = NULL;
  char *title = NULL;

  char *depdate = NULL;
  char *hdate = NULL;
  char *pstatus = NULL;
  char *rstatus = NULL;
  char date1[30];
  char fnx[L_tmpnam+1];

  tmpnam(fnx);

  fobjR = new CifFileObj(fnx, WRITE_MODE, 80, "?", 1);
  fobjR->Read(iFile,diags);
  if (diags != NULL) printf("Diags: %s\n",diags);
  fobjR->CloseFile();
  if (fobjR) delete fobjR;

  fobjR = new CifFileObj(fnx, READ_MODE);
  blockId = fobjR->GetBlockName(0);

  sid   = get_attribute_value(fobjR, blockId, "NDB_HEADER", "Structure_ID");
  pdbID = get_attribute_value(fobjR, blockId, "NDB_PDBFIL", "PDB_ID");
  authors = get_attribute_value(fobjR, blockId, "NDB_AUTHOR", "Author1");
  title = get_attribute_value(fobjR, blockId, "NDB_TITLE", "Title");
  depdate = get_attribute_value(fobjR, blockId, "NDB_HEADER", "Initial_Deposition_Date");
  hdate = get_attribute_value(fobjR, blockId, "NDB_RELEAS", "Coordinate_On_Hold_Until_Date");

  pstatus = get_attribute_value(fobjR, blockId, "NDB_NDBFIL", "Status_Code");
  rstatus = get_attribute_value(fobjR, blockId, "NDB_RECVD2", "Author_Release_Status_Code");

  printf("#\n");
  printf("# ENTRY ID = %s\n", sid);

  if (pdbID != NULL && strlen(pdbID) ) {
    printf("\'%s\'\n", pdbID);
  } else {
    printf("?\n");
  }

  printf("?\n");  // tracking no

  if (title != NULL && strlen(title) ) {
    //    printf(";\n%s\n;\n", title);

    format_text(title, 70);

  } else {
    printf("?\n");
  }

  if (authors != NULL && strlen(authors) ) {
    format_text(authors, 70);
  } else {
    printf("?\n");
  }

  if (pstatus != NULL && strlen(pstatus) ) {
    printf("\'%s\'\n", pstatus);
  } else {
    printf("?\n");
  }

  if (rstatus != NULL && strlen(rstatus) ) {
    if (!strcmp(rstatus,"HOLD")) 
      printf("\'HLD\'\n");
    else if (!strcmp(rstatus,"REL")) 
      printf("\'rel\'\n");
    else if (!strcmp(rstatus,"HPUB")) 
      printf("\'PUB\'\n");
    else
      printf("\'%s\'\n", rstatus);

  } else {
    printf("?\n");
  }

  if (depdate != NULL && strlen(depdate) > 1 ) {
    memset(date1,0,30);
    //    printf("\'%s\'\n", depdate);

    if (depdate[4] == '-' && depdate[7] == '-' && ( strlen(depdate) == 10 || strlen(depdate) == 16) ) {
      dformat_2(depdate,date1);
      printf("\'%s\'\n", date1);
      printf("\'%s\'\n", date1);
    } else if (depdate[2] == '-' && depdate[6] == '-' && strlen(depdate) == 9) {
      dformat_1(depdate,date1);
      printf("\'%s\'\n", date1);
      printf("\'%s\'\n", date1);
    } else {
      fprintf(stderr,"Deposition date format for %s is noncompliant: %s\n", sid, depdate);
      printf("?\n");
      printf("?\n");
    }
  } else {
    printf("?\n");
    printf("?\n");
  }

  if (hdate != NULL && strlen(hdate) > 1 ) {
    memset(date1,0,30);

    if (hdate[4] == '-' && hdate[7] == '-' && ( strlen(hdate) >= 10 )) {
      dformat_2(hdate,date1);
      printf("\'%s\'\n", date1);
    } else if (hdate[2] == '-' && hdate[6] == '-' && strlen(hdate) == 9) {
      dformat_1(hdate,date1);
      printf("\'%s\'\n", date1);
    } else {
      fprintf(stderr,"Hold date format for %s is noncompliant: %s\n", sid, hdate);
      printf("?\n");
    }
  } else {
    printf("?\n");
  }
  remove(fnx);
  fobjR->CloseFile();
  if (fobjR) delete fobjR;
}


static void dformat_1(char *date, char *odate) {
  // reformat  dd-mmm-yy  to Mon dd, yyyy

  if (date == NULL ||  date[2] != '-' || date[6] != '-' || strlen(date) != 9) return;


  strcpy (odate,"");
  strncpy(odate,&date[3],3);
  odate[1] = tolower(odate[1]);
  odate[2] = tolower(odate[2]);

  strcat(odate," ");
  if (date[0] == '0') date[0] = ' ';
  strncat(odate,&date[0],2);

  strcat(odate," ");
  if (date[7] == '0' || date[7] == '1' || date[7] == '2' || date[7] == '3' || date[7] == '4' || date[7] == '5') {
    strcat(odate,"20");
    strncat(odate,&date[7],2);
  } else {
    strcat(odate,"19");
    strncat(odate,&date[7],2);
  }
}

static void dformat_2(char *date, char *odate) {
  //
  // reformat  CIF data = yyyy-mm-dd[:00:00]  to Mon dd, yyyy
  //                      2000-01-27  to Jan 27, 2000
  //

  if (date == NULL ||  date[4] != '-' || date[7] != '-' ) return;

  char month[5];
  memset(month,0,5);
  strncpy(month,&date[5],2);


  strcpy (odate,"");

  if (!strcmp(month,"01"))
    strcpy(odate,"Jan ");
  else   if (!strcmp(month,"02"))
    strcpy(odate,"Feb ");
  else   if (!strcmp(month,"03"))
    strcpy(odate,"Mar ");
  else   if (!strcmp(month,"04"))
    strcpy(odate,"Apr ");
  else   if (!strcmp(month,"05"))
    strcpy(odate,"May ");
  else   if (!strcmp(month,"06"))
    strcpy(odate,"Jun ");
  else   if (!strcmp(month,"07"))
    strcpy(odate,"Jul ");
  else   if (!strcmp(month,"08"))
    strcpy(odate,"Aug ");
  else   if (!strcmp(month,"09"))
    strcpy(odate,"Sep ");
  else   if (!strcmp(month,"10"))
    strcpy(odate,"Oct ");
  else   if (!strcmp(month,"11"))
    strcpy(odate,"Nov ");
  else   if (!strcmp(month,"12"))
    strcpy(odate,"Dec ");

  if (date[8] == '0') date[8] = ' ';
  strncat(odate,&date[8],2);
  strcat(odate," ");
  strncat(odate,&date[0],4);

}



static void dformat_3(char *date, char *odate) {
  //
  // reformat  CIF date = Mon dd yyyy  to  Sybase date yyyy-Mon-dd hh:mm
  //                      2000-01-27  to 2000-Jan-27 00:00  odate[18]
  //
  if (odate == NULL) return;
  strcpy(odate,"");

  if (date == NULL ||  date[3] != ' ' || date[6] != ' ' ) return;

  char month[5];
  memset(month,0,5);
  strncpy(month,&date[0],3);
  strcpy (odate,"");
  strncat(odate,&date[7],4);
  strcat(odate,"-");
  strcat(odate,month);
  strcat(odate,"-");
  strncat(odate,&date[4],2);
  if (odate[9] == ' ') odate[9] = '0';
  strcat(odate," 00:00");
}



static void dformat_4(const char *date, char *odate) {
  //
  // Mon dd yyyy to CIF date yyyy-mm-dd  odate[11]
  //
  if (odate == NULL) return;
  strcpy(odate,"");

  if (date == NULL ||  date[3] != ' ' || date[6] != ' ' ) return;

  char month[5];
  memset(month,0,5);
  strncpy(month,date,3);


  strcpy (odate,"");
  strncat(odate,&date[7],4);
  strcat(odate,"-");

  if (!strcmp(month,"Jan"))
    strcat(odate,"01");
  else   if (!strcmp(month,"Feb"))
    strcat(odate,"02");
  else   if (!strcmp(month,"Mar"))
    strcat(odate,"03");
  else   if (!strcmp(month,"Apr"))
    strcat(odate,"04");
  else   if (!strcmp(month,"May"))
    strcat(odate,"05");
  else   if (!strcmp(month,"Jun"))
    strcat(odate,"06");
  else   if (!strcmp(month,"Jul"))
    strcat(odate,"07");
  else   if (!strcmp(month,"Aug"))
    strcat(odate,"08");
  else   if (!strcmp(month,"Sep"))
    strcat(odate,"09");
  else   if (!strcmp(month,"Oct"))
    strcat(odate,"10");
  else   if (!strcmp(month,"Nov"))
    strcat(odate,"11");
  else   if (!strcmp(month,"Dec"))
    strcat(odate,"12");

  strcat(odate,"-");
  strncat(odate,&date[4],2);
  if (odate[8] ==  ' ') odate[8] = '0';

}



static void format_text(char *text, int width)
{
  int i, ibeg, iend, iend1, ilen;

  ilen = strlen(text);

  if ((ilen == 1) && text[0] == '?') {
    printf("?\n");
    return;
  }

  if (ilen < width) {
    printf(";    ");
    for (i=0; i < ilen; i++ ) {
      printf("%c", toupper(text[i]));
    }
    printf("\n;\n");
    return;
  }    

  //  printf(" strlen %d \n",strlen(text));

  // Remove newlines... 
  //
  for (i=0; i < ilen; i++) {
    if (text[i] == '\n') text[i] = ' ';
  }


  ibeg = 0;
  printf(";");
  while (ibeg < ilen-1) {
    iend = ibeg + width; 
    if (iend >= ilen - 1) {
      iend = ilen -1;

    } else {
      
      //      printf(" ilen = %d ibeg = %d iend = %d \n",ilen, ibeg,iend);
      iend1 = iend;
      while ((iend >= ibeg ) && text[iend] != ' ' && text[iend] != ',' ) {
	--iend;
      }
      if (iend ==ibeg) iend = iend1;
    }

    //    printf("Adjust ibeg = %d iend = %d \n",ibeg,iend);

    printf("   ");
    for (i=ibeg; i <= iend; i++) {
      printf("%c",toupper(text[i]));
    }
    printf("\n");
    ibeg = iend;
  }
  printf(";\n");
}


void CIFToBCPFile1(char *iFile, const char *oFile, const char *hFile, const char *sqlFile,
		      char **idList, int nList, int idOp, int stripReleased, const char *dbName)
{
  // Old schema... 
  FILE *fp1=NULL, *fp2=NULL;
  char *diags=NULL;
  CifFileObj * fobjR = NULL;
  char *blockId=NULL;
  ISTable *t = NULL;
  ReVarCifArray<CifString> * aRow = NULL;
  int i, j, nRows=0;
  char fnx[L_tmpnam+1];

  char date1[30];
  char date2[30];
  char auth[255];
  char title[255];
  char titleUC[255];
  char rStatus[100];
  char status[100];

  time_t curr_time;
  struct tm *tmptr;

  char timeString[20];

  time(&curr_time);
  tmptr = localtime(&curr_time);
  strcpy(timeString,"");
  strftime(timeString,18,"%Y-%b-%d %R", tmptr);

  tmpnam(fnx);




  fobjR = new CifFileObj(fnx, WRITE_MODE, 80, "?", 1);
  fobjR->Read(iFile,diags);
  if (diags != NULL) printf("Diags: %s\n",diags);
  fobjR->CloseFile();
  if (fobjR) delete fobjR;

  fobjR = new CifFileObj(fnx, READ_MODE);
  blockId = fobjR->GetBlockName(0);

  t = fobjR->GetTablePtr(blockId,"bnl_status");


  //
  // 
  // 0 _bnl_status.pdb_id_code  
  // 1 _bnl_status.pdb_tid_code
  // 2 _bnl_status.title
  // 3 _bnl_status.author_list
  // 4 _bnl_status.pstatus
  // 5 _bnl_status.rstatus
  // 6 _bnl_status.date_deposit
  // 7 _bnl_status.date_accession
  // 8 _bnl_status.date_hold
  //

  if (t != NULL) {
    fp1=fopen(oFile,"w");
    fp2=fopen(sqlFile,"w");
    if (dbName)
      fprintf(fp2,"use %s\n",dbName);
    else
      fprintf(fp2,"use pfndv5\n");

    fprintf(fp2,"go\n");
    nRows = t->GetNumRows();
    for (i=0; i < nRows; i++) {
      aRow = t->GetRow(i);
      //jdw
      if (nList && idOp && on_list((*aRow)[0].Text(),idList,nList))  continue;
      if (nList && !idOp && !on_list((*aRow)[0].Text(),idList,nList))  continue;
      if (!strcasecmp((*aRow)[4].Text(),"REL") && stripReleased) continue;
      if (!strcasecmp((*aRow)[5].Text(),"rel") && stripReleased) continue;
      if (!strcmp((*aRow)[0].Text(),"?")) continue;

      memset(date1,0,30);
      memset(date2,0,30);
      dformat_3((*aRow)[7].Text()  , date1);
      dformat_3((*aRow)[8].Text()  , date2);
      memset(auth,0,255);
      memset(title,0,255);
      memset(titleUC,0,255);

      strncpy(auth,(*aRow)[3].Text(),253);
      strncpy(title,(*aRow)[2].Text(),253);
      for (j = 0; j < strlen(auth); j++) {
	if (auth[j] == '\n') auth[j] = ' ';
	if (auth[j] == '\t') auth[j] = ' ';
      }
      
      for (j = 0; j < strlen(title); j++) {
	if (title[j] == '\n') auth[j] = ' ';
	if (title[j] == '\t') auth[j] = ' ';
	titleUC[j]  = toupper(title[j]);
      }
      ndb_clean_string1(auth);
      ndb_clean_string1(title);
      ndb_clean_string1(titleUC);

      // Set all to REVIEW
      if (!strcmp((*aRow)[4].Text(),"INCOMPLETE")) {
	strcpy(rStatus,"REVIEW");
      } else if (!strcmp((*aRow)[4].Text(),"PROCESSING") || !strcmp((*aRow)[4].Text(),"PROC")) {
	strcpy(rStatus,"REVIEW");
      } else if (!strcmp((*aRow)[4].Text(),"DEPOSITOR") || !strcmp((*aRow)[4].Text(),"WAIT")) {
	strcpy(rStatus,"REVIEW");
      } else if (!strcmp((*aRow)[4].Text(),"REVIEW")) {
	strcpy(rStatus,"REVIEW");
      } else {
	strcpy(rStatus,"REVIEW");
      }

      strcpy(status, (*aRow)[5].Text() );
      if (!strcmp((*aRow)[5].Text(),"PUB")) {
	strcpy(status,"HPUB");
      } else if (!strcmp((*aRow)[5].Text(),"HLD")) {
	strcpy(status,"HOLD");
      } else if (!strcmp((*aRow)[5].Text(),"rel")) {
	strcpy(status,"REL");
      }
      

      fprintf(fp1,"PDB%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n",
	      (*aRow)[0].Text(),         // id
	      (*aRow)[0].Text(),         // id	      
	      "",                        // annot
	      status,                    // status (also set to release status.. )
	      status,                    // release status
	      "BNL",                     // deposit site
	      "RCSB",                    // process site
	      date1,                     // init dep date
	      "",                        // date author approv
	      "",                        // recvd author approv
	      "",                        // approv type
	      date2,                     // date hold coord
	      "",                        // date hold struct fact
	      "",                        // date hold nmr constr
	      "",                        // date of RCSB release
	      "",                        // date deposition form
	      "",                        //  date coords
	      "",                        //  date sf 
	      "",                        // date manuscript
	      "",                        // date nmr constraints
	      "",                        // replaced entry id
	      "",                        // revision description
	      auth,                      // authors
	      title,
	      titleUC);

      fprintf(fp2,"DELETE FROM rcsb_status WHERE Structure_ID = \"PDB%s\"\n", (*aRow)[0].Text());
      fprintf(fp2,"go\n");
	      
      if (aRow) delete aRow;;
    }
    fclose(fp1);
    fclose(fp2);
  }
  if (t) delete (t);
  remove(fnx);
}


void CIFToBCPFile2(char *iFile, const char *oFile, const char *hFile, const char *sqlFile,
		      char **idList, int nList, int idOp, int stripReleased, const char *dbName)
{

  FILE *fp1=NULL, *fp2=NULL;
  char *diags=NULL;
  CifFileObj * fobjR = NULL;
  char *blockId=NULL;
  ISTable *t = NULL;
  ReVarCifArray<CifString> * aRow = NULL;
  int i, j, nRows=0;
  char fnx[L_tmpnam+1];

  char date1[30];
  char date2[30];
  char auth[255];
  char title[255];
  char titleUC[255];
  char rStatus[100];
  char status[100];

  time_t curr_time;
  struct tm *tmptr;

  char timeString[20];

  time(&curr_time);
  tmptr = localtime(&curr_time);
  strcpy(timeString,"");
  strftime(timeString,18,"%Y-%b-%d %R", tmptr);

  tmpnam(fnx);




  fobjR = new CifFileObj(fnx, WRITE_MODE, 80, "?", 1);
  fobjR->Read(iFile,diags);
  if (diags != NULL) printf("Diags: %s\n",diags);
  fobjR->CloseFile();
  if (fobjR) delete fobjR;

  fobjR = new CifFileObj(fnx, READ_MODE);
  blockId = fobjR->GetBlockName(0);

  t = fobjR->GetTablePtr(blockId,"bnl_status");


  //
  // 
  // 0 _bnl_status.pdb_id_code  
  // 1 _bnl_status.pdb_tid_code
  // 2 _bnl_status.title
  // 3 _bnl_status.author_list
  // 4 _bnl_status.pstatus
  // 5 _bnl_status.rstatus
  // 6 _bnl_status.date_deposit
  // 7 _bnl_status.date_accession
  // 8 _bnl_status.date_hold
  //

  if (t != NULL) {
    fp1=fopen(oFile,"w");
    fp2=fopen(sqlFile,"w");
    if (dbName)
      fprintf(fp2,"use %s\n",dbName);
    else
      fprintf(fp2,"use msd1\n");

    fprintf(fp2,"go\n");
    nRows = t->GetNumRows();
    for (i=0; i < nRows; i++) {
      aRow = t->GetRow(i);
      //jdw
      if (nList && idOp && on_list((*aRow)[0].Text(),idList,nList))  continue;
      if (nList && !idOp && !on_list((*aRow)[0].Text(),idList,nList))  continue;
      if (!strcasecmp((*aRow)[4].Text(),"REL") && stripReleased) continue;
      if (!strcasecmp((*aRow)[5].Text(),"rel") && stripReleased) continue;
      if (!strcmp((*aRow)[0].Text(),"?")) continue;

      memset(date1,0,30);
      memset(date2,0,30);
      dformat_3((*aRow)[7].Text()  , date1);
      dformat_3((*aRow)[8].Text()  , date2);
      memset(auth,0,255);
      memset(title,0,255);
      memset(titleUC,0,255);

      strncpy(auth,(*aRow)[3].Text(),253);
      strncpy(title,(*aRow)[2].Text(),253);
      for (j = 0; j < strlen(auth); j++) {
	if (auth[j] == '\n') auth[j] = ' ';
	if (auth[j] == '\t') auth[j] = ' ';
      }
      
      for (j = 0; j < strlen(title); j++) {
	if (title[j] == '\n') auth[j] = ' ';
	if (title[j] == '\t') auth[j] = ' ';
	titleUC[j]  = toupper(title[j]);
      }
      ndb_clean_string1(auth);
      ndb_clean_string1(title);
      ndb_clean_string1(titleUC);

      // Set all to REVIEW
      if (!strcmp((*aRow)[4].Text(),"INCOMPLETE")) {
	strcpy(rStatus,"REVIEW");
      } else if (!strcmp((*aRow)[4].Text(),"PROCESSING") || !strcmp((*aRow)[4].Text(),"PROC")) {
	strcpy(rStatus,"REVIEW");
      } else if (!strcmp((*aRow)[4].Text(),"DEPOSITOR") || !strcmp((*aRow)[4].Text(),"WAIT")) {
	strcpy(rStatus,"REVIEW");
      } else if (!strcmp((*aRow)[4].Text(),"REVIEW")) {
	strcpy(rStatus,"REVIEW");
      } else {
	strcpy(rStatus,"REVIEW");
      }

      strcpy(status, (*aRow)[5].Text() );
      if (!strcmp((*aRow)[5].Text(),"PUB")) {
	strcpy(status,"HPUB");
      } else if (!strcmp((*aRow)[5].Text(),"HLD")) {
	strcpy(status,"HOLD");
      } else if (!strcmp((*aRow)[5].Text(),"rel")) {
	strcpy(status,"REL");
      }
      

      fprintf(fp1,"PDB%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n",
	      (*aRow)[0].Text(),         // id
	      (*aRow)[0].Text(),         // id	      
	      "",                        // annot
	      status,                    // status  (also set to release status jdw 13-sep)
	      status,                    // release status
	      "BNL",                     // deposit site
	      "RCSB",                    // process site
	      date1,                     // init dep date
	      "",                        // date author approv
	      "",                        // recvd author approv
	      "",                        // approv type
	      date2,                     // date hold coord
	      "",                        // date hold struct fact
	      "",                        // date hold nmr constr
	      "",                        // date of RCSB release
	      "",                        // date deposition form
	      "",                        //  date coords
	      "",                        //  date sf 
	      "",                        // date manuscript
	      "",                        // date nmr constraints
	      "",                        // replaced entry id
	      "",                        // revision description
	      auth,                      // authors
	      title,
	      titleUC,                   //title
              timeString);               // current date 

      fprintf(fp2,"DELETE FROM rcsb_status WHERE Structure_ID = \"PDB%s\"\n", (*aRow)[0].Text());
      fprintf(fp2,"go\n");
	      
      if (aRow) delete aRow;;
    }
    fclose(fp1);
    fclose(fp2);
  }
  if (t) delete (t);
  remove(fnx);
}




void CIFToPendingFile(char *iFile, const char *oFile, const char *hFile,
		      char **idList, int nList, int idOp, int stripReleased, 
		      int printHead)
{

  FILE *fp1=NULL;
  char *diags=NULL;
  CifFileObj * fobjR = NULL;
  char *blockId=NULL;
  ISTable *t = NULL;
  ReVarCifArray<CifString> * aRow = NULL;
  int i, nRows=0;

  time_t curr_time;
  struct tm *tmptr;
  char *curt;
  char timeString[31];
  char fnx[L_tmpnam+1];
  char prvID[80];

  tmpnam(fnx);


  fobjR = new CifFileObj(fnx, WRITE_MODE, 80, "?", 1);
  fobjR->Read(iFile,diags);
  if (diags != NULL) printf("Diags for file %s: %s\n",iFile,diags);
  fobjR->CloseFile();
  if (fobjR) delete fobjR;

  fobjR = new CifFileObj(fnx, READ_MODE);
  blockId = fobjR->GetBlockName(0);

  t = fobjR->GetTablePtr(blockId,"bnl_status");


  time(&curr_time);
  tmptr = localtime(&curr_time);
  curt = ctime(&curr_time);
  if (curt[strlen(curt)-1] == '\n') curt[strlen(curt)-1] = '\0';
  strftime(timeString,30,"%b %e, %Y", tmptr);

  //
  // 
  if (t != NULL) {
    fp1=fopen(oFile,"w");
    if (printHead) {
      fprintf(fp1,head1,timeString);
      fprintf(fp1,"\n\n");
    }
    strcpy(prvID,"");
    nRows = t->GetNumRows();
    for (i=0; i < nRows; i++) {
      aRow = t->GetRow(i);
      //jdw
      if (nList && idOp && on_list((*aRow)[0].Text(),idList,nList))  continue;
      if (nList && !idOp && !on_list((*aRow)[0].Text(),idList,nList))  continue;
      //processing status
      if (!strncasecmp((*aRow)[4].Text(),"rel",3) && stripReleased) continue;
      // 13-dec-2001
      if (!strncasecmp((*aRow)[4].Text(),"obs",3)) continue;
      if (!strncasecmp((*aRow)[4].Text(),"del",3)) continue;
      if (!strncasecmp((*aRow)[4].Text(),"wdrn",4)) continue;

      if (!strncasecmp((*aRow)[5].Text(),"rel",3) && stripReleased) continue;
      if (!strncasecmp((*aRow)[5].Text(),"obs",3)) continue;
      if (!strncasecmp((*aRow)[5].Text(),"del",3)) continue;
      if (!strncasecmp((*aRow)[5].Text(),"wdrn",4)) continue;

      if ( (*aRow)[0].Length() < 4 ) {
	printf("Skipping entry with missing/bad Idcode - category row number %d\n",i+1);
	printf("Previous Idcode was %s\n",prvID);
	continue;
      }
      if ( (*aRow)[4].Length() < 2) {
	printf("Skipping entry %s with missing/bad processing status code.\n",
	       (*aRow)[0].Text());
	continue;
      }

      if ( (*aRow)[5].Length() < 2) {
	printf("Skipping entry %s with missing/bad release status code.\n",
	       (*aRow)[0].Text());
	continue;
      }


      fprintf(fp1,"Idcode:  %4s          Tracking Number:  %6s\n",
	      (*aRow)[0].Text(),
	      (*aRow)[1].Text());
      fprintf(fp1,"Entry Description:\n");
      fprintf(fp1,"%s\n",	      (*aRow)[2].Text());
      fprintf(fp1,"Authors:\n");
      fprintf(fp1,"%s\n",	      (*aRow)[3].Text());

      fprintf(fp1,"Status for %4s:   ",	      (*aRow)[0].Text());

#if 1      
      if (!strcmp((*aRow)[4].Text(),"INCOMPLETE")) {
	fprintf(fp1,"**INCOMPLETE**>--processing-->--depositor-->--review-->");
      } else if (!strcmp((*aRow)[4].Text(),"PROCESSING") || !strcasecmp((*aRow)[4].Text(),"PROC")) {
      	fprintf(fp1,"--incomplete-->**PROCESSING**>--depositor-->--review-->");
      } else if (!strcmp((*aRow)[4].Text(),"DEPOSITOR") || !strcasecmp((*aRow)[4].Text(),"WAIT")) {
      	fprintf(fp1,"--incomplete-->--processing-->**DEPOSITOR**>--review-->");
      } else if (!strcmp((*aRow)[4].Text(),"REVIEW")) {
      	fprintf(fp1,"--incomplete-->--processing-->--depositor-->**REVIEW**>");
      } else {
      	fprintf(fp1,"--incomplete-->--processing-->--depositor-->**REVIEW**>");
      }
#else
      // Set all to REVIEW
      if (!strcmp((*aRow)[4].Text(),"INCOMPLETE")) {
	fprintf(fp1,"--incomplete-->--processing-->--depositor-->**REVIEW**>");
      } else if (!strcmp((*aRow)[4].Text(),"PROCESSING") || !strcmp((*aRow)[4].Text(),"PROC")) {
      	fprintf(fp1,"--incomplete-->--processing-->--depositor-->**REVIEW**>");
      } else if (!strcmp((*aRow)[4].Text(),"DEPOSITOR") || !strcmp((*aRow)[4].Text(),"WAIT")) {
      	fprintf(fp1,"--incomplete-->--processing-->--depositor-->**REVIEW**>");
      } else if (!strcmp((*aRow)[4].Text(),"REVIEW")) {
      	fprintf(fp1,"--incomplete-->--processing-->--depositor-->**REVIEW**>");
      } else {
      	fprintf(fp1,"--incomplete-->--processing-->--depositor-->**REVIEW**>");
      }
#endif

      if (!strcasecmp((*aRow)[5].Text(),"rel")) {
	fprintf(fp1,"--rel\n");
      } else if (!strcmp((*aRow)[5].Text(),"HLD")) {
	fprintf(fp1,"**HLD\n");
	fprintf(fp1,"On hold until:  %s\n",(*aRow)[8].Text());
	if ((*aRow)[8].Length()< 2) 
	  printf("Entry %s has missing hold date.\n",(*aRow)[0].Text());

      } else if (!strcmp((*aRow)[5].Text(),"PUB")) {
	fprintf(fp1,"**PUB\n");
	fprintf(fp1,"This entry will be released on publication.\n");
      } else {
	fprintf(fp1,"**UNK\n");
	printf("Entry %s has unrecognized release status code -> %s\n",
	       (*aRow)[0].Text(),(*aRow)[5].Text());
      }
 
      fprintf(fp1,"All materials arrived as of:  %s\n",(*aRow)[6].Text());
      if ((*aRow)[6].Length()< 2) 
	  printf("Entry %s has missing deposition date.\n",(*aRow)[0].Text());
      fprintf(fp1,"Accession Date:  %s\n",(*aRow)[7].Text());
      if ((*aRow)[7].Length()< 2) 
	  printf("Entry %s has missing accession date.\n",(*aRow)[0].Text());
      fprintf(fp1,"\n\n");
      strncpy(prvID,(*aRow)[0].Text(),79);
      if (aRow) delete aRow;;
    }
    fclose(fp1);

    //

    fp1=fopen(hFile,"w");
    if (printHead) {
      fprintf(fp1,head2,timeString);
      fprintf(fp1,"\n\n");
    }
    nRows = t->GetNumRows();
    for (i=0; i < nRows; i++) {
      aRow = t->GetRow(i);
      if (strcmp((*aRow)[5].Text(),"HLD")) continue;
      if (nList && idOp && on_list((*aRow)[0].Text(),idList,nList))  continue;
      if (nList && !idOp && !on_list((*aRow)[0].Text(),idList,nList))  continue;
      //jdw... fix
      if (!strncasecmp((*aRow)[4].Text(),"rel",3) && stripReleased) continue;
      // 13-dec-2001
      if (!strncasecmp((*aRow)[4].Text(),"obs",3)) continue;
      if (!strncasecmp((*aRow)[4].Text(),"del",3)) continue;
      if (!strncasecmp((*aRow)[4].Text(),"wdrn",4)) continue;

      if (!strncasecmp((*aRow)[5].Text(),"rel",3) && stripReleased) continue;
      if (!strncasecmp((*aRow)[5].Text(),"del",3)) continue;
      if (!strncasecmp((*aRow)[5].Text(),"wdrn",4)) continue;
      if (!strncasecmp((*aRow)[5].Text(),"obs",2)) continue;

      if (!strcmp((*aRow)[0].Text(),"?")) continue;

      fprintf(fp1,"Idcode:  %4s          Tracking Number:  %6s\n",
	      (*aRow)[0].Text(),
	      (*aRow)[1].Text());
      fprintf(fp1,"Entry Description:\n");
      fprintf(fp1,"%s\n",	      (*aRow)[2].Text());
      fprintf(fp1,"Authors:\n");
      fprintf(fp1,"%s\n",	      (*aRow)[3].Text());

      fprintf(fp1,"Status for %4s:   ",	      (*aRow)[0].Text());

#if 1
      if (!strcmp((*aRow)[4].Text(),"INCOMPLETE")) {
	fprintf(fp1,"**INCOMPLETE**>--processing-->--depositor-->--review-->");
      } else if (!strcmp((*aRow)[4].Text(),"PROCESSING") || !strcasecmp((*aRow)[4].Text(),"PROC")) {
      	fprintf(fp1,"--incomplete-->**PROCESSING**>--depositor-->--review-->");
      } else if (!strcmp((*aRow)[4].Text(),"DEPOSITOR") || !strcasecmp((*aRow)[4].Text(),"WAIT")) {
      	fprintf(fp1,"--incomplete-->--processing-->**DEPOSITOR**>--review-->");
      } else if (!strcmp((*aRow)[4].Text(),"REVIEW")) {
      	fprintf(fp1,"--incomplete-->--processing-->--depositor-->**REVIEW**>");
      } else {
      	fprintf(fp1,"--incomplete-->--processing-->--depositor-->**REVIEW**>");
      }
#else
      if (!strcmp((*aRow)[4].Text(),"INCOMPLETE")) {
	fprintf(fp1,"**INCOMPLETE**>--processing-->--depositor-->--review-->");
      } else if (!strcmp((*aRow)[4].Text(),"PROCESSING")) {
      	fprintf(fp1,"--incomplete-->**PROCESSING**>--depositor-->--review-->");
      } else if (!strcmp((*aRow)[4].Text(),"DEPOSITOR")) {
      	fprintf(fp1,"--incomplete-->--processing-->**DEPOSITOR**>--review-->");
      } else if (!strcmp((*aRow)[4].Text(),"REVIEW")) {
      	fprintf(fp1,"--incomplete-->--processing-->--depositor-->**REVIEW**>");
      }
#endif
      if (!strcmp((*aRow)[5].Text(),"rel")) {
	fprintf(fp1,"--rel\n");
      } else if (!strcmp((*aRow)[5].Text(),"HLD")) {
	fprintf(fp1,"**HLD\n");
	fprintf(fp1,"On hold until:  %s\n",(*aRow)[8].Text());
      } else if (!strcmp((*aRow)[5].Text(),"PUB")) {
	fprintf(fp1,"**PUB\n");
	fprintf(fp1,"This entry will be released on publication.\n");
      } else {
	fprintf(fp1,"**UNK\n");
      }

      fprintf(fp1,"All materials arrived as of:  %s\n",(*aRow)[6].Text());
      fprintf(fp1,"Accession Date:  %s\n",(*aRow)[7].Text());
      fprintf(fp1,"\n\n");
      if (aRow) delete aRow;;
    }
    fclose(fp1);


  }
  if (t) delete (t);
  remove(fnx);
}


void PendingFileToStatusmmCIF(const char *iFile, 
		      char **idList, int nList, int idOp) {
  
  FILE *fp=NULL, *fp1=NULL;

  char line[132];

  char id[20];
  char idCode[20], trackingNo[20];
  char hpub[20];
  char hdate[20];
  char depdate[20];
  char rstatus[20];
  char status1[80];
  char pstatus[80];
  char accdate[20];

  char description[2048];
  char authors[2048];
  char tFilename[245], depdate1[18], hdate1[18];

  char *p=NULL;
  int j=0;



#if 0
   fprintf(fp1,"data_STATUS\n\n");
  fprintf(fp1,"##\n");
  fprintf(fp1,"##\n");
  fprintf(fp1,"loop_\n");
  fprintf(fp1,"_bnl_status.pdb_id_code\n");
  fprintf(fp1,"_bnl_status.pdb_tid_code\n");
  fprintf(fp1,"_bnl_status.title\n");
  fprintf(fp1,"_bnl_status.author_list\n");
  fprintf(fp1,"_bnl_status.pstatus\n");
  fprintf(fp1,"_bnl_status.rstatus\n");
  fprintf(fp1,"_bnl_status.date_deposit\n");
  fprintf(fp1,"_bnl_status.date_accession\n");
  fprintf(fp1,"_bnl_status.date_hold\n");
#endif




  fp=fopen(iFile,"r");


  fgets(line,130,fp);
  if (strlen(line)) line[strlen(line)-1] = '\0';
  while (!feof(fp)) {

    if (!strncmp(line,"Idcode: ",8)) {

      memset(idCode,0,20);
      memset(trackingNo,0,20);
      strncpy(idCode,&line[9],4);
      strncpy(trackingNo,&line[41],6);


    }

    if (!strncmp(line,"Entry Description:",18)) {
      memset(description,0,2048);
      strcpy(description,"");
      fgets(line,130,fp);
      //      if (strlen(line)) line[strlen(line)-1] = '\0';
      while (!feof(fp) && strncmp(line,"Authors:",8)) {
	strcat(description,line);
	fgets(line,130,fp);
	//	if (strlen(line)) line[strlen(line)-1] = '\0';
      }
    }

    if (!strncmp(line,"Authors:",8)) {
      memset(authors,0,2048);
      strcpy(authors,"");
      fgets(line,130,fp);
      //      if (strlen(line)) line[strlen(line)-1] = '\0';
      while (!feof(fp) && strncmp(line,"Status for ",11)) {
	strcat(authors,line);
	fgets(line,130,fp);
	//	if (strlen(line)) line[strlen(line)-1] = '\0';
      }
    }

    //  Get stuff starting from - Status for:
    if (!strncmp(line,"Status for ",11)) {
      if (strlen(line) && line[strlen(line)-1] == '\n') line[strlen(line)-1] = '\0';
      memset(id,0,20);
      memset(accdate,0,20);
      memset(hpub,0,20);
      memset(hdate,0,20);      
      memset(depdate,0,20);
      memset(status1,0,80);
      memset(pstatus,0,80);
      memset(rstatus,0,20);

      strncpy(id,&line[11],4);

      //      strcpy(status1,&line[19]);
      strcpy(status1,line);

      strcpy(rstatus,&status1[strlen(status1)-3]);
      p = strtok(status1,"*");
      p = strtok(NULL,"*");
      if (p) strcpy(pstatus,p);

      for (int i=0; i < 3; i++) {
	fgets(line,130,fp);
	if (strlen(line)) line[strlen(line)-1] = '\0';
	if (!strncmp(line,"Accession Date: ",16)) {
	  strcpy(accdate,&line[17]);
	}
	if (!strncmp(line,"This ",5)) {
	  strcpy(hpub,"HPUB");
	}
	if (!strncmp(line,"All materials arrived as of:",28)) {
	  strcpy(depdate,&line[30]);
	}
	if (!strncmp(line,"On hold ",8)) {
	  strcpy(hdate,&line[16]);
	}
      }



      // Write out the CIF data items....

     if ( !nList || (nList && idOp && !on_list(idCode,idList,nList)) ||
	  (nList && !idOp && on_list(idCode,idList,nList))) {
       j++;

	dformat_4(hdate,hdate1);
	dformat_4(depdate,depdate1);
	strcpy(tFilename,"PDB");
	strcat(tFilename,idCode);
	strcat(tFilename,".cif");
	ndb_set_string_to_lower1(tFilename);
	fp1=fopen(tFilename,"w");

	fprintf(fp1,"data_PDB%s\n\n",idCode);
	fprintf(fp1,"##\n");
	fprintf(fp1,"##\n");
	fprintf(fp1,"loop_\n");
	fprintf(fp1,"_database_2.database_id\n");
	fprintf(fp1,"_database_2.database_code\n");
        fprintf(fp1,"PDB   %s\n",idCode);
	fprintf(fp1,"RCSB  PDB%s\n",idCode);
	fprintf(fp1,"BNL   \'%s\'\n",trackingNo);
#
	if (strlen(description)) description[strlen(description)-1] = '\0';
	ndb_clean_string1(description);
	fprintf(fp1,"##\n");
	fprintf(fp1,"_struct.entry_id         PDB%s\n",idCode);
	fprintf(fp1,"_struct.title\n;%s\n;\n",description);
#
	fprintf(fp1,"##\n");
	fprintf(fp1,"##\n");
	fprintf(fp1,"_ndb_database_status.entry_id                           PDB%s\n",idCode);
	fprintf(fp1,"_ndb_database_status.status_code                        \'%s\'\n",pstatus);
	fprintf(fp1,"_ndb_database_status.ndb_tid                            ?\n");
	fprintf(fp1,"_ndb_database_status.status_coordinates_in_NDB          ?\n");
	fprintf(fp1,"_ndb_database_status.recvd_deposit_form                 ?\n");
	fprintf(fp1,"_ndb_database_status.date_deposition_form               ?\n");
	fprintf(fp1,"_ndb_database_status.recvd_coordinates                  Y\n");
	fprintf(fp1,"_ndb_database_status.date_coordinates                   \'%s\'\n",depdate1);
	fprintf(fp1,"_ndb_database_status.recvd_struct_fact                  ?\n");
	fprintf(fp1,"_ndb_database_status.date_struct_fact                   ?\n");
	fprintf(fp1,"_ndb_database_status.recvd_internal_approval            ?\n");
	fprintf(fp1,"_ndb_database_status.recvd_nmr_constraints              ?\n");
	fprintf(fp1,"_ndb_database_status.date_nmr_constraints               ?\n");
	fprintf(fp1,"_ndb_database_status.recvd_manuscript                   ?\n");
	fprintf(fp1,"_ndb_database_status.date_manuscript                    ?\n");
	if (strlen(authors)) authors[strlen(authors)-1] = '\0';
	ndb_clean_string1(authors);
	fprintf(fp1,"_ndb_database_status.name_depositor\n");
	fprintf(fp1,";%s\n;\n",authors);
	fprintf(fp1,"_ndb_database_status.rcsb_annotator                     UNASSIGNED\n");
	fprintf(fp1,"_ndb_database_status.recvd_author_approval              ?\n");
        fprintf(fp1,"_ndb_database_status.author_approval_type               ?\n");
	fprintf(fp1,"_ndb_database_status.date_author_approval               ?\n");
	fprintf(fp1,"_ndb_database_status.recvd_initial_deposition_date      \'%s\'\n",depdate1);
	fprintf(fp1,"_ndb_database_status.date_submitted                     ?\n");
	fprintf(fp1,"_ndb_database_status.author_release_status_code         \'%s\'\n",rstatus);
	fprintf(fp1,"_ndb_database_status.date_revised                       ?\n");
	fprintf(fp1,"_ndb_database_status.revision_id                        ?\n");
	fprintf(fp1,"_ndb_database_status.replaced_entry_id                  ?\n");
	fprintf(fp1,"_ndb_database_status.revision_description               ?\n");
	fprintf(fp1,"_ndb_database_status.date_of_NDB_release                ?\n");
	fprintf(fp1,"_ndb_database_status.date_released_to_PDB               ?\n");
	fprintf(fp1,"_ndb_database_status.date_of_PDB_release                ?\n");
	fprintf(fp1,"_ndb_database_status.date_hold_coordinates              \'%s\'\n",hdate1);
	fprintf(fp1,"_ndb_database_status.date_hold_struct_fact              ?\n");
	fprintf(fp1,"_ndb_database_status.date_hold_nmr_constraints          ?\n");

	if (!strcmp(hpub,"HPUB")) 
	  fprintf(fp1,"_ndb_database_status.hold_for_publication               Y\n");
	else
	  fprintf(fp1,"_ndb_database_status.hold_for_publication               N\n");

	fprintf(fp1,"_ndb_database_status.dep_release_code_coordinates       ?\n");
	fprintf(fp1,"_ndb_database_status.dep_release_code_struct_fact       ?\n");
	fprintf(fp1,"_ndb_database_status.dep_release_code_nmr_constraints   ?\n");
	fprintf(fp1,"_ndb_database_status.pdb_date_of_author_approval        ?\n");
	fprintf(fp1,"_ndb_database_status.deposit_site                       BNL\n");
	fprintf(fp1,"_ndb_database_status.process_site                       BNL\n");

	fprintf(fp1,"_ndb_database_status.bnl_backlog_category               ?\n");
	fprintf(fp1,"_ndb_database_status.bnl_backlog_state                  ?\n");
	fprintf(fp1,"_ndb_database_status.bnl_layer1_flag                    N\n");
# 
	fprintf(fp1,"##\n");
	fprintf(fp1,"##\n");
	fclose(fp1);
      }
    }
    fgets(line,130,fp);
    if (strlen(line)) line[strlen(line)-1] = '\0';
  }

  printf("done %d\n",j); 
  fclose(fp);

}



void PendingFileToCIF(const char *iFile, const char *oFile,
		      char **idList, int nList, int idOp) {
  
  FILE *fp=NULL, *fp1=NULL;

  char line[132];

  char id[20];
  char idCode[20], trackingNo[20];
  char hpub[20];
  char hdate[20];
  char depdate[20];
  char rstatus[20];
  char status1[80];
  char pstatus[80];
  char accdate[20];

  char description[2048];
  char authors[2048];
  char *p=NULL;
  int j=0;

  fp1=fopen(oFile,"w");
  fprintf(fp1,"data_BNLPENDING\n\n");
  fprintf(fp1,"##\n");
  fprintf(fp1,"##\n");
  fprintf(fp1,"loop_\n");
  fprintf(fp1,"_bnl_status.pdb_id_code\n");
  fprintf(fp1,"_bnl_status.pdb_tid_code\n");
  fprintf(fp1,"_bnl_status.title\n");
  fprintf(fp1,"_bnl_status.author_list\n");
  fprintf(fp1,"_bnl_status.pstatus\n");
  fprintf(fp1,"_bnl_status.rstatus\n");
  fprintf(fp1,"_bnl_status.date_deposit\n");
  fprintf(fp1,"_bnl_status.date_accession\n");
  fprintf(fp1,"_bnl_status.date_hold\n");



  fp=fopen(iFile,"r");


  fgets(line,130,fp);
  if (strlen(line)) line[strlen(line)-1] = '\0';
  while (!feof(fp)) {

    if (!strncmp(line,"Idcode: ",8)) {

      memset(idCode,0,20);
      memset(trackingNo,0,20);
      strncpy(idCode,&line[9],4);
      strncpy(trackingNo,&line[41],6);


    }

    if (!strncmp(line,"Entry Description:",18)) {
      memset(description,0,2048);
      strcpy(description,"");
      fgets(line,130,fp);
      //      if (strlen(line)) line[strlen(line)-1] = '\0';
      while (!feof(fp) && strncmp(line,"Authors:",8)) {
	strcat(description,line);
	fgets(line,130,fp);
	//	if (strlen(line)) line[strlen(line)-1] = '\0';
      }
    }

    if (!strncmp(line,"Authors:",8)) {
      memset(authors,0,2048);
      strcpy(authors,"");
      fgets(line,130,fp);
      //      if (strlen(line)) line[strlen(line)-1] = '\0';
      while (!feof(fp) && strncmp(line,"Status for ",11)) {
	strcat(authors,line);
	fgets(line,130,fp);
	//	if (strlen(line)) line[strlen(line)-1] = '\0';
      }
    }

    //  Get stuff starting from - Status for:
    if (!strncmp(line,"Status for ",11)) {
      if (strlen(line) && line[strlen(line)-1] == '\n') line[strlen(line)-1] = '\0';
      memset(id,0,20);
      memset(accdate,0,20);
      memset(hpub,0,20);
      memset(hdate,0,20);      
      memset(depdate,0,20);
      memset(status1,0,80);
      memset(pstatus,0,80);
      memset(rstatus,0,20);

      strncpy(id,&line[11],4);

      //      strcpy(status1,&line[19]);
      strcpy(status1,line);

      strcpy(rstatus,&status1[strlen(status1)-3]);
      p = strtok(status1,"*");
      p = strtok(NULL,"*");
      if (p) strcpy(pstatus,p);

      for (int i=0; i < 3; i++) {
	fgets(line,130,fp);
	if (strlen(line)) line[strlen(line)-1] = '\0';
	if (!strncmp(line,"Accession Date: ",16)) {
	  strcpy(accdate,&line[17]);
	}
	if (!strncmp(line,"This ",5)) {
	  strcpy(hpub,"HPUB");
	}
	if (!strncmp(line,"All materials arrived as of:",28)) {
	  strcpy(depdate,&line[30]);
	}
	if (!strncmp(line,"On hold ",8)) {
	  strcpy(hdate,&line[16]);
	}
      }


#if 0
      if (strlen(hpub))
	fprintf(fp1,"%s: %s %12s %s %s\n", id, accdate, hpub, rstatus, pstatus);
      else       if (strlen(hdate))
	fprintf(fp1,"%s: %s %12s %s %s\n", id, accdate, hdate, rstatus, pstatus);
      else
	fprintf(fp1,"%s: %s %12s %s %s\n",id, accdate, " ", rstatus, pstatus);
#endif

      // Write out the CIF data items....

      if ( !nList || (nList && idOp && !on_list(idCode,idList,nList)) ||
         (nList && !idOp && on_list(idCode,idList,nList))) {
	j++;
	fprintf(fp1,"\'%s\'\n",idCode);
	fprintf(fp1,"\'%s\'\n",trackingNo);
	if (strlen(description)) description[strlen(description)-1] = '\0';
	fprintf(fp1,";%s\n;\n",description);
	if (strlen(authors)) authors[strlen(authors)-1] = '\0';
	fprintf(fp1,";%s\n;\n",authors);
	fprintf(fp1,"\'%s\'\n",pstatus);
	fprintf(fp1,"\'%s\'\n",rstatus);
	fprintf(fp1,"\'%s\'\n",depdate);
	fprintf(fp1,"\'%s\'\n",accdate);
	fprintf(fp1,"\'%s\'\n",hdate);
      }
    }
    fgets(line,130,fp);
    if (strlen(line)) line[strlen(line)-1] = '\0';
  }

  fprintf(fp1,"##\n");
  fprintf(fp1,"##\n");
  fclose(fp1);
  printf("done %d\n",j); 
  fclose(fp);

}

//


void ndb_name_conversion(char *string)
{
  int i;
  if (string == NULL || string[0] == '\0') return;

  char first[50], last[50];

  for (i = strlen(string) - 1; i >= 0; i--)
            if (string[i] == ',') break;

  if (i > 0) {
    strcpy(first, &string[i+1]);
    string[i] = '\0';
    strcpy(last, string);
    //            ndb_clean_string1(first);
    //            ndb_clean_string1(last);
    if (first[strlen(first)-1] == '.')
      sprintf(string, "%s%s\0", first, last);
    else sprintf(string, "%s %s\0", first, last);
  }
}

static int on_list(const char *target, char **list, int nList)  {
  int i, iMatch;


  iMatch = 0;
  for (i=0; i < nList; i++) {
    if (!strcasecmp(target,list[i])) {
      iMatch = 1;
      break;
    }
  }
  return(iMatch);

}


static char **get_sid_list(const char *fname, int ibeg, int iend, int *numSid)
{
  FILE *fp1;
  int i, j, nsid;
  char **list=NULL;
  char sid[100];
  
  *numSid=0;
  if ((fp1 = fopen(fname, "r")) == NULL) {
    printf("Unable to open %s",fname);
    return(list);
  }
  
  i=nsid=0;
  fgets(sid,99,fp1);
  while (!feof(fp1)) {
    i++;
    if ((i >= ibeg) && (i <= iend))   nsid++;
    fgets(sid,99,fp1); 
  }
  fclose(fp1);
  if (nsid == 0) return(list);

  list = (char **) calloc(nsid+1,sizeof(char*));
  for (i=0; i<nsid+1; i++) {
    list[i] = (char *) calloc(100,sizeof(char));
  } 
  
  if ((fp1 = fopen(fname, "r")) == NULL) {
    printf("Unable to open %s",fname);
    return(list);
  }
  
  i=j=0;
  while (!feof(fp1)) {
    i++;
    if ((i >= ibeg) && (i <= iend)) {
      memset(list[j],0,100);
      fgets(list[j],99,fp1); 
      ndb_clean_string1(list[j]);
      j++;
    } else {
      fgets(sid,99,fp1); 
    }
  }
  fclose(fp1);
  *numSid=nsid; 
  return(list);
}


/*
 *  ndb_clean_string1() strip leading and trailing white space from a string.
 *  Also compress multiple internal white space characters.
 *
 */
static void ndb_clean_string1(char *string)
{
  int i, stringlen;

  if (string == NULL) return;
  stringlen = strlen(string);
  for (i=0; i< stringlen; i++) { 
    if (string[i] == ' ' || string[i] == '\t' ||  string[i] == '\n') {
      strcpy(string, &string[i+1]);     
      stringlen--;
      i--;
    }
    else break;
  }
  for (i=0; i< stringlen; i++) {
    if (i != 0 && 
        (string[i-1] == ' ' || string[i-1] == '\t' || string[i-1] == '\n')  && 
        (string[i]   == ' ' || string[i]   == '\t' || string[i]   == '\n' )) {
      strcpy(&string[i-1], &string[i]);
      i--;
      stringlen--;
    }
  }
  stringlen = strlen(string);
  for (i=0; i< stringlen; i++) {
    if (string[i] == '\t' || string[i] == '\n' ) {
      string[i] = ' ';
    }
  }
  for (i=stringlen -1; i>= 0; i--) {
    if (string[i] == ' ' || string[i] == '\t' ||  string[i] == '\n')
      string[i] = '\0';
    else break;
  }
}


//

char *get_attribute_value(CifFileObj *fobj, const char *blockId, const char *category, const char *attribute)
{
  ISTable *t=NULL;

  ReVarCifArray<CifString> *col=NULL;
  int nRow, iCol;
  char *string=NULL;

  //  ndb_log_message_text(NDB_MSG_DEBUG,"Searching for %s and %s",category,attribute);

  if (fobj->IsTablePresent(blockId,category)) {
    t=fobj->GetTablePtr(blockId,category);
    nRow = t->GetNumRows();
    // ndb_log_message_text(NDB_MSG_DEBUG,"Got %d rows in %s",nRow,category);
    if (nRow > 0) {
      iCol = t->GetColumnIndex(attribute);
      // ndb_log_message_text(NDB_MSG_DEBUG,"Attribute %s has index %d",attribute,iCol);
      if (iCol >= 0) {
	col=t->GetColumn(iCol);
	if (col && ((*col)[0].Length() > 0) ) {
	  string = new char[(*col)[0].Length() + 1];
	  strcpy(string,(*col)[0].Text());
	  //  ndb_log_message_text(NDB_MSG_DEBUG,"Attribute %s has value %s",attribute,string);
	}
      }
    }
  }
  return(string);
}


char **get_attribute_values(CifFileObj *fobj, const char *blockId, const char *category, const char *attribute,
			    int *nValues)
{
  ISTable *t=NULL;

  ReVarCifArray<CifString> *col=NULL;
  int nRow, iCol;
  char **strings=NULL;

  //  ndb_log_message_text(NDB_MSG_DEBUG,"Searching for %s and %s",category,attribute);

  if (fobj->IsTablePresent(blockId,category)) {
    t=fobj->GetTablePtr(blockId,category);
    nRow = t->GetNumRows();
    // ndb_log_message_text(NDB_MSG_DEBUG,"Got %d rows in %s",nRow,category);
    if (nRow > 0) {
      iCol = t->GetColumnIndex(attribute);
      // ndb_log_message_text(NDB_MSG_DEBUG,"Attribute %s has index %d",attribute,iCol);
      if (iCol >= 0) {
	col=t->GetColumn(iCol);
	if (col && (col->Length() > 0) ) {
	  strings = new char *[ col->Length() + 1];
	  for (int i=0;  i < col->Length(); i++) {
	    strings[i] = new char[(*col)[i].Length() + 1];
	    strcpy(strings[i],(*col)[i].Text());
	  }
	  *nValues = col->Length();
	}
      }
    }
  }
  return(strings);
}




//* JDW ----

char *set_attribute_value_where(CifFileObj *fobj, const char *blockId, const char *category, const char *setValA,
				const char *attributeA, const char *attributeB, const char *valB)
{
  ISTable *t=NULL;

  ReVarCifArray<CifString> *col=NULL;
  ReVarPCifArray<int>  *iResult=NULL;
  int nRow, iCol1, iCol2, errCode;
  char *string=NULL;
  CifString cs;

  //   ndb_log_message_text(NDB_MSG_DEBUG,"Searching for %s and %s where %s = %s",
  //		       category,attributeA, attributeB, valB);
  t=fobj->GetTablePtr(blockId,category);

  if (t != NULL) {
    nRow = t->GetNumRows();
    //    ndb_log_message_text(NDB_MSG_DEBUG,"Got %d rows in %s",nRow,category);
    if (nRow > 0) {
      iCol1 = t->GetColumnIndex(attributeB);
      iCol2 = t->GetColumnIndex(attributeA);
      //      ndb_log_message_text(NDB_MSG_DEBUG,"iCol1 %d iCol2 %d",iCol1, iCol2);
      if ( !t->isIndexed(iCol1) ) {     // make column searchable if required ... 
	ReVarCifArray<CifString> *col0 = t->GetColumn(iCol1);
	t->RemoveColumn(iCol1); t->InsertColumn(attributeB,iCol1,1);
	t->FillColumn(*col0,iCol1);
      }
      //      ndb_log_message_text(NDB_MSG_DEBUG,"iCol1 %d can be searched",iCol1);
      cs.Clear(); cs.Copy(valB);
      iResult = t->SearchColumn(cs, iCol1, errCode);
      if (!errCode && iResult && (iResult->Length() > 0)) {
	//ndb_log_message_text(NDB_MSG_DEBUG,"Found %d values of %s matching %s",iResult->Length(),
	//			     attributeB, valB);
	
	col = t->GetSubRow((*iResult)[0],iCol2, iCol2);

	string = new char[(*col)[0].Length() + 1];
	strcpy(string,(*col)[0].Text());
	//	ndb_log_message_text(NDB_MSG_DEBUG,"Attribute %s has value %s",attributeA,string);
      } else {
	//	ndb_log_message_text(NDB_MSG_DEBUG,"Search returned nothing");
      }
    }
  }
  return(string);
}


char *get_attribute_value_where(CifFileObj *fobj, const char *blockId, const char *category, 
				const char *attributeA, const char *attributeB, const char *valB)
{
  ISTable *t=NULL;

  ReVarCifArray<CifString> *col=NULL;
  ReVarPCifArray<int>  *iResult=NULL;
  int nRow, iCol1, iCol2, errCode;
  char *string=NULL;
  CifString cs;

//   ndb_log_message_text(NDB_MSG_DEBUG,"Searching for %s and %s where %s = %s",
  //		       category,attributeA, attributeB, valB);
  t=fobj->GetTablePtr(blockId,category);

  if (t != NULL) {
    nRow = t->GetNumRows();
//    ndb_log_message_text(NDB_MSG_DEBUG,"Got %d rows in %s",nRow,category);
    if (nRow > 0) {
      iCol1 = t->GetColumnIndex(attributeB);
      iCol2 = t->GetColumnIndex(attributeA);
//      ndb_log_message_text(NDB_MSG_DEBUG,"iCol1 %d iCol2 %d",iCol1, iCol2);
      if ( !t->isIndexed(iCol1) ) {     // make column searchable if required ... 
	ReVarCifArray<CifString> *col0 = t->GetColumn(iCol1);
	t->RemoveColumn(iCol1); t->InsertColumn(attributeB,iCol1,1);
	t->FillColumn(*col0,iCol1);
      }
//      ndb_log_message_text(NDB_MSG_DEBUG,"iCol1 %d can be searched",iCol1);
      cs.Clear(); cs.Copy(valB);
      iResult = t->SearchColumn(cs, iCol1, errCode);
      if (!errCode && iResult && (iResult->Length() > 0)) {
//	ndb_log_message_text(NDB_MSG_DEBUG,"Found %d values of %s matching %s",iResult->Length(),
//			     attributeB, valB);
	col = t->GetSubRow((*iResult)[0],iCol2, iCol2);
	string = new char[(*col)[0].Length() + 1];
	strcpy(string,(*col)[0].Text());
//	ndb_log_message_text(NDB_MSG_DEBUG,"Attribute %s has value %s",attributeA,string);
      } else {
//	ndb_log_message_text(NDB_MSG_DEBUG,"Search returned nothing");
      }
    }
  }
  return(string);
}



char **get_attribute_values_where(CifFileObj *fobj, const char *blockId, const char *category, 
				  const char *attributeA, const char *attributeB, const char *valB,
				  int *nValues)
{
  ISTable *t=NULL;

  ReVarCifArray<CifString> *col=NULL;
  ReVarPCifArray<int>  *iResult=NULL;
  int nRow, iCol1, iCol2, errCode;
  char **strings=NULL;
  CifString cs;

  //  ndb_log_message_text(NDB_MSG_DEBUG,"Searching for %s and %s where %s = %s",
  //		       category,attributeA, attributeB, valB);
  t=fobj->GetTablePtr(blockId,category);

  if (t != NULL) {
    nRow = t->GetNumRows();
    //    ndb_log_message_text(NDB_MSG_DEBUG,"Got %d rows in %s",nRow,category);
    if (nRow > 0) {
      iCol1 = t->GetColumnIndex(attributeB);
      iCol2 = t->GetColumnIndex(attributeA);
      //      ndb_log_message_text(NDB_MSG_DEBUG,"iCol1 %d iCol2 %d",iCol1, iCol2);
      if ( !t->isIndexed(iCol1) ) {     // make column searchable if required ... 
	ReVarCifArray<CifString> *col0 = t->GetColumn(iCol1);
	t->RemoveColumn(iCol1); t->InsertColumn(attributeB,iCol1,1);
	t->FillColumn(*col0,iCol1);
      }
      //      ndb_log_message_text(NDB_MSG_DEBUG,"iCol1 %d can be searched",iCol1);
      cs.Clear(); cs.Copy(valB);
      iResult = t->SearchColumn(cs, iCol1, errCode);
      if (!errCode && iResult && (iResult->Length() > 0)) {
	//	ndb_log_message_text(NDB_MSG_DEBUG,"Found %d values of %s matching %s",iResult->Length(),
	//	     attributeB, valB);
	col = t->GetSubColumnByIndex(iCol2,iResult);
	//	col = t->GetSubRow((*iResult)[0],iCol2, iCol2);

	strings = new char *[ iResult->Length() + 1];
	for (int i=0;  i < iResult->Length(); i++) {
	  strings[i] = new char[(*col)[i].Length() + 1];
	  strcpy(strings[i],(*col)[i].Text());
	  //	  ndb_log_message_text(NDB_MSG_DEBUG,"Attribute %s  %d  has value %s",attributeA,i,strings[i]);
	}
	*nValues = iResult->Length();
      } else {
	//	ndb_log_message_text(NDB_MSG_DEBUG,"Search returned nothing");
	*nValues = 0;
      }
    }
  }
  return(strings);
}

static void ndb_set_string_to_lower1(char *string)
{
  int i, str_len;

  if (string == NULL) return;
  str_len = strlen(string);
  for (i=0; i<str_len; i++)
    string[i] = tolower(string[i]);
}
