/*
FILE:     adit-session-driver.C
*/
/*
VERSION:  0.904-beta
*/
/*
DATE:     8/14/2003
*/
/*
  Comments and Questions to: sw-help@rcsb.rutgers.edu
*/
/*
COPYRIGHT 1999-2003 Rutgers - The State University of New Jersey

This software is provided WITHOUT WARRANTY OF MERCHANTABILITY OR
FITNESS FOR A PARTICULAR PURPOSE OR ANY OTHER WARRANTY, EXPRESS OR
IMPLIED.  RUTGERS MAKE NO REPRESENTATION OR WARRANTY THAT THE
SOFTWARE WILL NOT INFRINGE ANY PATENT, COPYRIGHT OR OTHER
PROPRIETARY RIGHT.

The user of this software shall indemnify, hold harmless and defend
Rutgers, its governors, trustees, officers, employees, students,
agents and the authors against any and all claims, suits,
losses, liabilities, damages, costs, fees, and expenses including
reasonable attorneys' fees resulting from or arising out of the
use of this software.  This indemnification shall include, but is
not limited to, any and all claims alleging products liability.
*/
/*
               PDB SOFTWARE LICENSE AGREEMENT

BY CLICKING THE ACCEPTANCE BUTTON OR INSTALLING OR USING 
THIS "SOFTWARE, THE INDIVIDUAL OR ENTITY LICENSING THE  
SOFTWARE ("LICENSEE") IS CONSENTING TO BE BOUND BY AND IS 
BECOMING A PARTY TO THIS AGREEMENT.  IF LICENSEE DOES NOT 
AGREE TO ALL OF THE TERMS OF THIS AGREEMENT
THE LICENSEE MUST NOT INSTALL OR USE THE SOFTWARE.

1. LICENSE AGREEMENT

This is a license between you ("Licensee") and the Protein Data Bank (PDB) 
at Rutgers, The State University of New Jersey (hereafter referred to 
as "RUTGERS").   The software is owned by RUTGERS and protected by 
copyright laws, and some elements are protected by laws governing 
trademarks, trade dress and trade secrets, and may be protected by 
patent laws. 

2. LICENSE GRANT

RUTGERS grants you, and you hereby accept, non-exclusive, royalty-free 
perpetual license to install, use, modify, prepare derivative works, 
incorporate into other computer software, and distribute in binary 
and source code format, or any derivative work thereof, together with 
any associated media, printed materials, and on-line or electronic 
documentation (if any) provided by RUTGERS (collectively, the "SOFTWARE"), 
subject to the following terms and conditions: (i) any distribution 
of the SOFTWARE shall bind the receiver to the terms and conditions 
of this Agreement; (ii) any distribution of the SOFTWARE in modified 
form shall clearly state that the SOFTWARE has been modified from 
the version originally obtained from RUTGERS.  

2. COPYRIGHT; RETENTION OF RIGHTS.  

The above license grant is conditioned on the following: (i) you must 
reproduce all copyright notices and other proprietary notices on any 
copies of the SOFTWARE and you must not remove such notices; (ii) in 
the event you compile the SOFTWARE, you will include the copyright 
notice with the binary in such a manner as to allow it to be easily 
viewable; (iii) if you incorporate the SOFTWARE into other code, you 
must provide notice that the code contains the SOFTWARE and include 
a copy of the copyright notices and other proprietary notices.  All 
copies of the SOFTWARE shall be subject to the terms of this Agreement.  

3. NO MAINTENANCE OR SUPPORT; TREATMENT OF ENHANCEMENTS 

RUTGERS is under no obligation whatsoever to: (i) provide maintenance 
or support for the SOFTWARE; or (ii) to notify you of bug fixes, patches, 
or upgrades to the features, functionality or performance of the 
SOFTWARE ("Enhancements") (if any), whether developed by RUTGERS 
or third parties.  If, in its sole discretion, RUTGERS makes an 
Enhancement available to you and RUTGERS does not separately enter 
into a written license agreement with you relating to such bug fix, 
patch or upgrade, then it shall be deemed incorporated into the SOFTWARE 
and subject to this Agreement. You are under no obligation whatsoever 
to provide any Enhancements to RUTGERS or the public that you may 
develop over time; however, if you choose to provide your Enhancements 
to RUTGERS, or if you choose to otherwise publish or distribute your 
Enhancements, in source code form without contemporaneously requiring 
end users or RUTGERS to enter into a separate written license agreement 
for such Enhancements, then you hereby grant RUTGERS a non-exclusive,
royalty-free perpetual license to install, use, modify, prepare
derivative works, incorporate into the SOFTWARE or other computer
software, distribute, and sublicense your Enhancements or derivative
works thereof, in binary and source code form.

4. FEES.  There is no license fee for the SOFTWARE.  If Licensee
wishes to receive the SOFTWARE on media, there may be a small charge
for the media and for shipping and handling.  Licensee is
responsible for any and all taxes.

5. TERMINATION.  Without prejudice to any other rights, Licensor
may terminate this Agreement if Licensee breaches any of its terms
and conditions.  Upon termination, Licensee shall destroy all
copies of the SOFTWARE.

6. PROPRIETARY RIGHTS.  Title, ownership rights, and intellectual
property rights in the Product shall remain with RUTGERS.  Licensee 
acknowledges such ownership and intellectual property rights and will 
not take any action to jeopardize, limit or interfere in any manner 
with RUTGERS' ownership of or rights with respect to the SOFTWARE.  
The SOFTWARE is protected by copyright and other intellectual 
property laws and by international treaties.  Title and related 
rights in the content accessed through the SOFTWARE is the property 
of the applicable content owner and is protected by applicable law.  
The license granted under this Agreement gives Licensee no rights to such
content.

7. DISCLAIMER OF WARRANTY.  THE SOFTWARE IS PROVIDED FREE OF 
CHARGE, AND, THEREFORE, ON AN "AS IS" BASIS, WITHOUT WARRANTY OF 
ANY KIND, INCLUDING WITHOUT LIMITATION THE WARRANTIES THAT IT 
IS FREE OF DEFECTS, MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE 
OR NON-INFRINGING.  THE ENTIRE RISK AS TO THE QUALITY AND 
PERFORMANCE OF THE SOFTWARE IS BORNE BY LICENSEE.  SHOULD THE 
SOFTWARE PROVE DEFECTIVE IN ANY RESPECT, THE LICENSEE AND NOT 
LICENSOR ASSUMES THE ENTIRE COST OF ANY SERVICE AND REPAIR.  
THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF 
THIS AGREEMENT.  NO USE OF THE PRODUCT IS AUTHORIZED HEREUNDER 
EXCEPT UNDER THIS DISCLAIMER.

8. LIMITATION OF LIABILITY.  TO THE MAXIMUM EXTENT PERMITTED BY
APPLICABLE LAW,  IN NO EVENT WILL LICENSOR BE LIABLE FOR ANY 
INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
OUT OF THE USE OF OR INABILITY TO USE THE SOFTWARE, INCLUDING, 
WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK 
STOPPAGE, COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL 
OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF ADVISED OF THE
POSSIBILITY THEREOF. 
*/
/*
  PURPOSE:      Top-level session driver module.

*/


#include <iostream.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "CifBuilder.h"
#include "CifFileObj.h"
#include "CifViewObj.h"
#include "CifStateObj.h"

#include "ndb_htmllib.h"
#include "ndb_misclib.h"

#define  HTML_ENV_GLOBAL_INIT
#include "interface-defs.h"


//
// --------------------------------------------------------------------------------
//                   Text for deposition server (X-ray)
//
static char p0_en[]="\
<h3>Running an ADIT Session</h3> \n\
<UL> \n\
<LI>Enter the name of your coordinate file.  (Use the <B>Browse...</B> button to find your file.) \n\
<LI>Select the file type (PDB or mmCIF) \n\
<LI>    <b>Depositors are urged to submit structure factors according to <a href=\"http://journals.iucr.org/d/issues/2000/01/00/gr0998/index.html\">IUCR guidelines.</a></b><br> \n\
   Enter the name of your structure factor file, if available.  (Use the <B>Browse...</B> button to find your file.) \n\
<LI>Select the file type (mmCIF or Other ASCII)<BR> \n\
Please note that SFCHECK can only be run on structure factor files in mmCIF format. \n\
<LI>Select the ADIT Operation you wish to perform:  precheck, validation, or deposition. \n\
<LI>Press the <font COLOR=\"blue\" size=4>BEGIN</font> button to start this operation. \n\
</UL> \n\
";


static char p1_en[]="\
<h3>About ADIT</h3> \n\
 \n\
An ADIT session involves three steps: a data format precheck, a \n\
validation precheck, and the actual deposition of the structural data \n\
to the database.<P> \n\
 \n\
In the first step, the data format precheck confirms that the data \n\
files you enter are in a format that can be automatically processed by \n\
our software.  \n\
 \n\
The format and required data items for coordinate files are described for the \n\
<a href=\"/adit/docs/pdb_atom_format.html\">PDB</a> \n\
and <a href=\"/adit/docs/mmcif_atom_format.html\">mmCIF</a> formats.   \n\
The required mmCIF data \n\
items for <a href=\"/adit/docs/mmcif_sf_format.html\">structure factor data</a> \n\
are also described. \n\
 \n\
The precheck will produce a brief report identifying any changes that need  \n\
to be made in your data files in order to obtain a validation report or to deposit \n\
your structure.<P> \n\
 \n\
In the second step, the validation precheck creates a validation \n\
report for the structure which includes an RCSB Atlas page and \n\
stereochemical checks. The validation may take a few minutes depending \n\
on the size of your structure, so please be patient.<P> \n\
 \n\
The third and final step is the deposition of your structure to the \n\
database. A WWW editor is provided to help you enter information about \n\
your structure.<P> \n\
<a href=\"/adit/docs/tutorial.html\">ADIT Tutorials</a><P> \n\
";

//
// --------------------------------------------------------------------------------
//                   Text for deposition server (NMR)
//
static char p0_en_nmr[]="\
<h3>Running an ADIT Session</h3> \n\
<UL> \n\
<LI>Enter the name of your coordinate file.  (Use the <B>Browse...</B> button to find your file.) \n\
<LI>Select the file type (PDB or mmCIF) \n\
<LI><b>Depositors are strongly encouraged to submit constraint files. </b><br> \n\
    Enter the name of your constraint file, if available.  (Use the <B>Browse...</B> button to find your file.) \n\
<LI>You may provide files containing coupling constants and chemical shift data,  \n\
    and we will pass this information on to the BioMagResBank to be included in their archive. \n\
<LI>Select the ADIT Operation you wish to perform:  precheck, validation, or deposition. \n\
<LI>Press the <font COLOR=\"blue\" size=4>BEGIN</font> button to start this operation. \n\
</UL> \n\
";


static char p1_en_nmr[]="\
<h3>About ADIT</h3> \n\
 \n\
An ADIT session involves three steps: a data format precheck, a \n\
validation precheck, and the actual deposition of the structural data \n\
to the database.<P> \n\
 \n\
In the first step, the data format precheck confirms that the data \n\
files you enter are in a format that can be automatically processed by \n\
our software.  \n\
 \n\
The format and required data items for coordinate files are described for the \n\
<a href=\"/adit/docs/pdb_atom_format.html\">PDB</a> \n\
and <a href=\"/adit/docs/mmcif_atom_format.html\">mmCIF</a> formats.   \n\
 \n\
The precheck will produce a brief report identifying any changes that need  \n\
to be made in your data files in order to obtain a validation report or to deposit \n\
your structure.<P> \n\
 \n\
In the second step, the validation precheck creates a validation \n\
report for the structure which includes an RCSB Atlas page and \n\
stereochemical checks. The validation may take a few minutes depending \n\
on the size of your structure, so please be patient.<P> \n\
 \n\
The third and final step is the deposition of your structure to the \n\
database. A WWW editor is provided to help you enter information about \n\
your structure.<P> \n\
<a href=\"/adit/docs/tutorial.html\">ADIT Tutorials</a><P> \n\
";

// --------------------------------------------------------------------------------
//                   Text for deposition server (X-ray)
//
#if 0
static char p1_en[]="\
<p>A deposition session involves three steps: a data format precheck, a validation \n\
precheck, and the actual deposition of the structural data to the database.  \n\
<p> \n\
In the first step, the data format precheck confirms that the data files you \n\
enter are in a format that can be automatically processed by our software. \n\
The format and required data items for coordinate files are described for the \n\
<a href=\"/adit/docs/pdb_atom_format.html\">PDB</a> \n\
and <a href=\"/adit/docs/mmcif_atom_format.html\">mmCIF</a> formats.   \n\
The required mmCIF data \n\
items for <a href=\"/adit/docs/mmcif_sf_format.html\">structure factor data</a> are also described. \n\
The precheck will produce a brief report identifying any \n\
changes thatneed to be made in your data files in order to obtain a validation \n\
report or to deposit your structure. \n\
<p>In the second step, the validation precheck creates a validation report for the \n\
structure which includes an RCSB Atlas page and stereochemical checks.  \n\
The validation may take a few minutes depending on the size of your structure, \n\
so please be patient. \n\
<p>The third and final step is the deposition of your structure to the \n\
database. A WWW editor is provided to help you enter information about your \n\
structure. \n\
<p> \n\
To start a new deposition session, enter the names of your files, choosing \n\
the formats from the pull-down menus.  Select the operation you wish to preform \n\
(precheck, validate or deposit) and then press the BEGIN button. \n\
<p> \n\
A <a href=\"/adit/docs/tutorial.html\">tutorial</a> describing the use of the ADIT system is available.\n\
";
//
static char p1_enX[]="\
<p>The RCSB ADIT system is an \n\
alternative to e-mail deposition.  A deposition session involves three \n\
steps:  a data format precheck, a validation precheck, and the actual deposition of the \n\
structural data to the database. \n\
The first step is a precheck which confirms that the data files you enter are in a format that \n\
can be automatically processed by our software.   The required data items \n\
and format for coordinate data are described for the <a href=\"/adit/docs/pdb_atom_format.html\">PDB</a> \n\
and <a href=\"/adit/docs/mmcif_atom_format.html\">mmCIF</a> formats.   The required mmCIF data \n\
items for <a href=\"/adit/docs/mmcif_sf_format.html\">structure factor data</a> are also described. \n\
<p> \n\
The precheck will produce a brief report identifying any changes that need \n\
to be made in your data files in order to obtain a validation report or to \n\
deposit your structure.  \n\
The second step creates a validation report for the structure which includes \n\
an RCSB Atlas page and stereochemical checks.  \n\
The validation may take a few minutes depending on the size of your structure, so please be patient. \n\
The third and final step is the deposition of your structure to the database.  \n\
A WWW editor is provided to help you enter information about your structure. \n\
<P>\n\
To start a new deposition session, enter the names and formats of your data files, \n\
select  precheck or validate or deposit, and then press the <font COLOR=\"blue\" size=4>BEGIN</font> button. \n\
<p> \n\
A <a href=\"/adit/docs/tutorial.html\">tutorial</a> describing the use of the ADIT system is available.\n\
";
#endif

static char p1a_en[]="\
<p>To continue working with this set of data files, \n\
select precheck, validate or deposit from the <b>Operation</b> menu, and then press the <font COLOR=\"blue\" size=4>BEGIN</font> button. \n\
To begin a new session with another set of data files, select  <b>Adit Home Page</b> above. \n\
<p> \n\
";

//
// --------------------------------------------------------------------------------
//                   Text for validation server (X-ray)
//

static char v0_en[]="\
<h3>Running the ADIT Validation Server</h3> \n\
<UL> \n\
<LI>Enter the name of your coordinate file.  (Use the <B>Browse...</B> button to find your file.) \n\
<LI>Select the file type (PDB or mmCIF) \n\
<LI>Enter the name of your structure factor file, if available.  (Use the <B>Browse...</B> button to find your file.) \n\
<LI>Select the file type (mmCIF or Other ASCII)<BR> \n\
Please note that SFCHECK can only be run on structure factor files in mmCIF format. \n\
<LI>Select the operation you wish to perform:  precheck or validation. \n\
<LI>Press the <font COLOR=\"blue\" size=4>BEGIN</font> button to start this operation. \n\
    Please be patient, creating the validation report may take several minutes. \n\
</UL> \n\
";


static char v0_en_nmr[]="\
<h3>Running the ADIT Validation Server</h3> \n\
<UL> \n\
<LI>Enter the name of your coordinate file.  (Use the <B>Browse...</B> button to find your file.) \n\
<LI>Select the file type (PDB or mmCIF) \n\
<LI>Enter the name of your constraint file, if available.  (Use the <B>Browse...</B> button to find your file.) \n\
<LI>Select the operation you wish to perform:  precheck or validation. \n\
<LI>Press the <font COLOR=\"blue\" size=4>BEGIN</font> button to start this operation. \n\
    Please be patient, creating the validation report may take several minutes. \n\
</UL> \n\
";

static char v2_en[]="\
<h3>About the ADIT Validation Server</h3> \n\
 \n\
<p>There are two steps required to use this validation server. \n\
The first step is a precheck which confirms that the data files you enter are in a format that \n\
can be automatically processed by our validation software.   The required data items \n\
and format for coordinate data are described for the <a href=\"/adit/docs/pdb_atom_format.html\">PDB</a> \n\
and <a href=\"/adit/docs/mmcif_atom_format.html\">mmCIF</a> formats.   The required mmCIF data \n\
items for <a href=\"/adit/docs/mmcif_sf_format.html\">structure factor data</a> are also described. \n\
The precheck will produce a brief report identifying any changes that need \n\
to be made in your data files in order to obtain a validation report.\n\
<p> \n\
The second step creates a validation report for the structure which includes \n\
an RCSB Atlas page and stereochemical checks.  \n\
The validation may take a few minutes depending on the size of your structure, so please be patient. \n\
To produce a report, enter the names of data files, select the file format, \n\
select precheck or validate, and then press the <font COLOR=\"blue\" size=4>BEGIN</font> button. \n\
<p> \n\
A <a href=\"/adit/docs/tutorial.html\">tutorial</a> describing the use of ADIT is available.\n\
";

static char v2_en_nmr[]="\
<h3>About the ADIT Validation Server</h3> \n\
 \n\
<p>There are two steps required to use this validation server. \n\
The first step is a precheck which confirms that the data files you enter are in a format that \n\
can be automatically processed by our validation software.   The required data items \n\
and format for coordinate data are described for the <a href=\"/adit/docs/pdb_atom_format.html\">PDB</a> \n\
and <a href=\"/adit/docs/mmcif_atom_format.html\">mmCIF</a> formats.   \n\
The precheck will produce a brief report identifying any changes that need \n\
to be made in your data files in order to obtain a validation report.\n\
<p> \n\
The second step creates a validation report for the structure which includes \n\
an RCSB Atlas page and stereochemical checks.  \n\
The validation may take a few minutes depending on the size of your structure, so please be patient. \n\
To produce a report, enter the names of data files, select the file format, \n\
select precheck or validate, and then press the <font COLOR=\"blue\" size=4>BEGIN</font> button. \n\
<p> \n\
A <a href=\"/adit/docs/tutorial.html\">tutorial</a> describing the use of ADIT is available.\n\
";


// ---------------------------------------------------------------------------------------------

static char p2a_en[]="\
<p>To continue working with this set of data files, \n\
select precheck or validate from the <b>Operation</b> menu, and then press the <font COLOR=\"blue\" size=4>BEGIN</font> button. \n\
To begin a new session with another set of data files, select <b>Validate Home Page</b> above. \n\
<p> \n\
";


//
// ---------------------------------------------------------------------------------------------
// 
static char *root_file_name(char *path);
static int get_index_in_list(const char **list, const char *target);
   
//static char TBL_BG_COLOR[]="#c0c0c0";
static char TBL_BG_COLOR[]="#9999cc";

//static char *formats1[]={"pdb","cif",""};
//static char *formats1Display[]={"PDB","mmCIF",""};

static char *formats1[]={"cif","cifeps","pdb",""};
static char *formats1Display[]={"mmCIF","mmCIF (Enc. PDB)","PDB",""};

static char *formats2[]={"cifeps","cif","pdb",""};
static char *formats2Display[]={"mmCIF (Enc. XYZ)","mmCIF","PDB",""};

#if 0
static char *formatsSF[]={"cif","x-plor","cns","mtz","other",""};
static char *formatsSFDisplay[]={"mmCIF","X-PLOR","CNS","MTZ","other",""};
#endif

static char *formatsSF[]={"cif","other",""};
static char *formatsSFDisplay[]={"mmCIF","Other ASCII",""};

static char *formatsNC[]={"any",""};
static char *formatsNCDisplay[]={"any",""};

static char *aditOps[]={"precheck","validate","edit",""};
static char *aditOpsDisplay[]={"Precheck","Validate","Edit",""};

static char *aditOps1[]={"precheck","validate","edit",""};
static char *aditOpsDisplay1[]={"Precheck","Validate","Input Tool",""};



static char *valOps[]={"precheck","validate",""};
static char *valOpsDisplay[]={"Precheck","Validate",""};

//static char eMail[]="annotator@rcsb.rutgers.edu";
//static char dbName[]="msd1";

//static char eMailHelp[]="annotator@rcsb.rutgers.edu";


void write_top_session_screen_deposit_1(FILE *io, CifStateObj *sobj, const char *ufid, 
					const char *projDir);
void write_top_session_screen_deposit_2(FILE *io, CifStateObj *sobj, const char *ufid, 
				      const char *projDir);

void write_top_session_screen_validate_1(FILE *io, CifStateObj *sobj, const char *ufid,
					 const char *projDir);
void write_top_session_screen_validate_2(FILE *io, CifStateObj *sobj, const char *ufid,
					 const char *projDir);



int main(int argc, char ** argv) {
  entry entries[HT_MAX_ENTRIES];
  int  nentries;
  char  *prog_name=NULL;
  char *ufid=NULL, *expmeth=NULL, *moltype=NULL, *context=NULL;
  char *projDir=NULL, *dbname=NULL;
  CifStateObj * sobj = NULL;
  char dirPath[CIF_MAX_NAME_LENGTH], statePath[CIF_MAX_NAME_LENGTH];  
  char stateFile[CIF_MAX_NAME_LENGTH];  
  char uname[CIF_MAX_NAME_LENGTH];  
  int filesLoaded;

  prog_name = argv[0];
  nentries = ht_decode_cgi_query(entries);
  ht_set_httpd_environment(entries,nentries);

  ht_set_server_url(HT_SERVER_NAME);
  ht_set_server_cgi_path(HT_CGI_PATH);
  ndb_open_log(prog_name,HT_LOGFILE_NAME);
  ndb_set_debug_level(NDB_GLOBAL_DEBUG_LEVEL);
  
  ufid      = ht_get_keyword_value(HT_ATTRIBUTE_UNIQUE_ID,entries,nentries);
  context   = ht_get_keyword_value(HT_ATTRIBUTE_CONTEXT,entries,nentries);
  expmeth   = ht_get_keyword_value(HT_ATTRIBUTE_EXP_METH,entries,nentries);
  moltype   = ht_get_keyword_value(HT_ATTRIBUTE_MOL_TYPE,entries,nentries);
  projDir   = ht_get_keyword_value(HT_ATTRIBUTE_PROJDIR,entries,nentries);
  dbname    = ht_get_keyword_value(HT_ATTRIBUTE_DBNAME,entries,nentries);

  ndb_log_message_text(NDB_MSG_DEBUG,"Got %d entries",nentries);
  for (int i=0; i < nentries; i++) {
    ndb_log_message_text(NDB_MSG_DEBUG,"adit-session_driver: entry=%d name=%s val=%s file=%s isfile=%d",
			 i,
			 entries[i].name,
			 entries[i].val,
			 entries[i].file,
			 entries[i].isFile);
  }
  
  ndb_log_message_text(NDB_MSG_DEBUG,"Got ufid     %s",ufid);
  ndb_log_message_text(NDB_MSG_DEBUG,"Got context  %s",context);
  ndb_log_message_text(NDB_MSG_DEBUG,"Got moltype  %s",HT_MOL_TYPE);
  ndb_log_message_text(NDB_MSG_DEBUG,"Got expmeth  %s",HT_EXP_METHOD);
  ndb_log_message_text(NDB_MSG_DEBUG,"Got projDir  %s",projDir);

  // Check for existing state
  
  if (ufid) {
    strcpy(dirPath,HT_SESSION_ROOT_PATH);
    strcat(dirPath,ufid);
    strcat(dirPath,"/");

    if (html_advise_missing_directory(stdout, dirPath, HT_HOME_PATH)) exit(0);

    //
    //jdw 12-apr-99 block restart of deposit sessions if deposited.
    //
    if (HT_APPL_CONTEXT_TYPE == HT_CONTEXT_DEPOSIT) {
      html_display_redeposit_block_message(stdout, ufid, dirPath, RCSB_STYLE_DEFAULT);
    }

    strcpy(statePath,dirPath);
    strcat(statePath,HT_STATE_FILE);
    
    sobj = new CifStateObj(statePath, "state", APPEND_MODE, 0);
    if (sobj && !expmeth) {
      expmeth=sobj->GetSessionExpMethod();
      strcpy(HT_EXP_METHOD,expmeth);
    }
    if (sobj && !moltype) {
      moltype=sobj->GetSessionMolType();
      strcpy(HT_MOL_TYPE,moltype);
    }
    filesLoaded = 1;


  } else {
    //    
    //   Create a new session and temporary directories and initialize state object ...
    //
    ufid = ht_get_unique_name();
    strcpy(uname,HT_SESSION_ROOT_PATH);
    strcat(uname,ufid);
    ndb_log_message_text(NDB_MSG_DEBUG,"Starting session %s",ufid);
    ndb_log_message_text(NDB_MSG_DEBUG,"Creating session directory %s",uname);
    mkdir(uname,0770);

    strcpy(uname,HT_SESSION_TMP_ROOT_PATH);
    strcat(uname,ufid);
    strcat(uname,"/");
    mkdir(uname,0770);
    ndb_log_message_text(NDB_MSG_DEBUG,"Creating session temporary directory %s",uname);

    strcpy(dirPath,HT_SESSION_ROOT_PATH);
    strcat(dirPath,ufid);
    strcat(dirPath,"/");

    strcpy(statePath,dirPath);
    strcat(statePath,HT_STATE_FILE);

    sobj = new CifStateObj(statePath, "state", WRITE_MODE, 0);
    sobj->SetSessionId(ufid);
    sobj->SetSessionExpMethod(HT_EXP_METHOD);
    sobj->SetSessionMolType(HT_MOL_TYPE);
    sobj->SetSessionContext(context);
    filesLoaded = 0;
    sobj->Close();
    delete sobj;

    sobj = new CifStateObj(statePath, "state", APPEND_MODE, 0);
  }

  //
  // Project directory depends on: moltype and expmeth
  //
  projDir = ht_get_projdir(moltype,expmeth);
  if (projDir) strcpy(HT_ARCHIVE_DIRECTORY,projDir);

  //
  // DB also depends on: moltype and expmeth
  //
  if (!dbname) {
    dbname = ht_get_dbname(moltype,expmeth);
    // set the global database name ... 
    if (dbname) strcpy(HT_TARGET_DBNAME,dbname);
  }

  ndb_log_message_text(NDB_MSG_DEBUG,"Got ufid   : %s",ufid);
  ndb_log_message_text(NDB_MSG_DEBUG,"Got context: %s",context);
  ndb_log_message_text(NDB_MSG_DEBUG,"Got projDir: %s",projDir);
  ndb_log_message_text(NDB_MSG_DEBUG,"Got dbname : %s",dbname);

  if (!strcmp(context,"validate")) {

    if (filesLoaded)
      write_top_session_screen_validate_2(stdout, sobj, ufid, projDir);
    else
      write_top_session_screen_validate_1(stdout, sobj, ufid, projDir);

  } else {

    if (filesLoaded)
      write_top_session_screen_deposit_2(stdout, sobj, ufid, projDir);
    else
      write_top_session_screen_deposit_1(stdout, sobj, ufid, projDir);

  }
  
  if (sobj) {
    ndb_log_message_text(NDB_MSG_DEBUG,"adit-session-driver: Updating state");
    //    sobj->Update();
    strcpy(stateFile,dirPath);
    strcat(stateFile,"stateObj.cif");
    ndb_log_message_text(NDB_MSG_DEBUG,"adit-session-driver: Wrote state %s",stateFile);
    sobj->Write(stateFile);
    sobj->Close();
  }
    
  ndb_close_log();    
  exit(0);
}


//
void write_top_session_screen_deposit_1(FILE *io, CifStateObj *sobj, const char *ufid, 
				      const char *projDir)
{
  /*
   *  Provides file upload options ... 
   */
  char *fileid=NULL,   *formatType=NULL;
  char *fileidSF=NULL, *formatTypeSF=NULL;
  char *fileidNC=NULL, *formatTypeNC=NULL;
  char *fileidDF=NULL, *lastOp=NULL, *sidCode=NULL;
  char *fileidCC=NULL, *fileidCS=NULL;

  int i, nRow;

  if (sobj) {
    fileid       = sobj->GetSessionFileCoords();
    formatType   = sobj->GetSessionFormatCoords();
    fileidSF     = sobj->GetSessionFileSF();
    formatTypeSF = sobj->GetSessionFormatSF();
    fileidNC     = sobj->GetSessionFileNC();
    formatTypeNC = sobj->GetSessionFormatNC();
    fileidDF     = sobj->GetSessionFileDF();
    fileidCC     = sobj->GetSessionFileCC();
    fileidCS     = sobj->GetSessionFileCS();
    sidCode      = sobj->GetSessionSIDCode();
    lastOp       = sobj->GetSessionOperation();
  }
  ndb_log_message_text(NDB_MSG_DEBUG,"In write_top_session_screen_deposit_1 *************");
  ndb_log_message_text(NDB_MSG_DEBUG,"fileid:       %s",fileid);
  ndb_log_message_text(NDB_MSG_DEBUG,"formatType:   %s",formatType);
  ndb_log_message_text(NDB_MSG_DEBUG,"fileidSF:     %s",fileidSF);
  ndb_log_message_text(NDB_MSG_DEBUG,"formatTypeSF: %s",formatTypeSF);
  ndb_log_message_text(NDB_MSG_DEBUG,"fileidNC:     %s",fileidNC);
  ndb_log_message_text(NDB_MSG_DEBUG,"formatTypeNC: %s",formatTypeNC);
  ndb_log_message_text(NDB_MSG_DEBUG,"fileidDF:     %s",fileidDF);
  ndb_log_message_text(NDB_MSG_DEBUG,"fileidCC:     %s",fileidCC);
  ndb_log_message_text(NDB_MSG_DEBUG,"fileidCS:     %s",fileidCS);
  ndb_log_message_text(NDB_MSG_DEBUG,"sidCode:      %s",sidCode);


  
  ht_put_content_type_target(io,"_top");
  html_display_open_page(io,"Deposition Options", RCSB_STYLE_DEFAULT);

  //jdw blank title... 

  html_display_page_title_logo(io,"", RCSB_STYLE_DEFAULT);
  html_display_return_to_page(io, "Adit Home Page", HT_HOME_PATH);


  if (!strcmp(HT_EXP_METHOD,"NMR")) {
    fprintf(io,"%s",p0_en_nmr);
  } else {
    fprintf(io,"%s",p0_en);
  }

  //
  //    
  //
  fprintf(io,"<FORM METHOD=\"POST\" ENCTYPE=\"multipart/form-data\" ACTION=\"%s%s%s\" TARGET=\"_top\">\n",
	  HT_SERVER_NAME,
	  HT_CGI_PATH,
	  HT_INTERFACE_OPERATION_DRIVER);


  fprintf(io,"<table cellpadding=10 bgcolor=\"%s\">\n",TBL_BG_COLOR);

  //
  // Coordinates ... 
  //

  fprintf(io,"<tr><td align=center><b>Coordinate<BR>File: <b></td>\n");
  fprintf(io,"<td><INPUT name=\"%s\" type=file  size=\"23\" value=\"%s\"></td>\n",
	  HT_ATTRIBUTE_FILE_ID,  fileid, fileid);

  fprintf(io,"<td align=center><b>File<BR>Type: <b></td>\n");
  fprintf(io,"<td><SELECT NAME=\"%s\" WIDTH=10>\n",HT_ATTRIBUTE_FORMAT_TYPE);

  if (HT_APPL_CONTEXT_TYPE == HT_CONTEXT_DEPOSIT) {  // Restrict deposited formats... 
    i=0;
    while ( strlen(formats1Display[i]) ) {
      if (formatType &&  !strcmp(formats1[i],formatType)) 
	fprintf(io,"<OPTION SELECTED VALUE=\"%s\">%s\n",formats1[i],formats1Display[i]);
      else
	fprintf(io,"<OPTION VALUE=\"%s\">%s\n",formats1[i],formats1Display[i]);
      i++;
    }
  } else {
    i=0;
    while ( strlen(formats2Display[i]) ) {
      if (formatType &&  !strcmp(formats2[i],formatType)) 
	fprintf(io,"<OPTION SELECTED VALUE=\"%s\">%s\n",formats2[i],formats2Display[i]);
      else
	fprintf(io,"<OPTION VALUE=\"%s\">%s\n",formats2[i],formats2Display[i]);
      i++;
    }
  }
  fprintf(io,"</SELECT></td>\n");

  nRow = 3;
  if (HT_APPL_CONTEXT_TYPE == HT_CONTEXT_ASSEMBLE) nRow = 4;

  // JDW - CC and CS files ...
  if (!strcmp(HT_EXP_METHOD,"NMR")) nRow+=2;

  fprintf(io,"<td align=center rowspan=%d><font COLOR=\"blue\" size=4>\n",nRow);
  fprintf(io,"<INPUT TYPE=\"SUBMIT\" VALUE=\"BEGIN\"></font></td></tr>\n");

  //
  // NMR Constraints or Structure factors
  //

  if (!strcmp(HT_EXP_METHOD,"NMR")) {
    //
    // Constraints ... 
    //
    fprintf(io,"<tr><td align=center><b>Constraint<BR>File: <b></td>\n");
    fprintf(io,"<td><INPUT name=\"%s\" type=file  size=\"23\" value=\"%s\"></td>\n",
	    HT_ATTRIBUTE_FILE_ID_NC,  fileidNC, fileidNC);
    
    fprintf(io,"<td align=center><b>File<BR>Type: <b></td>\n");
    fprintf(io,"<td><SELECT NAME=\"%s\" WIDTH=10>\n",HT_ATTRIBUTE_FORMAT_TYPE_NC);
    i=0;
    while ( strlen(formatsNCDisplay[i]) ) {
      if (formatTypeNC &&  !strcmp(formatsNC[i],formatTypeNC)) 
	fprintf(io,"<OPTION SELECTED VALUE=\"%s\">%s\n",formatsNC[i],formatsNCDisplay[i]);
      else
	fprintf(io,"<OPTION VALUE=\"%s\">%s\n",formatsNC[i],formatsNCDisplay[i]);
      i++;
    }
    fprintf(io,"</SELECT></td></tr>\n");

#if 1
    //  JDW -17-APR-99
    //  NMR - Coupling constants and chemical shift
    //
    
    fprintf(io,"<tr><td align=center><b>Coupling<BR>Constants: <b></td>\n");
    fprintf(io,"<td colspan=3 align=left><INPUT name=\"%s\" type=file  size=\"23\" value=\"%s\"></td>\n",
	    HT_ATTRIBUTE_FILE_ID_CC, fileidCC, fileidCC);
    fprintf(io,"</tr>");

    fprintf(io,"<tr><td align=center><b>Chemical<BR>Shifts: <b></td>\n");
    fprintf(io,"<td colspan=3 align=left><INPUT name=\"%s\" type=file  size=\"23\" value=\"%s\"></td>\n",
	    HT_ATTRIBUTE_FILE_ID_CS, fileidCS, fileidCS);
    fprintf(io,"</tr>");
#endif

  } else if (!strcmp(HT_EXP_METHOD,"X-ray")) {
    //
    // Structure factors  ... 
    //
    fprintf(io,"<tr><td align=center><b>Structure<br>Factor<BR>File: <b></td>\n");
    fprintf(io,"<td><INPUT name=\"%s\" type=file  size=\"23\" value=\"%s\"></td>\n",
	    HT_ATTRIBUTE_FILE_ID_SF,  fileidSF, fileidSF);
    
    fprintf(io,"<td align=center><b>File<BR>Type: <b></td>\n");
    fprintf(io,"<td><SELECT NAME=\"%s\" WIDTH=10>\n",HT_ATTRIBUTE_FORMAT_TYPE_SF);
    i=0;
    while ( strlen(formatsSFDisplay[i]) ) {
      if (formatTypeSF &&  !strcmp(formatsSF[i],formatTypeSF)) 
	fprintf(io,"<OPTION SELECTED VALUE=\"%s\">%s\n",formatsSF[i],formatsSFDisplay[i]);
      else
	fprintf(io,"<OPTION VALUE=\"%s\">%s\n",formatsSF[i],formatsSFDisplay[i]);
      i++;
    }
    fprintf(io,"</SELECT></td></tr>\n");
  }
  //
  // Deposition form ...
  //
  if (HT_APPL_CONTEXT_TYPE == HT_CONTEXT_ASSEMBLE) {
    fprintf(io,"<tr><td align=center><b>Deposition<BR>Form: <b></td>\n");
    fprintf(io,"<td colspan=3 align=left><INPUT name=\"%s\" type=file  size=\"23\" value=\"%s\"></td>\n",
	    HT_ATTRIBUTE_FILE_ID_DF, fileidDF, fileidDF);
    fprintf(io,"</tr>");
  }

  //  
  //  Operation ...
  //


  if (HT_APPL_CONTEXT_TYPE == HT_CONTEXT_DEPOSIT) {
    fprintf(io,"<tr><td align=center><b>Operation: <b></td>\n");
    fprintf(io,"<td colspan=4 align=left><SELECT NAME=\"%s\" WIDTH=10>\n",HT_ATTRIBUTE_OPERATION);
    i=0;
    while ( strlen(aditOpsDisplay[i]) ) {
      if (lastOp &&  !strcmp(aditOps[i],lastOp)) 
	fprintf(io,"<OPTION SELECTED VALUE=\"%s\">%s\n",aditOps[i],aditOpsDisplay[i]);
      else
	fprintf(io,"<OPTION VALUE=\"%s\">%s\n",aditOps[i],aditOpsDisplay[i]);
      i++;
    }
    fprintf(io,"</SELECT></td></tr>\n");
  } else {
    fprintf(io,"<tr><td align=center><b>Operation: <b></td>\n");
    fprintf(io,"<td align=left><SELECT NAME=\"%s\" WIDTH=10>\n",HT_ATTRIBUTE_OPERATION);
    i=0;
    while ( strlen(aditOpsDisplay1[i]) ) {
      if (lastOp &&  !strcmp(aditOps1[i],lastOp)) 
	fprintf(io,"<OPTION SELECTED VALUE=\"%s\">%s\n",aditOps1[i],aditOpsDisplay1[i]);
      else
	fprintf(io,"<OPTION VALUE=\"%s\">%s\n",aditOps1[i],aditOpsDisplay1[i]);
      i++;
    }
    fprintf(io,"</SELECT></td>\n");

    fprintf(io,"<td  align=center><b>ID:<b></td>\n");
    fprintf(io,"<td align=center><INPUT name=\"%s\" type=text  size=\"10\"></td>\n",
	    HT_ATTRIBUTE_SID_CODE);
    fprintf(io,"</tr>");


  }

  fprintf(io,"</table>\n");

  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_UNIQUE_ID, ufid);
  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_CONTEXT, HT_APPL_CONTEXT);
  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_DBNAME,  HT_TARGET_DBNAME);
  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_EMAIL,   HT_ANNOTATOR_EMAIL);
  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_PROJDIR, HT_ARCHIVE_DIRECTORY);
  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_MOL_TYPE, HT_MOL_TYPE);
  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_EXP_METH, HT_EXP_METHOD);
  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_VIEW_TYPE, HT_VIEW_TYPE);
  //  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_FILES_LOADED, "TRUE");

  
  fprintf(io,"</FORM>\n");

  if (!strcmp(HT_EXP_METHOD,"NMR")) {
    fprintf(io,"%s",p1_en_nmr);
  } else {
    fprintf(io,"%s",p1_en);
  }
  html_display_close_page(io, RCSB_STYLE_DEFAULT);
}


void write_top_session_screen_deposit_2(FILE *io, CifStateObj *sobj, const char *ufid, 
					const char *projDir)
/*
 * For the case where data files have already been uploaded ...
 *
 * Simplified display with hardwired filenames.
 *
 */  
{
  char *fileid=NULL,   *formatType=NULL;
  char *fileidSF=NULL, *formatTypeSF=NULL;
  char *fileidNC=NULL, *formatTypeNC=NULL;
  char *fileidDF=NULL, *lastOp=NULL, *sidCode=NULL;
  char *fileidCC=NULL, *fileidCS=NULL;

  int i, in, nRow;

  if (sobj) {
    fileid       = sobj->GetSessionFileCoords();
    formatType   = sobj->GetSessionFormatCoords();
    fileidSF     = sobj->GetSessionFileSF();
    formatTypeSF = sobj->GetSessionFormatSF();
    fileidNC     = sobj->GetSessionFileNC();
    formatTypeNC = sobj->GetSessionFormatNC();
    fileidDF     = sobj->GetSessionFileDF();
    fileidCC     = sobj->GetSessionFileCC();
    fileidCS     = sobj->GetSessionFileCS();
    sidCode      = sobj->GetSessionSIDCode();
    lastOp       = sobj->GetSessionOperation();
  }

  ndb_log_message_text(NDB_MSG_DEBUG,"In write_top_session_screen_deposit_2 *************");
  ndb_log_message_text(NDB_MSG_DEBUG,"fileid:       %s",fileid);
  ndb_log_message_text(NDB_MSG_DEBUG,"formatType:   %s",formatType);
  ndb_log_message_text(NDB_MSG_DEBUG,"fileidSF:     %s",fileidSF);
  ndb_log_message_text(NDB_MSG_DEBUG,"formatTypeSF: %s",formatTypeSF);
  ndb_log_message_text(NDB_MSG_DEBUG,"fileidNC:     %s",fileidNC);
  ndb_log_message_text(NDB_MSG_DEBUG,"formatTypeNC: %s",formatTypeNC);
  ndb_log_message_text(NDB_MSG_DEBUG,"fileidDF:     %s",fileidDF);  
  ndb_log_message_text(NDB_MSG_DEBUG,"fileidCC:     %s",fileidCC);  
  ndb_log_message_text(NDB_MSG_DEBUG,"fileidCS:     %s",fileidCS);  
  ndb_log_message_text(NDB_MSG_DEBUG,"sidCode:      %s",sidCode);  
  
  ht_put_content_type_target(io,"_top");
  html_display_open_page(io,"Deposition Options", RCSB_STYLE_DEFAULT);

  //jdw blank title... 

  html_display_page_title_logo(io,"", RCSB_STYLE_DEFAULT);
  html_display_return_to_page(io, "Adit Home Page", HT_HOME_PATH);
  
  fprintf(io,"%s",p1a_en);

  //
  //    
  //
  fprintf(io,"<FORM METHOD=\"POST\" ENCTYPE=\"multipart/form-data\" ACTION=\"%s%s%s\" TARGET=\"_top\">\n",
	  HT_SERVER_NAME,
	  HT_CGI_PATH,
	  HT_INTERFACE_OPERATION_DRIVER);
  fprintf(io,"<table cellpadding=10 bgcolor=\"%s\">\n",TBL_BG_COLOR);

  //
  // Coordinates ... 
  //

  fprintf(io,"<tr><td align=center><b>Coordinate<BR>File: </b></td>\n");
  fprintf(io,"<td width=200><b>%s</b></td>\n",root_file_name(fileid));

  fprintf(io,"<td align=center><b>File<BR>Type: </b></td>\n");
  in = get_index_in_list((const char **)formats1, formatType);
  if (in >= 0) 
    fprintf(io,"<td><b>%s</b></td>\n",formats1Display[in]);
  else
    fprintf(io,"<td>&nbsp</td>\n");

  nRow = 2;
  if (HT_APPL_CONTEXT_TYPE == HT_CONTEXT_ASSEMBLE) nRow = 3;

  // JDW CS and CC files
  if (!strcmp(HT_EXP_METHOD,"NMR")) nRow+=2;

  fprintf(io,"<td align=center rowspan=%d><font COLOR=\"blue\" size=4>\n",nRow);
  fprintf(io,"<INPUT TYPE=\"SUBMIT\" VALUE=\"BEGIN\"></font></td></tr>\n");

  //
  // NMR Constraints and  Structure factors
  //

  if (!strcmp(HT_EXP_METHOD,"NMR")) {
    //
    // Constraints ... 
    //
    fprintf(io,"<tr><td align=center><b>Constraint<BR>File: <b></td>\n");
    fprintf(io,"<td><b>%s</b></td>\n",root_file_name(fileidNC));
    
    fprintf(io,"<td align=center><b>File<BR>Type: </b></td>\n");
    in = get_index_in_list((const char **)formatsNC, formatTypeNC);
    if (in >= 0) 
      fprintf(io,"<td><b>%s</b></td>\n",formatsNCDisplay[in]);
    else
      fprintf(io,"<td>&nbsp</td>\n");


    fprintf(io,"<tr><td align=center><b>Coupling<BR>Constants: </b></td>\n");
    if (strlen(fileidCC))
	fprintf(io,"<td colspan=3 align=left><b>%s</b></td></tr>\n",root_file_name(fileidCC));
    else
      fprintf(io,"<td colspan=3 align=left>&nbsp</td></tr>\n");


    fprintf(io,"<tr><td align=center><b>Chemical<BR>Shifts: </b></td>\n");
    if (strlen(fileidCS))
	fprintf(io,"<td colspan=3 align=left><b>%s</b></td></tr>\n",root_file_name(fileidCS));
    else
      fprintf(io,"<td colspan=3 align=left>&nbsp</td></tr>\n");



  } else if (!strcmp(HT_EXP_METHOD,"X-ray")) {
    //
    // Structure factors  ... 
    //
    fprintf(io,"<tr><td align=center><b>Structure<br>Factor<BR>File: </b></td>\n");
    fprintf(io,"<td><b>%s</b></td>\n",root_file_name(fileidSF));
    
    fprintf(io,"<td align=center><b>File<BR>Type: </b></td>\n");

    in = get_index_in_list((const char **)formatsSF, formatTypeSF);
    if (in >= 0) 
      fprintf(io,"<td><b>%s</b></td>\n",formatsSFDisplay[in]);
    else
      fprintf(io,"<td>&nbsp</td>\n");

  }
  //
  // Deposition form ...
  //
  if (HT_APPL_CONTEXT_TYPE == HT_CONTEXT_ASSEMBLE) {
    fprintf(io,"<tr><td align=center><b>Deposition<BR>Form: </b></td>\n");
    if (strlen(fileidDF))
	fprintf(io,"<td colspan=3 align=left><b>%s</b></td></tr>\n",root_file_name(fileidDF));
    else
      fprintf(io,"<td colspan=3 align=left>&nbsp</td></tr>\n");
  }

  //  
  //  Operation Menu...
  //


  if (HT_APPL_CONTEXT_TYPE == HT_CONTEXT_DEPOSIT) {
    fprintf(io,"<tr><td align=center><b>Operation: </b></td>\n");
    fprintf(io,"<td colspan=4 align=left><SELECT NAME=\"%s\" WIDTH=10>\n",HT_ATTRIBUTE_OPERATION);
    i=0;
    while ( strlen(aditOpsDisplay[i]) ) {
      if (lastOp &&  !strcmp(aditOps[i],lastOp)) 
	fprintf(io,"<OPTION SELECTED VALUE=\"%s\">%s\n",aditOps[i],aditOpsDisplay[i]);
      else
	fprintf(io,"<OPTION VALUE=\"%s\">%s\n",aditOps[i],aditOpsDisplay[i]);
      i++;
    }
    fprintf(io,"</SELECT></td></tr>\n");
  } else {
    fprintf(io,"<tr><td align=center><b>Operation: </b></td>\n");
    fprintf(io,"<td colspan=2 align=left><SELECT NAME=\"%s\" WIDTH=10>\n",HT_ATTRIBUTE_OPERATION);
    i=0;
    while ( strlen(aditOpsDisplay1[i]) ) {
      if (lastOp &&  !strcmp(aditOps1[i],lastOp)) 
	fprintf(io,"<OPTION SELECTED VALUE=\"%s\">%s\n",aditOps1[i],aditOpsDisplay1[i]);
      else
	fprintf(io,"<OPTION VALUE=\"%s\">%s\n",aditOps1[i],aditOpsDisplay1[i]);
      i++;
    }
    fprintf(io,"</SELECT></td>\n");


    fprintf(io,"<td align=center><b>ID<b></td>\n");

    if (strlen(sidCode))
	fprintf(io,"<td align=left><b>%s</b></td></tr>\n",sidCode);
    else
      fprintf(io,"<td align=left>&nbsp</td></tr>\n");

  }


  fprintf(io,"</table>\n");

  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_UNIQUE_ID, ufid);
  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_CONTEXT, HT_APPL_CONTEXT);
  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_DBNAME,  HT_TARGET_DBNAME);
  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_EMAIL,   HT_ANNOTATOR_EMAIL);
  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_PROJDIR, HT_ARCHIVE_DIRECTORY);
  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_MOL_TYPE, HT_MOL_TYPE);
  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_EXP_METH, HT_EXP_METHOD);
  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_VIEW_TYPE, HT_VIEW_TYPE);
  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_FILES_LOADED, "TRUE");

  
  fprintf(io,"</FORM>\n");

  
  html_display_close_page(io, RCSB_STYLE_DEFAULT);
}



//
void write_top_session_screen_validate_1(FILE *io, CifStateObj *sobj, const char *ufid, 
					 const char *projDir)	

{
  /*
   *  Provides file upload options... 
   */

  char *fileid=NULL,   *formatType=NULL;
  char *fileidSF=NULL, *formatTypeSF=NULL;
  //  char *fileidNC=NULL, *formatTypeNC=NULL;
  //  char *fileidDF=NULL, 
  char *lastOp=NULL;
  int i, nRow;

  if (sobj) {
    fileid       = sobj->GetSessionFileCoords();
    formatType   = sobj->GetSessionFormatCoords();
    fileidSF     = sobj->GetSessionFileSF();
    formatTypeSF = sobj->GetSessionFormatSF();

    // fileidNC     = sobj->GetSessionFileNC();
    // formatTypeNC = sobj->GetSessionFormatNC();
    // fileidDF     = sobj->GetSessionFileDF();

    lastOp       = sobj->GetSessionOperation();
  }
  
  
  ht_put_content_type_target(io,"_top");
  html_display_open_page(io,"Validation Options", RCSB_STYLE_DEFAULT);

  //jdw blank title... 

  html_display_page_title_logo(io,"", RCSB_STYLE_DEFAULT);
  html_display_return_to_page(io, "Validation Home Page", HT_HOME_PATH);


  if (!strcmp(HT_EXP_METHOD,"NMR")) {
    fprintf(io,"%s",v0_en_nmr);
  } else {
    fprintf(io,"%s",v0_en);
  }

  //
  //    
  //
  fprintf(io,"<FORM METHOD=\"POST\" ENCTYPE=\"multipart/form-data\" ACTION=\"%s%s%s\" TARGET=\"_top\">\n",
	  HT_SERVER_NAME,
	  HT_CGI_PATH,
	  HT_INTERFACE_OPERATION_DRIVER);


  fprintf(io,"<table cellpadding=10 bgcolor=\"%s\">\n",TBL_BG_COLOR);

  //
  // Coordinates ... 
  //

  fprintf(io,"<tr><td align=center><b>Coordinate<BR>File: <b></td>\n");
  fprintf(io,"<td><INPUT name=\"%s\" type=file  size=\"23\" value=\"%s\"></td>\n",
	  HT_ATTRIBUTE_FILE_ID,  fileid, fileid);

  fprintf(io,"<td align=center><b>File<BR>Type: </b></td>\n");
  fprintf(io,"<td><SELECT NAME=\"%s\" WIDTH=10>\n",HT_ATTRIBUTE_FORMAT_TYPE);


  i=0;
  while ( strlen(formats1Display[i]) ) {
    if (formatType &&  !strcmp(formats1[i],formatType)) 
      fprintf(io,"<OPTION SELECTED VALUE=\"%s\">%s\n",formats1[i],formats1Display[i]);
    else
      fprintf(io,"<OPTION VALUE=\"%s\">%s\n",formats1[i],formats1Display[i]);
    i++;
  }
  fprintf(io,"</SELECT></td>\n");

  nRow = 3;

  fprintf(io,"<td align=center rowspan=%d><font COLOR=\"blue\" size=4>\n",nRow);
  fprintf(io,"<INPUT TYPE=\"SUBMIT\" VALUE=\"BEGIN\"></font></td></tr>\n");

  //
  // NMR Constraints or Structure factors
  //

  if (!strcmp(HT_EXP_METHOD,"NMR")) {

  } else if (!strcmp(HT_EXP_METHOD,"X-ray")) {
    //
    // Structure factors  ... 
    //
    fprintf(io,"<tr><td align=center><b>Structure<br>Factor<BR>File: </b></td>\n");
    fprintf(io,"<td><INPUT name=\"%s\" type=file  size=\"23\" value=\"%s\"></td>\n",
	    HT_ATTRIBUTE_FILE_ID_SF,  fileidSF, fileidSF);
    
    fprintf(io,"<td align=center><b>File<BR>Type: </b></td>\n");
    fprintf(io,"<td><SELECT NAME=\"%s\" WIDTH=10>\n",HT_ATTRIBUTE_FORMAT_TYPE_SF);
    i=0;
    while ( strlen(formatsSFDisplay[i]) ) {
      if (formatTypeSF &&  !strcmp(formatsSF[i],formatTypeSF)) 
	fprintf(io,"<OPTION SELECTED VALUE=\"%s\">%s\n",formatsSF[i],formatsSFDisplay[i]);
      else
	fprintf(io,"<OPTION VALUE=\"%s\">%s\n",formatsSF[i],formatsSFDisplay[i]);
      i++;
    }
    fprintf(io,"</SELECT></td></tr>\n");
  }

  //  
  //  Operation ...
  //
  fprintf(io,"<tr><td align=center><b>Operation: </b></td>\n");
  fprintf(io,"<td colspan=3 align=left><SELECT NAME=\"%s\" WIDTH=10>\n",HT_ATTRIBUTE_OPERATION);

  i=0;
  while ( strlen(valOpsDisplay[i]) ) {
    if (lastOp &&  !strcmp(valOps[i],lastOp)) 
      fprintf(io,"<OPTION SELECTED VALUE=\"%s\">%s\n",valOps[i],valOpsDisplay[i]);
    else
      fprintf(io,"<OPTION VALUE=\"%s\">%s\n",valOps[i],valOpsDisplay[i]);
    i++;
  }
  fprintf(io,"</SELECT></td></tr>\n");


  fprintf(io,"</table>\n");

  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_UNIQUE_ID, ufid);
  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_CONTEXT, HT_APPL_CONTEXT);
  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_DBNAME,  HT_TARGET_DBNAME);
  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_EMAIL,   HT_ANNOTATOR_EMAIL);
  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_PROJDIR, HT_ARCHIVE_DIRECTORY);
  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_MOL_TYPE, HT_MOL_TYPE);
  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_EXP_METH, HT_EXP_METHOD);
  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_VIEW_TYPE, HT_VIEW_TYPE);
  //  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_FILES_LOADED, "TRUE");

  
  fprintf(io,"</FORM>\n");

  if (!strcmp(HT_EXP_METHOD,"NMR")) {
    fprintf(io,"%s",v2_en_nmr);
  } else {
    fprintf(io,"%s",v2_en);
  }



  html_display_close_page(io, RCSB_STYLE_DEFAULT);
}


void write_top_session_screen_validate_2(FILE *io, CifStateObj *sobj, const char *ufid, 
					 const char *projDir)
/*
 * Simplified display with hardwired filenames.
 *
 */  
{
  char *fileid=NULL,   *formatType=NULL;
  char *fileidSF=NULL, *formatTypeSF=NULL;
  //  char *fileidNC=NULL, *formatTypeNC=NULL;
  //   char *fileidDF=NULL;
  char *lastOp=NULL;
  

  int i, in, nRow;

  if (sobj) {
    fileid       = sobj->GetSessionFileCoords();
    formatType   = sobj->GetSessionFormatCoords();
    fileidSF     = sobj->GetSessionFileSF();
    formatTypeSF = sobj->GetSessionFormatSF();
    // fileidNC     = sobj->GetSessionFileNC();
    // formatTypeNC = sobj->GetSessionFormatNC();
    // fileidDF     = sobj->GetSessionFileNC();
    lastOp       = sobj->GetSessionOperation();
  }
  
  
  ht_put_content_type_target(io,"_top");
  html_display_open_page(io,"Validation Options", RCSB_STYLE_DEFAULT);

  //jdw blank title... 

  html_display_page_title_logo(io,"", RCSB_STYLE_DEFAULT);
  html_display_return_to_page(io, "Validation Home Page", HT_HOME_PATH);
  
  fprintf(io,"%s",p2a_en);

  //
  //    
  //
  fprintf(io,"<FORM METHOD=\"POST\" ENCTYPE=\"multipart/form-data\" ACTION=\"%s%s%s\" TARGET=\"_top\">\n",
	  HT_SERVER_NAME,
	  HT_CGI_PATH,
	  HT_INTERFACE_OPERATION_DRIVER);
  fprintf(io,"<table cellpadding=10 bgcolor=\"%s\">\n",TBL_BG_COLOR);

  //
  // Coordinates ... 
  //

  fprintf(io,"<tr><td align=center><b>Coordinate<BR>File: </b></td>\n");
  fprintf(io,"<td width=200><b>%s</b></td>\n",root_file_name(fileid));

  fprintf(io,"<td align=center><b>File<BR>Type: <b></td>\n");
  in = get_index_in_list((const char **)formats1, formatType);
  if (in >= 0) 
    fprintf(io,"<td><b>%s</b></td>\n",formats1Display[in]);
  else
    fprintf(io,"<td>&nbsp</td>\n");

  nRow = 3;


  fprintf(io,"<td align=center rowspan=%d><font COLOR=\"blue\" size=4>\n",nRow);
  fprintf(io,"<INPUT TYPE=\"SUBMIT\" VALUE=\"BEGIN\"></font></td></tr>\n");

  //
  // NMR Constraints and  Structure factors
  //

  if (!strcmp(HT_EXP_METHOD,"NMR")) {


  } else if (!strcmp(HT_EXP_METHOD,"X-ray")) {
    //
    // Structure factors  ... 
    //
    fprintf(io,"<tr><td align=center><b>Structure<br>Factor<BR>File: </b></td>\n");
    fprintf(io,"<td><b>%s</b></td>\n",root_file_name(fileidSF));
    
    fprintf(io,"<td align=center><b>File<BR>Type: </b></td>\n");

    in = get_index_in_list((const char **)formatsSF, formatTypeSF);
    if (in >= 0) 
      fprintf(io,"<td><b>%s</b></td>\n",formatsSFDisplay[in]);
    else
      fprintf(io,"<td>&nbsp</td>\n");

  }

  //  
  //  Operation ...
  //
  fprintf(io,"<tr><td align=center><b>Operation: <b></td>\n");
  fprintf(io,"<td colspan=3 align=left><SELECT NAME=\"%s\" WIDTH=10>\n",HT_ATTRIBUTE_OPERATION);

  i=0;
  while ( strlen(valOpsDisplay[i]) ) {
    if (lastOp &&  !strcmp(valOps[i],lastOp)) 
      fprintf(io,"<OPTION SELECTED VALUE=\"%s\">%s\n",valOps[i],valOpsDisplay[i]);
    else
      fprintf(io,"<OPTION VALUE=\"%s\">%s\n",valOps[i],valOpsDisplay[i]);
    i++;
  }
  fprintf(io,"</SELECT></td></tr>\n");


  fprintf(io,"</table>\n");

  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_UNIQUE_ID, ufid);
  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_CONTEXT, HT_APPL_CONTEXT);
  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_DBNAME,  HT_TARGET_DBNAME);
  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_EMAIL,   HT_ANNOTATOR_EMAIL);
  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_PROJDIR, HT_ARCHIVE_DIRECTORY);
  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_MOL_TYPE, HT_MOL_TYPE);
  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_EXP_METH, HT_EXP_METHOD);
  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_VIEW_TYPE, HT_VIEW_TYPE);
  fprintf(io,"<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_FILES_LOADED, "TRUE");

  
  fprintf(io,"</FORM>\n");

  html_display_close_page(io, RCSB_STYLE_DEFAULT);
}


static int get_index_in_list(const char **list, const char *target) 
{
  int i,ret = -1;
  if (target == NULL) return(ret);
  i=0;
  while (strlen(list[i])) {
    if (!strcmp(target,list[i])) return(i);
    i++;
  }
  return(ret);
}


static char *root_file_name(char *path)
{
  char *p1=NULL, *root=NULL;
  int len=0;

  ndb_log_message_text(NDB_MSG_DEBUG,"root_file_name: input path %s",path);

  if (path == NULL || (strlen(path) == 0)) {
    root = (char *) calloc(6, sizeof(char));
    strcpy(root,"&nbsp");
    ndb_log_message_text(NDB_MSG_DEBUG,"root_file_name: return %s",root);
    return(root);
  }

  p1=strrchr(path,'/');
  if ( p1 == NULL || strlen(p1) <= 1) {
    root = (char *) calloc(6, sizeof(char));
    strcpy(root,"&nbsp");

  } else {
    p1++;
    len = strlen(p1);
    root = (char *) calloc(len+1, sizeof(char));
    strcpy(root,p1);

  }
  ndb_log_message_text(NDB_MSG_DEBUG,"root_file_name: return %s",root);
  return(root);
}


