/*
FILE:     check-view.C
*/
/*
VERSION:  0.904-beta
*/
/*
DATE:     8/14/2003
*/
/*
  Comments and Questions to: sw-help@rcsb.rutgers.edu
*/
/*
COPYRIGHT 1999-2003 Rutgers - The State University of New Jersey

This software is provided WITHOUT WARRANTY OF MERCHANTABILITY OR
FITNESS FOR A PARTICULAR PURPOSE OR ANY OTHER WARRANTY, EXPRESS OR
IMPLIED.  RUTGERS MAKE NO REPRESENTATION OR WARRANTY THAT THE
SOFTWARE WILL NOT INFRINGE ANY PATENT, COPYRIGHT OR OTHER
PROPRIETARY RIGHT.

The user of this software shall indemnify, hold harmless and defend
Rutgers, its governors, trustees, officers, employees, students,
agents and the authors against any and all claims, suits,
losses, liabilities, damages, costs, fees, and expenses including
reasonable attorneys' fees resulting from or arising out of the
use of this software.  This indemnification shall include, but is
not limited to, any and all claims alleging products liability.
*/
/*
               PDB SOFTWARE LICENSE AGREEMENT

BY CLICKING THE ACCEPTANCE BUTTON OR INSTALLING OR USING 
THIS "SOFTWARE, THE INDIVIDUAL OR ENTITY LICENSING THE  
SOFTWARE ("LICENSEE") IS CONSENTING TO BE BOUND BY AND IS 
BECOMING A PARTY TO THIS AGREEMENT.  IF LICENSEE DOES NOT 
AGREE TO ALL OF THE TERMS OF THIS AGREEMENT
THE LICENSEE MUST NOT INSTALL OR USE THE SOFTWARE.

1. LICENSE AGREEMENT

This is a license between you ("Licensee") and the Protein Data Bank (PDB) 
at Rutgers, The State University of New Jersey (hereafter referred to 
as "RUTGERS").   The software is owned by RUTGERS and protected by 
copyright laws, and some elements are protected by laws governing 
trademarks, trade dress and trade secrets, and may be protected by 
patent laws. 

2. LICENSE GRANT

RUTGERS grants you, and you hereby accept, non-exclusive, royalty-free 
perpetual license to install, use, modify, prepare derivative works, 
incorporate into other computer software, and distribute in binary 
and source code format, or any derivative work thereof, together with 
any associated media, printed materials, and on-line or electronic 
documentation (if any) provided by RUTGERS (collectively, the "SOFTWARE"), 
subject to the following terms and conditions: (i) any distribution 
of the SOFTWARE shall bind the receiver to the terms and conditions 
of this Agreement; (ii) any distribution of the SOFTWARE in modified 
form shall clearly state that the SOFTWARE has been modified from 
the version originally obtained from RUTGERS.  

2. COPYRIGHT; RETENTION OF RIGHTS.  

The above license grant is conditioned on the following: (i) you must 
reproduce all copyright notices and other proprietary notices on any 
copies of the SOFTWARE and you must not remove such notices; (ii) in 
the event you compile the SOFTWARE, you will include the copyright 
notice with the binary in such a manner as to allow it to be easily 
viewable; (iii) if you incorporate the SOFTWARE into other code, you 
must provide notice that the code contains the SOFTWARE and include 
a copy of the copyright notices and other proprietary notices.  All 
copies of the SOFTWARE shall be subject to the terms of this Agreement.  

3. NO MAINTENANCE OR SUPPORT; TREATMENT OF ENHANCEMENTS 

RUTGERS is under no obligation whatsoever to: (i) provide maintenance 
or support for the SOFTWARE; or (ii) to notify you of bug fixes, patches, 
or upgrades to the features, functionality or performance of the 
SOFTWARE ("Enhancements") (if any), whether developed by RUTGERS 
or third parties.  If, in its sole discretion, RUTGERS makes an 
Enhancement available to you and RUTGERS does not separately enter 
into a written license agreement with you relating to such bug fix, 
patch or upgrade, then it shall be deemed incorporated into the SOFTWARE 
and subject to this Agreement. You are under no obligation whatsoever 
to provide any Enhancements to RUTGERS or the public that you may 
develop over time; however, if you choose to provide your Enhancements 
to RUTGERS, or if you choose to otherwise publish or distribute your 
Enhancements, in source code form without contemporaneously requiring 
end users or RUTGERS to enter into a separate written license agreement 
for such Enhancements, then you hereby grant RUTGERS a non-exclusive,
royalty-free perpetual license to install, use, modify, prepare
derivative works, incorporate into the SOFTWARE or other computer
software, distribute, and sublicense your Enhancements or derivative
works thereof, in binary and source code form.

4. FEES.  There is no license fee for the SOFTWARE.  If Licensee
wishes to receive the SOFTWARE on media, there may be a small charge
for the media and for shipping and handling.  Licensee is
responsible for any and all taxes.

5. TERMINATION.  Without prejudice to any other rights, Licensor
may terminate this Agreement if Licensee breaches any of its terms
and conditions.  Upon termination, Licensee shall destroy all
copies of the SOFTWARE.

6. PROPRIETARY RIGHTS.  Title, ownership rights, and intellectual
property rights in the Product shall remain with RUTGERS.  Licensee 
acknowledges such ownership and intellectual property rights and will 
not take any action to jeopardize, limit or interfere in any manner 
with RUTGERS' ownership of or rights with respect to the SOFTWARE.  
The SOFTWARE is protected by copyright and other intellectual 
property laws and by international treaties.  Title and related 
rights in the content accessed through the SOFTWARE is the property 
of the applicable content owner and is protected by applicable law.  
The license granted under this Agreement gives Licensee no rights to such
content.

7. DISCLAIMER OF WARRANTY.  THE SOFTWARE IS PROVIDED FREE OF 
CHARGE, AND, THEREFORE, ON AN "AS IS" BASIS, WITHOUT WARRANTY OF 
ANY KIND, INCLUDING WITHOUT LIMITATION THE WARRANTIES THAT IT 
IS FREE OF DEFECTS, MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE 
OR NON-INFRINGING.  THE ENTIRE RISK AS TO THE QUALITY AND 
PERFORMANCE OF THE SOFTWARE IS BORNE BY LICENSEE.  SHOULD THE 
SOFTWARE PROVE DEFECTIVE IN ANY RESPECT, THE LICENSEE AND NOT 
LICENSOR ASSUMES THE ENTIRE COST OF ANY SERVICE AND REPAIR.  
THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF 
THIS AGREEMENT.  NO USE OF THE PRODUCT IS AUTHORIZED HEREUNDER 
EXCEPT UNDER THIS DISCLAIMER.

8. LIMITATION OF LIABILITY.  TO THE MAXIMUM EXTENT PERMITTED BY
APPLICABLE LAW,  IN NO EVENT WILL LICENSOR BE LIABLE FOR ANY 
INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
OUT OF THE USE OF OR INABILITY TO USE THE SOFTWARE, INCLUDING, 
WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK 
STOPPAGE, COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL 
OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF ADVISED OF THE
POSSIBILITY THEREOF. 
*/
/* 
  PURPOSE:      This module performs a variety of useful checks on view files.

*/    


#include <iostream.h>

#include "CifFileObj.h"
#include "CifFileObj.h"
#include "CifViewObj.h"
#include "CifBuilder.h"

static void  print_view_obj_viewable(ostream& fs, CifViewObj *vobj, CifBuilder& cb);

static void  check_view_obj(ostream& fs, CifViewObj *vobj, CifBuilder& cb);
static void  check_view_obj_viewable(ostream& fs, CifViewObj *vobj, CifBuilder& cb);
static void  mk_template_from_view(ostream& fs, CifViewObj *vobj, CifBuilder& cb);
static void  print_view_obj(ostream& fs, CifViewObj *vobj);

static void format_cif_definition(ostream &fs, const char *string);
static void format_cif_definition_template(ostream &fs, const char *string);

static int nws_strlen(const char *line);
static int count_leading_ws(const char *line);
static int get_a_line(const char *s, char *line, int *iPos);
static int get_a_line_x_html(const char *s, char *buf, int *iPos);

static void check_category_dict_features(ostream &fs, const char *category, CifBuilder &cb);
static void check_item_dict_features(ostream &fs, const char *item, CifBuilder &cb);
static void print_category_dict_features(ostream &fs, const char *category, CifBuilder &cb);
static void print_item_dict_features(ostream &fs, const char *item, CifBuilder &cb);

static void template_category_dict_features(ostream &fs, const char *category, CifBuilder &cb);
static void template_item_dict_features(ostream &fs, const char *item, CifBuilder &cb);
static void format_cif_example_template(ostream &fs, const char *string);
static void  make_view(ostream& fs, CifBuilder& cb);
static void  add_space(ostream& fs, const char *field, int iwidth);

void usage(const char * pname) {
  cerr << pname << " usage =  " << endl;
  cerr << "             -df <dict_file>  -dict <dict_name> " << endl;
  cerr << "             -vf <view_file>  -view <view_name>" << endl;
  cerr << "             -funct <template|print|printview|check|checkall>" << endl;
  cerr << "                    template = print CIF ASCII template" << endl;                 
  cerr << "                    print    = print CIF ASCII template" << endl;                 
  cerr << "                    printview= print CIF ASCII template" << endl;                 
  cerr << "                    check    = extract dictionary parameters for items in view" << endl;
  cerr << "                    checkall = extract dictionary parameters for items in view (all items)" << endl;
  cerr << "                    makeview = make preliminary view from dictionary (all items)" << endl;
}

int main(int argc, char *argv[]) {

  char *pname=NULL;
  char *dFile=NULL, *vFile=NULL, *dictName=NULL, *viewName=NULL, funct[20];
  

  pname = argv[0];
  strcpy(funct,"template");

  if (argc < 3) {
    usage(pname);
    exit(1);
  }
  cerr << "check-view - starting" << endl;
  for (int i=1; i < argc; i++) {
    if ( argv[i][0] == '-' ) {
      if (strcmp(argv[i],"-df") == 0 ) {
        i++;
        dFile = (char *) calloc(strlen(argv[i]) + 1, sizeof(char));
        strcpy(dFile,argv[i]);          
      } else if (strcmp(argv[i],"-vf") == 0 ) {
        i++;
        vFile = (char *) calloc(strlen(argv[i]) + 1, sizeof(char));
        strcpy(vFile,argv[i]);          
      } else if (strcmp(argv[i],"-dict") == 0 ) {
        i++;
        dictName = (char *) calloc(strlen(argv[i]) + 1, sizeof(char));
        strcpy(dictName,argv[i]);          
      } else if (strcmp(argv[i],"-view") == 0 ) {
        i++;
        viewName = (char *) calloc(strlen(argv[i]) + 1, sizeof(char));
        strcpy(viewName,argv[i]);          
      } else if (strcmp(argv[i],"-funct") == 0 ) {
        i++;
        strncpy(funct,argv[i],19);          

      } else {
	usage(pname);
        exit(1);
      }
    } else {
      usage(pname);
      exit(1);
    }
  }

  if (!dFile || !strlen(dFile)) {
    usage(pname); exit(2);
  }



  if (!dictName || !strlen(dictName)) {
    usage(pname); exit(4);
  }


  //
  //  Read dictionary object ...
  //

  cerr << "check-view - reading dictionary" << endl;
  CifBuilder cb(dFile);
  cb.BuildDictionaryList();
  cb.SetContext(dictName);

  //
  //  Read view file 
  //
  if (!strcmp(funct,"makeview")) {
    cerr << "Searching for " << dFile << " dictionary " << dictName << endl;
    make_view(cout, cb);
    exit(0);
  }

  if (!vFile || !strlen(vFile)) {
    usage(pname); exit(3);
  }
  if (!viewName || !strlen(viewName)) {
    usage(pname); exit(5);
  }

  cerr << "check-view - reading view file " << vFile << " view name " << viewName << endl;
  CifViewObj *vobj = new CifViewObj(vFile, "/var/tmp/view.odb", viewName, 1);
  if (!vobj) {
    usage(pname); exit(6);
  }

  cerr << "check-view - beginning function "<< funct << endl;
  if (!strcmp(funct,"template")) {
    mk_template_from_view(cout,vobj,cb);
  } else if (!strcmp(funct,"check")) {
    cout << "Searching for " << dFile << " dictionary " << dictName << endl;
    cout << "Searching for " << vFile << " data block " << viewName << endl;
    cout << "View Diagnostics " << vobj->GetViewDiagnostics() << endl;
    check_view_obj_viewable(cout,vobj,cb);
  }  else if (!strcmp(funct,"checkall")) {
    cout << "Searching for " << dFile << " dictionary " << dictName << endl;
    cout << "Searching for " << vFile << " data block " << viewName << endl;
    cout << "View Diagnostics " << vobj->GetViewDiagnostics() << endl;
    check_view_obj(cout,vobj,cb);
  }  else if (!strcmp(funct,"printview")) {
    cout << "Searching for " << vFile << " data block " << viewName << endl;
    cout << "View Diagnostics " << vobj->GetViewDiagnostics() << endl;
    print_view_obj_viewable(cout,vobj, cb);
  }  else if (!strcmp(funct,"print")) {
    cout << "Searching for " << vFile << " data block " << viewName << endl;
    cout << "View Diagnostics " << vobj->GetViewDiagnostics() << endl;
    print_view_obj(cout,vobj);
  }
}


static void  make_view(ostream& fs, CifBuilder& cb)
{
  int i, j, ilen;
  
  uWord ngroups, ncats, nitems;
  char **categoryGroups=NULL;
  char **categories=NULL, **groups=NULL, **items=NULL; 

  categoryGroups = cb.GetDictionaryCategoryGroupList(ngroups);

  fs << "data_view" << endl << endl;

  fs << "loop_" << endl;
  fs << "_ndb_view_category_group.view_group_id" << endl;
  fs << "_ndb_view_category_group.description" << endl;
  for (i=0; i < ngroups; i++) {
    if (!strcmp("inclusive_group",categoryGroups[i])) continue;
    fs << "'" << categoryGroups[i] << "'";
    add_space(fs,categoryGroups[i], 30);
    fs << "'Group " << categoryGroups[i] << "'" <<  endl;
  }

  categories = cb.GetDictionaryCategories(ncats);

  fs << endl << endl;
  fs << "loop_" << endl;
  fs << "_ndb_view_category.category_id" << endl;
  fs << "_ndb_view_category.category_view_name" << endl;
  fs << "_ndb_view_category.view_group_id" << endl;
  fs << "_ndb_view_category.category_view_mandatory_code" << endl;
  fs << "_ndb_view_category.category_view_display_code" << endl;

  for (i=0; i < ncats; i++) {
    fs << "'" << categories[i] << "'";
    add_space(fs,categories[i], 35);
    fs << "'" << categories[i] << "'";
    add_space(fs,categories[i], 35);
    groups = cb.GetCategoryCategoryGroups(categories[i],ngroups);
    if (ngroups > 1) {
      fs << "'" << groups[1] << "'";
      add_space(fs,groups[1], 30);
      fs << " N T6" << endl;

      for (j=0; j < ngroups; j++){
	if (groups[j] != NULL) delete groups[j];
      }
      if (groups != NULL)  delete groups;
    } else if (ngroups > 0) {
      fs << "'" << groups[0] << "'";
      add_space(fs,groups[0], 30);
      fs << " H T6" << endl;
      for (j=0; j < ngroups; j++){
	if (groups[j] != NULL) delete groups[j];
      }
      if (groups != NULL)  delete groups;

    } else {
      fs << " unknown_group   Y R" << endl;
    }
  }

  items = cb.GetDictionaryItems(nitems);

  fs << endl << endl;
  fs << "loop_" << endl;
  fs << "_ndb_view_item.category_id" << endl;
  fs << "_ndb_view_item.item_name" << endl;
  fs << "_ndb_view_item.item_view_name" << endl;
  fs << "_ndb_view_item.item_view_mandatory_code" << endl;
  fs << "_ndb_view_item.item_view_allow_alternate_value" << endl;
  for (i=0; i < nitems; i++) {
    ilen =0;
    for (j=1; j< strlen(items[i]); j++) {
      if (items[i][j] != '.') {
	fs << items[i][j];
	ilen++;
      } else {
	break;
      }
    }

    for (j=0; j < 35-ilen; j++) {
      fs << " ";
    }
    fs << "'" << items[i] << "'";
    add_space(fs,items[i], 55);
    fs << "'" << items[i] << "'";
    add_space(fs,items[i], 55);
    fs <<  " N  N"  << endl;
  }
}

static void  add_space(ostream& fs, const char *field, int iwidth) 
{
  int i, ispaces;
  ispaces =  iwidth - strlen(field);
  if (ispaces <= 0) {
    fs << " ";
    return;
  }
  for (i=0; i < ispaces; i++) {
    fs << " ";
  }
}


static void  print_view_obj(ostream& fs, CifViewObj *vobj)
{
  ReVarCifArray<CifString> *c0 = NULL;
  ReVarCifArray<CifString> *c2 = NULL;
  ReVarCifArray<CifString> *c3 = NULL;
  ReVarCifArray<CifString> *c4 = NULL;
  
  int i, j, k;

  char viewName[100];
  int mcode, acode;

  
  c0= vobj->GetGroups();
  for (i=0; i < c0->Length(); i++) {
    fs << "View group " << (*c0)[i].Text() << endl;
    
    c3 = vobj->GetGroupCategories((*c0)[i].Text());
    if (c3 && c3->Length() > 0) {
      
      c4=vobj->GetGroupViewCategories((*c0)[i].Text());
      if (!c4) {
	fs << "No categories in group " << (*c0)[i].Text() << endl;
	continue;
      } else if ( c4->Length() != c3->Length() ) {
	fs << "Group category length mismatch " 
	   <<  c3->Length() << " .ne. " << c4->Length() << endl;
	exit(1);
      }
      
      for (j=0; j < c3->Length(); j++) {
	
	fs << " Category " << (*c3)[j].Text() << " " << j+1 << " of " << c3->Length() << endl;
	fs << " View category name " << (*c4)[j].Text() << endl;
	c2 = vobj->GetCategoryItems((*c3)[j].Text());
	fs << " Number of items " << c2->Length() << endl;
	//	iCat = vobj->GetViewCategoryIndex((*c3)[j].Text());


	for (k=0; k < c2->Length(); k++) {
	  fs   << "    ++ Item " << k << " " << (*c2)[k].Text() 
	       << " View Name " << vobj->GetViewItem((*c2)[k].Text())->Text()
	       << " mcode "     << vobj->GetItemMandatoryCode((*c2)[k].Text()) 
	       << " altcode "   << vobj->GetItemAllowAlternateValue((*c2)[k].Text()) << endl;
	  vobj->GetItemInfo((*c2)[k].Text(), viewName, mcode,acode);
	  fs   << "    -- Item " << k << " " << (*c2)[k].Text() 
	       << " view name " << viewName << " mcode " << mcode 
	       << " acode " << acode << endl;
	}
	if (c2) delete c2;
      }
      if (c4) delete c4;
    }
    if (c3) {delete c3; c3 = NULL;}

  }
  if (c0) delete c0;
}


static void  check_view_obj(ostream& fs, CifViewObj *vobj, CifBuilder& cb)
{
  ReVarCifArray<CifString> *c0 = NULL;
  ReVarCifArray<CifString> *c2 = NULL;
  ReVarCifArray<CifString> *c3 = NULL;
  ReVarCifArray<CifString> *c4 = NULL;
  
  int i, j, k;

  char viewName[100];
  int mcode, acode;
  
  c0= vobj->GetGroups();
  for (i=0; i < c0->Length(); i++) {
    fs << endl;
    fs << "----------------------------------------------------------------------------" << endl;
    fs << "View group " << (*c0)[i].Text() <<  "  " << i+1 << " of " << c0->Length() << endl;
    
    c3 = vobj->GetGroupCategories((*c0)[i].Text());
    if (c3 && c3->Length() > 0) {
      
      for (j=0; j < c3->Length(); j++) {
	fs << endl;
	fs << "-CATEGORY--CATEGORY--CATEGORY--CATEGORY--CATEGORY--CATEGORY--CATEGORY--CATEGORY-" << endl;	
	fs << "Category " << (*c3)[j].Text() << " " << j+1 << " of " << c3->Length() << endl;
	fs << "++ View category name " << (*c3)[j].Text() << endl;
	fs << "++ Category mandatory code " << vobj->GetCategoryMandatoryCode((*c3)[j].Text()) << endl;
	fs << "++ Category display code   " << vobj->GetCategoryDisplayCode((*c3)[j].Text()) << endl;

	//	iCat = vobj->GetViewCategoryIndex((*c3)[j].Text());

	check_category_dict_features(fs,(*c3)[j].Text(),cb);

	c2 = vobj->GetCategoryItems((*c3)[j].Text());
	for (k=0; k < c2->Length(); k++) {
	  fs << endl;
	  fs << "-ITEM--ITEM--ITEM--ITEM--ITEM--ITEM--ITEM--ITEM--ITEM--ITEM--ITEM--ITEM--ITEM-" << endl;
	  fs   << "Item " << k+1 << " of " << c2->Length() << " is " << (*c2)[k].Text() << endl;
	  fs   << "++ view name " <<  vobj->GetViewItem((*c2)[k].Text())->Text() << endl;
	  fs   << "++ mcode     "   <<  vobj->GetItemMandatoryCode((*c2)[k].Text())  << endl;
	  fs   << "++ altcode   "   <<  vobj->GetItemAllowAlternateValue((*c2)[k].Text()) << endl;
	  vobj->GetItemInfo((*c2)[k].Text(), viewName, mcode,acode);
	  //	  fs   << "    -- Item " << k << " " << (*c2)[k].Text()  << endl;
	  // fs   << "    -- view name " << viewName << endl;
          // fs   << "    -- mcode     " << mcode << endl;
	  // fs   << "    -- altcode   " << acode << endl;
	  check_item_dict_features(fs,(*c2)[k].Text(),cb);
	}
	if (c2) delete c2;
      }
      if (c4) delete c4;
    }
    if (c3) {delete c3; c3 = NULL;}
  }
  if (c0) delete c0;
}



static void  mk_template_from_view(ostream& fs, CifViewObj *vobj, CifBuilder& cb)
{
  ReVarCifArray<CifString> *c0 = NULL;
  ReVarCifArray<CifString> *c2 = NULL;
  ReVarCifArray<CifString> *c3 = NULL;
  ReVarCifArray<CifString> *c4 = NULL;
  
  int i, j, k;
  int m, il, ilmax;
  int iDisplay;
  
  c0= vobj->GetGroups();

  fs << "data_template" << endl;
  fs << "#" << endl;
  for (i=0; i < c0->Length(); i++) {
    fs << endl;
    fs << "#----------------------------------------------------------------------------" << endl;
    fs << "#" << endl;
    fs << "#                      Data about " << (*c0)[i].Text() << endl;
    fs << "#" << endl;
    fs << "#----------------------------------------------------------------------------" << endl;
    fs << "#" << endl;
    
    c3 = vobj->GetGroupCategories((*c0)[i].Text());
    if (c3 && c3->Length() > 0) {
      
      for (j=0; j < c3->Length(); j++) {
	
	if (vobj->GetCategoryMandatoryCode((*c3)[j].Text()) == 2) {
	  continue;
	}

	c2 = vobj->GetCategoryItemsViewable((*c3)[j].Text());
	if (c2->Length() <= 0) continue;
	fs << "# " << endl;
	fs << "#                      Section = " << (*c3)[j].Text() << endl;

	iDisplay = vobj->GetCategoryDisplayCode((*c3)[j].Text());
	template_category_dict_features(fs,(*c3)[j].Text(),cb);

	for (k=0; k < c2->Length(); k++) {
	  fs << "# ==> " << vobj->GetViewItem((*c2)[k].Text())->Text();
	  if (strcmp(vobj->GetViewItem((*c2)[k].Text())->Text(), (*c2)[k].Text()) ) {
	     fs << "  (" 
	     << (*c2)[k].Text()
	     << ")";
	  }
	  fs << endl; 
	  template_item_dict_features(fs,(*c2)[k].Text(),cb);
	}

	

	if (iDisplay > 0) { // loop_
        fs << "# ==>  The question marks in the template below are placeholders for one row   <== " << endl;
        fs << "# ==>  of data in this section.  Replace the question marks with your data and <== " << endl;
        fs << "# ==>  add as many rows as required.                                           <== " << endl;
	  fs << "loop_" << endl;
	  for (k=0; k < c2->Length(); k++) {
	    fs << (*c2)[k].Text() << endl;
	  }
	  for (k=0; k < c2->Length(); k++) {
	    fs << " ?";
	  }
	  fs << endl;

	} else {  
	  fs << "# ==>  Replace the question marks in the template below with your data.      <== " << endl;
	  il = ilmax = 0;
	  for (k=0; k < c2->Length(); k++) {
	    il = strlen((*c2)[k].Text());
	    if ( il > ilmax ) ilmax = il;

	  }
	  for (k=0; k < c2->Length(); k++) {
	    fs << (*c2)[k].Text();
	    for (m = strlen((*c2)[k].Text()); m < ilmax + 2; m++) {
	      fs << " ";
	    }
	    fs << "?" << endl;
	  }
	}

	if (c2) delete c2;
      }
      if (c4) delete c4;
    }
    if (c3) {delete c3; c3 = NULL; }
  }
  if (c0) delete c0;
}



static void  print_view_obj_viewable(ostream& fs, CifViewObj *vobj, CifBuilder& cb)
{
  ReVarCifArray<CifString> *c0 = NULL;
  ReVarCifArray<CifString> *c2 = NULL;
  ReVarCifArray<CifString> *c3 = NULL;
  ReVarCifArray<CifString> *c4 = NULL;
  
  int i, j, k;

  char viewName[100];
  int mcode, acode;
  
  c0= vobj->GetGroups();
  for (i=0; i < c0->Length(); i++) {
    fs << endl;
    fs << "----------------------------------------------------------------------------" << endl;
    fs << "View group " << (*c0)[i].Text() << endl;
    
    c3 = vobj->GetGroupCategories((*c0)[i].Text());
    if (c3 && c3->Length() > 0) {
      
      for (j=0; j < c3->Length(); j++) {
	if (vobj->GetCategoryMandatoryCode((*c3)[j].Text()) == 2) {
	  //	  fs << "++ Category is hidden in this view" << endl;
	  continue;
	}
	fs << endl;
	fs << "CATEGORY--CATEGORY--CATEGORY--CATEGORY--CATEGORY--CATEGORY--CATEGORY" << endl;
	fs << endl;
	fs << "mmCIF Category " << (*c3)[j].Text() << endl;
	fs << "View name      " << vobj->GetViewCategory((*c3)[j].Text())->Text() << endl;

		print_category_dict_features(fs,(*c3)[j].Text(),cb);

	c2 = vobj->GetCategoryItems((*c3)[j].Text());
	c4 = vobj->GetCategoryItemsViewable((*c3)[j].Text());

	for (k=0; k < c2->Length(); k++) {
	  vobj->GetItemInfo((*c2)[k].Text(), viewName, mcode,acode);
	  if (mcode == 2) {
	    // fs << "++ Item is hidden in this view" << endl;
	    continue;
	  }
	  fs << endl;
	  fs << "ITEM--ITEM--ITEM--ITEM--ITEM--ITEM--ITEM--ITEM--ITEM--ITEM--ITEM--ITEM" << endl;
	  fs << endl;
	  fs   << "mmCIF item name " << (*c2)[k].Text() << endl;
	  fs   << "View name       " <<  vobj->GetViewItem((*c2)[k].Text())->Text() << endl;


	  //	  fs   << "    -- Item " << k << " " << (*c2)[k].Text()  << endl;
	  //      fs   << "    -- view name " << viewName << endl;
          //      fs   << "    -- mcode     " << mcode << endl;
	  //      fs   << "    -- altcode   " << acode << endl;

	  	  print_item_dict_features(fs,(*c2)[k].Text(),cb);
	}

	if (c2) delete c2;
	if (c4) delete c4;
      }

    }
    if (c3) {delete c3; c3 = NULL;}
  }
  if (c0) delete c0;
}



static void  check_view_obj_viewable(ostream& fs, CifViewObj *vobj, CifBuilder& cb)
{
  ReVarCifArray<CifString> *c0 = NULL;
  ReVarCifArray<CifString> *c2 = NULL;
  ReVarCifArray<CifString> *c3 = NULL;
  ReVarCifArray<CifString> *c4 = NULL;
  
  int i, j, k, iv;

  char viewName[100];
  int mcode, acode;
  
  c0= vobj->GetGroups();
  for (i=0; i < c0->Length(); i++) {
    fs << endl;
    fs << "----------------------------------------------------------------------------" << endl;
    fs << "View group " << (*c0)[i].Text() << endl;
    
    c3 = vobj->GetGroupCategories((*c0)[i].Text());
    if (c3 && c3->Length() > 0) {
      
      for (j=0; j < c3->Length(); j++) {
	fs << endl;
	fs << "CATEGORY--CATEGORY--CATEGORY--CATEGORY--CATEGORY--CATEGORY--CATEGORY" << endl;
	fs << "Category " << (*c3)[j].Text() << " " << j+1 << " of " << c3->Length() << endl;
	fs << "++ View category name " << (*c3)[j].Text() << endl;
	fs << "++ Category mandatory code " << vobj->GetCategoryMandatoryCode((*c3)[j].Text()) << endl;
	fs << "++ Category display code   " << vobj->GetCategoryDisplayCode((*c3)[j].Text()) << endl;

	if (vobj->GetCategoryMandatoryCode((*c3)[j].Text()) == 2) {
	  fs << "++ Category is hidden in this view" << endl;
	  continue;
	}

	check_category_dict_features(fs,(*c3)[j].Text(),cb);

	c2 = vobj->GetCategoryItems((*c3)[j].Text());
	c4 = vobj->GetCategoryItemsViewable((*c3)[j].Text());
	iv = 0;
	for (k=0; k < c2->Length(); k++) {
	  fs << endl;
	  fs << "ITEM--ITEM--ITEM--ITEM--ITEM--ITEM--ITEM--ITEM--ITEM--ITEM--ITEM--ITEM" << endl;
	  fs   << "Item " << k << " " << (*c2)[k].Text() << endl;
	  fs   << "++ view name " <<  vobj->GetViewItem((*c2)[k].Text())->Text() << endl;
	  fs   << "++ mcode     "   <<  vobj->GetItemMandatoryCode((*c2)[k].Text())  << endl;
	  fs   << "++ altcode   "   <<  vobj->GetItemAllowAlternateValue((*c2)[k].Text()) << endl;
	  vobj->GetItemInfo((*c2)[k].Text(), viewName, mcode,acode);
	  //	  fs   << "    -- Item " << k << " " << (*c2)[k].Text()  << endl;
	  //      fs   << "    -- view name " << viewName << endl;
          //      fs   << "    -- mcode     " << mcode << endl;
	  //      fs   << "    -- altcode   " << acode << endl;
	  if (mcode == 2) {
	    fs << "++ Item is hidden in this view" << endl;
	    continue;
	  }
	  iv++;
	  check_item_dict_features(fs,(*c2)[k].Text(),cb);
	}

	if (iv != c4->Length()) {
	  cerr << "Consistency error in view iv = " << iv << " view length " <<  c4->Length() << endl;
	  exit(1);
	}

	if (c2) delete c2;
	if (c4) delete c4;
      }
    }
    if (c3) {delete c3; c3 = NULL; }
  }
  if (c0) delete c0;
}





static void check_item_dict_features(ostream &fs, const char *item, CifBuilder &cb) {
  uWord n, i;
  char *string=NULL;
  char **strings=NULL;

  string      =  cb.GetItemDescription(item);
  if (!string) 
    fs << "++ Missing standard item description" << endl;
  else {
    fs << "++ Found standard item description" << endl;
    format_cif_definition(fs, string);
    if (string) delete string;
  }

  string      =  cb.GetItemDescriptionNDB(item);
  if (!string) 
    fs << "++ Missing local item description" << endl;
  else {
    fs << "++ Found local item description" << endl;
    format_cif_definition(fs, string);
    if (string) delete string;
  }

  string      =  cb.GetItemType(item);
  if (!string) 
    fs << "++ Missing item type" << endl;
  else {
    fs << "++ Found item type: " << string << endl;
    if (string) delete string;
  }

  string      =  cb.GetItemPrimitiveCode(item);
  if (!string) 
    fs << "++ Missing item primitive code" << endl;
  else {
    fs << "++ Found item primitive code: " << string << endl;
    if (string) delete string;
  }

  string      =  cb.GetItemRegex(item);
  if (!string) 
    fs << "++ Missing item regex" << endl;
  else {
    fs << "++ Found item regex: " << string << endl;
    if (string) delete string;
  }

  string      =  cb.GetItemMandatoryCode(item);
  if (!string) 
    fs << "++ Missing item mandatory code" << endl;
  else {
    fs << "++ Found item mandatory code: " << string << endl;
    if (string) delete string;
  }

  string      =  cb.GetItemDefaultValue(item);
  if (!string) 
    fs << "++ No item default value" << endl;
  else {
    fs << "++ Found item default value: " << string << endl;
    if (string) delete string;
  }


  strings =  cb.GetItemRangeMin(item,n);
  if (!strings) 
    fs << "++ No item minimum range" << endl;
  else {
    fs << "++ Found " << n << " item minimum boundaries" << endl;
    for (i=0; i < n; i++) {
      fs << "** Minimum value " << i+1 << " " << strings[i] << endl;
      if (strings[i]) delete strings[i];
    }
    if (strings) delete strings;
  }

  strings =  cb.GetItemRangeMax(item,n);
  if (!strings) 
    fs << "++ No item maximum range" << endl;
  else {
    fs << "++ Found " << n << " item maximum boundaries" << endl;
    for (i=0; i < n; i++) {
      fs << "** Maximum value " << i+1 << " " << strings[i] << endl;
      if (strings[i]) delete strings[i];
    }
    if (strings) delete strings;
  }


  strings =  cb.GetItemExamplesCase(item,n);
  if (!strings) 
    fs << "++ Missing standard item examples (CASE)" << endl;
  else {
    fs << "++ Found " << n << " standard item examples (CASE)" << endl;
    for (i=0; i < n; i++) {
      fs << "** CASE " << i+1 << endl;
      format_cif_definition(fs, strings[i]);
      if (strings[i]) delete strings[i];
    }
    if (strings) delete strings;
  }

  strings =  cb.GetItemExamplesDetail(item,n);
  if (!strings) 
    fs << "++ Missing standard item examples (DETAIL)" << endl;
  else {
    fs << "++ Found " << n << " standard item examples (DETAIL)" << endl;
    for (i=0; i < n; i++) {
      fs << "** DETAIL " << i+1 << endl;
      format_cif_definition(fs, strings[i]);
      if (strings[i]) delete strings[i];
    }
    if (strings) delete strings;
  }



  strings =  cb.GetItemExamplesCaseNDB(item,n);
  if (!strings) 
    fs << "++ Missing local item examples (CASE)" << endl;
  else {
    fs << "++ Found " << n << " local item examples (CASE)" << endl;
    for (i=0; i < n; i++) {
      fs << "** CASE " << i+1 << endl;
      format_cif_definition(fs, strings[i]);
      if (strings[i]) delete strings[i];
    }
    if (strings) delete strings;
  }

  strings =  cb.GetItemExamplesDetailNDB(item,n);
  if (!strings) 
    fs << "++ Missing local item examples (DETAIL)" << endl;
  else {
    fs << "++ Found " << n << " local item examples (DETAIL)" << endl;
    for (i=0; i < n; i++) {
      fs << "** DETAIL " << i+1 << endl;
      format_cif_definition(fs, strings[i]);
      if (strings[i]) delete strings[i];
    }
    if (strings) delete strings;
  }
}


static void print_item_dict_features(ostream &fs, const char *item, CifBuilder &cb) {
  uWord n, i;
  char *string=NULL;
  char **strings=NULL;

  string      =  cb.GetItemDescription(item);
  if (!string) 
    fs << "++ Missing mmCIF item description" << endl;
  else {
    fs << endl;
    fs << "++ mmCIF item description" << endl;
    format_cif_definition(fs, string);
    if (string) delete string;
  }

  string      =  cb.GetItemDescriptionNDB(item);
  if (!string) 
    fs << "++ Missing local item description" << endl;
  else {
    fs << endl;
    fs << "++ Local item description" << endl;
    format_cif_definition(fs, string);
    if (string) delete string;
  }

#if 0
  string      =  cb.GetItemType(item);
  if (!string) 
    fs << "++ Missing item type" << endl;
  else {
    fs << "++ Found item type: " << string << endl;
    if (string) delete string;
  }

  string      =  cb.GetItemPrimitiveCode(item);
  if (!string) 
    fs << "++ Missing item primitive code" << endl;
  else {
    fs << "++ Found item primitive code: " << string << endl;
    if (string) delete string;
  }

  string      =  cb.GetItemRegex(item);
  if (!string) 
    fs << "++ Missing item regex" << endl;
  else {
    fs << "++ Found item regex: " << string << endl;
    if (string) delete string;
  }

  string      =  cb.GetItemMandatoryCode(item);
  if (!string) 
    fs << "++ Missing item mandatory code" << endl;
  else {
    fs << "++ Found item mandatory code: " << string << endl;
    if (string) delete string;
  }

  string      =  cb.GetItemDefaultValue(item);
  if (!string) 
    fs << "++ No item default value" << endl;
  else {
    fs << "++ Found item default value: " << string << endl;
    if (string) delete string;
  }


  strings =  cb.GetItemRangeMin(item,n);
  if (!strings) 
    fs << "++ No item minimum range" << endl;
  else {
    fs << "++ Found " << n << " item minimum boundaries" << endl;
    for (i=0; i < n; i++) {
      fs << "** Minimum value " << i+1 << " " << strings[i] << endl;
      if (strings[i]) delete strings[i];
    }
    if (strings) delete strings;
  }

  strings =  cb.GetItemRangeMax(item,n);
  if (!strings) 
    fs << "++ No item maximum range" << endl;
  else {
    fs << "++ Found " << n << " item maximum boundaries" << endl;
    for (i=0; i < n; i++) {
      fs << "** Maximum value " << i+1 << " " << strings[i] << endl;
      if (strings[i]) delete strings[i];
    }
    if (strings) delete strings;
  }

#endif

  strings =  cb.GetItemExamplesCase(item,n);
  if (!strings) 
    fs << "++ Missing mmCIF item examples (CASE)" << endl;
  else {
    fs << endl;
    fs << "++ Found " << n << " mmCIF item examples (CASE)" << endl;
    for (i=0; i < n; i++) {
      fs << "** CASE " << i+1 << endl;
      format_cif_definition(fs, strings[i]);
      if (strings[i]) delete strings[i];
    }
    if (strings) delete strings;
  }

  strings =  cb.GetItemExamplesDetail(item,n);
  if (!strings) 
    fs << "++ Missing mmCIF item examples (DETAIL)" << endl;
  else {
    fs << endl;
    fs << "++ Found " << n << " mmCIF item examples (DETAIL)" << endl;
    for (i=0; i < n; i++) {
      fs << "** DETAIL " << i+1 << endl;
      format_cif_definition(fs, strings[i]);
      if (strings[i]) delete strings[i];
    }
    if (strings) delete strings;
  }


  strings =  cb.GetItemExamplesCaseNDB(item,n);
  if (!strings) 
    fs << "++ Missing local item examples (CASE)" << endl;
  else {
    fs << endl;
    fs << "++ Found " << n << " local item examples (CASE)" << endl;
    for (i=0; i < n; i++) {
      fs << "** CASE " << i+1 << endl;
      format_cif_definition(fs, strings[i]);
      if (strings[i]) delete strings[i];
    }
    if (strings) delete strings;
  }

  strings =  cb.GetItemExamplesDetailNDB(item,n);
  if (!strings) 
    fs << "++ Missing local item examples (DETAIL)" << endl;
  else {
    fs << endl;
    fs << "++ Found " << n << " local item examples (DETAIL)" << endl;
    for (i=0; i < n; i++) {
      fs << "** DETAIL " << i+1 << endl;
      format_cif_definition(fs, strings[i]);
      if (strings[i]) delete strings[i];
    }
    if (strings) delete strings;
  }
}



static void print_category_dict_features(ostream &fs, const char *category, CifBuilder &cb) {
  
  uWord nexamples, i;
  char **exampleCase=NULL,         **exampleDetail=NULL;
  char **exampleCaseNDB=NULL,      **exampleDetailNDB=NULL;
  char *categoryDescription=NULL, *categoryDescriptionNDB=NULL;


  categoryDescription      =  cb.GetCategoryDescription(category);
  if (!categoryDescription) 
    fs << "++ Missing mmCIF description" << endl;
  else {
    fs << endl;
    fs << "++ Found mmCIF description" << endl;
    format_cif_definition(fs, categoryDescription);
    if (categoryDescription) delete categoryDescription;
  }


  //
    
  categoryDescriptionNDB   =  cb.GetCategoryDescriptionNDB(category);
  if (!categoryDescriptionNDB) 
    fs << "++ Missing local description" << endl;
  else {
    fs << endl;
    fs << "++ Found local description" << endl;
    format_cif_definition(fs, categoryDescriptionNDB);
    if (categoryDescriptionNDB) delete categoryDescriptionNDB;
  }


  
  exampleCase   =  cb.GetCategoryExamplesCase(category,nexamples);
  if (!exampleCase) 
    fs << "++ Missing mmCIF examples (CASE)" << endl;
  else {
    fs << endl;
    fs << "++ Found " << nexamples << " mmCIF examples (CASE)" << endl;
    for (i=0; i < nexamples; i++) {
      fs << "** CASE " << i+1 << endl;
      format_cif_definition(fs, exampleCase[i]);
      if (exampleCase[i]) delete exampleCase[i];
    }
    if (exampleCase) delete exampleCase;
  }

  exampleDetail   =  cb.GetCategoryExamplesDetail(category,nexamples);
  if (!exampleDetail) 
    fs << "++ Missing mmCIF examples (DETAIL)" << endl;
  else {
    fs << endl;
    fs << "++ Found " << nexamples << " mmCIF examples (DETAIL)" << endl;
    for (i=0; i < nexamples; i++) {
      fs << "** DETAIL " << i+1 << endl;
      format_cif_definition(fs, exampleDetail[i]);
      if (exampleDetail[i]) delete exampleDetail[i];
    }
    if (exampleDetail) delete exampleDetail;
  }

  exampleCaseNDB   =  cb.GetCategoryExamplesCaseNDB(category,nexamples);
  if (!exampleCaseNDB) 
    fs << "++ Missing local examples (CASE)" << endl;
  else {
    fs << endl;
    fs << "++ Found " << nexamples << " local examples (CASE)" << endl;
    for (i=0; i < nexamples; i++) {
      fs << "** CASE " << i+1 << endl;
      format_cif_definition(fs, exampleCaseNDB[i]);
      if (exampleCaseNDB[i]) delete exampleCaseNDB[i];
    }
    if (exampleCaseNDB) delete exampleCaseNDB;
  }

  exampleDetailNDB   =  cb.GetCategoryExamplesDetailNDB(category,nexamples);
  if (!exampleDetailNDB) 
    fs << "++ Missing local examples (DETAIL)" << endl;
  else {
    fs << endl;
    fs << "++ Found " << nexamples << " local examples (DETAIL)" << endl;
    for (i=0; i < nexamples; i++) {
      fs << "** DETAIL " << i+1 << endl;
      format_cif_definition(fs, exampleDetailNDB[i]);
      if (exampleDetailNDB[i]) delete exampleDetailNDB[i];
    }
    if (exampleDetailNDB) delete exampleDetailNDB;
  }

}


static void check_category_dict_features(ostream &fs, const char *category, CifBuilder &cb) {
  
  uWord nexamples, i;
  char **exampleCase=NULL,         **exampleDetail=NULL;
  char **exampleCaseNDB=NULL,      **exampleDetailNDB=NULL;
  char *categoryDescription=NULL, *categoryDescriptionNDB=NULL;


  categoryDescription      =  cb.GetCategoryDescription(category);
  if (!categoryDescription) 
    fs << "++ Missing standard description" << endl;
  else {
    fs << "++ Found standard description" << endl;
    format_cif_definition(fs, categoryDescription);
    if (categoryDescription) delete categoryDescription;
  }


  //
    
  categoryDescriptionNDB   =  cb.GetCategoryDescriptionNDB(category);
  if (!categoryDescriptionNDB) 
    fs << "++ Missing local description" << endl;
  else {
    fs << "++ Found local description" << endl;
    format_cif_definition(fs, categoryDescriptionNDB);
    if (categoryDescriptionNDB) delete categoryDescriptionNDB;
  }


  
  exampleCase   =  cb.GetCategoryExamplesCase(category,nexamples);
  if (!exampleCase) 
    fs << "++ Missing standard examples (CASE)" << endl;
  else {
    fs << "++ Found " << nexamples << " standard examples (CASE)" << endl;
    for (i=0; i < nexamples; i++) {
      fs << "** CASE " << i+1 << endl;
      format_cif_definition(fs, exampleCase[i]);
      if (exampleCase[i]) delete exampleCase[i];
    }
    if (exampleCase) delete exampleCase;
  }

  exampleDetail   =  cb.GetCategoryExamplesDetail(category,nexamples);
  if (!exampleDetail) 
    fs << "++ Missing standard examples (DETAIL)" << endl;
  else {
    fs << "++ Found " << nexamples << " standard examples (DETAIL)" << endl;
    for (i=0; i < nexamples; i++) {
      fs << "** DETAIL " << i+1 << endl;
      format_cif_definition(fs, exampleDetail[i]);
      if (exampleDetail[i]) delete exampleDetail[i];
    }
    if (exampleDetail) delete exampleDetail;
  }

  exampleCaseNDB   =  cb.GetCategoryExamplesCaseNDB(category,nexamples);
  if (!exampleCaseNDB) 
    fs << "++ Missing local examples (CASE)" << endl;
  else {
    fs << "++ Found " << nexamples << " local examples (CASE)" << endl;
    for (i=0; i < nexamples; i++) {
      fs << "** CASE " << i+1 << endl;
      format_cif_definition(fs, exampleCaseNDB[i]);
      if (exampleCaseNDB[i]) delete exampleCaseNDB[i];
    }
    if (exampleCaseNDB) delete exampleCaseNDB;
  }

  exampleDetailNDB   =  cb.GetCategoryExamplesDetailNDB(category,nexamples);
  if (!exampleDetailNDB) 
    fs << "++ Missing local examples (DETAIL)" << endl;
  else {
    fs << "++ Found " << nexamples << " local examples (DETAIL)" << endl;
    for (i=0; i < nexamples; i++) {
      fs << "** DETAIL " << i+1 << endl;
      format_cif_definition(fs, exampleDetailNDB[i]);
      if (exampleDetailNDB[i]) delete exampleDetailNDB[i];
    }
    if (exampleDetailNDB) delete exampleDetailNDB;
  }

}



static void template_category_dict_features(ostream &fs, const char *category, CifBuilder &cb) {
  

  char *categoryDescription=NULL, *categoryDescriptionNDB=NULL;

  // use local description in preference to standard description.
  //
  
  categoryDescriptionNDB   =  cb.GetCategoryDescriptionNDB(category);
  if (!categoryDescriptionNDB) {
    categoryDescription      =  cb.GetCategoryDescription(category);
    if (categoryDescription) {
      format_cif_definition_template(fs, categoryDescription);
      if (categoryDescription) delete categoryDescription;
    }
  } else {
    format_cif_definition_template(fs, categoryDescriptionNDB);
    if (categoryDescriptionNDB) delete categoryDescriptionNDB;
  }
  
  //
  //
#if 0
  uWord nexamples, ndetails, i;
  char **exampleCase=NULL,         **exampleDetail=NULL;
  char **exampleCaseNDB=NULL,      **exampleDetailNDB=NULL;
  
  exampleCaseNDB   =  cb.GetCategoryExamplesCaseNDB(category,nexamples);
  if (!exampleCaseNDB) {
    exampleCase     =  cb.GetCategoryExamplesCase(category,nexamples);
    exampleDetail   =  cb.GetCategoryExamplesDetail(category,ndetails);
    if (exampleCase) {
      for (i=0; i < nexamples; i++) {
	if (exampleDetail && ndetails > i) {
	  format_cif_definition_template(fs, exampleDetail[i]);
	  if (exampleDetail[i]) delete exampleDetail[i];
	}
	format_cif_definition_template(fs, exampleCase[i]);
	if (exampleCase[i]) delete exampleCase[i];
      }
      if (exampleCase) delete exampleCase;
    }

  }  else {

    exampleDetailNDB   =  cb.GetCategoryExamplesDetailNDB(category,ndetails);
    for (i=0; i < nexamples; i++) {
      if (exampleDetailNDB && ndetails > i) {
	format_cif_definition_template(fs, exampleDetailNDB[i]);
	if (exampleDetailNDB[i]) delete exampleDetailNDB[i];
      }
      format_cif_definition_template(fs, exampleCaseNDB[i]);
      if (exampleCaseNDB[i]) delete exampleCaseNDB[i];

    }
    if (exampleCaseNDB) delete exampleCaseNDB;
    if (exampleDetailNDB) delete exampleDetailNDB;
  }
#endif
}


static void template_item_dict_features(ostream &fs, const char *item, CifBuilder &cb) {
  uWord i;
  char *string=NULL;


  string      =  cb.GetItemDescriptionNDB(item);
  if (!string)     string      =  cb.GetItemDescription(item);
  if (string) {
    format_cif_definition_template(fs, string);
    if (string) delete string;
  } 

  uWord nexamples, ndetails;
  char **exampleCase=NULL,         **exampleDetail=NULL;
  char **exampleCaseNDB=NULL,      **exampleDetailNDB=NULL;
  
  exampleCaseNDB   =  cb.GetItemExamplesCaseNDB(item,nexamples);
  if (!exampleCaseNDB) {
    exampleCase     =  cb.GetItemExamplesCase(item,nexamples);
    exampleDetail   =  cb.GetItemExamplesDetail(item,ndetails);
    if (exampleCase && nexamples > 0) {
      for (i=0; i < 1; i++) {
	fs << "# For example: " << endl;
	if (exampleDetail && ndetails > i) {
	  format_cif_definition_template(fs, exampleDetail[i]);
	  if (exampleDetail[i]) delete exampleDetail[i];
	}
	format_cif_definition_template(fs, exampleCase[i]);
	if (exampleCase[i]) delete exampleCase[i];
      }
      if (exampleCase) delete exampleCase;
    }

  }  else {

    exampleDetailNDB   =  cb.GetItemExamplesDetailNDB(item,ndetails);
    if (nexamples > 0) {
      for (i=0; i < 1; i++) {
	//	fs << "# For example: " << endl;
	//	if (exampleDetailNDB && ndetails > i) {
	//  format_cif_definition_template(fs, exampleDetailNDB[i]);
	//  if (exampleDetailNDB[i]) delete exampleDetailNDB[i];
	// }
	format_cif_example_template(fs, exampleCaseNDB[i]);
	if (exampleCaseNDB[i]) delete exampleCaseNDB[i];
	
      }
    }
    if (exampleCaseNDB) delete exampleCaseNDB;
    if (exampleDetailNDB) delete exampleDetailNDB;
  }


}



static void format_cif_definition_template(ostream &fs, const char *string)
{
  char line[512];
  int indent, i, iPos;
  if (!string || (strlen(string) == 0)) return;

  iPos=0;
  indent = 512;

  fs << "#" << endl;

  /* 
   *  Get the minimum indentation for non blank lines.
   */  
  while (get_a_line_x_html(string, line, &iPos)) {

    if (nws_strlen(line)) {
      i = count_leading_ws(line);
      if (i < indent) indent = i;
    }
  }
  iPos = i= 0;
  while (get_a_line_x_html(string, line, &iPos)) {
    if (nws_strlen(line)) {
      if ((i==0) && (indent != 0)) {
	fs << "# " << &line[indent-1]; i++;
      } else {
	fs << "# " << &line[indent];
      }
    } else {
      fs << "#" << endl;
    }
  }
  fs << endl << "#" << endl;

}



static void format_cif_example_template(ostream &fs, const char *string)
{
  char line[512];
  int indent, i, iPos, nLines;
  if (!string || (strlen(string) == 0)) return;

  iPos=0;
  nLines=0;
  indent = 512;

  /* 
   *  Get the minimum indentation for non blank lines.
   */  
  while (get_a_line_x_html(string, line, &iPos)) {
    
    if (nws_strlen(line)) {
      nLines++;
      i = count_leading_ws(line);
      if (i < indent) indent = i;
    }
  }
  iPos = i= 0;
  if (nLines > 1) {
    fs << "# For example:" << endl;
    fs << "# " << endl;
    while (get_a_line_x_html(string, line, &iPos)) {
      if (nws_strlen(line)) {
	if ((i==0) && (indent != 0)) {
	  fs << "# " << &line[indent-1]; i++;
	} else {
	  fs << "# " << &line[indent];
	}
      } else {
	fs << "#" << endl;
      }
    }
  } else {
    while (get_a_line_x_html(string, line, &iPos)) {
      if (nws_strlen(line)) {
	fs << "# For example:   " << &line[indent];
      }
    }
  } 
  fs << endl << "#" << endl;
  
}



static void format_cif_definition(ostream &fs, const char *string)
{
  char line[512];
  int indent, i, iPos;
  if (!string || (strlen(string) == 0)) {
    fs << "    +++BLANK++++" << endl;
    return;
  }
  iPos=0;
  indent = 512;

  //  fs << "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" << endl;

  /* 
   *  Get the minimum indentation for non blank lines.
   */  
  while (get_a_line(string, line, &iPos)) {

    if (nws_strlen(line)) {
      i = count_leading_ws(line);
      if (i < indent) indent = i;
    }
  }
  iPos = i= 0;
  while (get_a_line(string, line, &iPos)) {
    if (nws_strlen(line)) {
      if ((i==0) && (indent != 0)) {
	fs << &line[indent-1]; i++;
      } else
	fs << &line[indent];
    } else
    fs << endl;
  }
  fs << endl;
  //  fs << endl << "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" << endl;

}

static int nws_strlen(const char *line)
{
  int i,len;
  len=0;
  if (line == NULL) return(len);
  for (i=0; i < strlen(line); i++) {
    if (!isspace(line[i])) len++;
  }
  return(len);
}



static int count_leading_ws(const char *line)
{
  int i;
  i=0;
  if (line == NULL) return(i);
  while( isspace(line[i])) {
    i++;
  }
  return(i);
}



static int get_a_line(const char *s, char *line, int *iPos)
  /*
   * Get the next new line terminated string in s[iPos] and return
   * this in line[]. Update the nPos. Return line length.
   */
{
  int i, j;
  i=*iPos;
  j=0;
  while (s[i] != '\n' && s[i] != '\0') {
    line[j] = s[i];
    i++; j++;
  }
  if (s[i] == '\n') {
    line[j]='\n'; j++;  i++;
  }
  line[j]='\0'; 
  *iPos = i;
  return(j);
}


static int get_a_line_x_html(const char *s, char *buf, int *iPos)
  /*
   * Get the next new line terminated string in s[iPos] and return
   * this in buf[]. Update the nPos. Return line length.
   *  strip <> along the way.
   */
{
  int i, j, k, ion;
  char line[512];
  
  i=*iPos;
  j=0;
  while (s[i] != '\n' && s[i] != '\0') {
    line[j] = s[i];
    i++; j++;
  }
  if (s[i] == '\n') {
    line[j]='\n'; j++;  i++;
  }
  line[j]='\0'; 
  *iPos = i;

  ion = 1; 
  k = 0;
  for (i=0; i < strlen(line); i++) {
    if (ion && line[i] != '<') {
      buf[k] = line[i];
      k++;
    } else {
      if (line[i] == '>') ion = 1;
      if (line[i] == '<') ion = 0;
    }
  }
  buf[k] = '\0'; k++;
  j = strlen(buf);
  return(j);
}
