/*
FILE:     db-load-op.C
*/
/*
VERSION:  0.904-beta
*/
/*
DATE:     8/14/2003
*/
/*
  Comments and Questions to: sw-help@rcsb.rutgers.edu
*/
/*
COPYRIGHT 1999-2003 Rutgers - The State University of New Jersey

This software is provided WITHOUT WARRANTY OF MERCHANTABILITY OR
FITNESS FOR A PARTICULAR PURPOSE OR ANY OTHER WARRANTY, EXPRESS OR
IMPLIED.  RUTGERS MAKE NO REPRESENTATION OR WARRANTY THAT THE
SOFTWARE WILL NOT INFRINGE ANY PATENT, COPYRIGHT OR OTHER
PROPRIETARY RIGHT.

The user of this software shall indemnify, hold harmless and defend
Rutgers, its governors, trustees, officers, employees, students,
agents and the authors against any and all claims, suits,
losses, liabilities, damages, costs, fees, and expenses including
reasonable attorneys' fees resulting from or arising out of the
use of this software.  This indemnification shall include, but is
not limited to, any and all claims alleging products liability.
*/
/*
               PDB SOFTWARE LICENSE AGREEMENT

BY CLICKING THE ACCEPTANCE BUTTON OR INSTALLING OR USING 
THIS "SOFTWARE, THE INDIVIDUAL OR ENTITY LICENSING THE  
SOFTWARE ("LICENSEE") IS CONSENTING TO BE BOUND BY AND IS 
BECOMING A PARTY TO THIS AGREEMENT.  IF LICENSEE DOES NOT 
AGREE TO ALL OF THE TERMS OF THIS AGREEMENT
THE LICENSEE MUST NOT INSTALL OR USE THE SOFTWARE.

1. LICENSE AGREEMENT

This is a license between you ("Licensee") and the Protein Data Bank (PDB) 
at Rutgers, The State University of New Jersey (hereafter referred to 
as "RUTGERS").   The software is owned by RUTGERS and protected by 
copyright laws, and some elements are protected by laws governing 
trademarks, trade dress and trade secrets, and may be protected by 
patent laws. 

2. LICENSE GRANT

RUTGERS grants you, and you hereby accept, non-exclusive, royalty-free 
perpetual license to install, use, modify, prepare derivative works, 
incorporate into other computer software, and distribute in binary 
and source code format, or any derivative work thereof, together with 
any associated media, printed materials, and on-line or electronic 
documentation (if any) provided by RUTGERS (collectively, the "SOFTWARE"), 
subject to the following terms and conditions: (i) any distribution 
of the SOFTWARE shall bind the receiver to the terms and conditions 
of this Agreement; (ii) any distribution of the SOFTWARE in modified 
form shall clearly state that the SOFTWARE has been modified from 
the version originally obtained from RUTGERS.  

2. COPYRIGHT; RETENTION OF RIGHTS.  

The above license grant is conditioned on the following: (i) you must 
reproduce all copyright notices and other proprietary notices on any 
copies of the SOFTWARE and you must not remove such notices; (ii) in 
the event you compile the SOFTWARE, you will include the copyright 
notice with the binary in such a manner as to allow it to be easily 
viewable; (iii) if you incorporate the SOFTWARE into other code, you 
must provide notice that the code contains the SOFTWARE and include 
a copy of the copyright notices and other proprietary notices.  All 
copies of the SOFTWARE shall be subject to the terms of this Agreement.  

3. NO MAINTENANCE OR SUPPORT; TREATMENT OF ENHANCEMENTS 

RUTGERS is under no obligation whatsoever to: (i) provide maintenance 
or support for the SOFTWARE; or (ii) to notify you of bug fixes, patches, 
or upgrades to the features, functionality or performance of the 
SOFTWARE ("Enhancements") (if any), whether developed by RUTGERS 
or third parties.  If, in its sole discretion, RUTGERS makes an 
Enhancement available to you and RUTGERS does not separately enter 
into a written license agreement with you relating to such bug fix, 
patch or upgrade, then it shall be deemed incorporated into the SOFTWARE 
and subject to this Agreement. You are under no obligation whatsoever 
to provide any Enhancements to RUTGERS or the public that you may 
develop over time; however, if you choose to provide your Enhancements 
to RUTGERS, or if you choose to otherwise publish or distribute your 
Enhancements, in source code form without contemporaneously requiring 
end users or RUTGERS to enter into a separate written license agreement 
for such Enhancements, then you hereby grant RUTGERS a non-exclusive,
royalty-free perpetual license to install, use, modify, prepare
derivative works, incorporate into the SOFTWARE or other computer
software, distribute, and sublicense your Enhancements or derivative
works thereof, in binary and source code form.

4. FEES.  There is no license fee for the SOFTWARE.  If Licensee
wishes to receive the SOFTWARE on media, there may be a small charge
for the media and for shipping and handling.  Licensee is
responsible for any and all taxes.

5. TERMINATION.  Without prejudice to any other rights, Licensor
may terminate this Agreement if Licensee breaches any of its terms
and conditions.  Upon termination, Licensee shall destroy all
copies of the SOFTWARE.

6. PROPRIETARY RIGHTS.  Title, ownership rights, and intellectual
property rights in the Product shall remain with RUTGERS.  Licensee 
acknowledges such ownership and intellectual property rights and will 
not take any action to jeopardize, limit or interfere in any manner 
with RUTGERS' ownership of or rights with respect to the SOFTWARE.  
The SOFTWARE is protected by copyright and other intellectual 
property laws and by international treaties.  Title and related 
rights in the content accessed through the SOFTWARE is the property 
of the applicable content owner and is protected by applicable law.  
The license granted under this Agreement gives Licensee no rights to such
content.

7. DISCLAIMER OF WARRANTY.  THE SOFTWARE IS PROVIDED FREE OF 
CHARGE, AND, THEREFORE, ON AN "AS IS" BASIS, WITHOUT WARRANTY OF 
ANY KIND, INCLUDING WITHOUT LIMITATION THE WARRANTIES THAT IT 
IS FREE OF DEFECTS, MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE 
OR NON-INFRINGING.  THE ENTIRE RISK AS TO THE QUALITY AND 
PERFORMANCE OF THE SOFTWARE IS BORNE BY LICENSEE.  SHOULD THE 
SOFTWARE PROVE DEFECTIVE IN ANY RESPECT, THE LICENSEE AND NOT 
LICENSOR ASSUMES THE ENTIRE COST OF ANY SERVICE AND REPAIR.  
THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF 
THIS AGREEMENT.  NO USE OF THE PRODUCT IS AUTHORIZED HEREUNDER 
EXCEPT UNDER THIS DISCLAIMER.

8. LIMITATION OF LIABILITY.  TO THE MAXIMUM EXTENT PERMITTED BY
APPLICABLE LAW,  IN NO EVENT WILL LICENSOR BE LIABLE FOR ANY 
INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
OUT OF THE USE OF OR INABILITY TO USE THE SOFTWARE, INCLUDING, 
WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK 
STOPPAGE, COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL 
OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF ADVISED OF THE
POSSIBILITY THEREOF. 
*/
/*

  PURPOSE:      Checkpoint and build loadable database archive.

*/
#include <iostream.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>

#include "ndb_htmllib.h"
#include "ndb_misclib.h"

#define  HTML_ENV_GLOBAL_INIT
#include "interface-defs.h"
#include "view-obj-util.h"


static void send_local_update_notification(const char *dirPath, const char *idName, 
					   const char *email);

static char *mk_ndb_db_load_object(const char *idNameLC, const char *archPath, 
				   const char *ndbID, const char *blockId);

static char *mk_db_load_object(const char *idNameLC, const char *archPath, const char *dbName);


int main(int argc, char ** argv) {
    entry entries[HT_MAX_ENTRIES];
    int   nentries;
    char  *ufid=NULL, *prog_name=NULL,  *rcsbID=NULL, *ndbID=NULL; 
    char  *diags=NULL, *diagst=NULL, *ndbdiagst=NULL;

    char  dirPath[HT_MAX_NAME_LENGTH];    
    char  fobjPath[HT_MAX_NAME_LENGTH];    
    char  vobjPath[HT_MAX_NAME_LENGTH];   
    char  archPath[HT_MAX_NAME_LENGTH];     
    char  cifPath[HT_MAX_NAME_LENGTH];
    char  idlc[20];
    
    char idName[20];

    char category[CIF_MAX_NAME_LENGTH], viewName[CIF_MAX_NAME_LENGTH];
    char blockId[CIF_MAX_NAME_LENGTH];
    char  dbNameSave[HT_MAX_NAME_LENGTH];
    int iLoadNDB=0;

    prog_name = argv[0];
    nentries = ht_decode_get_query(entries);
    ht_set_httpd_environment(entries,nentries);

    ht_set_server_url(HT_SERVER_NAME);
    ht_set_server_cgi_path(HT_CGI_PATH);
   
    ndb_open_log(prog_name,HT_LOGFILE_NAME);
    ndb_set_debug_level(NDB_GLOBAL_DEBUG_LEVEL);

    ufid    = ht_get_keyword_value(HT_ATTRIBUTE_UNIQUE_ID,entries,nentries);

    ndb_log_message_text(NDB_MSG_DEBUG,"Got ufid:    %s",ufid);

    strcpy(dirPath,HT_SESSION_ROOT_PATH);
    strcat(dirPath,ufid);
    strcat(dirPath,"/");


    strcpy(fobjPath,dirPath);
    strcat(fobjPath,HT_TABLE_OBJECT_FILE);
    ndb_log_message_text(NDB_MSG_DEBUG,"Opening table object at %s",fobjPath);
    CifFileObj *fobj  = new CifFileObj(fobjPath,APPEND_MODE);

    strcpy(vobjPath,dirPath);
    strcat(vobjPath,HT_VIEW_OBJECT_FILE);
    ndb_log_message_text(NDB_MSG_DEBUG,"Opening view object at %s",vobjPath);
    CifViewObj *vobj = new CifViewObj(vobjPath,(const char *)HT_VIEW_NAME,0);
    ReVarCifArray<CifString> * categoriesV = vobj->GetCategoriesViewable();

    get_current_view_address(dirPath,blockId,category,viewName);
    
    ndb_log_message_text(NDB_MSG_DEBUG,"Current blockId %s category %s",blockId,category);



    rcsbID = get_attribute_value_where(fobj, blockId, "database_2", "database_code", 
				       "database_id", "RCSB");

    if (rcsbID ==NULL) {
      ndb_log_message_text(NDB_MSG_ERR,"Null RCSB ID code");
      ht_put_content_type_target(stdout,"other");
      
      printf("<HTML>\n");
      printf("<BODY BGCOLOR=\"#ffffff\" TEXT=\"#000080\" LINK=\"#000080\" VLINK=\"#000080\">\n");
      printf("<center>");
      printf("<FONT SIZE=7><BOLD>DB Loader Diagnostics</BOLD></FONT><br>\n");

      printf("<hr><br><br><BOLD>ERROR  -- Cannot DB without RCSB ID ... </BOLD><br>\n");
      printf("</center>");
      printf("</BODY>\n");
      printf("</HTML>\n");
      fflush(NULL);
      ndb_close_log();    
      exit(0);
    }

    ndbID=get_attribute_value_where(fobj, blockId, "database_2", "database_code", 
				    "database_id", "NDB");
    if (ndbID != NULL) ndb_set_string_to_upper(ndbID);


    ndb_set_string_to_upper(rcsbID);
    strcpy(idName,rcsbID);

    update_entry_ids(fobj,blockId,idName);

    //    scale_up_R_factor_values(fobj, blockId); // temporary fix

    strcpy(archPath,HT_ARCHIVE_PATH);
    strcat(archPath,HT_ARCHIVE_DIRECTORY);
    strcat(archPath,"/");

    strcpy(idlc,idName);
    ndb_set_string_to_lower(idlc);

    strcat(archPath,idlc);
    strcat(archPath,"/");

    strcpy(cifPath,archPath);
    strcat(cifPath,idlc);
    strcat(cifPath,".cif");

    fobj->RenameBlock(idName,0);
    
    // blockId may change here ... 
#if 1
    strcpy(blockId, idName);
#endif
    update_cif_context(fobj,vobj,dirPath,idlc);

    //
    // Make sure the archive is in sync  before overwriting files
    //
    update_cvs_archive_directory(idlc, HT_ARCHIVE_PATH);
    ndb_log_message_text(NDB_MSG_DEBUG,"Done with 1st CVS update");      
    fobj->Write(cifPath);

    ndb_log_message_text(NDB_MSG_DEBUG,"Done writing CIF");      
    // Update any auxiliary files  ... 
    update_aux_files(idlc,dirPath,archPath);

    ndb_log_message_text(NDB_MSG_DEBUG,"Done updating Aux files.");      

    //
    // If auxilary files have been loaded in this sessions 
    //  move these into the annotation area ... 
    //


    // Status update for RCSB Database only!  Must have a well formed RCSBXXXXX id.
    if (!strncmp(rcsbID,"RCSB",4)) {
      diagst=update_db_status(fobj,dirPath,blockId);
    }

    // Set database context.  This selects between NDB and RCSB databases.

    if (ndbID != NULL && !strcmp(ndbID,rcsbID) && strncmp(rcsbID,"RCSB",4)) {

      diags=mk_db_load_object(idlc, archPath,"ndbv3");
      //      diags = mk_ndb_db_load_object(idlc, archPath, ndbID, blockId);
      strcpy(dbNameSave,"ndbv3");

    } else {

      diags=mk_db_load_object(idlc, archPath, HT_TARGET_DBNAME);
      strcpy(dbNameSave,HT_TARGET_DBNAME);
    }

    // DB update for primary data ... 


    //
    // NDB DB update (optional)
    //
    iLoadNDB = 0;
    if (ndbID != NULL && strcmp(rcsbID,ndbID) ) {
      ndb_log_message_text(NDB_MSG_INFO,"Building load object for RCSB %s NDB %s", 
			   rcsbID, ndbID);
      ndbdiagst=mk_ndb_db_load_object(idlc, archPath, ndbID, blockId);
      iLoadNDB = 1;
    }

    send_local_update_notification(dirPath, idlc, HT_ANNOTATOR_EMAIL);

    update_cvs_archive_directory(idlc, HT_ARCHIVE_PATH);

    ht_put_content_type_target(stdout,"other");

    printf("<HTML>\n");
    printf("<BODY BGCOLOR=\"#ffffff\" TEXT=\"#000080\" LINK=\"#000080\" VLINK=\"#000080\">\n");

    printf("<center>");
    printf("<FONT SIZE=7><BOLD>DB Loader Diagnostics</BOLD></FONT><br>\n");
    printf("</center><br><br><hr>");

    if (diagst != NULL) {
      printf("<font color=\"black\"><b>Loading Status for ID %s</b><br> <pre>\n\n",rcsbID);
      printf("%s",diagst);
      printf("</pre>\n");
    } else {
      printf("<font color=\"black\">  <pre>\n");
      printf("<b>No diagnostics generated loading status DB ....</b>");
      printf("</pre>\n");

    }

    if (diags != NULL) {
      printf("<font color=\"black\"><b>Loading %s DB with ID %s </b><br> <pre>\n\n",dbNameSave,rcsbID);
      printf("%s",diags);
      printf("</pre>\n");
    } else {
      printf("<font color=\"black\">  <pre>\n");
      printf("<b>No diagnostics generated loading DB %s ....</b>",dbNameSave);
      printf("</pre>\n");

    }


    if (ndbdiagst != NULL) {
      printf("<font color=\"black\"> <b>Loading database NDBV3 with ID %s</b><br><br><pre>\n\n", ndbID);
      printf("%s",ndbdiagst);
      printf("</pre>\n");
    } else {
      if (iLoadNDB) {
	printf("<font color=\"black\">  <pre>\n");
	printf("<b>No diagnostics generated loading DB ....</b>");
	printf("</pre>\n");
      }

    }

    printf("</BODY>\n");
    printf("</HTML>\n");
    fflush(NULL);

    // Block id may have changed ... 
#if 1
    set_current_view_address(dirPath,blockId,category,viewName);
#endif
    ndb_close_log();    
    exit(0);
}

#if 0
static void update_cvs_archive_directory(const char *idNameLC, const char *annotPath) {
  char  command[HT_MAX_NAME_LENGTH];
  char  script[2048];
  
  strcpy(script,"");
  sprintf(command,"cd %s; ",annotPath);
  strcat(script,command);
  sprintf(command,"%s -f -d %s commit -m\"Update by Input Tool\" %s/%s >& /var/tmp/cvs.log;  ",
	  HT_CVS_COMMAND,HT_CVS_ARCHIVE_PATH,HT_ARCHIVE_DIRECTORY,idNameLC);
  strcat(script,command);
  ndb_log_message_text(NDB_MSG_DEBUG,"CVS commit script %s",script);
  wsystem(script);
}
#endif

static char *mk_db_load_object(const char *idNameLC, const char *archPath, const char *dbName) {

  char  dbPath[HT_MAX_NAME_LENGTH];
  char  errPath[HT_MAX_NAME_LENGTH];
  char  cifPath[HT_MAX_NAME_LENGTH];
  char  linkPath[HT_MAX_NAME_LENGTH];
  char  command[HT_MAX_NAME_LENGTH];
  char  script[4096];
  char *diags=NULL;
  char loadType[10];
  int len;

  if (!strncmp(dbName,"ndb",3)) {
    strcpy(loadType,"all");
  } else {
    strcpy(loadType,"general");
  }
  
  //
  // I. Make directory to hold db loadable object
  //

  strcpy(dbPath,archPath);
  strcat(dbPath,"db/");
  mkdir(dbPath,0755);

  //
  // II. Make link in this directory to the archival CIF
  //
  strcpy(cifPath,archPath);
  strcat(cifPath,idNameLC);
  strcat(cifPath,".cif");

  strcpy(linkPath,dbPath);
  strcat(linkPath,idNameLC);
  strcat(linkPath,".cif");
  remove(linkPath);
  symlink(cifPath,linkPath);

  //
  //  III.  build db update.
  //

  strcpy(script,"");
  sprintf(command,"cd %s; ",dbPath);
  strcat(script,command);

  // Create the PDB and NDB format files for convenience...
#if 0
  sprintf(command,"%s -o 2 -i %s.cif >& %s; cp -f %s.cif.pdb ../%s.pdb; ",
	  HT_TRANSLATOR_NAME, idNameLC, "TRANSLATOR0.ERR",idNameLC, idNameLC);
  strcat(script,command);

  sprintf(command,"%s -o 103 -i %s.cif >& %s; cp -f %s.cif.ndb ../%s.ndb; ",
	  HT_TRANSLATOR_NAME, idNameLC,"TRANSLATOR1.ERR", idNameLC, idNameLC);
  strcat(script,command);

  sprintf(command,"cp -f %s.cif.ndb %s%s.ndb ",
	  idNameLC,HT_PENDING_PATH,idNameLC);
  strcat(script,command);
  ndb_log_message_text(NDB_MSG_DEBUG,"DB loader setup script %s",script);
  wsystem(script);
  strcpy(script,"");
  sprintf(command,"cd %s; ",dbPath);
  strcat(script,command);
  sprintf(command,"%s -tabtype all -bcp -db %s -file %s.cif.ndb >& %s; ",
	  HT_DBLOADER_NAME, dbName, idNameLC,"DBLOAD.ERR");
  strcat(script,command);
#endif

  translate_coordinate_format("cif", "pdb", dbPath , idNameLC);
  translate_coordinate_format("cif", "ndb", dbPath , idNameLC);

  sprintf(command,"cp -f %s.pdb ../%s.pdb; ", idNameLC, idNameLC);
  strcat(script,command);

  sprintf(command,"cp -f %s.ndb ../%s.ndb; ", idNameLC, idNameLC);
  strcat(script,command);

  //  sprintf(command,"cp -f %s.cif.ndb %s%s.ndb ",
  //	  idNameLC,HT_PENDING_PATH,idNameLC);
  //  strcat(script,command);

  ndb_log_message_text(NDB_MSG_DEBUG,"DB loader setup script %s",script);
  wsystem(script);
  strcpy(script,"");
  sprintf(command,"cd %s; ",dbPath);
  strcat(script,command);

  // JDW TURN OFF COORDINATE LOADS FOR NOW
  //  sprintf(command,"%s -tabtype all -bcp -db %s -file %s.ndb >& %s; ",
  //	  HT_DBLOADER_NAME, dbName, idNameLC,"DBLOAD.ERR");

  sprintf(command,"%s -tabtype %s -bcp -db %s -file %s.ndb >& %s; ",
	  HT_DBLOADER_NAME, loadType, dbName, idNameLC,"DBLOAD.ERR");

  strcat(script,command);



  sprintf(command,"setenv SYBASE %s; setenv DSQUERY %s; setenv DSCONSOLE %s; setenv DSLISTEN %s; ", 
	  HT_SYBASE_PATH, HT_SYBASE_SERVER, HT_SYBASE_SERVER, HT_SYBASE_SERVER);
  strcat(script,command);

  sprintf(command,"setenv NDB_XDBUSER %s; setenv NDB_XDBPW %s; ",
	  HT_DBUSER, HT_DBPW);
  strcat(script,command);


  strcat(script,"./BCP_COMMANDS.csh >>& DBLOAD.ERR; ");

  ndb_log_message_text(NDB_MSG_DEBUG,"NDB DB loader environment U %s PW %s ",HT_DBUSER, HT_DBPW);
  ndb_log_message_text(NDB_MSG_DEBUG,"NDB DB loader environment S %s DB %s",HT_SYBASE_SERVER,dbName);
  ndb_log_message_text(NDB_MSG_DEBUG,"DB loader script %s",script);
  wsystem(script);

  FILE *fp=NULL;
  struct stat statbuf;

  strcpy(errPath,dbPath);
  strcat(errPath,"DBLOAD.ERR");

  if ( (fp=fopen(errPath,"r")) != NULL ) {
    stat(errPath,&statbuf);
    ndb_log_message(NDB_MSG_DEBUG,"Reading %s size %d",errPath,statbuf.st_size);
    diags = (char *) calloc(statbuf.st_size+1, sizeof(char));
    len=fread(diags, sizeof(char), statbuf.st_size+1, fp);
    ndb_log_message(NDB_MSG_DEBUG,"Read %d bytes in %s ",len, errPath);
    fclose(fp);    
  }
  return(diags); 

}



static char *mk_ndb_db_load_object(const char *idNameLC, const char *archPath, 
				   const char *ndbID, const char *blockId) {

  char  dbPath[HT_MAX_NAME_LENGTH];
  char  errPath[HT_MAX_NAME_LENGTH];
  char  cifPath[HT_MAX_NAME_LENGTH];
  char  linkPath[HT_MAX_NAME_LENGTH];
  char  command[HT_MAX_NAME_LENGTH];
  char  script[4096];
  char *diags=NULL;
  char *diagnostics=NULL;
  char  ndbIdLC[20], ndbIdUC[20];

  int len;

  
  if ( (idNameLC == NULL) || (archPath == NULL) || (ndbID == NULL) || 
      !strlen(idNameLC) || !strlen(archPath) || !strlen(ndbID) ) return (diags);

  ndb_log_message_text(NDB_MSG_INFO,"Loading data for NDB id  %s",ndbID);

  memset(ndbIdLC, 0, 20);
  memset(ndbIdUC, 0, 20);

  strcpy(ndbIdLC,ndbID);
  strcpy(ndbIdUC,ndbID);
  ndb_set_string_to_lower(ndbIdLC);
  ndb_set_string_to_upper(ndbIdUC);
  
  //
  // I. Make directory to hold NDB db loadable objects
  //

  strcpy(dbPath,archPath);
  strcat(dbPath,"ndb-db/");

  // Remove the directory first ... 

  strcpy(script,"");
  sprintf(command,"rm -rf %s ", dbPath);
  strcat(script,command);
  wsystem(script);

  mkdir(dbPath,0775);

  //
  // II. Make link in this directory to the archival CIF
  //
  strcpy(cifPath,archPath);
  strcat(cifPath,idNameLC);
  strcat(cifPath,".cif");

  strcpy(linkPath,dbPath);
  strcat(linkPath,idNameLC);
  strcat(linkPath,".cif");
  remove(linkPath);
  symlink(cifPath,linkPath);

  //
  //  IIa.  Make object for NDB file

  char  fobjPath[HT_MAX_NAME_LENGTH];    
  strcpy(fobjPath,dbPath);
  strcat(fobjPath,HT_TABLE_OBJECT_FILE);
  ndb_log_message_text(NDB_MSG_INFO,"Opening table object for NDB file at %s",fobjPath);
  CifFileObj *fobj = new CifFileObj(fobjPath,WRITE_MODE);
  ndb_log_message_text(NDB_MSG_INFO,"Reading CIF file %s", linkPath);
  fobj->Read(linkPath, diagnostics);
  ndb_log_message_text(NDB_MSG_INFO,"Diagnostics reading data %s",diagnostics);


  ndb_log_message_text(NDB_MSG_INFO,"Updating entry ids for NDB id %s in block %s",ndbIdUC,blockId);

  update_entry_ids(fobj,blockId,ndbIdUC);


  strcpy(cifPath,dbPath);
  strcat(cifPath,ndbIdLC);
  strcat(cifPath,".cif");
  
  fobj->RenameBlock(ndbIdUC,0);
  fobj->Write(cifPath);

  ndb_log_message_text(NDB_MSG_INFO,"Wrote CIF in file %s",cifPath);

  translate_coordinate_format("cif", "ndb", dbPath , ndbIdLC);
  translate_coordinate_format("cif", "pdb", dbPath , ndbIdLC);

  //
  //  III.  build db update.
  //
  //

  strcpy(script,"");
  sprintf(command,"cd %s; ",dbPath);
  strcat(script,command);

  //
  sprintf(command,"cp -f %s.pdb ../%s.pdb; ", ndbIdLC, ndbIdLC);
  strcat(script,command);

  sprintf(command,"cp -f %s.ndb ../%s.ndb; ", ndbIdLC, ndbIdLC);
  strcat(script,command);


  // Update pending directory for the public NDB database... 

  sprintf(command,"cp -f %s.ndb %s%s.ndb; ",
	  ndbIdLC, "/pending/ndbv5/",ndbIdLC);
  strcat(script,command);

  sprintf(command,"%s -tabtype all -bcp -db %s -file %s.ndb >& %s; ",
	  HT_DBLOADER_NAME, "ndbv3", ndbIdLC,"DBLOAD.ERR");
  strcat(script,command);

  sprintf(command,"setenv SYBASE %s; setenv DSQUERY %s; setenv DSCONSOLE %s; setenv DSLISTEN %s; ", 
	  HT_SYBASE_PATH, HT_SYBASE_SERVER, HT_SYBASE_SERVER, HT_SYBASE_SERVER);
  strcat(script,command);

  sprintf(command,"setenv NDB_XDBUSER %s; setenv NDB_XDBPW %s; ",
	  HT_DBUSER, HT_DBPW);
  strcat(script,command);

  strcat(script," ./BCP_COMMANDS.csh >>& DBLOAD.ERR; ");

  ndb_log_message_text(NDB_MSG_DEBUG,"NDB DB loader environment U %s PW %s ",HT_DBUSER, HT_DBPW);
  ndb_log_message_text(NDB_MSG_DEBUG,"NDB DB loader environment S %s ",HT_SYBASE_SERVER);
  ndb_log_message_text(NDB_MSG_DEBUG,"NDB DB loader script %s",script);
  wsystem(script);

  FILE *fp=NULL;
  struct stat statbuf;

  strcpy(errPath,dbPath);
  strcat(errPath,"DBLOAD.ERR");

  if ( (fp=fopen(errPath,"r")) != NULL ) {
    stat(errPath,&statbuf);
    ndb_log_message(NDB_MSG_DEBUG,"Reading %s size %d",errPath,statbuf.st_size);
    diags = (char *) calloc(statbuf.st_size+1, sizeof(char));
    len=fread(diags, sizeof(char), statbuf.st_size+1, fp);
    ndb_log_message(NDB_MSG_DEBUG,"Read %d bytes in %s ",len, errPath);
    fclose(fp);    
  }
  return(diags); 
}


static void send_local_update_notification(const char *dirPath, const char *idName, const char *email) {
  char  script[2048];
  char  idlc[20];
  FILE *fp;
  time_t curr_time;
  struct tm *tmptr;
  char *curt;
  char  msgPath[HT_MAX_NAME_LENGTH];    
  char timeString[31];

  static char letter1[]="\
This deposition has been updated in the CVS archive and has been loaded into\n\
the internal database %s.   To access the archival copy of this structure:\n\
\n\
                   cd /annotation/%s/%s\n\
\n\
Warmest regards,\n\
\n\
ADIT\n\
\n";

  time(&curr_time);
  tmptr = localtime(&curr_time);
  curt = ctime(&curr_time);
  ndb_clean_string(curt);
  if (curt[strlen(curt)-1] == '\n') curt[strlen(curt)-1] = '\0';
  strftime(timeString,30,"%Y-%m-%d", tmptr);
  strcpy(idlc,idName);
  ndb_set_string_to_lower(idlc);

  strcpy(msgPath,dirPath);
  strcat(msgPath,"Notify-annotator.let");
  fp=fopen(msgPath,"w");
  if (fp == NULL) return;
  fprintf(fp,"\n\nDate: %s\n\n",timeString);
  fprintf(fp,"Dear Annotator:\n\n");
  fprintf(fp,letter1,HT_TARGET_DBNAME,HT_ARCHIVE_DIRECTORY,idlc);
  fclose(fp);

  sprintf(script,"Mail -s \"Structure %s updated.\" %s < %s ",idName,email,msgPath);
  ndb_log_message_text(NDB_MSG_DEBUG,"Mail script %s",script);
  wsystem(script);
}
