/*
FILE:     html-env-util.C
*/
/*
VERSION:  0.904-beta
*/
/*
DATE:     8/14/2003
*/
/*
  Comments and Questions to: sw-help@rcsb.rutgers.edu
*/
/*
COPYRIGHT 1999-2003 Rutgers - The State University of New Jersey

This software is provided WITHOUT WARRANTY OF MERCHANTABILITY OR
FITNESS FOR A PARTICULAR PURPOSE OR ANY OTHER WARRANTY, EXPRESS OR
IMPLIED.  RUTGERS MAKE NO REPRESENTATION OR WARRANTY THAT THE
SOFTWARE WILL NOT INFRINGE ANY PATENT, COPYRIGHT OR OTHER
PROPRIETARY RIGHT.

The user of this software shall indemnify, hold harmless and defend
Rutgers, its governors, trustees, officers, employees, students,
agents and the authors against any and all claims, suits,
losses, liabilities, damages, costs, fees, and expenses including
reasonable attorneys' fees resulting from or arising out of the
use of this software.  This indemnification shall include, but is
not limited to, any and all claims alleging products liability.
*/
/*
               PDB SOFTWARE LICENSE AGREEMENT

BY CLICKING THE ACCEPTANCE BUTTON OR INSTALLING OR USING 
THIS "SOFTWARE, THE INDIVIDUAL OR ENTITY LICENSING THE  
SOFTWARE ("LICENSEE") IS CONSENTING TO BE BOUND BY AND IS 
BECOMING A PARTY TO THIS AGREEMENT.  IF LICENSEE DOES NOT 
AGREE TO ALL OF THE TERMS OF THIS AGREEMENT
THE LICENSEE MUST NOT INSTALL OR USE THE SOFTWARE.

1. LICENSE AGREEMENT

This is a license between you ("Licensee") and the Protein Data Bank (PDB) 
at Rutgers, The State University of New Jersey (hereafter referred to 
as "RUTGERS").   The software is owned by RUTGERS and protected by 
copyright laws, and some elements are protected by laws governing 
trademarks, trade dress and trade secrets, and may be protected by 
patent laws. 

2. LICENSE GRANT

RUTGERS grants you, and you hereby accept, non-exclusive, royalty-free 
perpetual license to install, use, modify, prepare derivative works, 
incorporate into other computer software, and distribute in binary 
and source code format, or any derivative work thereof, together with 
any associated media, printed materials, and on-line or electronic 
documentation (if any) provided by RUTGERS (collectively, the "SOFTWARE"), 
subject to the following terms and conditions: (i) any distribution 
of the SOFTWARE shall bind the receiver to the terms and conditions 
of this Agreement; (ii) any distribution of the SOFTWARE in modified 
form shall clearly state that the SOFTWARE has been modified from 
the version originally obtained from RUTGERS.  

2. COPYRIGHT; RETENTION OF RIGHTS.  

The above license grant is conditioned on the following: (i) you must 
reproduce all copyright notices and other proprietary notices on any 
copies of the SOFTWARE and you must not remove such notices; (ii) in 
the event you compile the SOFTWARE, you will include the copyright 
notice with the binary in such a manner as to allow it to be easily 
viewable; (iii) if you incorporate the SOFTWARE into other code, you 
must provide notice that the code contains the SOFTWARE and include 
a copy of the copyright notices and other proprietary notices.  All 
copies of the SOFTWARE shall be subject to the terms of this Agreement.  

3. NO MAINTENANCE OR SUPPORT; TREATMENT OF ENHANCEMENTS 

RUTGERS is under no obligation whatsoever to: (i) provide maintenance 
or support for the SOFTWARE; or (ii) to notify you of bug fixes, patches, 
or upgrades to the features, functionality or performance of the 
SOFTWARE ("Enhancements") (if any), whether developed by RUTGERS 
or third parties.  If, in its sole discretion, RUTGERS makes an 
Enhancement available to you and RUTGERS does not separately enter 
into a written license agreement with you relating to such bug fix, 
patch or upgrade, then it shall be deemed incorporated into the SOFTWARE 
and subject to this Agreement. You are under no obligation whatsoever 
to provide any Enhancements to RUTGERS or the public that you may 
develop over time; however, if you choose to provide your Enhancements 
to RUTGERS, or if you choose to otherwise publish or distribute your 
Enhancements, in source code form without contemporaneously requiring 
end users or RUTGERS to enter into a separate written license agreement 
for such Enhancements, then you hereby grant RUTGERS a non-exclusive,
royalty-free perpetual license to install, use, modify, prepare
derivative works, incorporate into the SOFTWARE or other computer
software, distribute, and sublicense your Enhancements or derivative
works thereof, in binary and source code form.

4. FEES.  There is no license fee for the SOFTWARE.  If Licensee
wishes to receive the SOFTWARE on media, there may be a small charge
for the media and for shipping and handling.  Licensee is
responsible for any and all taxes.

5. TERMINATION.  Without prejudice to any other rights, Licensor
may terminate this Agreement if Licensee breaches any of its terms
and conditions.  Upon termination, Licensee shall destroy all
copies of the SOFTWARE.

6. PROPRIETARY RIGHTS.  Title, ownership rights, and intellectual
property rights in the Product shall remain with RUTGERS.  Licensee 
acknowledges such ownership and intellectual property rights and will 
not take any action to jeopardize, limit or interfere in any manner 
with RUTGERS' ownership of or rights with respect to the SOFTWARE.  
The SOFTWARE is protected by copyright and other intellectual 
property laws and by international treaties.  Title and related 
rights in the content accessed through the SOFTWARE is the property 
of the applicable content owner and is protected by applicable law.  
The license granted under this Agreement gives Licensee no rights to such
content.

7. DISCLAIMER OF WARRANTY.  THE SOFTWARE IS PROVIDED FREE OF 
CHARGE, AND, THEREFORE, ON AN "AS IS" BASIS, WITHOUT WARRANTY OF 
ANY KIND, INCLUDING WITHOUT LIMITATION THE WARRANTIES THAT IT 
IS FREE OF DEFECTS, MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE 
OR NON-INFRINGING.  THE ENTIRE RISK AS TO THE QUALITY AND 
PERFORMANCE OF THE SOFTWARE IS BORNE BY LICENSEE.  SHOULD THE 
SOFTWARE PROVE DEFECTIVE IN ANY RESPECT, THE LICENSEE AND NOT 
LICENSOR ASSUMES THE ENTIRE COST OF ANY SERVICE AND REPAIR.  
THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF 
THIS AGREEMENT.  NO USE OF THE PRODUCT IS AUTHORIZED HEREUNDER 
EXCEPT UNDER THIS DISCLAIMER.

8. LIMITATION OF LIABILITY.  TO THE MAXIMUM EXTENT PERMITTED BY
APPLICABLE LAW,  IN NO EVENT WILL LICENSOR BE LIABLE FOR ANY 
INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
OUT OF THE USE OF OR INABILITY TO USE THE SOFTWARE, INCLUDING, 
WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK 
STOPPAGE, COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL 
OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF ADVISED OF THE
POSSIBILITY THEREOF. 
*/
/*
  PURPOSE:      Sets the global environment for all ADIT CGI applications.

*/

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <sys/types.h>

#include "ndb_misclib.h"
#include "ndb_htmllib.h"
#include "interface-defs.h"

#define BGCOLOR_TABLE_TITLE "khaki"
#define BGCOLOR_TABLE       "#000080"

// #define  STANDALONE_MODE 1
#undef   STANDALONE_MODE 


char *ht_get_projdir(const char *moltype, const char *expmeth)
  /*
   * Get the project directory according to exp and mol type.
   * Moltype is one of:  na, naprot, prot, biosync
   * Expmeth is one of:  NMR, X-ray
   *  
   * This is very site dependent... 
   */  
{
  char *st = NULL;
  
  st = (char *)calloc(HT_ENV_MAX_NAME_LEN,sizeof(char));
  if (!moltype || !strlen(moltype)) return(st);
  if (!expmeth || !strlen(expmeth)) return(st);

  if (!strcmp(moltype,"na"))
    strcpy(st,"ndb");
  else if (!strcmp(moltype,"prot"))
    strcpy(st,"prot");
  else if (!strcmp(moltype,"naprot"))
    strcpy(st,"prot");
  else if (!strcmp(moltype,"biosync"))
    strcpy(st,"biosync");

  if (!strcmp(expmeth,"NMR"))
    strcpy(st,"nmr");


  return(st);
}


char *ht_get_dbname(const char *moltype, const char *expmeth)
  /*
   * Get the DBNAME according to exp and mol type.
   * Moltype is one of:  na, naprot, prot
   * Expmeth is one of:  NMR, X-ray
   * 
   * This is very site dependent... 
   */  
{
  char *st = NULL;
  
  st = (char *)calloc(HT_ENV_MAX_NAME_LEN,sizeof(char));
  if (!moltype || !strlen(moltype)) return(st);
  if (!expmeth || !strlen(expmeth)) return(st);

  if (!strcmp(moltype,"na"))
    strcpy(st,"msd1");
  else if (!strcmp(moltype,"prot"))
    strcpy(st,"msd1");
  else if (!strcmp(moltype,"naprot"))
    strcpy(st,"msd1");
  else if (!strcmp(moltype,"biosync"))
    strcpy(st,"msd1");

  return(st);
}


char *ht_get_view_id(const char *moltype, const char *expmeth, const char *viewtype)
  /*
   * Get the view name according to type and current global context.
   * Moltype is one of:  na, naprot, prot
   * Expmeth is one of:  NMR, X-ray
   * viewtype is the designator for the view level of detail.
   */  
{
  char *st = NULL;

  ndb_log_message_text(NDB_MSG_DEBUG,"ht_get_view_id: moltype %s expmeth %s", moltype, expmeth);

  st = (char *)calloc(HT_ENV_MAX_NAME_LEN,sizeof(char));

  if (!moltype  || !strlen(moltype) || 
      !expmeth  || !strlen(expmeth) || 
      !viewtype || !strlen(viewtype) ) {
    strcpy(st,"View-pdbx-default.cif");
    return(st);
  }

#if 0
  if (!strcasecmp(viewtype,"sg1")) {
    strcpy(st,"View-pdbx-sg1.cif");
    return(st);
  } else if (!strcasecmp(viewtype,"sg2")) {
    strcpy(st,"View-pdbx-sg2.cif");
    return(st);
  } else if (!strcasecmp(viewtype,"sg3")) {
    strcpy(st,"View-pdbx-sg3.cif");
    return(st);
  }
#endif
    strcpy(st,viewtype);
    return(st);
  

  //
  // Assembler view is the same for all types and methods.
  //
  if (HT_APPL_CONTEXT_TYPE == HT_CONTEXT_ASSEMBLE) {
    strcpy(st,"View-assemble.cif");
    return(st);
  }

  // 

  strcpy(st,"View-pdbx-");

  //JDW 26-Jun-2003  No mol variation.
#if 0
  if (!strcmp(moltype,"na"))
    strcat(st,"na-");

  else if (!strcmp(moltype,"prot"))
    strcat(st,"prot-");

  else if (!strcmp(moltype,"naprot"))
    strcat(st,"naprot-");

  else if (!strcmp(moltype,"biosync"))
    strcat(st,"biosync-");
#endif 

  if (!strcasecmp(expmeth,"nmr"))
    strcat(st,"nmr-");
  else if (!strcasecmp(expmeth,"x-ray"))
    strcat(st,"xray-");

  if (HT_APPL_CONTEXT_TYPE == HT_CONTEXT_DEPOSIT) 
    strcat(st,"deposit");
  else   if (HT_APPL_CONTEXT_TYPE == HT_CONTEXT_ANNOTATE) 
    strcat(st,"deposit");


  strcat(st,".cif");
  return(st);
}


void ht_set_httpd_environment(entry entries[], int nentries)
/*
 *  set_httpd_environment(), here we set up the global environment for
 *  all cgi applications according SERVER and PORT returned by httpd.
 *  Parameters common to all applications are also extracted here.
 *
 *  This can be called in two way:  
 *  
 *  nentries = 0  picks up the environment but does not parse
 *                any cgi arguments.
 *  nentries > 0  picks up the environment and parses the cgi
 *                arguments as well.  
 *  
 *  The first case above is used to establish a minimum environment
 *  necessary to upload files in posts of multipart/form-data.
 */
{
    char  www_root[HT_ENV_MAX_NAME_LEN];         
    char  icon_path[HT_ENV_MAX_NAME_LEN];         
    char  tmp_path[HT_ENV_MAX_NAME_LEN];         
    char  server[HT_ENV_MAX_NAME_LEN];         
    char  port[HT_ENV_MAX_NAME_LEN];         
    char  rcsb_support_path[HT_ENV_MAX_NAME_LEN];         
    char *projDir=NULL, *eMail=NULL, *dbName=NULL, *context=NULL;
    char *moltype=NULL, *expmeth=NULL, *viewtype=NULL;
    char lang[3];
    int ilen=0, i_translator_version=8;
    


#if defined(STANDALONE_MODE)
    strcpy(server,"wombat.rutgers.edu");
    strcpy(port,"80");
#else
    char *server_env=NULL, *port_env=NULL;
    server_env = getenv("SERVER_NAME");
    memset(server,'\0',HT_ENV_MAX_NAME_LEN);
    strcpy(server,server_env);
    port_env  = getenv("SERVER_PORT");
    memset(port,'\0',HT_ENV_MAX_NAME_LEN);
    strcpy(port,port_env);
#endif


    // jdw -  Set the initial application name ...  
    //        When nentries=0 the application name below must
    //        point to a real file hierachy assumed by the
    //        adit environment.   This is required to locate
    //        binaries and icons before the full cgi
    //        environment is parsed.
    // 
    //
#if defined(ADITAPPL)
    strcpy(HT_APPLICATION_NAME,"adit");
#else
    strcpy(HT_APPLICATION_NAME,"validate");
#endif
    HT_OSAKA_SITE_FLAG = 0;
    strcpy(HT_CGI_PATH,   "/cgi-bin/");             
    strcat(HT_CGI_PATH,   HT_APPLICATION_NAME);             
    strcat(HT_CGI_PATH,   "/");             


    // ----------------------------------------------------------------------------
    //           Construct the URL for this  server ... 
    //

    strcpy(HT_SERVER_NAME, "http://");
    strcat(HT_SERVER_NAME, server);
    strcat(HT_SERVER_NAME, ":"); 
    strcat(HT_SERVER_NAME, port); 

    HT_APPL_CONTEXT_TEST=0;

    // ----------------------------------------------------------------------------
    //           Server/Port specific specifications go here ...
    //

    if ( !strcasecmp(server,"127.0.0.1") && !strcmp(port,"9876")  ) {
      strcpy(www_root,             "/wwwadit");                  
      strcpy(HT_SERVER_PATH,       "/wwwadit/docs");  


      strcpy(HT_ARCHIVE_PATH,      www_root);      
      strcat(HT_ARCHIVE_PATH,      "/adit/annotation/");
      strcpy(HT_CVS_ARCHIVE_PATH,  www_root);
      strcat(HT_CVS_ARCHIVE_PATH,  "/adit/data-archive/");
      strcpy(HT_PENDING_PATH,      www_root);
      strcpy(HT_PENDING_PATH,      "/adit/pending");

      strcpy(HT_RCSBAPPS_PATH,     www_root);  
      strcat(HT_RCSBAPPS_PATH,     "/adit/rcsbapps");  

      strcpy(HT_RCSBAPPS_PATH,     "/wwwadit/adit/rcsbapps");  
      strcpy(HT_GZCAT_COMMAND,     "/bin/gzcat");
      strcpy(HT_ZCAT_COMMAND,      "/bin/zcat");

      strcpy(HT_SYBASE_PATH,       "xxxxx");
      strcpy(HT_SYBASE_SERVER,     "xxxxx");
      strcpy(HT_DBPW,              "xxxxx");
      strcpy(HT_DBUSER,            "xxxxx");
      i_translator_version = 8;

    } else if ( !strcasecmp(server,"pengi.rutgers.edu") && !strcmp(port,"8000")  ) {
      strcpy(www_root,             "/WWWadit");                  
      strcpy(HT_SERVER_PATH,       "/WWWadit/docs");  


      strcpy(HT_ARCHIVE_PATH,      www_root);      
      strcat(HT_ARCHIVE_PATH,      "/adit/annotation/");
      strcpy(HT_CVS_ARCHIVE_PATH,  www_root);
      strcat(HT_CVS_ARCHIVE_PATH,  "/adit/data-archive/");
      strcpy(HT_PENDING_PATH,      www_root);
      strcpy(HT_PENDING_PATH,      "/adit/pending");

      strcpy(HT_RCSBAPPS_PATH,     www_root);  
      strcat(HT_RCSBAPPS_PATH,     "/adit/rcsbapps");  

      strcpy(HT_RCSBAPPS_PATH,     "/WWWadit/adit/rcsbapps");  
      strcpy(HT_GZCAT_COMMAND,     "/bin/gzcat");
      strcpy(HT_ZCAT_COMMAND,      "/bin/zcat");

      strcpy(HT_SYBASE_PATH,       "xxxxx");
      strcpy(HT_SYBASE_SERVER,     "xxxxx");
      strcpy(HT_DBPW,              "xxxxx");
      strcpy(HT_DBUSER,            "xxxxx");

      i_translator_version = 8;
      
    } else {
      ht_put_content_type(stdout);
      ht_put_title(stdout,"Configuration error ...");
      ht_put_open_body(stdout);
      ht_put_header1_text(stdout,"Configuration error ...");
      fprintf(stdout,"<pre>\n");
      fprintf(stdout,"Server = %s\n",server);
      fprintf(stdout,"Port = %s\n",port);
      fprintf(stdout,"</pre>\n");
      ht_put_ndb_address(stdout);
      ht_put_close_body(stdout);
      return;

    }


    //
    //  Leave here if called with empty list of CGI parameters
    //
    strcpy(HT_ASCII_FILTER_COMMAND,  HT_RCSBAPPS_PATH);
    strcat(HT_ASCII_FILTER_COMMAND,  "/bin/ascii_filter");

    strcpy(HT_SYS_TMP_PATH,www_root);
    strcat(HT_SYS_TMP_PATH,"/adit/scratch/");

    if (nentries == 0 ) {
      strcpy(HT_LOGFILE_NAME, HT_SYS_TMP_PATH);           
      strcat(HT_LOGFILE_NAME, "upload-");
      strcat(HT_LOGFILE_NAME, port);
      strcat(HT_LOGFILE_NAME, ".log");  
      strcat(HT_SYS_TMP_PATH,"upload/");
      return;
    }



    // ----------------------------------------------------------------------------
    //   Expect to find  - projdir, email, dbname, and context from the CGI 
    //   
    //

    projDir = ht_get_keyword_value(HT_ATTRIBUTE_PROJDIR,entries,nentries);
    eMail   = ht_get_keyword_value(HT_ATTRIBUTE_EMAIL,  entries,nentries);
    dbName  = ht_get_keyword_value(HT_ATTRIBUTE_DBNAME, entries,nentries);
    context = ht_get_keyword_value(HT_ATTRIBUTE_CONTEXT,entries,nentries);

    expmeth   = ht_get_keyword_value(HT_ATTRIBUTE_EXP_METH,entries,nentries);
    moltype   = ht_get_keyword_value(HT_ATTRIBUTE_MOL_TYPE,entries,nentries);
    viewtype  = ht_get_keyword_value(HT_ATTRIBUTE_VIEW_TYPE,entries,nentries);

    
    if (projDir && strlen(projDir)) 
      strcpy(HT_ARCHIVE_DIRECTORY,projDir);
    //    else
    //strcpy(HT_ARCHIVE_DIRECTORY,"ndb");
    
    if (eMail && strlen(eMail) ) 
      strcpy(HT_ANNOTATOR_EMAIL,eMail);    
    else 
      strcpy(HT_ANNOTATOR_EMAIL,"jwest@rcsb.rutgers.edu");    
    
    if (dbName && strlen(dbName) ) 
      strcpy(HT_TARGET_DBNAME, dbName);
    else
      strcpy(HT_TARGET_DBNAME,"msd1");

    //
    //
    if (expmeth && strlen(expmeth) ) 
      strcpy(HT_EXP_METHOD, expmeth);
    else
      strcpy(HT_EXP_METHOD,"");

    if (moltype && strlen(moltype) ) 
      strcpy(HT_MOL_TYPE, moltype);
    else
      strcpy(HT_MOL_TYPE,"");
    
    if (context && strlen(context) ) 
      strcpy(HT_APPL_CONTEXT, context);
    else
      strcpy(HT_APPL_CONTEXT, "annotate-en");      

    if (viewtype && strlen(viewtype))
      strcpy(HT_VIEW_TYPE,viewtype);
    else
      strcpy(HT_VIEW_TYPE,"default");


    
    HT_CURRENT_LANG = LANG_TYPE_EN;      
    ilen = strlen(HT_APPL_CONTEXT);
    if ( ilen > 3) {
      strcpy(lang,"");
      if (HT_APPL_CONTEXT[ilen-3] == '-') {
	lang[0] = HT_APPL_CONTEXT[ilen-2];
	lang[1] = HT_APPL_CONTEXT[ilen-1];
	if (!strcmp(lang,"es") )
	  HT_CURRENT_LANG = LANG_TYPE_ES;      
	else if (!strcmp(lang,"de"))
	  HT_CURRENT_LANG = LANG_TYPE_DE;      
	else if (!strcmp(lang,"fr"))
	  HT_CURRENT_LANG = LANG_TYPE_FR;      
      }
    }
    
    // ----------------------------------------------------------------------------
    // These are the 4 currently supported functional contexts...
    //
    if ( !strncmp(HT_APPL_CONTEXT,"deposit",7)) 
      HT_APPL_CONTEXT_TYPE=HT_CONTEXT_DEPOSIT;
    else if ( !strncmp(HT_APPL_CONTEXT,"assemble",8)) 
      HT_APPL_CONTEXT_TYPE=HT_CONTEXT_ASSEMBLE;
    else if ( !strncmp(HT_APPL_CONTEXT,"annotate",8)) 
      HT_APPL_CONTEXT_TYPE=HT_CONTEXT_ANNOTATE;
    else if ( !strncmp(HT_APPL_CONTEXT,"validate",8)) 
      HT_APPL_CONTEXT_TYPE=HT_CONTEXT_VALIDATE;

    //
    // Reset the HT_APPLICATION_NAME according to context
    //
    if (HT_APPL_CONTEXT_TYPE==HT_CONTEXT_VALIDATE) 
      strcpy(HT_APPLICATION_NAME,"validate");
    else
      strcpy(HT_APPLICATION_NAME,"adit");

    // JDW - This is to be used as part of a URL to access 
    //       temporary HTML files and images.
    //
    strcpy(tmp_path,   "/");
    strcat(tmp_path,   HT_APPLICATION_NAME);
    strcat(tmp_path,   "/tmp/");

    strcpy(icon_path,   "/");
    strcat(icon_path,   HT_APPLICATION_NAME);
    strcat(icon_path,   "/icons/");



    //    demo   = ht_get_keyword_value(HT_ATTRIBUTE_DEMO,entries,nentries);
    //    if (demo && (!strcmp(demo,"yes") || !strcmp(demo,"y")))  HT_APPL_CONTEXT_DEMO=1;



    // ----------------------------------------------------------------------------
    //  Some HTML STYLE Settings ...
    //

    strcpy(LANG_HTML_BODY_SCHEME,"<BODY BGCOLOR=\"#003399\" TEXT=\"#ffffff\" LINK=\"#ffff00\" VLINK=\"#ffff00\">");
    strcpy(LANG_HTML_BODY_SCHEME_ALT_1,"<BODY BGCOLOR=\"#ffffff\" TEXT=\"#000080\" LINK=\"#000080\" VLINK=\"#000080\">");
    strcpy(LANG_HTML_BODY_SCHEME_ALT_2,"<BODY BGCOLOR=\"#ffffff\" TEXT=\"#000000\" LINK=\"#000000\" VLINK=\"#000000\">");
    strcpy(LANG_HTML_FONT_FACE,"helvetica, arial");

    // ----------------------------------------------------------------------------
    // Language specific messages ...

    strcpy(LANG_MMCIFIT,"&nbsp&nbsp<B>AD<sub>IT</sub></B>");
    strcpy(LANG_RASMOL,"RasMol");
    strcpy(LANG_NDBVIEW,"NDBView");

    if (HT_CURRENT_LANG == LANG_TYPE_EN) {
      strcpy(LANG_CATEGORY_TITLE,"Categories");
      strcpy(LANG_ITEM_TITLE,"Data Items");
      strcpy(LANG_DISPLAY_TABLE,"Display Data as Table");
      strcpy(LANG_HELP,"Help");
      strcpy(LANG_DESCRIBE,"Describe");
      strcpy(LANG_EXAMPLES,"Example");
      strcpy(LANG_RESTART_ID,"Your restart ID =");
      strcpy(LANG_CHECK,"Check");
      strcpy(LANG_DEPOSIT,"Deposit");
      strcpy(LANG_QUIT,"Quit");
      strcpy(LANG_NOTIFY_AUTHOR,"Notify Author");
      strcpy(LANG_WRITE_CIF,"Write CIF");
      strcpy(LANG_WRITE_PDB,"Write PDB");
      strcpy(LANG_WRITE_NDB,"Write NDB");
      strcpy(LANG_DB_LOAD,"DB Load");
      strcpy(LANG_DESCRIPTION_TITLE,"Description of");
      strcpy(LANG_HELP_TITLE,"Description of");
      strcpy(LANG_EXAMPLE_TITLE,"Examples of");
      strcpy(LANG_ENTER_DATA_TITLE,"Enter Data in Category");
      strcpy(LANG_DATA_TABLE_TITLE,"Table of Data in Category");
      strcpy(LANG_DIAGNOSTICS_TITLE,"Diagnostics for Category");
    } else if (HT_CURRENT_LANG == LANG_TYPE_FR) {
      strcpy(LANG_CATEGORY_TITLE,"Cat&eacute;gories")	;
      strcpy(LANG_ITEM_TITLE,"Donn&eacute;es &eacute;l&eacute;mentaires");	
      strcpy(LANG_DISPLAY_TABLE,"Donn&eacute;es d'affichage comme Tableau");
      strcpy(LANG_HELP,"Aide");
      strcpy(LANG_DESCRIBE,"D&eacute;crivez");
      strcpy(LANG_EXAMPLES,"Exemple");
      strcpy(LANG_RESTART_ID,"Votre identification de relancement =");
      strcpy(LANG_CHECK,"Contr&ocirc;le");
      strcpy(LANG_DEPOSIT,"D&eacute;p&ocirc;t");
      strcpy(LANG_QUIT,"Quitt&eacute;");
      strcpy(LANG_NOTIFY_AUTHOR,"Informez L'Auteur");
      strcpy(LANG_WRITE_CIF,"&Eacute;crivez CIF");
      strcpy(LANG_WRITE_PDB,"&Eacute;crivez PDB");
      strcpy(LANG_WRITE_NDB,"&Eacute;crivez NDB");
      strcpy(LANG_DB_LOAD,"Chargement De DB");
      strcpy(LANG_DESCRIPTION_TITLE,"Description de");
      strcpy(LANG_HELP_TITLE,"Description de");
      strcpy(LANG_EXAMPLE_TITLE,"Exemples de");
      strcpy(LANG_ENTER_DATA_TITLE,"Saisissez les donn&eacute;es dans la cat&eacute;gorie");
      strcpy(LANG_DATA_TABLE_TITLE,"Tableau des donn&eacute;es dans la cat&eacute;gorie");
      strcpy(LANG_DIAGNOSTICS_TITLE,"Diagnostic pour la cat&eacute;gorie");
    } else if (HT_CURRENT_LANG == LANG_TYPE_DE){ 
      strcpy(LANG_CATEGORY_TITLE,"Kategorien");
      strcpy(LANG_ITEM_TITLE,"Datenelemente");
      strcpy(LANG_DISPLAY_TABLE,"Bildschirmanzeigedaten als Tabelle");
      strcpy(LANG_HELP,"Hilfe");
      strcpy(LANG_DESCRIBE,"Beschreiben Sie");
      strcpy(LANG_EXAMPLES,"Beispiele");
      strcpy(LANG_RESTART_ID,"Ihr Wiederanlauf Identifikation =");
      strcpy(LANG_CHECK,"&Uuml;berpr&uuml;fung");
      strcpy(LANG_DEPOSIT,"Ablagerung");
      strcpy(LANG_QUIT,"Beendet");
      strcpy(LANG_NOTIFY_AUTHOR,"Benachrichtigen Sie Autor");
      strcpy(LANG_WRITE_CIF,"Schreiben Sie CIF");
      strcpy(LANG_WRITE_PDB,"Schreiben Sie PDB");
      strcpy(LANG_WRITE_NDB,"Schreiben Sie NDB");
      strcpy(LANG_DB_LOAD,"DB-Eingabe");
      strcpy(LANG_DESCRIPTION_TITLE,"Beschreibung von");
      strcpy(LANG_HELP_TITLE,"Beschreibung von");
      strcpy(LANG_EXAMPLE_TITLE,"Beispiele von");
      strcpy(LANG_ENTER_DATA_TITLE,"Geben Sie Daten in der Kategorie ein");
      strcpy(LANG_DATA_TABLE_TITLE,"Tabelle von Daten in der Kategorie");
      strcpy(LANG_DIAGNOSTICS_TITLE,"Diagnose f&uuml;r Kategorie");
    } else if (HT_CURRENT_LANG == LANG_TYPE_ES) {
      strcpy(LANG_CATEGORY_TITLE,"Categor&iacute;as");
      strcpy(LANG_ITEM_TITLE,"Items De Datos");
      strcpy(LANG_DISPLAY_TABLE,"Datos de la visualizaci&oacute;n como vector");
      strcpy(LANG_HELP,"Ayuda");
      strcpy(LANG_DESCRIBE,"Describa");
      strcpy(LANG_EXAMPLES,"Ejemplo");
      strcpy(LANG_RESTART_ID,"Su identificaci&oacute;n del relanzar =");
      strcpy(LANG_CHECK,"Cheque");
      strcpy(LANG_DEPOSIT,"Dep&oacute;sito");
      strcpy(LANG_QUIT,"Salido");
      strcpy(LANG_NOTIFY_AUTHOR,"Notifique Al Autor");
      strcpy(LANG_WRITE_CIF,"Escriba CIF");
      strcpy(LANG_WRITE_PDB,"Escriba PDB");
      strcpy(LANG_WRITE_NDB,"Escriba NDB");
      strcpy(LANG_DB_LOAD,"Carga Del DB");
      strcpy(LANG_DESCRIPTION_TITLE,"Descripci&oacute;n de");
      strcpy(LANG_HELP_TITLE,"Descripci&oacute;n de");
      strcpy(LANG_EXAMPLE_TITLE,"Ejemplos de");
      strcpy(LANG_ENTER_DATA_TITLE,"Incorpore los datos en categor&iacute;a");
      strcpy(LANG_DATA_TABLE_TITLE,"Vector de datos en categor&iacute;a");
      strcpy(LANG_DIAGNOSTICS_TITLE,"Diagn&oacute;stico para la categor&iacute;a");
   }
    // ----------------------------------------------------------------------------
    //                     Images and icons ...
    //
    
#if defined(MSDTOOL)
    if (!strcmp(HT_APPLICATION_NAME,"adit")) {
      strcpy(HT_ICON_LOGO, icon_path);
      strcat(HT_ICON_LOGO, "adit-logo-1.gif");
    } else     if (!strcmp(HT_APPLICATION_NAME,"validate")) {
      strcpy(HT_ICON_LOGO, icon_path);
      strcat(HT_ICON_LOGO, "validate-logo-1.gif");
    } else {
      strcpy(HT_ICON_LOGO, icon_path);
      strcat(HT_ICON_LOGO, "adit-logo-1.gif");
    }
#elif defined(NDBTOOL)
    strcpy(HT_ICON_LOGO, icon_path);
    strcat(HT_ICON_LOGO, "ndb-logo-1.gif");

#else
    strcpy(HT_ICON_LOGO, icon_path);
    strcat(HT_ICON_LOGO, "adit-logo-1.gif");
#endif

#if 0
    strcpy(HT_BLANK_HTML_FILE,"/adit/icons/blank.html");
    strcpy(HT_ICON_KEY,"/adit/icons/key1.gif");
    strcpy(HT_ICON_BULLET,"/adit/icons/bulb1-22.gif");
    strcpy(HT_ICON_SAVE_ROW,"/adit/icons/arrow-1-15.gif");
    strcpy(HT_ICON_ITEM_ERROR,"/adit/icons/go.gif");
    strcpy(HT_ICON_EXAMPLES,"/adit/icons/book0b-15.gif");
    strcpy(HT_ICON_HELP,"/adit/icons/query1c-15.gif");
    strcpy(HT_ICON_INSERT,"/adit/icons/insert1-22.gif");
    strcpy(HT_ICON_DELETE,"/adit/icons/cross1-22.gif");
    strcpy(HT_ICON_NEXT,"/adit/icons/down-arrow-1-15.gif");
    strcpy(HT_ICON_PREVIOUS,"/adit/icons/up-arrow-1-15.gif");
#endif

    strcpy(rcsb_support_path,HT_RCSBAPPS_PATH);
    strcat(rcsb_support_path,"/ndb-support-files/datafiles");

    strcpy(HT_TRANSLATOR_NAME,      HT_RCSBAPPS_PATH);
    strcpy(HT_VALIDATOR_NAME,       HT_RCSBAPPS_PATH);

    strcpy(HT_DBLOADER_NAME,        HT_RCSBAPPS_PATH);
    strcpy(HT_CVS_COMMAND,          HT_RCSBAPPS_PATH);

    strcpy(HT_PDB_PREFILTER_NAME,   HT_RCSBAPPS_PATH);
    strcpy(HT_CIF_PREFILTER_NAME,   HT_RCSBAPPS_PATH);

    if (i_translator_version == 8) {
      strcat(HT_TRANSLATOR_NAME,       "/bin/maxit-v8-O -exchange_in -exchange_out -path ");
      strcat(HT_TRANSLATOR_NAME,       HT_RCSBAPPS_PATH);
      strcat(HT_VALIDATOR_NAME,        "/bin/validation-v8 -public -exchange -path ");
      strcat(HT_VALIDATOR_NAME,        HT_RCSBAPPS_PATH);
    } else {
      strcat(HT_TRANSLATOR_NAME,       "/bin/maxit-v7-O  -path ");
      strcat(HT_TRANSLATOR_NAME,       HT_RCSBAPPS_PATH);
      strcat(HT_VALIDATOR_NAME,        "/bin/validation  -path ");
      strcat(HT_VALIDATOR_NAME,        HT_RCSBAPPS_PATH);
    }
    strcat(HT_DBLOADER_NAME,         "/bin/ndbloader -path ");
    strcat(HT_DBLOADER_NAME,         rcsb_support_path);

    strcat(HT_CVS_COMMAND,           "/bin/cvs");
    strcat(HT_PDB_PREFILTER_NAME,    "/bin/prefilter");
    strcat(HT_CIF_PREFILTER_NAME,    "/bin/CheckCIFData");

    strcpy(HT_HTML_TRANSLATOR_NAME,   www_root);
    strcat(HT_HTML_TRANSLATOR_NAME,   HT_CGI_PATH);
    strcat(HT_HTML_TRANSLATOR_NAME,  "TableToText");

    strcpy(HT_SPLUS_INTERPRETER,  "/usr/local/bin/Splus5");

    strcpy(HT_CIFEXCH_IN_TRANSLATOR, HT_RCSBAPPS_PATH);
    strcat(HT_CIFEXCH_IN_TRANSLATOR, "/bin/Translator-in.csh");
    strcpy(HT_CIFEXCH_OUT_TRANSLATOR,HT_RCSBAPPS_PATH);
    strcat(HT_CIFEXCH_OUT_TRANSLATOR, "/bin/Translator-out.csh");

    // JDW - Navigation issue here... This is used only as a return url.
    strcpy(HT_HOME_PATH,  "/");
    strcat(HT_HOME_PATH,  HT_APPLICATION_NAME);
    strcat(HT_HOME_PATH,  "/");

    // JDW - Absolute path to the root  session directory... 
    strcpy(HT_SESSION_ROOT_PATH, www_root);
    strcat(HT_SESSION_ROOT_PATH, "/adit/sessions/");

    // JDW - Absolute path to the root session temporary directory... 
    strcpy(HT_SESSION_TMP_ROOT_PATH, HT_SERVER_PATH);
    strcat(HT_SESSION_TMP_ROOT_PATH, tmp_path);

    // JDW - Relative path to the root of temporary directory - used to build urls.
    strcpy(HT_SESSION_REL_TMP_ROOT_PATH,tmp_path);


    // JDW - Absolute path to config and site directories... 
    strcpy(HT_CONFIG_PATH, www_root);
    strcat(HT_CONFIG_PATH, "/adit/config/");
    strcat(HT_CONFIG_PATH, HT_APPLICATION_NAME);
    strcat(HT_CONFIG_PATH, "/");

    strcpy(HT_SITE_PATH, www_root);
    strcat(HT_SITE_PATH, "/adit/site/");

    strcpy(HT_PDB_ID_FILE, HT_SITE_PATH);
    strcat(HT_PDB_ID_FILE,"PDB_IDS.DAT");

    // JDW - Absolute path to pending directory ...
    strcat(HT_PENDING_PATH,HT_TARGET_DBNAME);
    strcat(HT_PENDING_PATH,"/");

    strcpy(HT_LOGFILE_NAME, HT_SYS_TMP_PATH);           
    strcat(HT_LOGFILE_NAME, HT_APPLICATION_NAME);
    strcat(HT_LOGFILE_NAME, "-");
    strcat(HT_LOGFILE_NAME, port);
    strcat(HT_LOGFILE_NAME, ".log");  


    if (!strncmp(HT_APPL_CONTEXT,"bmcd",4)) {
      strcpy(HT_CIFDB_DICT,"cif_bmcd.dic");
      strcpy(HT_VIEW_NAME,"view");
    } else {
      strcpy(HT_VIEW_NAME,"view");
      strcpy(HT_CIFDB_DICT,"cif_pdbx.dic");
    }

    strcpy(HT_SCHEMA_OBJ_PATH,HT_CONFIG_PATH);
    strcat(HT_SCHEMA_OBJ_PATH,"status_rcsb_schema.odb");

    strcpy(HT_SCHEMA_CIF_PATH,HT_CONFIG_PATH);
    strcat(HT_SCHEMA_CIF_PATH,"status_rcsb_schema.cif");


    strcpy(HT_CIFDB_PATH,HT_CONFIG_PATH);
    strcat(HT_CIFDB_PATH,"dict.odb");

#if 0
    if (!strcasecmp(HT_EXP_METHOD,"NMR")) {
      strcat(HT_CIFDB_PATH,"dict.odb");
    } else {
      strcat(HT_CIFDB_PATH,"dict.odb");
    }
#endif
    //    ndb_log_message_text(NDB_MSG_INFO,"html_env: Method is %s using dictionary path %s mol %s",
    //			 HT_EXP_METHOD,HT_CIFDB_PATH, HT_MOL_TYPE);

    strcpy(HT_VIEWFILE_NAME,"View.cif");
    strcpy(HT_STATE_FILE,"StateObj.odb");



#if 0
    if (!strcasecmp(HT_MOL_TYPE,"biosync")) {
      strcpy(HT_CIFDB_PATH,HT_CONFIG_PATH);
      strcat(HT_CIFDB_PATH,"dict-biosync.odb");
      strcpy(HT_CIFDB_DICT,"cif_biosync.dic");
      strcpy(HT_VIEW_NAME,"view");
    }
#endif

#if 0
    // not currently supported
    if (HT_CURRENT_LANG == LANG_TYPE_EN)
      strcat(HT_CIFDB_PATH,"dict.odb");
    else if (HT_CURRENT_LANG == LANG_TYPE_FR)    
      strcat(HT_CIFDB_PATH,"dict-fr.odb");
    else if (HT_CURRENT_LANG == LANG_TYPE_ES)    
      strcat(HT_CIFDB_PATH,"dict-es.odb");
    else if (HT_CURRENT_LANG == LANG_TYPE_DE)    
      strcat(HT_CIFDB_PATH,"dict-de.odb");
#endif




#if 0
    for (i=0; i < nentries; i++) {
      ndb_log_message_text(NDB_MSG_DEBUG,"html_env: entry=%d name=%s val=%s file=%s isfile=%d",
			   i,
			   entries[i].name,
			   entries[i].val,
			   entries[i].file,
			   entries[i].isFile);
    }
#endif
    return;
}

