/*
FILE:     interface-util.C
*/
/*
VERSION:  0.904-beta
*/
/*
DATE:     8/14/2003
*/
/*
  Comments and Questions to: sw-help@rcsb.rutgers.edu
*/
/*
COPYRIGHT 1999-2003 Rutgers - The State University of New Jersey

This software is provided WITHOUT WARRANTY OF MERCHANTABILITY OR
FITNESS FOR A PARTICULAR PURPOSE OR ANY OTHER WARRANTY, EXPRESS OR
IMPLIED.  RUTGERS MAKE NO REPRESENTATION OR WARRANTY THAT THE
SOFTWARE WILL NOT INFRINGE ANY PATENT, COPYRIGHT OR OTHER
PROPRIETARY RIGHT.

The user of this software shall indemnify, hold harmless and defend
Rutgers, its governors, trustees, officers, employees, students,
agents and the authors against any and all claims, suits,
losses, liabilities, damages, costs, fees, and expenses including
reasonable attorneys' fees resulting from or arising out of the
use of this software.  This indemnification shall include, but is
not limited to, any and all claims alleging products liability.
*/
/*
               PDB SOFTWARE LICENSE AGREEMENT

BY CLICKING THE ACCEPTANCE BUTTON OR INSTALLING OR USING 
THIS "SOFTWARE, THE INDIVIDUAL OR ENTITY LICENSING THE  
SOFTWARE ("LICENSEE") IS CONSENTING TO BE BOUND BY AND IS 
BECOMING A PARTY TO THIS AGREEMENT.  IF LICENSEE DOES NOT 
AGREE TO ALL OF THE TERMS OF THIS AGREEMENT
THE LICENSEE MUST NOT INSTALL OR USE THE SOFTWARE.

1. LICENSE AGREEMENT

This is a license between you ("Licensee") and the Protein Data Bank (PDB) 
at Rutgers, The State University of New Jersey (hereafter referred to 
as "RUTGERS").   The software is owned by RUTGERS and protected by 
copyright laws, and some elements are protected by laws governing 
trademarks, trade dress and trade secrets, and may be protected by 
patent laws. 

2. LICENSE GRANT

RUTGERS grants you, and you hereby accept, non-exclusive, royalty-free 
perpetual license to install, use, modify, prepare derivative works, 
incorporate into other computer software, and distribute in binary 
and source code format, or any derivative work thereof, together with 
any associated media, printed materials, and on-line or electronic 
documentation (if any) provided by RUTGERS (collectively, the "SOFTWARE"), 
subject to the following terms and conditions: (i) any distribution 
of the SOFTWARE shall bind the receiver to the terms and conditions 
of this Agreement; (ii) any distribution of the SOFTWARE in modified 
form shall clearly state that the SOFTWARE has been modified from 
the version originally obtained from RUTGERS.  

2. COPYRIGHT; RETENTION OF RIGHTS.  

The above license grant is conditioned on the following: (i) you must 
reproduce all copyright notices and other proprietary notices on any 
copies of the SOFTWARE and you must not remove such notices; (ii) in 
the event you compile the SOFTWARE, you will include the copyright 
notice with the binary in such a manner as to allow it to be easily 
viewable; (iii) if you incorporate the SOFTWARE into other code, you 
must provide notice that the code contains the SOFTWARE and include 
a copy of the copyright notices and other proprietary notices.  All 
copies of the SOFTWARE shall be subject to the terms of this Agreement.  

3. NO MAINTENANCE OR SUPPORT; TREATMENT OF ENHANCEMENTS 

RUTGERS is under no obligation whatsoever to: (i) provide maintenance 
or support for the SOFTWARE; or (ii) to notify you of bug fixes, patches, 
or upgrades to the features, functionality or performance of the 
SOFTWARE ("Enhancements") (if any), whether developed by RUTGERS 
or third parties.  If, in its sole discretion, RUTGERS makes an 
Enhancement available to you and RUTGERS does not separately enter 
into a written license agreement with you relating to such bug fix, 
patch or upgrade, then it shall be deemed incorporated into the SOFTWARE 
and subject to this Agreement. You are under no obligation whatsoever 
to provide any Enhancements to RUTGERS or the public that you may 
develop over time; however, if you choose to provide your Enhancements 
to RUTGERS, or if you choose to otherwise publish or distribute your 
Enhancements, in source code form without contemporaneously requiring 
end users or RUTGERS to enter into a separate written license agreement 
for such Enhancements, then you hereby grant RUTGERS a non-exclusive,
royalty-free perpetual license to install, use, modify, prepare
derivative works, incorporate into the SOFTWARE or other computer
software, distribute, and sublicense your Enhancements or derivative
works thereof, in binary and source code form.

4. FEES.  There is no license fee for the SOFTWARE.  If Licensee
wishes to receive the SOFTWARE on media, there may be a small charge
for the media and for shipping and handling.  Licensee is
responsible for any and all taxes.

5. TERMINATION.  Without prejudice to any other rights, Licensor
may terminate this Agreement if Licensee breaches any of its terms
and conditions.  Upon termination, Licensee shall destroy all
copies of the SOFTWARE.

6. PROPRIETARY RIGHTS.  Title, ownership rights, and intellectual
property rights in the Product shall remain with RUTGERS.  Licensee 
acknowledges such ownership and intellectual property rights and will 
not take any action to jeopardize, limit or interfere in any manner 
with RUTGERS' ownership of or rights with respect to the SOFTWARE.  
The SOFTWARE is protected by copyright and other intellectual 
property laws and by international treaties.  Title and related 
rights in the content accessed through the SOFTWARE is the property 
of the applicable content owner and is protected by applicable law.  
The license granted under this Agreement gives Licensee no rights to such
content.

7. DISCLAIMER OF WARRANTY.  THE SOFTWARE IS PROVIDED FREE OF 
CHARGE, AND, THEREFORE, ON AN "AS IS" BASIS, WITHOUT WARRANTY OF 
ANY KIND, INCLUDING WITHOUT LIMITATION THE WARRANTIES THAT IT 
IS FREE OF DEFECTS, MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE 
OR NON-INFRINGING.  THE ENTIRE RISK AS TO THE QUALITY AND 
PERFORMANCE OF THE SOFTWARE IS BORNE BY LICENSEE.  SHOULD THE 
SOFTWARE PROVE DEFECTIVE IN ANY RESPECT, THE LICENSEE AND NOT 
LICENSOR ASSUMES THE ENTIRE COST OF ANY SERVICE AND REPAIR.  
THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF 
THIS AGREEMENT.  NO USE OF THE PRODUCT IS AUTHORIZED HEREUNDER 
EXCEPT UNDER THIS DISCLAIMER.

8. LIMITATION OF LIABILITY.  TO THE MAXIMUM EXTENT PERMITTED BY
APPLICABLE LAW,  IN NO EVENT WILL LICENSOR BE LIABLE FOR ANY 
INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
OUT OF THE USE OF OR INABILITY TO USE THE SOFTWARE, INCLUDING, 
WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK 
STOPPAGE, COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL 
OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF ADVISED OF THE
POSSIBILITY THEREOF. 
*/

#include <iostream.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/wait.h>
#include <time.h>
#include <signal.h>
#include <sys/socket.h>
#include "ndb_misclib.h"
#include "interface-defs.h"
#include "ndb_htmllib.h"
#include "interface-lang.h"


static int nws_strlen(const char *line);
static int count_leading_ws(const char *line);
static int get_a_line(const char *s, char *line, int *iPos);

static void html_open_body_with_attributes(FILE *fph, const char *bgcolor,  const char *vlink, 
				    const char *link, const char *alink, const char *text,
				    int fontSize, const char *fontFace);
static void html_wrt_header(FILE *fph, const char *title);

static void html_wrt_header_w_target(FILE *fph, const char *title,
				     const char *target);

// ----------------------------------------------------------------------------------------
static char s1_en[] = "\
<table  width=100%%> \n\
<tr><td valign=top> \n\
<a href=\"http://www.rcsb.org/pdb/index.html\"><img  src=\"/%s/icons/pdb-logo-small.gif\"  \n\
   alt=\"PDB Home\" border=\"0\" height=\"50\" width=\"130\"></a> \n\
</TD> \n\
<TD><center> \n\
<IMG align=center SRC=\"/%s/icons/adit-logo-1.gif\" \n\
     WIDTH=\"272\" HEIGHT=\"131\" ALIGN=\"LEFT\" BORDER=\"0\" HSPACE=\"10\"> \n\
</center></td> \n\
<TD valign=top align=right> \n\
<table  cellspacing=\"0\" cellpadding=\"0\"> \n\
<tr><td width=\"33%%\" align=\"center\" valign=\"middle\"><a href=\"/%s/docs/tutorial.html\"> \n\
<img height=\"25\" width=\"25\" src=\"/%s/icons/questionMark_w.gif\" border=\"0\" alt=\"Tutorial\"></a></td> \n\
<td align=\"center\" valign=\"middle\" width=\"33%%\"><a href=\"http://www.rcsb.org/pdb/index.html\"> \n\
<img height=\"25\" width=\"25\" src=\"/%s/icons/home-button.gif\" border=\"0\" alt=\"PDB Home\"></a></td> \n\
<td align=\"center\" valign=\"middle\" width=\"33%\"><a href=\"mailto:help@rcsb.rutgers.edu\"> \n\
<img height=\"25\" width=\"25\" src=\"/%s/icons/info-button.gif\" border=\"0\"  \n\
     alt=\"help@rcsb.rutgers.edu\"></a></td> \n\
</tr> \n\
<tr> \n\
<td width=\"33%%\" align=\"center\" valign=\"middle\"><a href=\"/%s/docs/tutorial.html\"> \n\
<font size=\"1\" face=\"Arial,Helvetica,Geneva,Swiss,SunSans-Regular\">Tutorial</font></a></td> \n\
<td align=\"center\" valign=\"middle\" width=\"33%%\"><a href=\"http://www.rcsb.org/pdb/index.html\"> \n\
<font face=\"Arial,Helvetica,Geneva,Swiss,SunSans-Regular\" size=\"1\">PDB Home</font></a> \n\
</td> \n\
<td align=\"center\" valign=\"middle\" width=\"33%%\"><a href=\"mailto:help@rcsb.rutgers.edu\"> \n\
<font face=\"Arial,Helvetica,Geneva,Swiss,SunSans-Regular\" size=\"1\">Contact us</font></a> \n\
</td></tr></table></TD></tr></table> \n\
";

static char s5_en[] = "\
<table  width=100%%> \n\
<tr><td valign=top> \n\
<a href=\"http://www.rcsb.org/pdb/index.html\"><img  src=\"/%s/icons/pdb-logo-small.gif\"  \n\
   alt=\"PDB Home\" border=\"0\" height=\"50\" width=\"130\"></a> \n\
</TD> \n\
<TD><center> \n\
<IMG align=center SRC=\"/%s/icons/download-logo-1.gif\" \n\
     WIDTH=\"291\" HEIGHT=\"45\" ALIGN=\"LEFT\" BORDER=\"0\" HSPACE=\"10\"> \n\
</center></td> \n\
<TD valign=top align=right> \n\
<table  cellspacing=\"0\" cellpadding=\"0\"> \n\
<tr>\n\
<td align=\"center\" valign=\"middle\" width=\"33%%\"><a href=\"http://www.rcsb.org/pdb/index.html\"> \n\
<img height=\"25\" width=\"25\" src=\"/%s/icons/home-button.gif\" border=\"0\" alt=\"PDB Home\"></a></td> \n\
<td align=\"center\" valign=\"middle\" width=\"33%\"><a href=\"mailto:help@rcsb.rutgers.edu\"> \n\
<img height=\"25\" width=\"25\" src=\"/%s/icons/info-button.gif\" border=\"0\"  \n\
     alt=\"help@rcsb.rutgers.edu\"></a></td> \n\
</tr> \n\
<tr> \n\
\n\
<td align=\"center\" valign=\"middle\" width=\"33%%\"><a href=\"http://www.rcsb.org/pdb/index.html\"> \n\
<font face=\"Arial,Helvetica,Geneva,Swiss,SunSans-Regular\" size=\"1\">PDB Home</font></a> \n\
</td> \n\
<td align=\"center\" valign=\"middle\" width=\"33%%\"><a href=\"mailto:help@rcsb.rutgers.edu\"> \n\
<font face=\"Arial,Helvetica,Geneva,Swiss,SunSans-Regular\" size=\"1\">Contact us</font></a> \n\
</td></tr></table></TD></tr></table> \n\
";


static char rcsb_trailer_en[]="\
Questions, comments, and suggestions should be sent to \n\
<a href=\"mailto:help@rcsb.rutgers.edu\">help@rcsb.rutgers.edu</a>. \n\
 \n\
<hr noshade size=\"1\"><p align=right> \n\
<font align=right size=\"2\"><i>&copy; <a href=\"http://www.rcsb.org/\">RCSB</a></i></font> <P>  \n\
";



// ----------------------------------------------------------------------------------------

static void html_wrt_header(FILE *fph, const char *title) 
{
  fprintf(fph,"<HTML>\n");
  fprintf(fph,"<HEAD><TITLE>%s</TITLE></HEAD>\n",title);
}

static void html_wrt_header_w_target(FILE *fph, const char *title, 
				     const char * target) 
{
  fprintf(fph,"<HTML>\n");
  fprintf(fph,"<HEAD><TITLE>%s</TITLE><BASE TARGET=\"%s\"></BASE></HEAD>\n",
	  title,target);
}

static void html_open_body_with_attributes(FILE *fph, const char *bgcolor,  const char *vlink, 
				    const char *link, const char *alink, const char *text,
				    int fontSize, const char *fontFace)
{
  fprintf(fph,"<BODY BGCOLOR=\"%s\" VLINK=\"%s\" LINK=\"%s\" ALINK=\"%s\" TEXT=\"%s\">\n",
	  bgcolor,vlink,link,alink,text);
  fprintf(fph,"<BASEFONT SIZE=%d COLOR=\"%s\" FACE=\"%s\">\n",fontSize,text,fontFace);
}



void update_cvs_archive_directory(const char *idNameLC, const char *annotPath) {
  char  command[HT_MAX_NAME_LENGTH];
  char  script[2048];
  
  strcpy(script,"");
  sprintf(command,"cd %s; ",annotPath);
  strcat(script,command);
  sprintf(command,"%s -f -d %s commit -m\"Update by Input Tool\" %s/%s >& /var/tmp/cvs.log;  ",
	  HT_CVS_COMMAND,HT_CVS_ARCHIVE_PATH,HT_ARCHIVE_DIRECTORY,idNameLC);
  strcat(script,command);
  ndb_log_message_text(NDB_MSG_DEBUG,"CVS commit script %s",script);
  wsystem(script);
}
#if 0

char *mk_db_load_object(const char *idNameLC, const char *archPath) {

  char  dbPath[HT_MAX_NAME_LENGTH];
  char  errPath[HT_MAX_NAME_LENGTH];
  char  cifPath[HT_MAX_NAME_LENGTH];
  char  linkPath[HT_MAX_NAME_LENGTH];
  char  command[HT_MAX_NAME_LENGTH];
  char  script[2048];
  char *diags=NULL;
  int len;
  
  //
  // I. Make directory to hold db loadable object
  //

  strcpy(dbPath,archPath);
  strcat(dbPath,"db/");
  mkdir(dbPath,0755);

  //
  // II. Make link in this directory to the archival CIF
  //
  strcpy(cifPath,archPath);
  strcat(cifPath,idNameLC);
  strcat(cifPath,".cif");

  strcpy(linkPath,dbPath);
  strcat(linkPath,idNameLC);
  strcat(linkPath,".cif");
  remove(linkPath);
  symlink(cifPath,linkPath);

  //
  //  III.  build db update.
  //

  strcpy(script,"");
  sprintf(command,"cd %s; ",dbPath);
  strcat(script,command);

  // Create the PDB format file for convenience...
  sprintf(command,"%s -o 2 -i %s.cif >& %s; cp -f %s.cif.pdb ../%s.pdb; ",
	  HT_TRANSLATOR_NAME, idNameLC, "TRANSLATOR0.ERR",idNameLC, idNameLC);
  strcat(script,command);

  sprintf(command,"%s -o 103 -i %s.cif >& %s; cp -f %s.cif.ndb ../%s.ndb; ",
	  HT_TRANSLATOR_NAME, idNameLC,"TRANSLATOR1.ERR", idNameLC, idNameLC);
  strcat(script,command);

  sprintf(command,"cp -f %s.cif.ndb %s%s.ndb ",
	  idNameLC,HT_PENDING_PATH,idNameLC);
  strcat(script,command);
  ndb_log_message_text(NDB_MSG_DEBUG,"DB loader setup script %s",script);
  wsystem(script);



  strcpy(script,"");
  sprintf(command,"cd %s; ",dbPath);
  strcat(script,command);
  sprintf(command,"%s -tabtype all -bcp -db %s -file %s.cif.ndb >& %s; ",
	  HT_DBLOADER_NAME, HT_TARGET_DBNAME, idNameLC,"DBLOAD.ERR");
  strcat(script,command);

  sprintf(command,"setenv SYBASE %s; setenv DSQUERY %s; setenv DSCONSOLE %s; setenv DSLISTEN %s; ", 
	  HT_SYBASE_PATH, HT_SYBASE_SERVER, HT_SYBASE_SERVER, HT_SYBASE_SERVER);
  strcat(script,command);

  sprintf(command,"setenv NDB_XDBUSER %s; setenv NDB_XDBPW %s; ./BCP_COMMANDS.csh >>& DBLOAD.ERR; ", 
	  HT_DBUSER, HT_DBPW);
  strcat(script,command);

  ndb_log_message_text(NDB_MSG_DEBUG,"DB loader script %s",script);
  wsystem(script);

  FILE *fp=NULL;
  struct stat statbuf;

  strcpy(errPath,dbPath);
  strcat(errPath,"DBLOAD.ERR");

  if ( (fp=fopen(errPath,"r")) != NULL ) {
    stat(errPath,&statbuf);
    ndb_log_message(NDB_MSG_DEBUG,"Reading %s size %d",errPath,statbuf.st_size);
    diags = (char *) calloc(statbuf.st_size+1, sizeof(char));
    len=fread(diags, sizeof(char), statbuf.st_size+1, fp);
    ndb_log_message(NDB_MSG_DEBUG,"Read %d bytes in %s ",len, errPath);
    fclose(fp);    
  }
  return(diags); 
}
#endif

//

int check_input_data_syntax(FILE *io, const char * dirPath, 
			    int flagXYZ, const char *formatTypeXYZ, const char *rootNameXYZ,
			    int flagSF,  const char *formatTypeSF, const char *rootNameSF,
			    const char *returnText, const char *returnURL) {

  char diagPath[HT_MAX_NAME_LENGTH];
  char seqPath[HT_MAX_NAME_LENGTH];
  int iOk, iOpen;

  iOpen=0;
  if (flagXYZ ==1) {
    strcpy(diagPath,dirPath);
    strcpy(seqPath,dirPath);
    strcat(diagPath,"xyz.err");
    strcat(seqPath,"filter.out");
    iOk=coordinate_format_ok(formatTypeXYZ, dirPath, rootNameXYZ, diagPath);
    if (!iOpen) {
      html_display_open_page(io,"RCSB Data Syntax Check", NDB_STYLE_DEFAULT);
      html_display_page_title_logo(io, "Data Syntax Check", NDB_STYLE_DEFAULT);
      html_display_return_to_page(io, returnText, returnURL );
      iOpen=1;
    }
    if (!iOk) {
      fprintf(io,"<br><br><h2>Coordinate Syntax Diagnostics:</h2><ul>\n");
      display_raw_file(io,diagPath);
      fprintf(io,"</ul><br><br>");
    } else {
      fprintf(io,"<br><br><h2>Coordinate format is OK!</h2>\n");      
      fprintf(io,"<br><br><h2>Sequence information:</h2><br><pre>\n");      
      display_raw_file(io,seqPath);
      fprintf(io,"</pre>\n");      
    }
  }
  
  if (flagSF == 1) {
    strcpy(diagPath,dirPath);
    strcat(diagPath,"sf.err");
    iOk=sf_format_ok(formatTypeSF, dirPath, rootNameSF, diagPath);
    if (!iOpen) {
      html_display_open_page(io,"RCSB Data Syntax Check", NDB_STYLE_DEFAULT);
      html_display_page_title_logo(io, "Data Syntax Check", NDB_STYLE_DEFAULT);
      html_display_return_to_page(io, returnText, returnURL );
      iOpen=1;
    }

    if (!iOk) {
      fprintf(io,"<br><br><h2>Structure Factor Syntax Diagnostics:</h2><ul>");
      display_raw_file(io,diagPath);
      fprintf(io,"</ul><br><br>");
    } else {
      fprintf(io,"<br><br><h2>Structure Factor format is OK!</h2><br>\n");      
    }
  }
  if (iOpen) html_display_close_page(io, NDB_STYLE_DEFAULT);
  return(iOpen);
}


void html_display_return_tool_top(FILE *io, const char *ufid ) {
  fprintf(io,"<br><center>\n");
  fprintf(io,"<font size=4><a href=\"%s%s?%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s\">|Return to the Input Tool|</a></font>\n",
	  HT_CGI_PATH,
	  HT_INTERFACE_CALL_TOP_MENU_VIEW,
	  HT_ATTRIBUTE_UNIQUE_ID,
	  ufid,
	  HT_ATTRIBUTE_CONTEXT,
	  HT_APPL_CONTEXT,
	  HT_ATTRIBUTE_DBNAME,
	  HT_TARGET_DBNAME,
	  HT_ATTRIBUTE_EMAIL,
	  HT_ANNOTATOR_EMAIL,
	  HT_ATTRIBUTE_PROJDIR,
	  HT_ARCHIVE_DIRECTORY,
	  HT_ATTRIBUTE_EXP_METH,HT_EXP_METHOD,
	  HT_ATTRIBUTE_VIEW_TYPE,HT_VIEW_TYPE,
	  HT_ATTRIBUTE_MOL_TYPE,HT_MOL_TYPE);
  fprintf(io,"<br></center>\n");

}



void xhtml_display_return_tool_top(FILE *io, const char *ufid ) {
  fprintf(io,"<br><center>\n");
  fprintf(io,"<font size=4><a href=\"%s%s?%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s\">|Return to the Input Tool|</a></font>\n",
	  HT_CGI_PATH,
	  HT_INTERFACE_CALL_TOP_MENU_VIEW,
	  HT_ATTRIBUTE_UNIQUE_ID,
	  ufid,
	  HT_ATTRIBUTE_CONTEXT,
	  HT_APPL_CONTEXT,
	  HT_ATTRIBUTE_DBNAME,
	  HT_TARGET_DBNAME,
	  HT_ATTRIBUTE_EMAIL,
	  HT_ANNOTATOR_EMAIL,
	  HT_ATTRIBUTE_PROJDIR,
	  HT_ARCHIVE_DIRECTORY,
	  HT_ATTRIBUTE_EXP_METH,HT_EXP_METHOD,	  
	  HT_ATTRIBUTE_VIEW_TYPE,HT_VIEW_TYPE,
	  HT_ATTRIBUTE_MOL_TYPE,HT_MOL_TYPE);
  fprintf(io,"<br></center>\n");

}


void html_display_return_to_page(FILE *io, const char *text, const char *page ) {
/*
 * html_return_to_page() write centered anchor with input text which points to 
 * to the document in 'page'.
 */
  fprintf(io,"<center><font size=4><a href=\"%s\">|%s|</a></font></center>\n",page,text);
}


void html_display_page_title_logo(FILE *io, const char *title, int style) {

  switch (style) {
  case 0:
    fprintf(io,"<TABLE CELLPADDING=8><TR>");
    fprintf(io,"<TD><IMG SRC=\"%s\" ALIGN=\"CENTER\" ALT=\"Auto DEP Input Tool\"></TD>\n",HT_ICON_LOGO);
    fprintf(io,"<TD ALIGN=left WIDTH=400><FONT SIZE=8>&nbsp&nbsp<B>%s</B></FONT></TD>\n",title);
    fprintf(io,"</tr></table>\n");
    break;
  case 1:
    fprintf(io, s1_en, HT_APPLICATION_NAME,HT_APPLICATION_NAME,HT_APPLICATION_NAME,HT_APPLICATION_NAME,
	    HT_APPLICATION_NAME,HT_APPLICATION_NAME,HT_APPLICATION_NAME);
    if (title && strlen(title)) fprintf(io,"<center><H1>%s</H1></center>\n",title);
    break;
  case 5:
    fprintf(io, s5_en, HT_APPLICATION_NAME,HT_APPLICATION_NAME,HT_APPLICATION_NAME,HT_APPLICATION_NAME);
    if (title && strlen(title)) fprintf(io,"<center><H1>%s</H1></center>\n",title);
    break;
  default:
    fprintf(io,"<TABLE CELLPADDING=8><TR>");
    fprintf(io,"<TD><IMG SRC=\"%s\" ALIGN=\"CENTER\" ALT=\"Auto DEP Input Tool\"></TD>\n",HT_ICON_LOGO);
    fprintf(io,"<TD ALIGN=left WIDTH=400><FONT SIZE=8>&nbsp&nbsp<B>%s</B></FONT></TD>\n",title);
    fprintf(io,"</tr></table>\n");
    break;
  }
}



void html_display_open_page(FILE *io, const char *title,  int style) {
  if (io == NULL) return;
  //   ht_put_content_type_target(io,"_top");
  html_wrt_header(io,title);
  switch (style) {
  case 0:
    html_open_body_with_attributes(io, "#ffffff", "purple", "purple", "purple", 
				   "black", 3, "HELVETICA,ARIAL,TIMES");
    break;

  case 1:
    //    html_open_body_with_attributes(io, "#ffffff", "#333333", "#000006", "#000006", 
    //			   "black", 3, "HELVETICA,ARIAL,TIMES");
       html_open_body_with_attributes(io, "#ffffff", "#666666", "#000066", "#0000ff", 
    		   "black", 3, "HELVETICA,ARIAL,TIMES");

    break;

  default:
    html_open_body_with_attributes(io, "#ffffff", "purple", "purple", "purple", 
				   "black", 3, "HELVETICA,ARIAL,TIMES");
    break;
  }
}


void html_display_open_page_w_target(FILE *io, const char *title,  int style, const char *target) {
  if (io == NULL) return;
  //   ht_put_content_type_target(io,"_top");
  html_wrt_header_w_target(io,title,target);
  switch (style) {
  case 0:
    html_open_body_with_attributes(io, "#ffffff", "purple", "purple", "purple", 
				   "black", 3, "HELVETICA,ARIAL,TIMES");
    break;

  case 1:
    //    html_open_body_with_attributes(io, "#ffffff", "#333333", "#000006", "#000006", 
    //			   "black", 3, "HELVETICA,ARIAL,TIMES");
       html_open_body_with_attributes(io, "#ffffff", "#666666", "#000066", "#0000ff", 
    		   "black", 3, "HELVETICA,ARIAL,TIMES");

    break;

  default:
    html_open_body_with_attributes(io, "#ffffff", "purple", "purple", "purple", 
				   "black", 3, "HELVETICA,ARIAL,TIMES");
    break;
  }
}


void html_display_close_page(FILE *io, int style) {

  switch (style) {
  case 0:
    break;
  case 1:
    fprintf(io,"<pre>\n\n\n</pre>\n");
    fprintf(io,"%s", rcsb_trailer_en);
    break;
  default:
    break;
  }
  fprintf(io,"</BODY>\n");
  fprintf(io,"</HTML>\n");
}


void html_display_raw_file(FILE *io, const char *fName, const char *title) {
  
  if (io == NULL) return;
  html_display_open_page(io,title, NDB_STYLE_DEFAULT);

  fprintf(io,"<pre>\n\n\n\n");
  display_raw_file(io,fName);
  fprintf(io,"</pre>");

  html_display_close_page(io, NDB_STYLE_DEFAULT);
}



int display_raw_file(FILE *io, const char *fName) {
  /*
   * display_raw_file() copies the contents of named file to 
   * to the specified output unit.  Returns 1 for success
   * or 0 if the file is missing or on error.
   */
  
  FILE *fp=NULL;
  struct stat statbuf;

  char *dataFF=NULL;
  int len, iret;

  iret=0;
  if (fName == NULL || strlen(fName) == 0) return(iret);

  if ( (fp = fopen(fName,"r")) != NULL )  {
    stat(fName,&statbuf);
    ndb_log_message_text(NDB_MSG_DEBUG,"Reading %s size %d",fName,statbuf.st_size);
    dataFF = (char *) calloc(statbuf.st_size+1, sizeof(char));
    len=fread(dataFF, sizeof(char), statbuf.st_size+1, fp);
    ndb_log_message_text(NDB_MSG_DEBUG,"Read %d bytes in %s ",len, fName);
    fclose(fp);

    if (dataFF != NULL) {
      fprintf(io,"%s",dataFF);
      free(dataFF);
      iret=1;
    }
  }
  return(iret);
}

void html_display_redeposit_block_message(FILE *io, const char *ufid, const char *dirPath,  int style) {

  FILE *fp=NULL;
  char idName[100], id_file[HT_MAX_NAME_LENGTH];

  strcpy(id_file,dirPath);
  strcat(id_file,NDB_DEPOSITION_ID_FILE_NAME);
  
  if ((fp=fopen(id_file,"r")) != NULL) {
    getfield(fp,idName,19);
    fclose(fp);
    
    ht_put_content_type_target(io,"_top");
    html_display_open_page(io,"Structure Deposited", style);
    html_display_page_title_logo(io,"", style);
    html_display_return_to_page(io, "Continue", HT_HOME_PATH);
    
    fprintf(io,"<br><p><center><H2>Your deposition has <font color=red>already</font> been accepted!</h2></center>\n");
#if defined(MSDTOOL)
    fprintf(io,"<center><H2>Your RCSB ID code is %s</h2></center>\n",idName);
    fprintf(io,"<center><H2>Thanks for using the RCSB's ADIT.</h2></center>\n");
#else 
    fprintf(io,"<center><H2>Your NDB ID code is %s</h2></center>\n",idName);
    fprintf(io,"<center><H2>Thanks for using the NDB.</h2></center>\n");
#endif

    html_display_close_page(io, style);
    fflush(io);    

    ndb_close_log();    
    exit(0);
  }
}




int translate_coordinate_format(const char *formatTypeIn, const char *formatTypeOut, 
				const char *dirPath, const char *rootName) {

  /*
   *  translate_coordinate_format() translate coordinate format from formatTypeIn
   *  to formatTypeOut.
   *
   *  returns 0  if a problem is detected or 1 otherwise
   */

  char  script[4096]; 

  if ((formatTypeIn == NULL) || (formatTypeOut == NULL) || 
      (dirPath == NULL) || (rootName == NULL)) {
    ndb_log_message(NDB_MSG_ERR,"NULL arguments passed to translation translate_coordinate_format");
    return(0);
  }

  if (!strcmp(formatTypeIn,formatTypeOut)) return(1);

  memset(script,'\0',4096);

  if (!strcmp(formatTypeIn,"pdb")) {   // pdb ->

    if (!strcmp(formatTypeOut,"cifeps")) {  // pdb -> cifeps
      sprintf(script,"cd %s; %s -o 7 -i %s.pdb >& %s;  mv -f %s.pdb.cif %s.cifeps; mv -f %s.pdb.maxit.err maxit.diags; ",
	      dirPath, HT_TRANSLATOR_NAME, rootName, "TRANSLATOR.ERR",rootName,rootName,rootName);
      
    } else if (!strcmp(formatTypeOut,"cif")) {  // pdb->mmcif

	sprintf(script,"cd %s; %s -o 1 -i %s.pdb >& %s;  mv -f %s.pdb.cif %s.cif; mv -f %s.pdb.maxit.err maxit.diags; ",
		dirPath, HT_TRANSLATOR_NAME, rootName, "TRANSLATOR.ERR",rootName,rootName,rootName);

    } else if (!strcmp(formatTypeOut,"ndb")) { // pdb->ndb
      sprintf(script,"cd %s; %s -o 3 -i %s.pdb >& %s;  mv -f %s.pdb.ndb %s.ndb; mv -f %s.ndb.maxit.err maxit.diags; ",
	      dirPath, HT_TRANSLATOR_NAME, rootName, "TRANSLATOR.ERR",rootName,rootName,rootName);
    }
    
  } else if (!strcmp(formatTypeIn,"cifeps")) {  //cifeps ->
    
    if (!strcmp(formatTypeOut,"cif")) { //cifeps -> mmcif
      sprintf(script,"cd %s; %s -o 8  -i %s.cifeps >& %s;  mv -f %s.cifeps.cif %s.cif; mv -f %s.cifeps.maxit.err maxit.diags; ",
	      dirPath, HT_TRANSLATOR_NAME, rootName, "TRANSLATOR.ERR",rootName,rootName,rootName);
      
    } else if (!strcmp(formatTypeOut,"ndb")) {  //cifeps -> ndb
      sprintf(script,"cd %s; %s -o 6  -i %s.cifeps >& %s;  mv -f %s.cifeps.ndb %s.ndb; mv -f %s.cifeps.maxit.err maxit.diags; ",
	      dirPath, HT_TRANSLATOR_NAME, rootName, "TRANSLATOR.ERR",rootName,rootName,rootName);
      
    }

  } else if (!strcmp(formatTypeIn,"cif")) {  // mmcif -> 
    
    if (!strcmp(formatTypeOut,"pdb")) {   // mmcif -> pdb
      sprintf(script,"cd %s; %s -o 2 -i %s.cif >& %s;  mv -f %s.cif.pdb %s.pdb; mv -f %s.cif.maxit.err maxit.diags; ",
	      dirPath, HT_TRANSLATOR_NAME, rootName, "TRANSLATOR.ERR",rootName,rootName,rootName);

    } else if (!strcmp(formatTypeOut,"ndb")) {  // mmcif -> ndb
      sprintf(script,"cd %s; %s -o 6 -i %s.cif >& %s;  mv -f %s.cif.ndb %s.ndb; mv -f %s.cif.maxit.err maxit.diags; ",
	      dirPath, HT_TRANSLATOR_NAME, rootName, "TRANSLATOR.ERR",rootName,rootName,rootName);
      
    }
  } else if (!strcmp(formatTypeIn,"ndb")) {  // ndb ->
    
    if (!strcmp(formatTypeOut,"pdb")) {  // ndb->pdb
      sprintf(script,"cd %s; %s -o 4 -i %s.ndb >& %s;  mv -f %s.ndb.pdb %s.pdb; mv -f %s.pdb.maxit.err maxit.diags; ",
	      dirPath, HT_TRANSLATOR_NAME, rootName, "TRANSLATOR.ERR",rootName,rootName,rootName);

    } else if (!strcmp(formatTypeOut,"cif")) {  // ndb->mmcif
      sprintf(script,"cd %s; %s -o 5 -i %s.ndb >& %s;  mv -f %s.ndb.cif %s.cif; mv -f %s.ndb.maxit.err maxit.diags; ",
	      dirPath, HT_TRANSLATOR_NAME, rootName, "TRANSLATOR.ERR",rootName,rootName,rootName);
      
    }
  } else {
    ndb_log_message(NDB_MSG_ERR,"Unsupported translation: %s to %s",formatTypeIn,formatTypeOut);
    return(0);
  }

  strcat(script," cat -s maxit.err >> maxit.diags; ");
  ndb_log_message(NDB_MSG_INFO,"File format translation command: %s",script);      
  wsystem(script);
  return(1);
}



int coordinate_format_ok(const char *formatType, const char *dirPath, const char *rootName, const char *rptFile) {
  /*
   *  coordinate_format_ok() checks format syntax compliance 
   *  rptFile contains absolute path name to report file.
   *
   *  returns 0  if a problem is detected or 1 otherwise
   */

  char  script[HT_MAX_NAME_LENGTH]; 
  FILE *fp=NULL;

  if ((rptFile == NULL) || (formatType == NULL) || 
      (dirPath == NULL) || (rootName == NULL)) return(0);

  if ((strlen(rptFile) == 0) || (strlen(formatType) == 0) || 
      (strlen(dirPath) == 0) || (strlen(rootName) == 0)) return(0);


  if ( !strcmp(formatType,"pdb")) {
    if (!strcmp(HT_EXP_METHOD,"NMR")) {
      sprintf(script,"cd %s; setenv RCSBROOT %s ;%s -nmr -i %s.pdb >& %s; mv -f filter.err %s >>& %s ",
	      dirPath, HT_RCSBAPPS_PATH, HT_PDB_PREFILTER_NAME, rootName, 
	      "PREFILTER.ERR", rptFile, "PREFILTER.ERR");
    } else {
      sprintf(script,"cd %s; setenv RCSBROOT %s; %s -i %s.pdb >& %s; mv -f filter.err %s >>& %s ",
	      dirPath, HT_RCSBAPPS_PATH, HT_PDB_PREFILTER_NAME, rootName, 
	      "PREFILTER.ERR", rptFile, "PREFILTER.ERR");

    }
    wsystem(script);
  }

  if ( !strcmp(formatType,"cif")) {
    sprintf(script,"cd %s; %s -f %s.cif -diags %s  -dictobj %s  -dflag xyz  >>& %s ",
	    dirPath, HT_CIF_PREFILTER_NAME, rootName, rptFile,HT_CIFDB_PATH, "PREFILTER.ERR");
    wsystem(script);
  }
  
  if ( (fp = fopen(rptFile,"r")) != NULL )  {
    fclose(fp);        
    return(0);
  }

  return(1);
}


int sf_format_ok(const char *formatType, const char *dirPath, const char *rootName, const char *rptFile) {
  /*
   *  sf_format_ok() checks format syntax compliance 
   *  rptFile contains absolute path name to report file.
   *
   *  returns 0  if a problem is detected or 1 otherwise
   */

  char  script[HT_MAX_NAME_LENGTH]; 
  FILE *fp=NULL;

  if ((rptFile == NULL) || (formatType == NULL) || 
      (dirPath == NULL) || (rootName == NULL)) return(0);

  if ((strlen(rptFile) == 0) || (strlen(formatType) == 0) || 
      (strlen(dirPath) == 0) || (strlen(rootName) == 0)) return(0);


  if ( !strcmp(formatType,"cif")) {
    sprintf(script,"cd %s; %s -f %s.cif -diags %s  -dictobj %s  -dflag sf  >>& %s ",
	    dirPath, HT_CIF_PREFILTER_NAME, rootName, rptFile,HT_CIFDB_PATH, "PREFILTER.ERR");
    wsystem(script);
  }

  if ( (fp = fopen(rptFile,"r")) != NULL )  {
    fclose(fp);        
    return(0);
  }

  return(1);
}


void html_failure_message(FILE *io, const char *returnURL, const char *title, const char *message, int style) {

  ht_put_content_type(io);
  html_display_open_page(io,title, style);
  html_display_page_title_logo(io, title, style);
  if (returnURL && strlen(returnURL)) html_display_return_to_page(io, "Continue", returnURL );
  fprintf(io,"<pre>\n\n\n</pre>\n");
  fprintf(io,"<p><center><font size=7>%s</font></center><p>", message);
  fprintf(io,"<pre>\n\n\n\n\n\n</pre>\n");
  html_display_close_page(io, style);
  fflush(io);
  ndb_log_message_text(NDB_MSG_ERR,"html_failure_message: %s", message);
}


int html_advise_missing_file(FILE *io, const char *fileid, const char *returnURL) {

  FILE *fp=NULL;
  struct stat statbuf;

  if ( (fileid == NULL) || ((fileid != NULL) && (strlen(fileid) > 0) && (fp = fopen(fileid,"r")) == NULL ) ) {
    html_failure_message(io, returnURL, "File Upload Error ...", 
			 "Whoops, we were unable to up-load your input file  ...", RCSB_STYLE_DEFAULT);
    ndb_log_message_text(NDB_MSG_ERR,"Unable to open input file %s", fileid);
    return(1);
  } else if ((stat(fileid,&statbuf) != 0) || (statbuf.st_size == 0)) {

    html_failure_message(io, returnURL, "File Upload Error ...", 
			 "Whoops, your uploaded file is empty or corrupted ...", RCSB_STYLE_DEFAULT);
    ndb_log_message_text(NDB_MSG_DEBUG,"Can't read file %s size %d",fileid,statbuf.st_size);
    return(1);
  }

  fclose(fp);
  return(0);
}

int html_advise_missing_directory(FILE *io, const char *dirId, const char *returnURL) {

  struct stat statbuf;
  int istat; 

  

  if ( (dirId == NULL) || ((dirId != NULL) && (strlen(dirId) > 0) && ((istat = stat(dirId,&statbuf))!= 0)) ) {

    html_failure_message(io, returnURL, "Restart Error ...", 
			 "Whoops, we were unable to locate your restart directory  ...", RCSB_STYLE_DEFAULT);

    ndb_log_message_text(NDB_MSG_ERR,"Unable to locate restart directory %s", dirId);
    return(1);

  } else if (( istat == 0) && !S_ISDIR(statbuf.st_mode))  {

    html_failure_message(io, returnURL, "Restart Error ...", 
			 "Whoops, we were unable to access your restart directory ...", RCSB_STYLE_DEFAULT);
    ndb_log_message_text(NDB_MSG_DEBUG,"Can't read/write directory %s",dirId);
    return(1);
  }


  return(0);
}

int is_directory(char *dirId) {

  struct stat statbuf;
  int istat; 



  if ( (dirId == NULL) || ((dirId != NULL) && (strlen(dirId) > 0) && (  (istat = stat(dirId,&statbuf))!= 0)) ) {

    ndb_log_message_text(NDB_MSG_ERR,"Unable to locate restart directory %s", dirId);
    return(0);

  } else if (( istat == 0) && !S_ISDIR(statbuf.st_mode))  {

    ndb_log_message_text(NDB_MSG_DEBUG,"Can't read/write directory %s",dirId);
    return(0);
  }

  return(1);
}



#if 0

int capture_input_data(const char *tmpPath, const char *fileid, const char *formatType, 
		       const char *dirPath, const char *rootName) {
/*
 *  capture_input_data()  
 */
  int i;
  char  script[HT_MAX_NAME_LENGTH]; 

  if ((fileid == NULL) || (formatType == NULL) || 
      (tmpPath == NULL) || (rootName == NULL) || (dirPath == NULL)) return(0);

  if ((strlen(fileid) == 0) || (strlen(formatType) == 0) || 
      (strlen(tmpPath) == 0) || (strlen(rootName) == 0) || (strlen(dirPath)==0)) return(0);

  //
  //  Handle compressed input files and/or standardize input file names  ...
  //
  
  i = strlen(fileid);

  if ( (i > 2 && fileid[i-2] == '.' && fileid[i-1] == 'Z') ) {
    memset(script,'\0',HT_MAX_NAME_LENGTH);
    if (!strcmp(formatType,"pdb")) 
      sprintf(script,"%s %s%s > %s%s.pdb",HT_ZCAT_COMMAND, tmpPath, fileid, dirPath, rootName);
    else  if (!strcmp(formatType,"cifeps")) 
      sprintf(script,"%s %s%s > %s%s.cifeps", HT_ZCAT_COMMAND, tmpPath, fileid, dirPath, rootName);
    else  if (!strcmp(formatType,"cif")) 
      sprintf(script,"%s %s%s > %s%s.cif", HT_ZCAT_COMMAND, tmpPath, fileid, dirPath, rootName);
    else  if (!strcmp(formatType,"txt")) 
      sprintf(script,"%s %s%s > %s%s.txt", HT_ZCAT_COMMAND, tmpPath, fileid, dirPath, rootName);
    else  if (!strcmp(formatType,"mr")) 
      sprintf(script,"%s %s%s > %s%s.mr", HT_ZCAT_COMMAND, tmpPath, fileid, dirPath, rootName);

    ndb_log_message_text(NDB_MSG_INFO,"Coordinate file uncompressed by command: %s",script);      
    if (strlen(script)) wsystem(script);
    return(1);

  } else if  ((i > 3 && fileid[i-3] == '.' && fileid[i-2] == 'g' && fileid[i-1] == 'z') ) {
    memset(script,'\0',HT_MAX_NAME_LENGTH);
    if (!strcmp(formatType,"pdb")) 
      sprintf(script,"%s %s%s > %s%s.pdb",    HT_GZCAT_COMMAND, tmpPath, fileid, dirPath, rootName);
    else  if (!strcmp(formatType,"cifeps")) 
      sprintf(script,"%s %s%s > %s%s.cifeps", HT_GZCAT_COMMAND, tmpPath, fileid, dirPath, rootName);
    else  if (!strcmp(formatType,"cif")) 
      sprintf(script,"%s %s%s > %s%s.cif",    HT_GZCAT_COMMAND, tmpPath, fileid, dirPath, rootName);
    else  if (!strcmp(formatType,"txt")) 
      sprintf(script,"%s %s%s > %s%s.txt",    HT_GZCAT_COMMAND, tmpPath, fileid, dirPath, rootName);
    else  if (!strcmp(formatType,"mr")) 
      sprintf(script,"%s %s%s > %s%s.mr",   HT_GZCAT_COMMAND, tmpPath, fileid, dirPath, rootName);
    
    ndb_log_message_text(NDB_MSG_INFO,"Coordinate file uncompressed by command: %s",script);      
    if (strlen(script)) wsystem(script);
    return(1);

  } else {
    memset(script,'\0',HT_MAX_NAME_LENGTH);
    if (!strcmp(formatType,"pdb")) 
      sprintf(script,"mv %s%s  %s%s.pdb",    tmpPath, fileid, dirPath, rootName);
    else  if (!strcmp(formatType,"cifeps")) 
      sprintf(script,"mv %s%s  %s%s.cifeps", tmpPath, fileid, dirPath, rootName);	    
    else if (!strcmp(formatType,"cif")) 
      sprintf(script,"mv %s%s  %s%s.cif",    tmpPath, fileid, dirPath, rootName);	    
    else if (!strcmp(formatType,"txt")) 
      sprintf(script,"mv %s%s  %s%s.txt",    tmpPath, fileid, dirPath, rootName);	    
    else if (!strcmp(formatType,"mr")) 
      sprintf(script,"mv %s%s  %s%s.mr",     tmpPath, fileid, dirPath, rootName);	    
    
    ndb_log_message_text(NDB_MSG_INFO,"Coordinate file renamed by command: %s",script);      
    if (strlen(script)) wsystem(script);
    return(1);
  }
}
#endif


#if 1
int capture_input_data(const char *fileid, const char *formatType, const char *dirPath, const char *rootName) {
/*
 *  capture_input_data()  
 */
  int i;
  char  script[4096]; 
  char *mvcmd = "cp";

  if ((fileid == NULL) || (formatType == NULL) || 
      (dirPath == NULL) || (rootName == NULL)) return(0);

  if ((strlen(fileid) == 0) || (strlen(formatType) == 0) || 
      (strlen(dirPath) == 0) || (strlen(rootName) == 0)) return(0);

  //
  //  Handle compressed input files and/or standardize input file names  ...
  //
  
  i = strlen(fileid);

  if ( (i > 2 && fileid[i-2] == '.' && fileid[i-1] == 'Z') ) {
    memset(script,'\0',HT_MAX_NAME_LENGTH);
    if (!strcmp(formatType,"pdb")) 
      sprintf(script,"cd %s; %s %s | %s > %s.pdb", dirPath, HT_ZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName);
    else  if (!strcmp(formatType,"cifeps")) 
      sprintf(script,"cd %s; %s %s | %s > %s.cifeps", dirPath, HT_ZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName);
    else  if (!strcmp(formatType,"cif")) 
      sprintf(script,"cd %s; %s %s | %s > %s.cif", dirPath, HT_ZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName);	    
    else  if (!strcmp(formatType,"txt")) 
      sprintf(script,"cd %s; %s %s | %s > %s.txt", dirPath, HT_ZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName); 
    else  if (!strcmp(formatType,"mr")) 
      sprintf(script,"cd %s; %s %s | %s > %s.mr", dirPath, HT_ZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName);	    
    else  if (!strcmp(formatType,"any")) 
      sprintf(script,"cd %s; %s %s | %s > %s.any", dirPath, HT_ZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName);	    
    else  if (!strcmp(formatType,"other")) 
      sprintf(script,"cd %s; %s %s | %s > %s.other", dirPath, HT_ZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName);	    
    else  if (!strcmp(formatType,"html")) 
      sprintf(script,"cd %s; %s %s | %s > %s.html", dirPath, HT_ZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName);	    
    else  
      sprintf(script,"cd %s; %s %s | %s > %s.unknown", dirPath, HT_ZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName);	    

    ndb_log_message_text(NDB_MSG_INFO,"Coordinate file uncompressed by command: %s",script);      
    if (strlen(script)) wsystem(script);
    return(1);

  } else if  ((i > 3 && fileid[i-3] == '.' && fileid[i-2] == 'g' && fileid[i-1] == 'z') ) {
    memset(script,'\0',HT_MAX_NAME_LENGTH);
    if (!strcmp(formatType,"pdb")) 
      sprintf(script,"cd %s; %s %s | %s > %s.pdb", dirPath, HT_GZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName);
    else  if (!strcmp(formatType,"cifeps")) 
      sprintf(script,"cd %s; %s %s | %s > %s.cifeps", dirPath, HT_GZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName);
    else  if (!strcmp(formatType,"cif")) 
      sprintf(script,"cd %s; %s %s | %s > %s.cif", dirPath, HT_GZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName);	    
    else  if (!strcmp(formatType,"txt")) 
      sprintf(script,"cd %s; %s %s | %s > %s.txt", dirPath, HT_GZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName);	    
    else  if (!strcmp(formatType,"mr")) 
      sprintf(script,"cd %s; %s %s | %s > %s.mr", dirPath, HT_GZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName);	    
    else  if (!strcmp(formatType,"any")) 
      sprintf(script,"cd %s; %s %s | %s > %s.any", dirPath, HT_GZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName);	    
    else  if (!strcmp(formatType,"other")) 
      sprintf(script,"cd %s; %s %s | %s > %s.other", dirPath, HT_GZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName);	    
    else  if (!strcmp(formatType,"html")) 
      sprintf(script,"cd %s; %s %s | %s > %s.html", dirPath, HT_GZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName);	    
    else  
      sprintf(script,"cd %s; %s %s | %s > %s.unknown", dirPath, HT_GZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName);	    

    
    ndb_log_message_text(NDB_MSG_INFO,"Coordinate file uncompressed by command: %s",script);      
    if (strlen(script)) wsystem(script);
    return(1);

  } else {
    memset(script,'\0',HT_MAX_NAME_LENGTH);
    if (!strcmp(formatType,"pdb")) 
      sprintf(script,"cd %s; %s  %s ./%s.pdb",    dirPath, mvcmd, fileid, rootName);
    else  if (!strcmp(formatType,"cifeps")) 
      sprintf(script,"cd %s; %s %s  ./%s.cifeps", dirPath, mvcmd, fileid, rootName);	    
    else if (!strcmp(formatType,"cif")) 
      sprintf(script,"cd %s; %s %s  ./%s.cif",    dirPath, mvcmd, fileid, rootName);	    
    else if (!strcmp(formatType,"txt")) 
      sprintf(script,"cd %s; %s %s  ./%s.txt",    dirPath, mvcmd, fileid, rootName);	    
    else if (!strcmp(formatType,"mr")) 
      sprintf(script,"cd %s; %s %s  ./%s.mr",    dirPath, mvcmd, fileid, rootName);	    
    else if (!strcmp(formatType,"any")) 
      sprintf(script,"cd %s; %s %s  ./%s.any",    dirPath, mvcmd, fileid, rootName);	    
    else if (!strcmp(formatType,"other")) 
      sprintf(script,"cd %s; %s %s  ./%s.other",    dirPath, mvcmd, fileid, rootName);	    
    else if (!strcmp(formatType,"html")) 
      sprintf(script,"cd %s; %s %s  ./%s.html",    dirPath, mvcmd, fileid, rootName);	    
    else  
      sprintf(script,"cd %s; %s %s  ./%s.unknown",    dirPath, mvcmd, fileid, rootName);	    

    
    ndb_log_message_text(NDB_MSG_INFO,"Coordinate file renamed by command: %s",script);      
    if (strlen(script)) wsystem(script);
    return(1);
  }

}
#else
int capture_input_data(const char *fileid, const char *formatType, const char *dirPath, const char *rootName) {
/*
 *  capture_input_data()  
 */
  int i;
  char  script[4096]; 
  char *mvcmd = "cp";

  if ((fileid == NULL) || (formatType == NULL) || 
      (dirPath == NULL) || (rootName == NULL)) return(0);

  if ((strlen(fileid) == 0) || (strlen(formatType) == 0) || 
      (strlen(dirPath) == 0) || (strlen(rootName) == 0)) return(0);

  //
  //  Handle compressed input files and/or standardize input file names  ...
  //
  
  i = strlen(fileid);

  if ( (i > 2 && fileid[i-2] == '.' && fileid[i-1] == 'Z') ) {
    memset(script,'\0',HT_MAX_NAME_LENGTH);
    if (!strcmp(formatType,"pdb")) 
      sprintf(script,"cd %s; %s %s | %s > %s.pdb", dirPath, HT_ZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName);
    else  if (!strcmp(formatType,"cifeps")) 
      sprintf(script,"cd %s; %s %s | %s > %s.cifeps; %s %s.cifeps", dirPath, HT_ZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName,HT_CIFEXCH_OUT_TRANSLATOR,rootName);
    else  if (!strcmp(formatType,"cif")) 
      sprintf(script,"cd %s; %s %s | %s > %s.cif; %s %s.cif", dirPath, HT_ZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName,HT_CIFEXCH_OUT_TRANSLATOR,rootName);	    
    else  if (!strcmp(formatType,"txt")) 
      sprintf(script,"cd %s; %s %s | %s > %s.txt", dirPath, HT_ZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName); 
    else  if (!strcmp(formatType,"mr")) 
      sprintf(script,"cd %s; %s %s | %s > %s.mr", dirPath, HT_ZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName);	    
    else  if (!strcmp(formatType,"any")) 
      sprintf(script,"cd %s; %s %s | %s > %s.any", dirPath, HT_ZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName);	    
    else  if (!strcmp(formatType,"other")) 
      sprintf(script,"cd %s; %s %s | %s > %s.other", dirPath, HT_ZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName);	    
    else  if (!strcmp(formatType,"html")) 
      sprintf(script,"cd %s; %s %s | %s > %s.html", dirPath, HT_ZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName);	    
    else  
      sprintf(script,"cd %s; %s %s | %s > %s.unknown", dirPath, HT_ZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName);	    

    ndb_log_message_text(NDB_MSG_INFO,"Coordinate file uncompressed by command: %s",script);      
    if (strlen(script)) wsystem(script);
    return(1);

  } else if  ((i > 3 && fileid[i-3] == '.' && fileid[i-2] == 'g' && fileid[i-1] == 'z') ) {
    memset(script,'\0',HT_MAX_NAME_LENGTH);
    if (!strcmp(formatType,"pdb")) 
      sprintf(script,"cd %s; %s %s | %s > %s.pdb", dirPath, HT_GZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName);
    else  if (!strcmp(formatType,"cifeps")) 
      sprintf(script,"cd %s; %s %s | %s > %s.cifeps; %s %s.cifeps", dirPath, HT_GZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName,HT_CIFEXCH_OUT_TRANSLATOR,rootName);
    else  if (!strcmp(formatType,"cif")) 
      sprintf(script,"cd %s; %s %s | %s > %s.cif; %s %s.cif", dirPath, HT_GZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName,HT_CIFEXCH_OUT_TRANSLATOR,rootName);	    
    else  if (!strcmp(formatType,"txt")) 
      sprintf(script,"cd %s; %s %s | %s > %s.txt", dirPath, HT_GZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName);	    
    else  if (!strcmp(formatType,"mr")) 
      sprintf(script,"cd %s; %s %s | %s > %s.mr", dirPath, HT_GZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName);	    
    else  if (!strcmp(formatType,"any")) 
      sprintf(script,"cd %s; %s %s | %s > %s.any", dirPath, HT_GZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName);	    
    else  if (!strcmp(formatType,"other")) 
      sprintf(script,"cd %s; %s %s | %s > %s.other", dirPath, HT_GZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName);	    
    else  if (!strcmp(formatType,"html")) 
      sprintf(script,"cd %s; %s %s | %s > %s.html", dirPath, HT_GZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName);	    
    else  
      sprintf(script,"cd %s; %s %s | %s > %s.unknown", dirPath, HT_GZCAT_COMMAND,
	      fileid,HT_ASCII_FILTER_COMMAND,rootName);	    

    
    ndb_log_message_text(NDB_MSG_INFO,"Coordinate file uncompressed by command: %s",script);      
    if (strlen(script)) wsystem(script);
    return(1);

  } else {
    memset(script,'\0',HT_MAX_NAME_LENGTH);
    if (!strcmp(formatType,"pdb")) 
      sprintf(script,"cd %s; %s  %s ./%s.pdb",    dirPath, mvcmd, fileid, rootName);
    else  if (!strcmp(formatType,"cifeps")) 
      sprintf(script,"cd %s; %s %s  ./%s.cifeps; %s %s.cifeps", dirPath, mvcmd, fileid, rootName,
	      HT_CIFEXCH_OUT_TRANSLATOR,rootName);	    
    else if (!strcmp(formatType,"cif")) 
      sprintf(script,"cd %s; %s %s  ./%s.cif; %s %s.cif",    dirPath, mvcmd, fileid, rootName,
	      HT_CIFEXCH_OUT_TRANSLATOR,rootName);	    
    else if (!strcmp(formatType,"txt")) 
      sprintf(script,"cd %s; %s %s  ./%s.txt",    dirPath, mvcmd, fileid, rootName);	    
    else if (!strcmp(formatType,"mr")) 
      sprintf(script,"cd %s; %s %s  ./%s.mr",    dirPath, mvcmd, fileid, rootName);	    
    else if (!strcmp(formatType,"any")) 
      sprintf(script,"cd %s; %s %s  ./%s.any",    dirPath, mvcmd, fileid, rootName);	    
    else if (!strcmp(formatType,"other")) 
      sprintf(script,"cd %s; %s %s  ./%s.other",    dirPath, mvcmd, fileid, rootName);	    
    else if (!strcmp(formatType,"html")) 
      sprintf(script,"cd %s; %s %s  ./%s.html",    dirPath, mvcmd, fileid, rootName);	    
    else  
      sprintf(script,"cd %s; %s %s  ./%s.unknown",    dirPath, mvcmd, fileid, rootName);	    

    
    ndb_log_message_text(NDB_MSG_INFO,"Coordinate file renamed by command: %s",script);      
    if (strlen(script)) wsystem(script);
    return(1);
  }

}
#endif


// jdw in development Dec 20, 1999
#if 1
void update_aux_files(const char *idname, const char *dirPath, const char *archPath) {

  char  command[HT_MAX_NAME_LENGTH];
  char  script[4096];
  char  mvCmd[] = "cp ";
  

  //
  // Copy up-loaded files to annotation area using standardized file names.
  //
  strcpy(script,"");
  sprintf(command,"cd %s; ",dirPath);
  strcat(script,command);

  sprintf(command,"%s -f %s.cif  %s%s-sf.cif; ",
	  mvCmd,NDB_DEF_ROOT_FILE_NAME_SF,archPath,idname);
  strcat(script,command);

  sprintf(command,"%s -f %s.any %s%s-sf.any; ",
	  mvCmd,NDB_DEF_ROOT_FILE_NAME_SF,archPath,idname);
  strcat(script,command);

  sprintf(command,"%s -f %s.other %s%s-sf.other; ",
	  mvCmd,NDB_DEF_ROOT_FILE_NAME_SF,archPath,idname);
  strcat(script,command);

  sprintf(command,"%s -f %s.mr  %s%s.mr; ",
	  mvCmd,NDB_DEF_ROOT_FILE_NAME_NC,archPath,idname);
  strcat(script,command);

  sprintf(command,"%s -f %s.any  %s%s-cc.any; ",
	  mvCmd,NDB_DEF_ROOT_FILE_NAME_CC,archPath,idname);
  strcat(script,command);

  sprintf(command,"%s -f %s.any  %s%s-cs.any; ",
	  mvCmd,NDB_DEF_ROOT_FILE_NAME_CS,archPath,idname);
  strcat(script,command);

  ndb_log_message_text(NDB_MSG_DEBUG,"Aux file update script: %s",script);
  wsystem(script);

  ndb_log_message_text(NDB_MSG_DEBUG,"Done with aux file update.");

}

#endif



int acquire_lock(const char *path)
{
  FILE *fp;
  struct stat statbuf;
  char  lfn[HT_MAX_NAME_LENGTH];

  strcpy(lfn, path);
  strcat(lfn,".lock");
  
  if (stat(lfn,&statbuf) == 0) {
    return 0;
  } else {
    fp = fopen(lfn,"w");
    fclose(fp);
    return 1;
  }
}

void release_lock(const char *path)
{
  char  lfn[HT_MAX_NAME_LENGTH];
  strcpy(lfn, path);
  strcat(lfn,".lock");
  remove(lfn);
}

void  mk_help_frame(char *fname, char *ufid)
{
    FILE *fp;
    char  fn[HT_MAX_NAME_LENGTH];

    memset(fn,0,HT_MAX_NAME_LENGTH);
    strcpy(fn,HT_SERVER_PATH);
    strcat(fn,fname);

    ndb_log_message(NDB_MSG_INFO,"fn is  %s\n",fn);
    fp=fopen(fn,"w");
    mk_help_frame_fp(fp, ufid);
    fclose(fp);
}

void  mk_help_frame_fp(FILE *fp, char *ufid)
{
  char *str;
  int ilen;
  if (HT_CURRENT_LANG == LANG_TYPE_EN) 
    if (!strncmp(HT_APPL_CONTEXT,"deposit",7)) {
      ilen = strlen(help_en_deposit);
      str = (char *) calloc(2*ilen, sizeof(char));
      if (str) {
	sprintf(str,help_en_deposit,ufid);
	fprintf(fp,"%s\n",str);
	free(str);
      }
    } else { 
      fprintf(fp,"%s\n",help_en_annot);
    }

  else  if (HT_CURRENT_LANG == LANG_TYPE_FR) 
    fprintf(fp,"%s\n",help_fr);
  else  if (HT_CURRENT_LANG == LANG_TYPE_ES) 
    fprintf(fp,"%s\n",help_es);
  else  if (HT_CURRENT_LANG == LANG_TYPE_DE) 
    fprintf(fp,"%s\n",help_de);

  if ((HT_APPL_CONTEXT_TYPE == HT_CONTEXT_ANNOTATE) ||
      (HT_APPL_CONTEXT_TYPE == HT_CONTEXT_ASSEMBLE)) {  
    fprintf(fp,"<H3>Annotation Utilities:</H3>\n");
    fprintf(fp,"<br><ul>\n");
    fprintf(fp,"<li><A HREF=\"%s%s%s?%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s\"><FONT SIZE=4>%s</FONT></A>\n",
	      HT_SERVER_NAME,HT_CGI_PATH,HT_INTERFACE_MAXIT_CIF_OP,
	      HT_ATTRIBUTE_UNIQUE_ID,ufid,
	      HT_ATTRIBUTE_MAXIT_OP,"108",
	      HT_ATTRIBUTE_CONTEXT,HT_APPL_CONTEXT,
	      HT_ATTRIBUTE_DBNAME,HT_TARGET_DBNAME,
	      HT_ATTRIBUTE_EMAIL, HT_ANNOTATOR_EMAIL,
	      HT_ATTRIBUTE_PROJDIR, HT_ARCHIVE_DIRECTORY,
	      HT_ATTRIBUTE_EXP_METH, HT_EXP_METHOD,
	      HT_ATTRIBUTE_MOL_TYPE, HT_MOL_TYPE,
	      "Relabel H, O1P/O2P");

    fprintf(fp,"<li><A HREF=\"%s%s%s?%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s\"><FONT SIZE=4>%s</FONT></A>\n",
	      HT_SERVER_NAME,HT_CGI_PATH,HT_INTERFACE_MAXIT_CIF_OP,
	      HT_ATTRIBUTE_UNIQUE_ID,ufid,
	      HT_ATTRIBUTE_MAXIT_OP,"109",
	      HT_ATTRIBUTE_CONTEXT,HT_APPL_CONTEXT,
	      HT_ATTRIBUTE_DBNAME,HT_TARGET_DBNAME,
	      HT_ATTRIBUTE_EMAIL, HT_ANNOTATOR_EMAIL,
	      HT_ATTRIBUTE_PROJDIR, HT_ARCHIVE_DIRECTORY,
	      HT_ATTRIBUTE_EXP_METH, HT_EXP_METHOD,
	      HT_ATTRIBUTE_MOL_TYPE, HT_MOL_TYPE,
	      "Remove H atoms");
    fprintf(fp,"<li><A HREF=\"%s%s%s?%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s\"><FONT SIZE=4>%s</FONT></A>\n",
	      HT_SERVER_NAME,HT_CGI_PATH,HT_INTERFACE_MAXIT_CIF_OP,
	      HT_ATTRIBUTE_UNIQUE_ID,ufid,
	      HT_ATTRIBUTE_MAXIT_OP,"110",
	      HT_ATTRIBUTE_CONTEXT,HT_APPL_CONTEXT,
	      HT_ATTRIBUTE_DBNAME,HT_TARGET_DBNAME,
	      HT_ATTRIBUTE_EMAIL, HT_ANNOTATOR_EMAIL,
	      HT_ATTRIBUTE_PROJDIR, HT_ARCHIVE_DIRECTORY,
	      HT_ATTRIBUTE_EXP_METH, HT_EXP_METHOD,
	      HT_ATTRIBUTE_MOL_TYPE, HT_MOL_TYPE,
	      "Make NMR H atom correspondence table");
    fprintf(fp,"<li><A HREF=\"%s%s%s?%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s\"><FONT SIZE=4>%s</FONT></A>\n",
	      HT_SERVER_NAME,HT_CGI_PATH,HT_INTERFACE_MAXIT_CIF_OP,
	      HT_ATTRIBUTE_UNIQUE_ID,ufid,
	      HT_ATTRIBUTE_MAXIT_OP,"111",
	      HT_ATTRIBUTE_CONTEXT,HT_APPL_CONTEXT,
	      HT_ATTRIBUTE_DBNAME,HT_TARGET_DBNAME,
	      HT_ATTRIBUTE_EMAIL, HT_ANNOTATOR_EMAIL,
	      HT_ATTRIBUTE_PROJDIR, HT_ARCHIVE_DIRECTORY,
	      HT_ATTRIBUTE_EXP_METH, HT_EXP_METHOD,
	      HT_ATTRIBUTE_MOL_TYPE, HT_MOL_TYPE,
	      "Reposition water molecules");
    fprintf(fp,"</ul>\n");
  }
}


void  mk_diag_help_frame(char *fname)
{
    FILE *fp;
    char  fn[HT_MAX_NAME_LENGTH];
    memset(fn,0,HT_MAX_NAME_LENGTH);
    strcpy(fn,HT_SERVER_PATH);
    strcat(fn,fname);

    ndb_log_message(NDB_MSG_INFO,"fn is  %s\n",fn);
    fp=fopen(fn,"w");

    if (HT_CURRENT_LANG == LANG_TYPE_EN) 
      if (!strncmp(HT_APPL_CONTEXT,"deposit",7)) {
	fprintf(fp,"%s\n",other_en_deposit);
      } else {
	fprintf(fp,"%s\n",other_en_annot);
      }

    else  if (HT_CURRENT_LANG == LANG_TYPE_FR) 
      fprintf(fp,"%s\n",other_fr);
    else  if (HT_CURRENT_LANG == LANG_TYPE_ES) 
      fprintf(fp,"%s\n",other_es);
    else  if (HT_CURRENT_LANG == LANG_TYPE_DE) 
      fprintf(fp,"%s\n",other_de);

    fclose(fp);
}



void putfield(FILE *fp, const char *field)
{
    int i=0;
    if ((field == NULL) || (strlen(field) <= 0)) {
	fputc('?',fp);
	fputc('\0',fp);
	return;
    }
    
    while (field[i] != '\0') {
      if (field[i] != '\r') fputc(field[i],fp);
	i++;
    }
    fputc('\0',fp);
    return;
}

int getfield(FILE *fp, char *field, int Mxlen)
/*
 * Get the next NULL terminated string from file pointed
 * to by fp.  The string is copied into 'field' up to 
 * Mxlen characters.
 */
{
    int c, i=0;
    memset(field,'\0',Mxlen);

    c = fgetc(fp);
    while ( c != EOF ) {
	if (i < Mxlen-1) field[i] = c;
	if (c == '\0') break;
	i++;
	c = fgetc(fp);
    }
    return(i);
}

void skipfield(FILE *fp)
/*
 *   Skip a NULL terminated field.
 */
{
    int c;

    c = fgetc(fp);
    while ( c != EOF ) {
	if (c == '\0') break;
	c=fgetc(fp);
    }
    return;
}




void set_deposition_count(int count)
/*
 *  Sets the deposition count...
 *
 */
{
    FILE *fp;
    char fname[CIF_MAX_NAME_LENGTH];
    char tString[10];
    strcpy(fname,HT_SITE_PATH);
    strcat(fname,"DEPOSITION_COUNT.DAT");
    fp = fopen(fname,"w");
    if (fp == NULL) return;
    sprintf(tString,"%d",count);
    putfield(fp,tString);
    fclose(fp);
}


int get_current_deposition_count(void)
/*
 *  Gets the current deposition index ...
 *
 */
{
    FILE *fp;
    int icount=-1;

    char fname[CIF_MAX_NAME_LENGTH];
    char count[10];
    
    strcpy(fname,HT_SITE_PATH);
    strcat(fname,"DEPOSITION_COUNT.DAT");

    fp = fopen(fname,"r");
    if (fp == NULL) return(-1);
    getfield(fp,count,9);
    fclose(fp);

    if (strlen(count) > 0) icount = atoi(count);
    return(icount);
}



void set_current_view_address(const char *dirpath,  const char *block, 
			      const char *category, const char *viewCategory)
/*
 *  Sets the current category and its view name.
 *
 */
{
    FILE *fp;
    char fname[CIF_MAX_NAME_LENGTH];
    if ((category == NULL) || (strlen(category) <= 0) ) return;
    memset(fname,'\0',CIF_MAX_NAME_LENGTH);
    strcpy(fname,dirpath);
    strcat(fname,"Category.current");
    fp = fopen(fname,"w");
    if (fp == NULL) return;
    putfield(fp,category);
    putfield(fp,viewCategory);
    putfield(fp,block);
    fclose(fp);
}

void get_current_view_address(const char *dirpath, char *block, 
			      char *category, char *viewCategory)
/*
 *  Gets the current block, category and view name ...
 *
 */
{
    FILE *fp;
    
    char fname[CIF_MAX_NAME_LENGTH];
    char cat[CIF_MAX_NAME_LENGTH];
    char catV[CIF_MAX_NAME_LENGTH];
    char blockName[CIF_MAX_NAME_LENGTH];

    memset(fname,'\0',CIF_MAX_NAME_LENGTH);
    strcpy(fname,dirpath);
    strcat(fname,"Category.current");
    fp = fopen(fname,"r");
    if (fp == NULL) return;
    getfield(fp,cat, CIF_MAX_NAME_LENGTH);
    getfield(fp,catV,CIF_MAX_NAME_LENGTH);
    getfield(fp,blockName,CIF_MAX_NAME_LENGTH);
    strcpy(category,cat);
    strcpy(viewCategory,catV);
    strcpy(block,blockName);
    fclose(fp);
}


void format_cif_definition(FILE *fp, const char *string)
{
  char line[512];
  int indent, i, iPos;
  if (!string || (strlen(string) == 0)) return;
  iPos=0;
  indent = 512;
  /* 
   *  Get the minimum indentation for non blank lines.
   */  
  while (get_a_line(string, line, &iPos)) {

    if (nws_strlen(line)) {
      i = count_leading_ws(line);
      if (i < indent) indent = i;
    }
  }
  iPos = i= 0;
  while (get_a_line(string, line, &iPos)) {
    if (nws_strlen(line)) {
      if ((i==0) && (indent != 0)) {
	fprintf(fp,"%s",&line[indent-1]); i++;
      } else
	fprintf(fp,"%s",&line[indent]);
    } else
      fprintf(fp,"\n");
  }

}

static int nws_strlen(const char *line)
{
  int i,len;
  len=0;
  if (line == NULL) return(len);
  for (i=0; i < strlen(line); i++) {
    if (!isspace(line[i])) len++;
  }
  return(len);
}



static int count_leading_ws(const char *line)
{
  int i;
  i=0;
  if (line == NULL) return(i);
  while( isspace(line[i])) {
    i++;
  }
  return(i);
}



static int get_a_line(const char *s, char *line, int *iPos)
  /*
   * Get the next new line terminated string in s[iPos] and return
   * this in line[]. Update the nPos. Return line length.
   */
{
  int i, j;
  i=*iPos;
  j=0;
  while (s[i] != '\n' && s[i] != '\0') {
    line[j] = s[i];
    i++; j++;
  }
  if (s[i] == '\n') {
    line[j]='\n'; j++;  i++;
  }
  line[j]='\0'; 
  *iPos = i;
  return(j);
}


int wsystem(char *cmdstring)
/*
 * Execute the command string under the csh interpreter and wait for completion.
 *
 * Redirect stdout and stderr - .
 * Bump up shell limits... .
 */
{
  int status;
  int fdout, fderr;
  pid_t pid;
  char *cmdstr1=NULL;
  
  char fName[HT_ENV_MAX_NAME_LEN];         

  if ((cmdstring == NULL) || (strlen(cmdstring)== 0)) return(-1);

  // jdw -  Raise csh limits to max.
  //
  cmdstr1 = (char *) calloc(strlen(cmdstring)+160,sizeof(char));
  strcpy(cmdstr1,"limit stacksize unlimited >& /dev/null; limit datasize unlimited >& /dev/null; ");
  strcat(cmdstr1,"limit descriptors unlimited >& /dev/null;  ");
  strcat(cmdstr1,cmdstring);
  //
  // JDW -  There following is obviously not very clever, but 
  //        I was not able to redirect to /dev/null on SGI.
  //
  fdout = -1;
  fderr = -1;
  strcpy(fName,HT_SYS_TMP_PATH);
  strcat(fName,"SYS-STDOUT");
  if ( (fdout=creat(fName, 0666)) < 0) {
    return(-1);
  }
  strcpy(fName,HT_SYS_TMP_PATH);
  strcat(fName,"SYS-STDERR");
  if ( (fderr=creat(fName, 0666)) < 0) {
    return(-1);
  }

  /*
   * Get a child process.
   */
  if ((pid = fork()) < 0) {
    exit(-1);
  }
  /*
   * The child executes the code inside the if.
   */
  if (pid == 0) {

#if 1
    if (fderr > 0)  {
      //    dup2(fdout, STDOUT_FILENO);
      dup2(fderr, STDERR_FILENO);
    }
    
    if (fdout > 0)  {
      dup2(fdout, STDOUT_FILENO);
    }
#endif

    execl("/bin/csh", "csh", "-c", cmdstr1, (char *) 0 ) ;
    _exit(1);
  }
  
  /*
   * The parent executes the wait.
   */
  while (wait(&status) != pid) 
    /* empty */ ;

    //    if ( errno != EINTR) {
    //      status = -1;
    //      break;
    //    }

  close(fdout);
  close(fderr);
  if (cmdstr1) delete cmdstr1;
  return(status);
}

int wsystem2(char *cmdstring)
/*
 * Execute the command string under the csh interpreter and wait for completion.
 * Redirect stdout and stderr to /dev/null ....
 */
{
  int status;
  extern int errno;
  int fdout;
  pid_t pid;
  //
  sigset_t mask, savemask;
  struct sigaction ignore, saveint, savequit;

  
  fdout = -1;

  if ( (fdout=open("/dev/null",O_WRONLY)) < 0) {
    return(-1);
  }

  /*
   * Get a child process.
   */


  if ((pid = fork()) < 0) {
    exit(1);
  }


  sigemptyset(&ignore.sa_mask);
  ignore.sa_handler = SIG_IGN;
  ignore.sa_flags = 0;

  sigaction(SIGINT, &ignore, &saveint);
  sigaction(SIGQUIT, &ignore, &savequit);

  sigemptyset(&mask);
  sigaddset(&mask, SIGCHLD);
  sigprocmask(SIG_BLOCK, &mask, &savemask);

  if (fdout > 0)  {
    dup2(fdout, STDOUT_FILENO);
    dup2(fdout, STDERR_FILENO);
  }


  /*
   * The child executes the code inside the if.
   */
  if (pid == 0) {
    execl("/bin/csh", "csh", "-c", cmdstring, (char *) 0 ) ;
    _exit(1);
  }
  
  /*
   * The parent executes the wait.
   */
  while (wait(&status) != pid) {


    //    if ( errno != EINTR) {
    //      status = -1;
    //      break;
    //    }

  }
  sigaction(SIGINT, &saveint, (struct sigaction *) 0);
  sigaction(SIGQUIT, &savequit, (struct sigaction *) 0);
  sigprocmask(SIG_SETMASK, &savemask, (sigset_t *) 0);

  close(fdout);
  return(status);
}



int wsystem1(char *cmdstring)
/*
 * Execute the command string under the csh interpreter and wait for completion
 */
{
  int status;
  pid_t pid;

  /*
   * Get a child process.
   */
  
  if ((pid = fork()) < 0) {
    perror("fork");
    exit(1);
  }
  
  /*
     * The child executes the code inside the if.
     */
  if (pid == 0) {
    execl("/bin/csh", "csh", "-c", cmdstring, (char *) 0 ) ;
    perror("exec error");
        exit(1);
  }
  
  /*
   * The parent executes the wait.
   */
  while (wait(&status) != pid)
    /* empty */ ;

  return status;
}



char *ndb_root_id_name(char *path)
{
  char *p1=NULL, *p2=NULL, *p3=NULL, *root=NULL;
  int len=0, i;

  if (path == NULL || (strlen(path) == 0)) return(root);

  p1=strrchr(path,'/');
  if (strlen(p1) <= 1) return(root); /* / is the last character in string */

  if (p1 != NULL) {
    p1++;
    p2 = strchr(p1,'.');
    if (p2 != NULL) 
      len =  (int) (p2 - p1);
    else
      len = strlen(p1);

    root = (char *) calloc(len+1, sizeof(char));
    for (i=0, p3=p1; i < len; i++) {
      root[i]=*p3;
      p3++;
    }
  } else {
    p2 = strchr(path,'.');
    if (p2 != NULL) 
      len = (int) (p2 - path);
    else
      len = strlen(p1);
    root = (char *) calloc(len+1, sizeof(char));
    for (i=0, p3=path; i < len; i++) {
      root[i]=*p3;
      p3++;
    }
  }
  return(root);
}





int html_decode_multipart_form_data(entry *entries, const char *path)
{
  int  c,  i, j, ibeg;
  char *contentType=NULL, *sp;
  //jdw  char boundary[60];
  char boundary[256];
  int lenCT, lenData;

  int numEntries = 0;

  FILE *fp=NULL;
  char tfile[HT_MAX_NAME_LENGTH];

  int bufLen, isCompressed;
  char *bufp, *buf, *cp0, *cp1, *cp2, *cpN, *datap;
  char contDisp[HT_MAX_LINE_LENGTH], contType[HT_MAX_LINE_LENGTH];
  char     name[HT_MAX_NAME_LENGTH],  fileName[HT_MAX_NAME_LENGTH];
  struct stat statbuf;
  int iStat = 0;

  // jdw - 19-apr-99  insert a pause here to let network catch up.
  sleep(3);

  ndb_log_message(NDB_MSG_DEBUG,"Entering ht_post_multipart_form_data");
  ht_check_post_request();
  /*
   *  Verify "content_type"  and get MIME "boundary" string.
   */
  if (strncmp(getenv("CONTENT_TYPE"),"multipart/form-data",19)) {
    ndb_log_message_text(NDB_MSG_ERR,"html_decode_multipart_form_data: Wrong CONTENT_TYPE %s",
		    getenv("CONTENT_TYPE"));
    return(0);
  }

  isCompressed = 0;
  lenCT = strlen(getenv("CONTENT_TYPE"));
  contentType=(char *) calloc(lenCT+1,sizeof(char));
  strcpy(contentType,getenv("CONTENT_TYPE"));
  ndb_log_message_text(NDB_MSG_ERR,"Content Type[%d] is %s\n", lenCT,contentType);


  for (i=0, sp = &contentType[0], ibeg=-1; i < lenCT; i++) {
    if (!strncmp(sp,"boundary=",9)) {
      ibeg = i; break;
    }
    sp++;
  }
  ndb_log_message_text(NDB_MSG_ERR,"html_decode_multipart_form_data: ibeg= %d boundary[Len=%d] (%s)",
		       ibeg, strlen(sp), sp);

#if 0
  if (ibeg > 0 && ibeg < strlen(contentType) - 9) {
    sp+=9;
    memset(boundary,0,60);
    strcpy(boundary,"--");  /* prefix boundary string with "--"  */
    strncat(boundary, sp , 49);
    ndb_log_message_text(NDB_MSG_DEBUG,"html_decode_multipart_form_data: Using boundary[Len=%d] %s",
		    strlen(boundary),boundary);
  } else {
    ndb_log_message_text(NDB_MSG_ERR,"html_decode_multipart_form_data: no boundary string");
    return(0);
  }
#else
  if (ibeg > 0 && ibeg < strlen(contentType) - 9) {
    sp+=9;
    memset(boundary,0,256);
    strcpy(boundary,"--");  /* prefix boundary string with "--"  */
    strncat(boundary, sp , 244);
    ndb_log_message_text(NDB_MSG_DEBUG,"html_decode_multipart_form_data: Using boundary[Len=%d] %s",
		    strlen(boundary),boundary);
  } else {
    ndb_log_message_text(NDB_MSG_ERR,"html_decode_multipart_form_data: no boundary string");
    return(0);
  }

#endif


  bufLen = atoi(getenv("CONTENT_LENGTH"));
  ndb_log_message_text(NDB_MSG_ERR,"html_decode_multipart_form_data: CONTENT LENGTH %d\n",bufLen); 
  if (bufLen > HT_MAX_UPLOAD_BYTES) {
    ndb_log_message_text(NDB_MSG_DEBUG,"html_decode_multipart_form_data: Upload %d exceeds %d byte limit",
		    bufLen,HT_MAX_UPLOAD_BYTES); 
    return(0);
  }

  i=0;
  //  fp=fopen("/var/tmp/tmp-UPLOAD.dat","w");
  buf = (char *) calloc(bufLen+1, sizeof(char));

  iStat = fstat(0,&statbuf); 
  ndb_log_message_text(NDB_MSG_ERR,"html_decode_multipart_form_data: Stat returns %d",iStat);
  ndb_log_message_text(NDB_MSG_ERR,"html_decode_multipart_form_data: Stat size %d",statbuf.st_size);

#if 0
  if (statbuf.st_size == 0) {
    html_failure_message(stdout, HT_HOME_PATH, "Data Transfer Error ...", 
			 "Whoops, no data was transfered from your browser ...", RCSB_STYLE_DEFAULT);
    //    shutdown(0, 2);
    abort();
    _exit(0);
  }
#endif

  // cannot continue here 

  while ((c=fgetc(stdin)) != EOF  && i < bufLen) {
    buf[i] = c;
    //    fputc(c,fp);
    i++;
  }
  //  fclose(fp);

  ndb_log_message_text(NDB_MSG_DEBUG,"html_decode_multipart_form_data: Uploaded %d bytes of %d",i,bufLen); 
  if (i !=  bufLen ) {

    ndb_log_message_text(NDB_MSG_ERR,"html_decode_multipart_form_data: Uploaded failed.");
  }

#if 0
  fp=fopen("/var/tmp/tmp-UPLOAD.dat","w");
  for (i=0; i < bufLen; i++) {
    fputc(buf[i],fp);
  }
  fclose(fp);
#endif

  numEntries = -1;
  for (bufp = &buf[0];  bufp;  bufp++) {
    //    ndb_log_message_text(NDB_MSG_DEBUG,"html_decode_multipart_form_data: %s", bufp);
    if (!strncmp(bufp,boundary,strlen(boundary))) {

      if (numEntries != -1 && lenData) {
	ndb_log_message_text(NDB_MSG_DEBUG,"html_decode_multipart_form_data: Got here numEntries %d lenData %d",numEntries,lenData);
	if (entries[numEntries].isFile) { 
	  i = strlen(fileName);

	  isCompressed = 0;
	  if (i > 0) { 
	    if ( ( (i > 2 && fileName[i-2] == '.' && fileName[i-1] == 'Z') ) ||
		 ((i > 3 && fileName[i-3] == '.' && fileName[i-2] == 'g' && fileName[i-1] == 'z') )) isCompressed = 1;
	    cp0 = ht_strip_leading_path(fileName);
	    strcpy(tfile,path);
	    strcat(tfile,cp0);
	    ndb_log_message_text(NDB_MSG_DEBUG,"html_decode_multipart_form_data: copying file to %s",tfile);
	    fp = fopen(tfile,"w");
	    if (fp == NULL) {
	      ndb_log_message_text(NDB_MSG_ERR,"html_decode_multipart_form_data: Unable to open upload file %s",tfile);
      //	    return(numEntries);
	    }

	    if (isCompressed) {
	      for (cp1 = datap, i=0; i < lenData-2; i++) {
		fputc(*cp1,fp);  
		cp1++;
	      }

	    } else { 
	      for (cp1 = datap, i=0; i < lenData-2; i++) {
		cpN=cp1; cpN++; 
		if ((*cp1 == 13) && (*cpN != 10)) *cp1 = 10;  // convert isolated cr to nl.
		if (*cp1 > 31 || *cp1 == 10 ) {// filter ascii control characters.
		  fputc(*cp1,fp);  
		} else if (*cp1 == 9 || *cp1 > 126) {// filter ascii control characters.
		  fputc(' ',fp);  
		}
		cp1++;
	      }
	    }

	    fclose(fp);
	  }
	} else {
	  entries[numEntries].val = (char *)calloc(lenData+2,sizeof(char));
	  strncpy(entries[numEntries].val, datap,lenData-2);
	  ndb_log_message_text(NDB_MSG_DEBUG,"html_decode_multipart_form_data: entry=%d name=%s value=%s\n",
			  numEntries,
			  entries[numEntries].name,
			  entries[numEntries].val); 
	}
      }
      lenData = 0;
      numEntries++;
      entries[numEntries].isFile = 0;
      entries[numEntries].name   = (char *)NULL;
      entries[numEntries].val    = (char *)NULL;
      entries[numEntries].file   = (char *)NULL;

      bufp += strlen(boundary); bufp++; bufp++;
      
      for (i = 0, cp1 = bufp; *cp1 && (*cp1 != '\r') ; cp1++) {
	contDisp[i] = *cp1; i++;
      }
      contDisp[i]='\0';
      ndb_log_message_text(NDB_MSG_DEBUG,"html_decode_multipart_form_data: Content string = %s",contDisp);

      if (strlen(contDisp) == 0) {
	break;
      }

      for (i = 0, cp1 = contDisp; *cp1; cp1++) {
	if ( !strncmp(cp1,"name=",5) ) {
	  cp1+=6;
	  for (j=0, cp2 = cp1; cp2 && *cp2 != '"'; cp2++) {
	    name[j] = *cp2; j++;
	  }
	  name[j]= '\0';
	  entries[numEntries].name = (char *)calloc(strlen(name)+1, sizeof(char));
	  strcpy(entries[numEntries].name,name);
	  ndb_log_message_text(NDB_MSG_DEBUG,"html_decode_multipart_form_data: extracted NAME = %s",
			  entries[numEntries].name);
	}
	if ( !strncmp(cp1,"filename=",9) ) {
	  cp1+=10;
	  for (j=0, cp2 = cp1; cp2 && *cp2 != '"'; cp2++) {
	    fileName[j] = *cp2; j++;
	  }
	  fileName[j]= '\0';
	  if ( strlen(fileName) > 0) {
	    cp1 = ht_strip_leading_path(fileName);
	    entries[numEntries].file = (char *)calloc(strlen(cp1)+strlen(path)+1, sizeof(char));
	    strcat(entries[numEntries].file,path);
	    strcat(entries[numEntries].file,cp1);
	  } else {
	    entries[numEntries].file=NULL;
	  }
	  entries[numEntries].isFile = 1;
	  ndb_log_message_text(NDB_MSG_INFO,"html_decode_multipart_form_data: storing FILENAME = %s in file %s",
			  cp1, entries[numEntries].file);
	}
      }
      bufp += strlen(contDisp) + 2;
      for (i = 0, cp1 = bufp; *cp1 && (*cp1 != '\r') ; cp1++) {
	contType[i] = *cp1; i++;
      }
      contType[i]='\0';
      ndb_log_message_text(NDB_MSG_DEBUG,"html_decode_multipart_form_data: Content Type = %s",contType);
      if (strlen(contType)) bufp+=2;
      bufp += strlen(contType) + 2;
      datap = bufp;
    }
    lenData++;
  }
  for (i=0; i < numEntries; i++) {
    if (entries[i].isFile && entries[i].file != NULL ) {
      entries[i].val = (char *) calloc(strlen(entries[i].file)+1,sizeof(char));
      strcpy(entries[i].val,entries[i].file);
    }  else if (!entries[i].isFile && entries[i].val != NULL ) {
      entries[i].file = (char *) calloc(strlen(entries[i].val)+1,sizeof(char));
      strcpy(entries[i].file,entries[i].val);
    }
    ndb_log_message_text(NDB_MSG_DEBUG,"html_decode_multipart_form_data: entry=%d name=%s value=%s file=%s isFile=%d\n",
			 i,
			 entries[i].name,
			 entries[i].val,
			 entries[i].file,
			 entries[i].isFile); 
  }
  //
  if (buf) free(buf);
  return(numEntries);
}  

void capture_cgi_stdin(void) {

  FILE *fp=NULL;
  int c,i;

  fp=fopen("/var/tmp/tmp-cgi-stdin.dat","w");
  while ((c=fgetc(stdin)) != EOF) {
    fputc(c,fp);
    i++;
  }
  fclose(fp);
  //
  fp=fopen("/var/tmp/tmp-cgi-env.dat","w");
  fprintf(fp,"#\n");
  fprintf(fp,"setenv CONTENT_TYPE \"%s\"\n", getenv("CONTENT_TYPE"));
  fprintf(fp,"setenv CONTENT_LENGTH %s\n", getenv("CONTENT_LENGTH"));
  fprintf(fp,"setenv SERVER_NAME  %s\n", getenv("SERVER_NAME"));
  fprintf(fp,"setenv SERVER_PORT  %s\n", getenv("SERVER_PORT"));
  fprintf(fp,"setenv REQUEST_METHOD  %s\n", getenv("REQUEST_METHOD"));
  fclose(fp);
}
// -----------------------------------------------------------------------------------
