/*
FILE:     make-validation-letter.C
*/
/*
VERSION:  0.904-beta
*/
/*
DATE:     8/14/2003
*/
/*
  Comments and Questions to: sw-help@rcsb.rutgers.edu
*/
/*
COPYRIGHT 1999-2003 Rutgers - The State University of New Jersey

This software is provided WITHOUT WARRANTY OF MERCHANTABILITY OR
FITNESS FOR A PARTICULAR PURPOSE OR ANY OTHER WARRANTY, EXPRESS OR
IMPLIED.  RUTGERS MAKE NO REPRESENTATION OR WARRANTY THAT THE
SOFTWARE WILL NOT INFRINGE ANY PATENT, COPYRIGHT OR OTHER
PROPRIETARY RIGHT.

The user of this software shall indemnify, hold harmless and defend
Rutgers, its governors, trustees, officers, employees, students,
agents and the authors against any and all claims, suits,
losses, liabilities, damages, costs, fees, and expenses including
reasonable attorneys' fees resulting from or arising out of the
use of this software.  This indemnification shall include, but is
not limited to, any and all claims alleging products liability.
*/
/*
               PDB SOFTWARE LICENSE AGREEMENT

BY CLICKING THE ACCEPTANCE BUTTON OR INSTALLING OR USING 
THIS "SOFTWARE, THE INDIVIDUAL OR ENTITY LICENSING THE  
SOFTWARE ("LICENSEE") IS CONSENTING TO BE BOUND BY AND IS 
BECOMING A PARTY TO THIS AGREEMENT.  IF LICENSEE DOES NOT 
AGREE TO ALL OF THE TERMS OF THIS AGREEMENT
THE LICENSEE MUST NOT INSTALL OR USE THE SOFTWARE.

1. LICENSE AGREEMENT

This is a license between you ("Licensee") and the Protein Data Bank (PDB) 
at Rutgers, The State University of New Jersey (hereafter referred to 
as "RUTGERS").   The software is owned by RUTGERS and protected by 
copyright laws, and some elements are protected by laws governing 
trademarks, trade dress and trade secrets, and may be protected by 
patent laws. 

2. LICENSE GRANT

RUTGERS grants you, and you hereby accept, non-exclusive, royalty-free 
perpetual license to install, use, modify, prepare derivative works, 
incorporate into other computer software, and distribute in binary 
and source code format, or any derivative work thereof, together with 
any associated media, printed materials, and on-line or electronic 
documentation (if any) provided by RUTGERS (collectively, the "SOFTWARE"), 
subject to the following terms and conditions: (i) any distribution 
of the SOFTWARE shall bind the receiver to the terms and conditions 
of this Agreement; (ii) any distribution of the SOFTWARE in modified 
form shall clearly state that the SOFTWARE has been modified from 
the version originally obtained from RUTGERS.  

2. COPYRIGHT; RETENTION OF RIGHTS.  

The above license grant is conditioned on the following: (i) you must 
reproduce all copyright notices and other proprietary notices on any 
copies of the SOFTWARE and you must not remove such notices; (ii) in 
the event you compile the SOFTWARE, you will include the copyright 
notice with the binary in such a manner as to allow it to be easily 
viewable; (iii) if you incorporate the SOFTWARE into other code, you 
must provide notice that the code contains the SOFTWARE and include 
a copy of the copyright notices and other proprietary notices.  All 
copies of the SOFTWARE shall be subject to the terms of this Agreement.  

3. NO MAINTENANCE OR SUPPORT; TREATMENT OF ENHANCEMENTS 

RUTGERS is under no obligation whatsoever to: (i) provide maintenance 
or support for the SOFTWARE; or (ii) to notify you of bug fixes, patches, 
or upgrades to the features, functionality or performance of the 
SOFTWARE ("Enhancements") (if any), whether developed by RUTGERS 
or third parties.  If, in its sole discretion, RUTGERS makes an 
Enhancement available to you and RUTGERS does not separately enter 
into a written license agreement with you relating to such bug fix, 
patch or upgrade, then it shall be deemed incorporated into the SOFTWARE 
and subject to this Agreement. You are under no obligation whatsoever 
to provide any Enhancements to RUTGERS or the public that you may 
develop over time; however, if you choose to provide your Enhancements 
to RUTGERS, or if you choose to otherwise publish or distribute your 
Enhancements, in source code form without contemporaneously requiring 
end users or RUTGERS to enter into a separate written license agreement 
for such Enhancements, then you hereby grant RUTGERS a non-exclusive,
royalty-free perpetual license to install, use, modify, prepare
derivative works, incorporate into the SOFTWARE or other computer
software, distribute, and sublicense your Enhancements or derivative
works thereof, in binary and source code form.

4. FEES.  There is no license fee for the SOFTWARE.  If Licensee
wishes to receive the SOFTWARE on media, there may be a small charge
for the media and for shipping and handling.  Licensee is
responsible for any and all taxes.

5. TERMINATION.  Without prejudice to any other rights, Licensor
may terminate this Agreement if Licensee breaches any of its terms
and conditions.  Upon termination, Licensee shall destroy all
copies of the SOFTWARE.

6. PROPRIETARY RIGHTS.  Title, ownership rights, and intellectual
property rights in the Product shall remain with RUTGERS.  Licensee 
acknowledges such ownership and intellectual property rights and will 
not take any action to jeopardize, limit or interfere in any manner 
with RUTGERS' ownership of or rights with respect to the SOFTWARE.  
The SOFTWARE is protected by copyright and other intellectual 
property laws and by international treaties.  Title and related 
rights in the content accessed through the SOFTWARE is the property 
of the applicable content owner and is protected by applicable law.  
The license granted under this Agreement gives Licensee no rights to such
content.

7. DISCLAIMER OF WARRANTY.  THE SOFTWARE IS PROVIDED FREE OF 
CHARGE, AND, THEREFORE, ON AN "AS IS" BASIS, WITHOUT WARRANTY OF 
ANY KIND, INCLUDING WITHOUT LIMITATION THE WARRANTIES THAT IT 
IS FREE OF DEFECTS, MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE 
OR NON-INFRINGING.  THE ENTIRE RISK AS TO THE QUALITY AND 
PERFORMANCE OF THE SOFTWARE IS BORNE BY LICENSEE.  SHOULD THE 
SOFTWARE PROVE DEFECTIVE IN ANY RESPECT, THE LICENSEE AND NOT 
LICENSOR ASSUMES THE ENTIRE COST OF ANY SERVICE AND REPAIR.  
THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF 
THIS AGREEMENT.  NO USE OF THE PRODUCT IS AUTHORIZED HEREUNDER 
EXCEPT UNDER THIS DISCLAIMER.

8. LIMITATION OF LIABILITY.  TO THE MAXIMUM EXTENT PERMITTED BY
APPLICABLE LAW,  IN NO EVENT WILL LICENSOR BE LIABLE FOR ANY 
INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
OUT OF THE USE OF OR INABILITY TO USE THE SOFTWARE, INCLUDING, 
WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK 
STOPPAGE, COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL 
OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF ADVISED OF THE
POSSIBILITY THEREOF. 
*/
/*
  PURPOSE:      Provide a command-line utility to create an ADIT
                validation letter.
*/    



#include <iostream.h>
#include <time.h>

#include "CifFileObj.h"
#include "search-util.h"

#define  HT_MAX_NAME_LENGTH 255


static char vlet_header_en[]="\
Dear Depositor, \n\
 \n\
    Thank you for depositing your structure with the Protein Data Bank (PDB).   \n\
 \n\
    Your entry has been assigned PDB ID %s and RCSB ID %s.\n\
A copy of your processed file in PDB format is appended to this message. \n\
 \n\
";


static char vlet_trailer_en[]="\
    Thank you for your time and effort in responding to these points.  \n\
 \n\
                                     Sincerely yours, \n\
 \n\
                                     PDB Data Processing Staff \n\
 \n\
Please direct all correspondence to: \n\
 \n\
The Protein Data Bank                           E-mail: \n\
Rutgers, The State University of New Jersey     deposit@rcsb.rutgers.edu \n\
Wright and Rieman Labs                          Fax:   (732) 445-4320 \n\
Department of Chemistry                         Phone: (732) 445-0103 \n\
610 Taylor Rd \n\
Piscataway, NJ 08854-8087 \n\
\n\
\n\
--------------------------------------------------------------------------------\n\
\n\
";


static char blet_en[]="\
    As part of converting your file to final format, we have performed some \n\
additional checking and have encountered some issues requiring your input \n\
before we can release the file. \n\
 \n\
 < ---   ANNOTATOR INSERTS COMMENTS HERE....   --->  \n\
 \n\
    Further results of our checking are given at the conclusion of this \n\
letter. If you wish to view further validation tests of your structure  \n\
please go to our validation site located at http://pdb.rutgers.edu/validate. \n\
 \n\
    After reviewing the entry, please respond to the above issues \n\
as soon as possible so that we may complete and release your file. \n\
 \n\
";

static char glet_en[]="\
    As part of converting your file to final format, we have performed some \n\
checking.    Some points you may wish to consider are given \n\
at the conclusion of this letter.  If you wish to view further validation \n\
tests of your structure please go to our validation site located at \n\
http://pdb.rutgers.edu/validate. \n\
 \n\
    After reviewing the entry, please alert us immediately to any changes \n\
you may have.  If we do not hear from you within THREE days, we will consider \n\
this entry to have been approved by you.  The entry will then be released \n\
according to the following release/hold instructions you have provided: \n\
 \n\
";

int reformat_summary_letter(const char *idName, CifFileObj *fobj, const char *blockId,
				   const char *HT_EXP_METHOD, const char *HT_MOL_TYPE);
void formatStringData(FILE *fp, const char *cs, int maxWidth);
int display_raw_file(FILE *io, const char *fName);


void usage(const char * pname) {
  cerr << pname << " usage =  " << endl;
  cerr << "             -f <input_cif_file> -exp <xray|nmr> -mol <na|prot>  <" << endl;
  cerr << endl;
}


int main(int argc, char *argv[]) {

  char *pname=NULL, *iFile=NULL, *diags=NULL;
  char tObjFile[L_tmpnam];
  char *blockId=NULL;
  char *molType=NULL, *expType=NULL;

  pname = argv[0];

  if (argc < 6) {
    usage(pname);
    exit(1);
  }

  for (int i=1; i < argc; i++) {
    if ( argv[i][0] == '-' ) {
      if (strcmp(argv[i],"-f") == 0 ) {
        i++;
        iFile = (char *) calloc(strlen(argv[i]) + 1, sizeof(char));
        strcpy(iFile,argv[i]);          
      } else if (strcmp(argv[i],"-mol") == 0 ) {
        i++;
        molType = (char *) calloc(strlen(argv[i]) + 1, sizeof(char));
        strcpy(molType,argv[i]);          

      } else if (strcmp(argv[i],"-exp") == 0 ) {
        i++;
        expType = (char *) calloc(strlen(argv[i]) + 1, sizeof(char));
        strcpy(expType,argv[i]);          
      } else {
	usage(pname);
        exit(1);
      }
    } else {
      usage(pname);
      exit(1);
    }
  }

  if (!iFile || !strlen(iFile) || !expType || !strlen(expType) || !molType || !strlen(molType) ) {
    usage(pname); exit(2);
  }

  FILE *fp0 = NULL;
  fp0 = fopen(iFile,"r");
  if (fp0 == NULL) {
    usage(pname); 
    cerr << endl << "Unable to open input file " << iFile << endl;
    exit(2);

  }

  //
  //  Read Cif file ... 
  //

  tmpnam(tObjFile);
  CifFileObj *fobj = new CifFileObj(tObjFile, WRITE_MODE, 80, "?", 1);
  fobj->Read(iFile,diags);
  if (diags) {
    cout << " Diagnostics [" << strlen(diags) << "] " << diags << endl;
    delete diags;
  }
  blockId = fobj->GetBlockName(0);

  char *idName = NULL;
  idName = get_attribute_value_where(fobj, blockId, "database_2", "database_code", 
				     "database_id", "RCSB");


  reformat_summary_letter(idName, fobj, blockId, expType, molType);

  //
  fobj->CloseFile();
  if (fobj) delete fobj;
  remove(tObjFile);
  exit(0);
}




int reformat_summary_letter(const char *idName, CifFileObj *fobj, const char *blockId,
				   const char *HT_EXP_METHOD, const char *HT_MOL_TYPE) 
{
  // idName is RCSBID
  // id is file system id.
  // returns 1 if letters are created... 

 char l1[HT_MAX_NAME_LENGTH];
 char sourceLetter[HT_MAX_NAME_LENGTH];
 char id[20];
 FILE *fp=NULL;
 time_t curr_time;
 struct tm *tmptr;
 char *curt;
 char timeString[31];
 char status1[150];
 char status2[150];


 memset(id,0,20);
 strncpy(id,idName,19);
 ndb_set_string_to_lower(id);

 strcpy(sourceLetter,id);
 strcat(sourceLetter,".letter");

 time(&curr_time);
 tmptr = localtime(&curr_time);
 curt = ctime(&curr_time);
 ndb_clean_string(curt);
 if (curt[strlen(curt)-1] == '\n') curt[strlen(curt)-1] = '\0';
 strftime(timeString,30,"%Y-%m-%d", tmptr);


 char *PDBID = NULL;
 char *RStatusCoords = NULL, *RStatusSF = NULL, *RStatusNC = NULL;
 char *HDateCoords = NULL, *HDateSF = NULL, *HDateNC = NULL;
 char *RecvdSF = NULL, *RecvdNC=NULL;

 char **email=NULL;
 int  nEmail=0;
 char idlc[30];



 PDBID=get_attribute_value_where(fobj, blockId, "database_2", "database_code", 
				 "database_id", "PDB");
 if (PDBID == NULL) {
   PDBID = new char[10];
   strcpy(PDBID,"UNASSIGNED");
 }
 ndb_log_message_text(NDB_MSG_DEBUG,"reformat_summary_letter PDB ID %s RCSB ID %s",PDBID,idName);

 //
 // -------------------------------------------------------------------------------
 memset(idlc,0,30);
 strncpy(idlc,idName,29);
 ndb_set_string_to_lower(idlc);
 email = get_attribute_values(fobj,blockId,"audit_contact_author","email",&nEmail);

 fp = fopen("Mail-script.csh","w");
 if (fp != NULL) {
   fprintf(fp,"#!/bin/csh\n#\n");
   fprintf(fp,"echo \"~rt deposit@rcsb.rutgers.edu\"   > tmp.message\n");
   fprintf(fp,"echo \"~c  ot-outgoing-val@rcsb.rutgers.edu\"   >> tmp.message\n");
   fprintf(fp,"cat %s-validate.letter               >> tmp.message\n",idlc);
   fprintf(fp,"if (-e %s.pdb) then\n",idlc);
   fprintf(fp,"echo \" \" >> tmp.message\n");
   fprintf(fp,"echo \"--------------------------------------------------------------------------------\" >> tmp.message\n");
   fprintf(fp,"echo \"PDB FILE PDB FILE PDB FILE PDB FILE PDB FILE PDB FILE PDB FILE PDB FILE PDB FILE\" >> tmp.message\n");
   fprintf(fp,"echo \"--------------------------------------------------------------------------------\" >> tmp.message\n");
   fprintf(fp,"cat %s.pdb               >> tmp.message\n",idlc);
   fprintf(fp,"endif\n#\n");
   fprintf(fp,"Mail -s \"Structure %s\" ", idName);

   if (nEmail == 0) {
     fprintf(fp," deposit@rcsb.rutgers.edu < tmp.message\n#\n");
   } else {
     for (int i=0; i < nEmail; i++) {
       if (i == nEmail-1) 
	 fprintf(fp,"%s ",email[i]);
       else
	 fprintf(fp,"%s,",email[i]);
     }
   }
   fprintf(fp," < tmp.message\n#\n",idlc);
   fprintf(fp,"rm -f tmp.message\n#\n");
   fclose(fp);
 } 
 //
 // -------------------------------------------------------------------------------
 //

 RStatusCoords = get_attribute_value(fobj,blockId,"ndb_database_status","author_release_status_code");
 if (RStatusCoords  == NULL) {
   RStatusCoords = new char[12];
   strcpy(RStatusCoords,"NONE");
 }

 HDateCoords = get_attribute_value(fobj,blockId,"ndb_database_status","date_hold_coordinates");
 if (HDateCoords == NULL) {
   HDateCoords = new char[12];
   strcpy(HDateCoords,"UNASSIGNED");
 }

 RStatusSF = get_attribute_value(fobj,blockId,"ndb_database_status","dep_release_code_struct_fact");
 if (RStatusSF  == NULL) {
   RStatusSF = new char[12];
   strcpy(RStatusSF,"UNASSIGNED");
 }

 RecvdSF = get_attribute_value(fobj,blockId,"ndb_database_status","recvd_struct_fact");
 HDateSF = get_attribute_value(fobj,blockId,"ndb_database_status","date_hold_struct_fact");
 if (HDateSF  == NULL) {
   HDateSF = new char[12];
   strcpy(HDateSF,"UNASSIGNED");
 }

 RecvdNC = get_attribute_value(fobj,blockId,"ndb_database_status","recvd_nmr_constraints");
 RStatusNC = get_attribute_value(fobj,blockId,"ndb_database_status","dep_release_code_nmr_constraints");
 if (RStatusNC  == NULL) {
   RStatusNC = new char[12];
   strcpy(RStatusNC,"UNASSIGNED");
 }

 HDateNC = get_attribute_value(fobj,blockId,"ndb_database_status","date_hold_nmr_constraints");
 if (HDateNC  == NULL) {
   HDateNC = new char[12];
   strcpy(HDateNC,"UNASSIGNED");
 }

 ndb_log_message_text(NDB_MSG_DEBUG,"reformat_summary_letter Status Coords %s HData %s",RStatusCoords, 
		      HDateCoords);
 ndb_log_message_text(NDB_MSG_DEBUG,"reformat_summary_letter Status SF     %s HData %s",RStatusSF, HDateSF);
 ndb_log_message_text(NDB_MSG_DEBUG,"reformat_summary_letter Status NC     %s HData %s",RStatusNC, HDateNC);


 memset(status1,0,150);
 memset(status2,0,150);

 if ( !strcasecmp(RStatusCoords,"HPUB") ) {
   sprintf(status1,"\n    ** Your coordinates will be held until the publication of your \n         primary citation.\n");

 } else  if (!strcasecmp(RStatusCoords,"HOLD") ) {
   sprintf(status1,"\n    ** Your coordinates will be held until: %s\n", HDateCoords);
   ndb_log_message_text(NDB_MSG_DEBUG,"reformat_summary_letter Status Coords %s msg %s",RStatusCoords, status1);

 } else  if ( !strcasecmp(RStatusCoords,"REL") ) {
   sprintf(status1,"\n    ** Your coordinates will be released immediately.\n");
   ndb_log_message_text(NDB_MSG_DEBUG,"reformat_summary_letter Status Coords %s msg %s",RStatusCoords, status1);

 } else {
   sprintf(status1,"\n    ** No release information has been provided for your coordinates.\n");
   ndb_log_message_text(NDB_MSG_DEBUG,"reformat_summary_letter Status Coords %s msg %s",RStatusCoords, status1);
 }

 
 if (!strcasecmp(HT_EXP_METHOD,"nmr")) {
   if ( RecvdNC && !strcasecmp(RecvdNC,"Y") ) {
     if (!strcasecmp(RStatusNC,"HOLD FOR PUBLICATION") ) {
       sprintf(status2,"    ** Your constraint data will be held until the publication of your \n         primary citation.\n");
     } else  if ( !strncasecmp(RStatusNC,"HOLD",4) ) {
       sprintf(status2,"    ** Your constraint data will be held until: %s\n", HDateNC);
     } else  if (!strcasecmp(RStatusNC,"RELEASE NOW") ) {
       sprintf(status2,"    ** Your constraint data will be released immediately.\n");
     } else  if (HDateNC && (strlen(HDateNC) > 1)) {
       sprintf(status2,"    ** Your constraint data will be held until: %s\n", HDateNC);
     } else {
       sprintf(status2,"    ** No release information has been provided for your constraint data.\n");
     }
   }
 } else {
   if ( RecvdSF && !strcasecmp(RecvdSF,"Y") ) {
     if ( !strcasecmp(RStatusSF,"HOLD FOR PUBLICATION") ) {
       sprintf(status2,"    ** Your structure factor data will be held until the publication of your \n         primary citation.\n");
     } else  if ( !strncasecmp(RStatusSF,"HOLD",4) ) {
       sprintf(status2,"    ** Your structure factor data will be held until: %s\n", HDateSF);
     } else  if ( !strcasecmp(RStatusSF,"RELEASE NOW") ) {
       sprintf(status2,"    ** Your structure factor data will be released immediately.\n");
     } else  if (HDateSF && (strlen(HDateSF) > 1)) {
       sprintf(status2,"    ** Your structure factore data will be held until: %s\n", HDateSF);
     } else  if ( !strcasecmp(RStatusCoords,"HPUB") ) {
       sprintf(status2,"    ** Your structure factor data will be held until the publication of your \n         primary citation.\n");
     } else {
       sprintf(status2,"    ** No release information has been provided for your structure factor data.\n");
     }
   }
 }


 char *xTitle=NULL;
 // char *xDescriptor=NULL;
 int i,wLen = 76;
 xTitle=     get_attribute_value(fobj,blockId, "struct", "title");
 // xDescriptor=get_attribute_value(fobj,blockId, "struct", "ndb_descriptor");


 strcpy(l1, id);
 strcat(l1, ".let1");
 fp = fopen(l1,"w");
 if (fp != NULL) {
   if (strcmp(HT_MOL_TYPE,"na")) {
     fprintf(fp,"\n                                                            %s\n",timeString);
     fprintf(fp,vlet_header_en, PDBID, idName);

     // Title etc...
     for (i=0; i < wLen/2-4; i++) {
       fprintf(fp,"=");
     }
     fprintf(fp," TITLE ");
     for (i=0; i < wLen/2-4; i++) {
       fprintf(fp,"=");
     }
     fprintf(fp,"\n");
     //     fprintf(fp,"Descriptor:  ");
     //     formatStringData(fp, xDescriptor, wLen-12);
     formatStringData(fp, xTitle, wLen-2);
     for (i=0; i < wLen-1; i++) {
       fprintf(fp,"=");
     }
     fprintf(fp,"\n\n");
     
     fprintf(fp,glet_en);
     fprintf(fp,status1);
     // 
     if (strlen(status2) < 1) {
       if (!strcasecmp(HT_EXP_METHOD,"nmr") ) {
	 fprintf(fp,"\n   We encourage the deposition of NMR constraint data.  If you would like \n");
         fprintf(fp,"to deposit this data, please attach your constraint data file when you \n");
         fprintf(fp,"respond to this message.\n");
       } else {
	 fprintf(fp,"\n   We encourage the deposition of structure factor data.  If you would like \n");
         fprintf(fp,"to deposit this data, please attach your structure factor file when you \n");
         fprintf(fp,"respond to this message. \n");
       }
     } else {
       fprintf(fp,status2);
     }
     fprintf(fp,"\n");
     fprintf(fp,vlet_trailer_en);
   }
   display_raw_file(fp,sourceLetter);
 } else {
   return(0);
 }

 strcpy(l1, id);
 strcat(l1, ".let2");
 fp = fopen(l1,"w");
 if (fp != NULL) {
   if (strcmp(HT_MOL_TYPE,"na")) {
     fprintf(fp,"\n                                                            %s\n",timeString);
     fprintf(fp,vlet_header_en, PDBID, idName);

     // Title etc...
     for (i=0; i < wLen/2-4; i++) {
       fprintf(fp,"=");
     }
     fprintf(fp," TITLE ");
     for (i=0; i < wLen/2-4; i++) {
       fprintf(fp,"=");
     }
     fprintf(fp,"\n");
     //     fprintf(fp,"Descriptor:  ");
     //     formatStringData(fp, xDescriptor, wLen-12);
     formatStringData(fp, xTitle, wLen-2);
     for (i=0; i < wLen-1; i++) {
       fprintf(fp,"=");
     }
     fprintf(fp,"\n\n");

     fprintf(fp,blet_en);
     //     fprintf(fp,status1);
     //     fprintf(fp,status2);
     //     fprintf(fp,"\n");
     fprintf(fp,vlet_trailer_en);
   }
   display_raw_file(fp,sourceLetter);
 } else {
   return(0);
 }
 return(1);
}

void formatStringData(FILE *fp, const char *cs, int maxWidth) 
{
  int len, icount;
  if (cs == NULL || (strlen(cs) == 0) || 
      ((strlen(cs) == 1) && (cs[0] =='?' || cs[0] =='.'))) return;
  len = strlen(cs);

  //  if (len > maxWidth) len = maxWidth;

  icount = 0;
  for (int i=0; i < len; i++) {
    
    if (isspace(cs[i]) && i==0)
      continue;
    else  if (isspace(cs[i]))
      fprintf(fp," ");
    else  if (cs[i] == '"') // string double quotes
      fprintf(fp," ");
    else
      fprintf(fp,"%c",cs[i]);
    
    +icount++;
    if (icount > maxWidth) {
      fprintf(fp,"\n");
      icount=0;
    }
  }

  fprintf(fp,"\n");

}



int display_raw_file(FILE *io, const char *fName) {
  /*
   * display_raw_file() copies the contents of named file to 
   * to the specified output unit.  Returns 1 for success
   * or 0 if the file is missing or on error.
   */
  
  FILE *fp=NULL;
  struct stat statbuf;

  char *dataFF=NULL;
  int len, iret;

  iret=0;
  if (fName == NULL || strlen(fName) == 0) return(iret);

  if ( (fp = fopen(fName,"r")) != NULL )  {
    stat(fName,&statbuf);
    ndb_log_message_text(NDB_MSG_DEBUG,"Reading %s size %d",fName,statbuf.st_size);
    dataFF = (char *) calloc(statbuf.st_size+1, sizeof(char));
    len=fread(dataFF, sizeof(char), statbuf.st_size+1, fp);
    ndb_log_message_text(NDB_MSG_DEBUG,"Read %d bytes in %s ",len, fName);
    fclose(fp);

    if (dataFF != NULL) {
      fprintf(io,"%s",dataFF);
      free(dataFF);
      iret=1;
    }
  }
  return(iret);
}
