/*
FILE:     mk-deposition.C
*/
/*
VERSION:  0.904-beta
*/
/*
DATE:     8/14/2003
*/
/*
  Comments and Questions to: sw-help@rcsb.rutgers.edu
*/
/*
COPYRIGHT 1999-2003 Rutgers - The State University of New Jersey

This software is provided WITHOUT WARRANTY OF MERCHANTABILITY OR
FITNESS FOR A PARTICULAR PURPOSE OR ANY OTHER WARRANTY, EXPRESS OR
IMPLIED.  RUTGERS MAKE NO REPRESENTATION OR WARRANTY THAT THE
SOFTWARE WILL NOT INFRINGE ANY PATENT, COPYRIGHT OR OTHER
PROPRIETARY RIGHT.

The user of this software shall indemnify, hold harmless and defend
Rutgers, its governors, trustees, officers, employees, students,
agents and the authors against any and all claims, suits,
losses, liabilities, damages, costs, fees, and expenses including
reasonable attorneys' fees resulting from or arising out of the
use of this software.  This indemnification shall include, but is
not limited to, any and all claims alleging products liability.
*/
/*
               PDB SOFTWARE LICENSE AGREEMENT

BY CLICKING THE ACCEPTANCE BUTTON OR INSTALLING OR USING 
THIS "SOFTWARE, THE INDIVIDUAL OR ENTITY LICENSING THE  
SOFTWARE ("LICENSEE") IS CONSENTING TO BE BOUND BY AND IS 
BECOMING A PARTY TO THIS AGREEMENT.  IF LICENSEE DOES NOT 
AGREE TO ALL OF THE TERMS OF THIS AGREEMENT
THE LICENSEE MUST NOT INSTALL OR USE THE SOFTWARE.

1. LICENSE AGREEMENT

This is a license between you ("Licensee") and the Protein Data Bank (PDB) 
at Rutgers, The State University of New Jersey (hereafter referred to 
as "RUTGERS").   The software is owned by RUTGERS and protected by 
copyright laws, and some elements are protected by laws governing 
trademarks, trade dress and trade secrets, and may be protected by 
patent laws. 

2. LICENSE GRANT

RUTGERS grants you, and you hereby accept, non-exclusive, royalty-free 
perpetual license to install, use, modify, prepare derivative works, 
incorporate into other computer software, and distribute in binary 
and source code format, or any derivative work thereof, together with 
any associated media, printed materials, and on-line or electronic 
documentation (if any) provided by RUTGERS (collectively, the "SOFTWARE"), 
subject to the following terms and conditions: (i) any distribution 
of the SOFTWARE shall bind the receiver to the terms and conditions 
of this Agreement; (ii) any distribution of the SOFTWARE in modified 
form shall clearly state that the SOFTWARE has been modified from 
the version originally obtained from RUTGERS.  

2. COPYRIGHT; RETENTION OF RIGHTS.  

The above license grant is conditioned on the following: (i) you must 
reproduce all copyright notices and other proprietary notices on any 
copies of the SOFTWARE and you must not remove such notices; (ii) in 
the event you compile the SOFTWARE, you will include the copyright 
notice with the binary in such a manner as to allow it to be easily 
viewable; (iii) if you incorporate the SOFTWARE into other code, you 
must provide notice that the code contains the SOFTWARE and include 
a copy of the copyright notices and other proprietary notices.  All 
copies of the SOFTWARE shall be subject to the terms of this Agreement.  

3. NO MAINTENANCE OR SUPPORT; TREATMENT OF ENHANCEMENTS 

RUTGERS is under no obligation whatsoever to: (i) provide maintenance 
or support for the SOFTWARE; or (ii) to notify you of bug fixes, patches, 
or upgrades to the features, functionality or performance of the 
SOFTWARE ("Enhancements") (if any), whether developed by RUTGERS 
or third parties.  If, in its sole discretion, RUTGERS makes an 
Enhancement available to you and RUTGERS does not separately enter 
into a written license agreement with you relating to such bug fix, 
patch or upgrade, then it shall be deemed incorporated into the SOFTWARE 
and subject to this Agreement. You are under no obligation whatsoever 
to provide any Enhancements to RUTGERS or the public that you may 
develop over time; however, if you choose to provide your Enhancements 
to RUTGERS, or if you choose to otherwise publish or distribute your 
Enhancements, in source code form without contemporaneously requiring 
end users or RUTGERS to enter into a separate written license agreement 
for such Enhancements, then you hereby grant RUTGERS a non-exclusive,
royalty-free perpetual license to install, use, modify, prepare
derivative works, incorporate into the SOFTWARE or other computer
software, distribute, and sublicense your Enhancements or derivative
works thereof, in binary and source code form.

4. FEES.  There is no license fee for the SOFTWARE.  If Licensee
wishes to receive the SOFTWARE on media, there may be a small charge
for the media and for shipping and handling.  Licensee is
responsible for any and all taxes.

5. TERMINATION.  Without prejudice to any other rights, Licensor
may terminate this Agreement if Licensee breaches any of its terms
and conditions.  Upon termination, Licensee shall destroy all
copies of the SOFTWARE.

6. PROPRIETARY RIGHTS.  Title, ownership rights, and intellectual
property rights in the Product shall remain with RUTGERS.  Licensee 
acknowledges such ownership and intellectual property rights and will 
not take any action to jeopardize, limit or interfere in any manner 
with RUTGERS' ownership of or rights with respect to the SOFTWARE.  
The SOFTWARE is protected by copyright and other intellectual 
property laws and by international treaties.  Title and related 
rights in the content accessed through the SOFTWARE is the property 
of the applicable content owner and is protected by applicable law.  
The license granted under this Agreement gives Licensee no rights to such
content.

7. DISCLAIMER OF WARRANTY.  THE SOFTWARE IS PROVIDED FREE OF 
CHARGE, AND, THEREFORE, ON AN "AS IS" BASIS, WITHOUT WARRANTY OF 
ANY KIND, INCLUDING WITHOUT LIMITATION THE WARRANTIES THAT IT 
IS FREE OF DEFECTS, MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE 
OR NON-INFRINGING.  THE ENTIRE RISK AS TO THE QUALITY AND 
PERFORMANCE OF THE SOFTWARE IS BORNE BY LICENSEE.  SHOULD THE 
SOFTWARE PROVE DEFECTIVE IN ANY RESPECT, THE LICENSEE AND NOT 
LICENSOR ASSUMES THE ENTIRE COST OF ANY SERVICE AND REPAIR.  
THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF 
THIS AGREEMENT.  NO USE OF THE PRODUCT IS AUTHORIZED HEREUNDER 
EXCEPT UNDER THIS DISCLAIMER.

8. LIMITATION OF LIABILITY.  TO THE MAXIMUM EXTENT PERMITTED BY
APPLICABLE LAW,  IN NO EVENT WILL LICENSOR BE LIABLE FOR ANY 
INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
OUT OF THE USE OF OR INABILITY TO USE THE SOFTWARE, INCLUDING, 
WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK 
STOPPAGE, COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL 
OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF ADVISED OF THE
POSSIBILITY THEREOF. 
*/
/*

  PURPOSE:      Deposition operation.

*/


#include <iostream.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>

#include "CifBuilder.h"

#include "ndb_htmllib.h"
#include "ndb_misclib.h"

#define  HTML_ENV_GLOBAL_INIT
#include "interface-defs.h"
#include "view-obj-util.h"


static void ndb_cif_encapsulate(char **a);
// static void ndb_cif_de_encapsulate(char ** a);

static int write_acknowledgment_letter(const char *fname , const char *idName, 
				       const char *authName, const char *pdbId);

ISTable *create_correspondence_table(const char *idName, const char *dirPath, 
				     const char *email, const char *name, const char *pdbId);
static void create_deposition_file_structure(const char *idname, const char *dirPath);

void write_correspondence_db(CifFileObj *fobj, const char *idName, const char *dirPath, const char *blockId,
			     ISTable *corTab);
static void import_cvs_new_deposition(const char *idname, const char *dirPath);

static void send_local_notification(const char *idName, const char *tmppath, 
				    const char *path, const char *dir, const char *email,
				    const char *annot, const char *pdbId);


void html_display_missing_data_message(FILE *io, CifFileObj *fobj, CifViewObj *vobj,  
				       const char *blockId, const char *ufid, int style);


char *assign_deposition_id(CifFileObj *fobj, const char *blockId);



void html_display_accept_deposition(FILE *io, CifFileObj *fobj, CifViewObj *vobj,  
				    const char *blockId, const char *ufid, 
				    const char *email, const char *idNameUC, int style, 
				    const char *pdbId);

static char *get_corresponding_pdb_id(const char *fName, const char *targetId);
static char *get_annotator_for_id(const char *targetId);


int main(int argc, char ** argv) {
    entry entries[HT_MAX_ENTRIES];
    int   nentries;
    char  *ufid=NULL, *prog_name=NULL;
    char  *email=NULL, *name=NULL;
    char *pdbId = NULL, *annot=NULL;

    //    char  fn_items[HT_MAX_NAME_LENGTH];    

    char  dirPath[HT_MAX_NAME_LENGTH];    
    char  fobjPath[HT_MAX_NAME_LENGTH];    
    char  vobjPath[HT_MAX_NAME_LENGTH];    
    char  cifPath[HT_MAX_NAME_LENGTH];    
    

    char *idName=NULL;
    char idNameUC[100], id_file[HT_MAX_NAME_LENGTH];

    char category[CIF_MAX_NAME_LENGTH], viewName[CIF_MAX_NAME_LENGTH];
    char blockId[CIF_MAX_NAME_LENGTH];

    FILE *fp=NULL;
    prog_name = argv[0];
    nentries = ht_decode_get_query(entries);

    ht_set_httpd_environment(entries,nentries);
    ht_set_server_url(HT_SERVER_NAME);
    ht_set_server_cgi_path(HT_CGI_PATH);
   
    ndb_open_log(prog_name,HT_LOGFILE_NAME);
    ndb_set_debug_level(NDB_GLOBAL_DEBUG_LEVEL);

    ufid    = ht_get_keyword_value(HT_ATTRIBUTE_UNIQUE_ID,entries,nentries);

    ndb_log_message_text(NDB_MSG_DEBUG,"Got ufid:    %s",ufid);

    strcpy(dirPath,HT_SESSION_ROOT_PATH);
    strcat(dirPath,ufid);
    strcat(dirPath,"/");

    //
    // This is simple test to prevent redeposition of a stucture during
    // a single input session...
    //

    if (HT_APPL_CONTEXT_TYPE == HT_CONTEXT_DEPOSIT) {
      html_display_redeposit_block_message(stdout, ufid, dirPath, RCSB_STYLE_DEFAULT);
    }

    strcpy(fobjPath,dirPath);
    strcat(fobjPath,HT_TABLE_OBJECT_FILE);
    ndb_log_message_text(NDB_MSG_DEBUG,"Opening table object at %s",fobjPath);
    CifFileObj *fobj  = new CifFileObj(fobjPath,APPEND_MODE);

    strcpy(vobjPath,dirPath);
    strcat(vobjPath,HT_VIEW_OBJECT_FILE);
    ndb_log_message_text(NDB_MSG_DEBUG,"Opening view object at %s",vobjPath);
    CifViewObj *vobj = new CifViewObj(vobjPath,(const char *)HT_VIEW_NAME,0);
    get_current_view_address(dirPath,blockId,category,viewName);

    ndb_log_message_text(NDB_MSG_DEBUG,"Current blockId %s category %s",blockId,category);
    email =get_attribute_value(fobj, blockId, "audit_contact_author","email");
    name  =get_attribute_value(fobj, blockId, "audit_contact_author","name");

    //
    //  Bail out here if we detect a critical missing item...
    //
    html_display_missing_data_message(stdout, fobj, vobj, blockId, ufid, RCSB_STYLE_DEFAULT);

    //
    // assign id
    //
    idName=assign_deposition_id(fobj, blockId);
    strcpy(idNameUC,idName);
    ndb_set_string_to_upper(idNameUC);

    //
    //  Store ID in local file.
    //
    strcpy(id_file,dirPath);
    strcat(id_file,NDB_DEPOSITION_ID_FILE_NAME);
    fp = fopen(id_file,"w");
    if (fp !=NULL) {
      putfield(fp,idNameUC);
      fclose(fp);
    }


    //
    // Set the intitial deposition date and status if DEPOSIT MODE.
    //
    if (HT_APPL_CONTEXT_TYPE == HT_CONTEXT_DEPOSIT) {
      set_initial_deposit_status(fobj,blockId);

      // Set the RCSB ID...
      set_attribute_value_where(fobj, blockId, "database_2",
				"database_code", idName, 
				"database_id", "RCSB");

#if 1
      ndb_log_message_text(NDB_MSG_DEBUG,"Search for PDB ID for %s in file %s",idName,HT_PDB_ID_FILE);
      pdbId = get_corresponding_pdb_id(HT_PDB_ID_FILE, idName);
      if (pdbId != NULL) set_attribute_value_where(fobj, blockId, "database_2",
						   "database_code", pdbId, 
						   "database_id", "PDB");
      annot = get_annotator_for_id(idName);
      if (annot != NULL) update_attribute_value(fobj,blockId,"ndb_database_status","rcsb_annotator",annot);    
      ndb_log_message_text(NDB_MSG_DEBUG,"Got PDB ID %s Annotator %s",pdbId, annot);

      // Set initial NDB ID to RCSB ID...
      set_attribute_value_where(fobj, blockId, "database_2",
				"database_code", idName, 
				"database_id", "NDB");
#endif

    }

    //
    // Propagate the ID through the file and write CIF
    //
    update_entry_ids(fobj,blockId,idName);

    //

    strcpy(cifPath,dirPath);
    strcat(cifPath,NDB_CUR_ROOT_FILE_NAME_XYZ);
    strcat(cifPath,".cif");

    fobj->RenameBlock(idNameUC,0);
    fobj->Write(cifPath);
    fobj->RenameBlock(blockId,0);

    //
    //   Archiving and bookkeeping ... 
    //
    ndb_log_message_text(NDB_MSG_DEBUG,"idName %s email %s name %s ",idName,email,name);

    ndb_set_string_to_lower(idName);
    ISTable *s = create_correspondence_table(idName, dirPath, email, name, pdbId);
    create_deposition_file_structure(idName, dirPath);
    write_correspondence_db(fobj, idName, dirPath, blockId, s);
    import_cvs_new_deposition(idName,dirPath);

    if (HT_APPL_CONTEXT_TEST == 0) {
      send_local_notification(idName, dirPath, HT_ARCHIVE_PATH, HT_ARCHIVE_DIRECTORY, 
			      HT_ANNOTATOR_EMAIL,  annot, pdbId);
    }


    //
    //  Display HTML report for user...
    //

    html_display_accept_deposition(stdout, fobj, vobj,  blockId, ufid, email, 
				   idNameUC, RCSB_STYLE_DEFAULT, pdbId);

    ndb_close_log();    
    exit(0);
}


static char *get_annotator_for_id(const char *targetId) {

  char *retAnnot=NULL;


#if 0
  char *annotators[]={"SJ","KB","VP","LI"};
  int numAnnotators = 4;
  char id[11];
  int numId, iDiv, rem;
  if (!targetId || strlen(targetId) != 10 ) return(retAnnot);

  memset(id,0,11);
  strncpy(id,&targetId[4],6); 
  numId = atoi(id);
  iDiv = numId/numAnnotators;
  rem = numId - numAnnotators * iDiv; 
  if (rem >=0 && rem < numAnnotators) {
    retAnnot = new char[strlen(annotators[rem]) + 1];
    strcpy(retAnnot,annotators[rem]);
  }

#else

  retAnnot = new char[12];
  strcpy(retAnnot,"UNASSIGNED");

#endif

  if ( !strcmp(HT_MOL_TYPE,"na") ) {
    strcpy(retAnnot,"UNASSIGNED");
  }

  if (HT_OSAKA_SITE_FLAG == 1) {
    strcpy(retAnnot,"TK");
  }

  return (retAnnot);  
}



static char *get_corresponding_pdb_id(const char *fName, const char *targetId) {

  char *retId=NULL;
  char rcsbId[12], pdbId[5];
  FILE *fp2=NULL;

  // Test the output file
  if (!fName    || !strlen(fName))    return(retId);
  if (!targetId || !strlen(targetId)) return(retId);

  fp2 = fopen(fName,"r");
  if (fp2 == NULL) return(retId);

  while (!feof(fp2)) {
    getfield(fp2,rcsbId,12);
    getfield(fp2,pdbId,5);
    if (rcsbId && !strcmp(rcsbId,targetId)) {
      retId = new char[6];
      memset(retId,0,6);
      strncpy(retId,pdbId,5);
      //      printf("Matched RCSBID %10s  PDBID %5s\n",rcsbId, retId);
      break;
    }
  }
  fclose(fp2);
  return(retId);
}


char *assign_deposition_id(CifFileObj *fobj, const char *blockId) {

  char *rcsbID=NULL;
  char *idName = NULL;
  int idNo;
  
  rcsbID = get_attribute_value_where(fobj, blockId, "database_2", "database_code", 
				     "database_id", "RCSB");
  if ((rcsbID == NULL )) rcsbID=get_attribute_value_where(fobj, blockId, "database_2", "database_code", 
							  "database_id", "NDB");
  
  if (HT_APPL_CONTEXT_TYPE == HT_CONTEXT_DEPOSIT || 
      rcsbID == NULL || !strcmp(rcsbID,"?") || !strcasecmp(rcsbID,NDB_DEFAULT_BLOCK_ID) ) {

    idNo=get_current_deposition_count();
    idNo++;
    set_deposition_count(idNo);
    //    idName = new char[12];
    idName = (char *) calloc(12,sizeof(char));

    //  IDs of the form RCSB000000  (10 characters)

    if (idNo < 10) 
      sprintf(idName,"RCSB00000%d",idNo);
    else if (idNo < 100) 
      sprintf(idName,"RCSB0000%d",idNo);
    else if (idNo < 1000) 
      sprintf(idName,"RCSB000%d",idNo);
    else if (idNo < 10000) 
      sprintf(idName,"RCSB00%d",idNo);
    else if (idNo < 100000) 
      sprintf(idName,"RCSB0%d",idNo);
    else if (idNo < 1000000) 
      sprintf(idName,"RCSB%d",idNo);
  } else {
    idName = new char[strlen(rcsbID)+1];
    strcpy(idName,rcsbID);
  }
  return(idName);
}


void html_display_accept_deposition(FILE *io, CifFileObj *fobj, CifViewObj *vobj,  
				    const char *blockId, const char *ufid, 
				    const char *email, const char *idNameUC, int style, 
				    const char *pdbId)  {

  char  return_url[HT_MAX_NAME_LENGTH];  
  int i;
  //
  //   Accept deposition
  //
  ht_put_content_type_target(io,"_top");
  html_display_open_page(io,"RCSB Deposition",style);
  html_display_page_title_logo(io,"", style);

  if (HT_APPL_CONTEXT_DEMO) { 
    html_display_return_to_page(io, "Continue", "/index.html");
  } else if (!strncmp(HT_APPL_CONTEXT,"deposit",7)) {
    strcpy(return_url,HT_HOME_PATH);
    strcat(return_url,"index.html");
    html_display_return_to_page(io, "Continue", return_url);
  } else {
    strcpy(return_url,HT_HOME_PATH);
    strcat(return_url,"index.html");
    html_display_return_to_page(io, "Continue", return_url);
  }


#if defined(MSDTOOL)
    fprintf(io,"<center><H2>Your deposition has been accepted.</h2></center>\n");
    fprintf(io,"<center><H2>Your RCSB ID code is %s.</h2></center>\n",idNameUC);
    if (pdbId !=NULL) 
      fprintf(io,"<center><H2>Your PDB ID code is %s.</h2></center>\n",pdbId);
    
    fprintf(io,"<center><H2>Thank you for using the ADIT.</h2></center>\n");
#else
    fprintf(io,"<center><H2>Your deposition has been accepted.</h2></center>\n");
    fprintf(io,"<center><H2>Your NDB ID code is %s.</h2></center>\n",idNameUC);
    fprintf(io,"<center><H2>Thank you for using the NDB.</h2></center>\n");
#endif

    ReVarCifArray<CifString> * categoriesV = vobj->GetCategoriesViewable();
    if (categoriesV && categoriesV->Length() > 0) {
      ndb_log_message(NDB_MSG_DEBUG,"Printing tables for %d view categories",categoriesV->Length());

      fprintf(io,"<HR><br>\n");
      fprintf(io,"<p> <font size=4> The following tables contain a summary of the data you have just deposited. ");
      if (pdbId != NULL)
	fprintf(io,"A complete data file including your coordinate data and a validation summary ");
      else
	fprintf(io,"A complete data file including your PDB 4-letter code, coordinate data and a validation summary ");

      fprintf(io,"will be sent to you by e-mail at the address <font color=red><b>%s</b></font> shortly.",email);
      fprintf(io,"If multiple authors have been entered, a copy of the coordinate file \n");
      fprintf(io,"and validation report will be sent to each author's e-mail address \n");
      fprintf(io,"<br><br>If your e-mail address here or in the table below is in error, please send a brief message to ");
      fprintf(io,"<a href=\"mailto:deposit@rcsb.rutgers.edu\">deposit@rcsb.rutgers.edu</a> which includes ");
      fprintf(io,"your session restart identifier, <font color=red>%s</font>, and your correct contact information.",ufid);

      printf("<br><br><br><p>\n");
      for (i=0; i < categoriesV->Length(); i++) {
	display_view_category_as_table(io,fobj, vobj, blockId, (*categoriesV)[i].Text());
      }
    }

    html_display_close_page(io, style);
    fflush(io);    
}



void html_display_missing_data_message(FILE *io, CifFileObj *fobj, CifViewObj *vobj,  
				       const char *blockId, const char *ufid, int style) {

  char *email=NULL;
  //  char *name=NULL;
  email =get_attribute_value(fobj, blockId, "audit_contact_author","email");
  //  name  =get_attribute_value(fobj, blockId, "audit_contact_author","name");
  
  if (email == NULL || !strcmp(email,"?")) {
    
    
    ht_put_content_type_target(io,"_top");
    html_display_open_page(io,"Missing Data", style);
    html_display_page_title_logo(io,"", style);
    html_display_return_tool_top(io, ufid );
    
    CifString * categoryViewName = vobj->GetViewCategory("audit_contact_author");
    
    printf("<center><H2>Missing Data</h2>\n");
    printf("<br><br><h2>We cannot process your deposition because of some missing information... </h2>\n");
    printf("<br><H2>Please enter your name and e-mail address in category %s </h2></center>\n",
	   categoryViewName->Text());

    
    html_display_close_page(io, style);
    fflush(io);    
    
    ndb_close_log();    
    exit(0);
    
  }

}



static void send_local_notification(const char *idName, const char *tmppath, 
				    const char *path, const char *dir, const char *email,
				    const char *annot, const char *pdbId) {
  char  script[2048];
  char  idlc[100],iduc[100];
  char  fName[HT_ENV_MAX_NAME_LEN];         
  FILE *fp;
  time_t curr_time;
  struct tm *tmptr;
  char *curt;
  char timeString[31];

  static char letter1[]="\
This deposition has been stored in the CVS archive and is waiting annotation.\n\
To access the CVS module for structure %s, do the following:\n\
\n\
                   cd %s%s/%s\n\
\n\
\n\
Temporary session files for this deposition are located in the following path: \n\
\n\
           %s\n\
\n\
Warmest regards,\n\
\n\
ADIT \n\
\n";


  static char letter2[]="\
This deposition has been stored in the CVS archive and is waiting annotation.\n\
To access the CVS module for structure %s, do the following:\n\
\n\
                   cd %s%s/%s\n\
\n\
\n\
Temporary session files for this deposition are located in the following path: \n\
\n\
           %s\n\
\n\
The PDB ID for this entry is: %s\n\
\n\
This entry has been initially assigned to annotator: %s\n\
\n\
Warmest regards,\n\
\n\
ADIT \n\
\n";

  time(&curr_time);
  tmptr = localtime(&curr_time);
  curt = ctime(&curr_time);
  ndb_clean_string(curt);
  if (curt[strlen(curt)-1] == '\n') curt[strlen(curt)-1] = '\0';
  strftime(timeString,30,"%Y-%m-%d", tmptr);
  strcpy(idlc,idName);
  strcpy(iduc,idName);
  ndb_set_string_to_lower(idlc);
  ndb_set_string_to_upper(iduc);

  strcpy(fName, tmppath);
  strcat(fName, "notify-annotator-letter.txt");

  fp=fopen(fName,"w");
  if (fp == NULL) return;
  fprintf(fp,"\n\nDate: %s\n\n",timeString);
  fprintf(fp,"Dear Annotator:\n\n");
  if (pdbId !=NULL && annot != NULL) 
    fprintf(fp,letter2,iduc,path,dir,idlc,tmppath,pdbId, annot);
  else
    fprintf(fp,letter1,iduc,path,dir,idlc,tmppath);
  fclose(fp);

  sprintf(script,"Mail -s \"Structure %s deposited\" %s < %s",iduc,email,fName);
  ndb_log_message_text(NDB_MSG_DEBUG,"Mail script %s",script);

  wsystem(script);
}


void write_correspondence_db(CifFileObj *fobj, const char *idName, const char *dirPath,
			     const char *blockId, ISTable *corTab)
{

  ISTable *s = NULL;

  char  cobjPath[HT_MAX_NAME_LENGTH];  
  char  filePath[HT_MAX_NAME_LENGTH];  
  char  idNameuc[20];

  ndb_log_message_text(NDB_MSG_DEBUG,"Write correspondence db for id %s",idName);

  strcpy(idNameuc,idName);
  ndb_set_string_to_upper(idNameuc);

  strcpy(cobjPath,dirPath);
  strcat(cobjPath,"Cordb.odb");
  CifFileObj *cobj  = new CifFileObj(cobjPath,WRITE_MODE);

  s = fobj->GetTablePtr(blockId,"audit_contact_author");
  if (s !=NULL) cobj->WriteTable(s, idNameuc,  s->GetName());

  s = fobj->GetTablePtr(blockId,"database");
  if (s != NULL)   cobj->WriteTable(s, idNameuc,  s->GetName());

  s = fobj->GetTablePtr(blockId,"database_2");
  if (s != NULL)   cobj->WriteTable(s, idNameuc,  s->GetName());

  if (corTab != NULL) cobj->WriteTable(corTab, idNameuc,  corTab->GetName());

  strcpy(filePath,dirPath);
  strcat(filePath, HT_ARCHIVE_DIRECTORY);
  strcat(filePath, "/");
  strcat(filePath,idName);
  strcat(filePath,"/");
  strcat(filePath,idName);
  strcat(filePath,".cor.cif");

  ndb_log_message_text(NDB_MSG_DEBUG,"Write correspondence file %s",filePath);

  cobj->Write(filePath);
  cobj->CloseFile();
  ndb_log_message_text(NDB_MSG_DEBUG,"Done with correspondence db",filePath);
}


ISTable *create_correspondence_table(const char *idName, const char *dirPath, 
				     const char *email, const char *name, const char *pdbId)  
  /* 
   *   Note: messages use local encapsulation mechanism...
   */
{
  FILE *fp=NULL;
  char *dataCIF=NULL, *dataPDB=NULL, *dataLETTER=NULL, *dataFORM=NULL;

  char  newPath[HT_MAX_NAME_LENGTH];
  char  idUpper[CIF_MAX_NAME_LENGTH];

  char  letterPath[HT_MAX_NAME_LENGTH];

  CifString cs;
  int i, nCols, len;

  time_t curr_time;
  struct tm *tmptr;
  char *curt;
  char timeString[31];


  struct stat statbuf;

  time(&curr_time);
  tmptr = localtime(&curr_time);
  curt = ctime(&curr_time);
  ndb_clean_string(curt);
  if (curt[strlen(curt)-1] == '\n') curt[strlen(curt)-1] = '\0';
  strftime(timeString,30,"%Y-%m-%d", tmptr);

  strcpy(letterPath,dirPath);
  strcat(letterPath,NDB_ACK_DEPOSIT_LETTER);
  write_acknowledgment_letter(letterPath, idName, name, pdbId);

  strcpy(newPath,dirPath);
  strcat(newPath,NDB_DEF_ROOT_FILE_NAME_XYZ);
  strcat(newPath,".cif");
  if ( (fp=fopen(newPath,"r")) != NULL ) {
    stat(newPath,&statbuf);
    ndb_log_message(NDB_MSG_DEBUG,"Reading %s size %d",newPath,statbuf.st_size);
    dataCIF = (char *) calloc(statbuf.st_size+1, sizeof(char));
    len=fread(dataCIF, sizeof(char), statbuf.st_size+1, fp);
    //jdw
    ndb_cif_encapsulate(&dataCIF);
    ndb_log_message(NDB_MSG_DEBUG,"Read %d bytes in %s ",len, newPath);
    fclose(fp);    
  }

  strcpy(newPath,dirPath);
  strcat(newPath,NDB_DEF_ROOT_FILE_NAME_XYZ);
  strcat(newPath,".pdb");
  if ( (fp=fopen(newPath,"r")) != NULL ) {
    stat(newPath,&statbuf);
    ndb_log_message(NDB_MSG_DEBUG,"Reading %s size %d",newPath,statbuf.st_size);
    dataPDB = (char *) calloc(statbuf.st_size+1, sizeof(char));
    len=fread(dataPDB, sizeof(char), statbuf.st_size+1, fp);
    //jdw
    ndb_cif_encapsulate(&dataPDB);
    fclose(fp);    
    ndb_log_message(NDB_MSG_DEBUG,"Read %d bytes in %s ",len, newPath);
  }


  strcpy(newPath,letterPath);
  if ( (fp=fopen(newPath,"r")) != NULL ) {
    stat(newPath,&statbuf);
    ndb_log_message(NDB_MSG_DEBUG,"Reading %s size %d",newPath,statbuf.st_size);
    dataLETTER = (char *) calloc(statbuf.st_size+1, sizeof(char));
    len=fread(dataLETTER, sizeof(char), statbuf.st_size+1, fp);
    fclose(fp);    
    //jdw
    ndb_cif_encapsulate(&dataLETTER);
    ndb_log_message(NDB_MSG_DEBUG,"Read %d bytes in %s ",len, newPath);
  }

  strcpy(newPath,dirPath);
  strcat(newPath,NDB_DEF_ROOT_FILE_NAME_FORM);
  strcat(newPath,".txt");
  if ( (fp=fopen(newPath,"r")) != NULL ) {
    stat(newPath,&statbuf);
    ndb_log_message(NDB_MSG_DEBUG,"Reading %s size %d",newPath,statbuf.st_size);
    dataFORM = (char *) calloc(statbuf.st_size+1, sizeof(char));
    len=fread(dataFORM, sizeof(char), statbuf.st_size+1, fp);
    fclose(fp);    
    ndb_cif_encapsulate(&dataFORM);
    ndb_log_message(NDB_MSG_DEBUG,"Read %d bytes in %s ",len, newPath);
  }


  ISTable *s = new ISTable("ndb_database_message");
  
  s->AddColumn("entry_id");
  s->AddColumn("message_id");
  s->AddColumn("date");
  s->AddColumn("content_type");
  s->AddColumn("message_type");
  s->AddColumn("sender");
  s->AddColumn("sender_address_fax");
  s->AddColumn("sender_address_phone");
  s->AddColumn("sender_address_email");
  s->AddColumn("sender_address_mail");
  s->AddColumn("receiver");
  s->AddColumn("receiver_address_fax");
  s->AddColumn("receiver_address_phone");
  s->AddColumn("receiver_address_email");
  s->AddColumn("receiver_address_mail");
  s->AddColumn("content");
  s->AddColumn("message");

  s->AddRow();

  cs.Clear();
  cs.Copy("?");

  ReVarCifArray<CifString> rowData;

  nCols = s->GetNumColumns();

  for (i=0; i < nCols; i++) {
    rowData.Add(cs);
  }
  ndb_log_message_text(NDB_MSG_DEBUG,"Got %d columns",nCols);

  if ((dataCIF != NULL) || (dataPDB != NULL)) {
    rowData[15].Clear();
    rowData[15].Copy("COORDINATES");
    if (dataCIF != NULL) {
      rowData[16].Clear();  
      rowData[16].Copy(dataCIF);
    } else {
      rowData[16].Clear();  
      rowData[16].Copy(dataPDB);
    }
    
    rowData[0].Clear(); 
    strcpy(idUpper,idName);
    ndb_set_string_to_upper(idUpper);
    rowData[0].Copy(idUpper);
    
    rowData[1].Clear(); 
    rowData[1].Copy("1");
    
    rowData[2].Clear(); 
    rowData[2].Copy(timeString);
    
    rowData[3].Clear(); 
    rowData[3].Copy("DEPOSIT");
    
    rowData[4].Clear(); 
    rowData[4].Copy("InputTool");

    if (name != NULL) {
      rowData[5].Clear();  
      rowData[5].Copy(name);
    }

    if (email != NULL) {
      rowData[8].Clear();  
      rowData[8].Copy(email);
    }
  }
  s->FillRow(rowData, 0);
  //
  //

  if (dataLETTER != NULL) {

    ndb_log_message_text(NDB_MSG_DEBUG,"Adding letter to correspondence file");

    s->AddRow();
    ReVarCifArray<CifString> rowData1;
    
    nCols = s->GetNumColumns();
    
    for (i=0; i < nCols; i++) {
      rowData1.Add(cs);
    }
    
    rowData1[15].Clear();  
    rowData1[15].Copy("ACKNOWLEDGEMENT");  

    rowData1[16].Clear();  
    rowData1[16].Copy(dataLETTER);
    
    rowData1[0].Clear(); 
    strcpy(idUpper,idName);
    ndb_set_string_to_upper(idUpper);
    rowData1[0].Copy(idUpper);
    
    rowData1[1].Clear(); 
    rowData1[1].Copy("2");
    
    rowData1[2].Clear(); 
    rowData1[2].Copy(timeString);
    
    rowData1[3].Clear(); 
    rowData1[3].Copy("OTHER");
    
    rowData1[4].Clear(); 
    rowData1[4].Copy("Annotator");
    
    if (name != NULL) {
      rowData1[5].Clear();  
      rowData1[5].Copy(name);
    }
    
    if (email != NULL) {
      rowData1[8].Clear();  
      rowData1[8].Copy(email);
    }
    s->FillRow(rowData1,1);
  }

  if (dataFORM != NULL) {

    ndb_log_message_text(NDB_MSG_DEBUG,"Adding deposition form to correspondence file");

    s->AddRow();
    ReVarCifArray<CifString> rowData2;
    
    nCols = s->GetNumColumns();
    
    for (i=0; i < nCols; i++) {
      rowData2.Add(cs);
    }
    
    rowData2[15].Clear();  
    rowData2[15].Copy("DEPOSITION_FORM");  

    rowData2[16].Clear();  
    rowData2[16].Copy(dataFORM);
    
    rowData2[0].Clear(); 
    strcpy(idUpper,idName);
    ndb_set_string_to_upper(idUpper);
    rowData2[0].Copy(idUpper);
    
    rowData2[1].Clear(); 
    rowData2[1].Copy("3");
    
    rowData2[2].Clear(); 
    rowData2[2].Copy(timeString);
    
    rowData2[3].Clear(); 
    rowData2[3].Copy("OTHER");
    
    rowData2[4].Clear(); 
    rowData2[4].Copy("Annotator");
    
    if (name != NULL) {
      rowData2[5].Clear();  
      rowData2[5].Copy(name);
    }
    
    if (email != NULL) {
      rowData2[8].Clear();  
      rowData2[8].Copy(email);
    }
    s->FillRow(rowData2,2);
  }

  ndb_log_message_text(NDB_MSG_DEBUG,"Done with correspondence table");
  return(s);
}


static void import_cvs_new_deposition(const char *idname, const char *dirPath) {

  char  newPath[HT_MAX_NAME_LENGTH];
  char  command[HT_MAX_NAME_LENGTH];
  char  script[2048];

  
  //
  // I. Make directory structure for new entry
  //

  strcpy(newPath,dirPath);
  strcat(newPath,HT_ARCHIVE_DIRECTORY);
  strcat(newPath,"/");
  strcat(newPath,idname);
  strcat(newPath,"/");

  //
  //
  //

  strcpy(script,"");
  sprintf(command,"cd %s; ",newPath);
  strcat(script,command);
  sprintf(command,"%s -d %s import -m\"Archived from ADIT\" %s/%s RCSB V100A >>& %s%s;  ",
	  HT_CVS_COMMAND,HT_CVS_ARCHIVE_PATH, HT_ARCHIVE_DIRECTORY, idname,dirPath,"cvs.log");
  strcat(script,command);
  sprintf(command,"cd %s; ",HT_ARCHIVE_PATH);
  strcat(script,command);
  sprintf(command,"%s  -f -d %s checkout  %s/%s >>& %s%s;  ",
	  HT_CVS_COMMAND,HT_CVS_ARCHIVE_PATH, HT_ARCHIVE_DIRECTORY, idname,dirPath,"cvs.log");
  strcat(script,command);
  ndb_log_message_text(NDB_MSG_DEBUG,"CVS script %s",script);
  wsystem(script);
}


static void create_deposition_file_structure(const char *idname, const char *dirPath) {

  char  newPath[HT_MAX_NAME_LENGTH];
  char  command[HT_MAX_NAME_LENGTH];
  char  script[4096];
  char  mvCmd[] = "cp ";
  
  //
  // I. Make directory structure for new entry
  //

  strcpy(newPath,dirPath);
  strcat(newPath,HT_ARCHIVE_DIRECTORY);
  ndb_log_message_text(NDB_MSG_DEBUG,"New path is %s",newPath);
  mkdir(newPath,0771);

  strcat(newPath,"/");
  strcat(newPath,idname);

  mkdir(newPath,0771);
  strcat(newPath,"/");
  ndb_log_message_text(NDB_MSG_DEBUG,"New path is %s",newPath);

  //
  //  Compatibility script... 
#if 1
  strcpy(script,"");
  sprintf(command,"cd %s; ",dirPath);
  strcat(script,command);
  sprintf(command,"%s -f %s.txt ./%s/%s/%s; ",
	  mvCmd,ONDB_DEF_ROOT_FILE_NAME_FORM, HT_ARCHIVE_DIRECTORY,
	  idname,idname,NDB_DEF_ROOT_FILE_NAME_FORM);
  strcat(script,command);
  sprintf(command,"%s -f %s.cif ./%s/%s/%s-sf.cif; ",
	  mvCmd,ONDB_DEF_ROOT_FILE_NAME_SF,HT_ARCHIVE_DIRECTORY,idname,idname);
  strcat(script,command);

  sprintf(command,"%s -f %s.any ./%s/%s/%s-sf.any; ",
	  mvCmd,ONDB_DEF_ROOT_FILE_NAME_SF,HT_ARCHIVE_DIRECTORY,idname,idname);
  strcat(script,command);

  sprintf(command,"%s -f %s.other ./%s/%s/%s-sf.other; ",
	  mvCmd,ONDB_DEF_ROOT_FILE_NAME_SF,HT_ARCHIVE_DIRECTORY,idname,idname);
  strcat(script,command);

  sprintf(command,"%s -f %s.pdb ./%s/%s/%s-deposited.pdb; ",
	  mvCmd,ONDB_DEF_ROOT_FILE_NAME_XYZ,HT_ARCHIVE_DIRECTORY,idname,idname);
  strcat(script,command);

  sprintf(command,"%s -f %s.cif ./%s/%s/%s-deposited.cif; ",
	  mvCmd,ONDB_DEF_ROOT_FILE_NAME_XYZ,HT_ARCHIVE_DIRECTORY,idname,idname);
  strcat(script,command);

  sprintf(command,"%s -f %s.cif  ./%s/%s/%s.cif; ",
	  mvCmd,ONDB_CUR_ROOT_FILE_NAME_XYZ,HT_ARCHIVE_DIRECTORY,idname,idname);
  strcat(script,command);

  sprintf(command,"%s -f %s.mr  ./%s/%s/%s-deposited.mr; ",
	  mvCmd,ONDB_DEF_ROOT_FILE_NAME_NC,HT_ARCHIVE_DIRECTORY,idname,idname);
  strcat(script,command);

  ndb_log_message_text(NDB_MSG_DEBUG,"File system script I: %s",script);
  wsystem(script);

#endif

  //
  // II. Copy deposited files to directory structure and standardize file names.
  //
  strcpy(script,"");
  sprintf(command,"cd %s; ",dirPath);
  strcat(script,command);
  sprintf(command,"%s -f %s.txt ./%s/%s/%s; ",
	  mvCmd,NDB_DEF_ROOT_FILE_NAME_FORM, HT_ARCHIVE_DIRECTORY,
	  idname,idname,NDB_DEF_ROOT_FILE_NAME_FORM);
  strcat(script,command);
  sprintf(command,"%s -f %s.cif ./%s/%s/%s-sf.cif; ",
	  mvCmd,NDB_DEF_ROOT_FILE_NAME_SF,HT_ARCHIVE_DIRECTORY,idname,idname);
  strcat(script,command);

  sprintf(command,"%s -f %s.any ./%s/%s/%s-sf.any; ",
	  mvCmd,NDB_DEF_ROOT_FILE_NAME_SF,HT_ARCHIVE_DIRECTORY,idname,idname);
  strcat(script,command);

  sprintf(command,"%s -f %s.other ./%s/%s/%s-sf.other; ",
	  mvCmd,NDB_DEF_ROOT_FILE_NAME_SF,HT_ARCHIVE_DIRECTORY,idname,idname);
  strcat(script,command);

  //  sprintf(command,"%s -f %s.cifeps ./%s/%s/%s.cifeps; ",
  //  mvCmd,NDB_DEF_ROOT_FILE_NAME_XYZ,HT_ARCHIVE_DIRECTORY,idname,idname);
  //  strcat(script,command);

  sprintf(command,"%s -f %s.pdb ./%s/%s/%s-deposited.pdb; ",
	  mvCmd,NDB_DEF_ROOT_FILE_NAME_XYZ,HT_ARCHIVE_DIRECTORY,idname,idname);
  strcat(script,command);

  sprintf(command,"%s -f %s.cif ./%s/%s/%s-deposited.cif; ",
	  mvCmd,NDB_DEF_ROOT_FILE_NAME_XYZ,HT_ARCHIVE_DIRECTORY,idname,idname);
  strcat(script,command);

  sprintf(command,"%s -f %s.cif  ./%s/%s/%s.cif; ",
	  mvCmd,NDB_CUR_ROOT_FILE_NAME_XYZ,HT_ARCHIVE_DIRECTORY,idname,idname);
  strcat(script,command);

  sprintf(command,"%s -f %s.cif  ./%s/%s/%s.cifeps; ",
	  mvCmd,NDB_CUR_ROOT_FILE_NAME_XYZ,HT_ARCHIVE_DIRECTORY,idname,idname);
  strcat(script,command);

  sprintf(command,"%s -f %s.mr  ./%s/%s/%s-deposited.mr; ",
	  mvCmd,NDB_DEF_ROOT_FILE_NAME_NC,HT_ARCHIVE_DIRECTORY,idname,idname);
  strcat(script,command);

  sprintf(command,"%s -f %s.any  ./%s/%s/%s-cc-deposited.any; ",
	  mvCmd,NDB_DEF_ROOT_FILE_NAME_CC,HT_ARCHIVE_DIRECTORY,idname,idname);
  strcat(script,command);

  sprintf(command,"%s -f %s.any  ./%s/%s/%s-cs-deposited.any; ",
	  mvCmd,NDB_DEF_ROOT_FILE_NAME_CS,HT_ARCHIVE_DIRECTORY,idname,idname);
  strcat(script,command);

  sprintf(command,"touch ./%s/%s/%s.ndb; ",HT_ARCHIVE_DIRECTORY,idname,idname);
  strcat(script,command);

  sprintf(command,"touch ./%s/%s/%s.pdb; ",HT_ARCHIVE_DIRECTORY,idname,idname);
  strcat(script,command);

  sprintf(command,"%s -f %s ./%s/%s/%s; ",
	  mvCmd,NDB_ACK_DEPOSIT_LETTER,HT_ARCHIVE_DIRECTORY,idname,NDB_ACK_DEPOSIT_LETTER);
  strcat(script,command);


  ndb_log_message_text(NDB_MSG_DEBUG,"File system script II: %s",script);
  wsystem(script);

  ndb_log_message_text(NDB_MSG_DEBUG,"Done making deposit file system");

}


static int write_acknowledgment_letter(const char *fname , const char *idName, 
				       const char *name, const char *pdbId)
{
  char *authName=NULL;
  FILE *fp=NULL;
  time_t curr_time;
  struct tm *tmptr;
  char *curt;
  char timeString[31];

#if defined(MSDTOOL)



static char letter1[]="\
Thank you for depositing your structure with the Protein Data Bank (PDB).\n\
Your entry has been assigned the RCSB id code %s.\n\
We will process your deposition as soon as possible.\n\
\n\
If you have any questions, you can contact the PDB at:\n\
\n\
The Protein Data Bank\n\
Rutgers, The State University of New Jersey\n\
Wright and Rieman Labs\n\
Departmen of Chemistry\n\
610 Taylor Rd\n\
Piscataway, NJ 08854-8087\n\
\n\
E-mail: deposit@rcsb.rutgers.edu\n\
Phone: (732) 445-0103\n\
Fax:   (732) 445-4320\n\
\n\
Thank you for your interest in the PDB.\n\
\n\
\n\
Sincerely yours,\n\
\n\
PDB Curration Staff\n\
\n";


static char letter2[]="\
Thank you for depositing your structure with the Protein Data Bank (PDB).\n\
Your entry has been assigned the RCSB id code %s and PDB id code %s.\n\
We will process your deposition as soon as possible.\n\
\n\
If you have any questions, you can contact the PDB at:\n\
\n\
The Protein Data Bank\n\
Rutgers, The State University of New Jersey\n\
Wright and Rieman Labs\n\
Departmen of Chemistry\n\
610 Taylor Rd\n\
Piscataway, NJ 08854-8087\n\
\n\
E-mail: deposit@rcsb.rutgers.edu\n\
Phone: (732) 445-0103\n\
Fax:   (732) 445-4320\n\
\n\
Thank you for your interest in the PDB.\n\
\n\
\n\
Sincerely yours,\n\
\n\
PDB Curration Staff\n\
\n";


#else
static char letter1[]="\
Thank you for depositing your structure with the Nucleic Acid Database (NDB).\n\
Your entry has been assigned the NDB id code %s.\n\
We will process your deposition as soon as possible.\n\
\n\
If you have any questions, you can contact the NDB at:\n\
\n\
The Nucleic Acid Database\n\
Rutgers, The State University of New Jersey\n\
Wright and Rieman Labs\n\
Departmen of Chemistry\n\
610 Taylor Rd\n\
Piscataway, NJ 08854-8087\n\
\n\
E-mail: deposit@ndbserver.rutgers.edu\n\
Phone: (732) 445-0103\n\
Fax:   (732) 445-4320\n\
\n\
Thank you for your interest in the NDB.\n\
\n\
\n\
Sincerely yours,\n\
\n\
NDB Curration Staff\n\
\n";


static char letter2[]="\
Thank you for depositing your structure with the Nucleic Acid Database (NDB).\n\
Your entry has been assigned the NDB id code %s and PDB id code %s.\n\
We will process your deposition as soon as possible.\n\
\n\
If you have any questions, you can contact the NDB at:\n\
\n\
The Nucleic Acid Database\n\
Rutgers, The State University of New Jersey\n\
Wright and Rieman Labs\n\
Departmen of Chemistry\n\
610 Taylor Rd\n\
Piscataway, NJ 08854-8087\n\
\n\
E-mail: deposit@ndbserver.rutgers.edu\n\
Phone: (732) 445-0103\n\
Fax:   (732) 445-4320\n\
\n\
Thank you for your interest in the NDB.\n\
\n\
\n\
Sincerely yours,\n\
\n\
NDB Curration Staff\n\
\n";

#endif


//
  if (name ==  NULL || !strcmp(name,"?")) {
    authName = (char *) calloc(20,sizeof(char));
    strcpy(authName,"Sir/Madam");
  } else {
    authName = (char *) calloc(strlen(name)+1,sizeof(char));
    strcpy(authName,name);
  }
  fp=fopen(fname,"w");
  if (fp == NULL) return(0);

  time(&curr_time);
  tmptr = localtime(&curr_time);
  curt = ctime(&curr_time);
  ndb_clean_string(curt);
  if (curt[strlen(curt)-1] == '\n') curt[strlen(curt)-1] = '\0';
  strftime(timeString,30,"%Y-%m-%d", tmptr);

  fprintf(fp,"\n\nDate: %s\n\n",timeString);
  fprintf(fp,"Dear %s:\n\n",authName);

  if (pdbId != NULL ) 
    fprintf(fp,letter2,idName, pdbId);
  else
    fprintf(fp,letter1,idName);
 
  fclose(fp);

  return(1);
}


static void ndb_cif_encapsulate(char **a)
/* puts the encapsulated characters into string. At the
 begining of string puts 5 $, and one $ at the begining of
 every new line in the string
*/
{
  int i,j;
  int len,len1;
  char * value;
  len=strlen(*a);
  len1=6;
  value = (char *)malloc(len1);

  strcpy(value,"$$$$$");
  j=5;
  for (i=0;i<=len;i++)
  {
  value[j]=(*a)[i];
  j++;
  len1++;
  value=(char *)realloc(value,len1);
  if ((*a)[i]=='\n') {
  value[j]='$';
  j++;
  len1++;
  value=(char *)realloc(value,len1);
  }
  }
  *a=value;
}

#if 0
static void ndb_cif_de_encapsulate(char ** a)
{
  int i,j;
  int len;
  char * value;
  len=strlen(*a);
  value = (char *)malloc(len);
  if (strncmp (*a,"$$$$$",5)==0) {
  j=0;
  for (i=5;i<=len;i++)
  {
    value[j]=(*a)[i];
    j++;
    if ((*a)[i]=='\n') {
    i++;
    }
  }
  *a=value;
  }
}
#endif
