/*
FILE:     search-util.C
*/
/*
VERSION:  0.904-beta
*/
/*
DATE:     8/14/2003
*/
/*
  Comments and Questions to: sw-help@rcsb.rutgers.edu
*/
/*
COPYRIGHT 1999-2003 Rutgers - The State University of New Jersey

This software is provided WITHOUT WARRANTY OF MERCHANTABILITY OR
FITNESS FOR A PARTICULAR PURPOSE OR ANY OTHER WARRANTY, EXPRESS OR
IMPLIED.  RUTGERS MAKE NO REPRESENTATION OR WARRANTY THAT THE
SOFTWARE WILL NOT INFRINGE ANY PATENT, COPYRIGHT OR OTHER
PROPRIETARY RIGHT.

The user of this software shall indemnify, hold harmless and defend
Rutgers, its governors, trustees, officers, employees, students,
agents and the authors against any and all claims, suits,
losses, liabilities, damages, costs, fees, and expenses including
reasonable attorneys' fees resulting from or arising out of the
use of this software.  This indemnification shall include, but is
not limited to, any and all claims alleging products liability.
*/
/*
               PDB SOFTWARE LICENSE AGREEMENT

BY CLICKING THE ACCEPTANCE BUTTON OR INSTALLING OR USING 
THIS "SOFTWARE, THE INDIVIDUAL OR ENTITY LICENSING THE  
SOFTWARE ("LICENSEE") IS CONSENTING TO BE BOUND BY AND IS 
BECOMING A PARTY TO THIS AGREEMENT.  IF LICENSEE DOES NOT 
AGREE TO ALL OF THE TERMS OF THIS AGREEMENT
THE LICENSEE MUST NOT INSTALL OR USE THE SOFTWARE.

1. LICENSE AGREEMENT

This is a license between you ("Licensee") and the Protein Data Bank (PDB) 
at Rutgers, The State University of New Jersey (hereafter referred to 
as "RUTGERS").   The software is owned by RUTGERS and protected by 
copyright laws, and some elements are protected by laws governing 
trademarks, trade dress and trade secrets, and may be protected by 
patent laws. 

2. LICENSE GRANT

RUTGERS grants you, and you hereby accept, non-exclusive, royalty-free 
perpetual license to install, use, modify, prepare derivative works, 
incorporate into other computer software, and distribute in binary 
and source code format, or any derivative work thereof, together with 
any associated media, printed materials, and on-line or electronic 
documentation (if any) provided by RUTGERS (collectively, the "SOFTWARE"), 
subject to the following terms and conditions: (i) any distribution 
of the SOFTWARE shall bind the receiver to the terms and conditions 
of this Agreement; (ii) any distribution of the SOFTWARE in modified 
form shall clearly state that the SOFTWARE has been modified from 
the version originally obtained from RUTGERS.  

2. COPYRIGHT; RETENTION OF RIGHTS.  

The above license grant is conditioned on the following: (i) you must 
reproduce all copyright notices and other proprietary notices on any 
copies of the SOFTWARE and you must not remove such notices; (ii) in 
the event you compile the SOFTWARE, you will include the copyright 
notice with the binary in such a manner as to allow it to be easily 
viewable; (iii) if you incorporate the SOFTWARE into other code, you 
must provide notice that the code contains the SOFTWARE and include 
a copy of the copyright notices and other proprietary notices.  All 
copies of the SOFTWARE shall be subject to the terms of this Agreement.  

3. NO MAINTENANCE OR SUPPORT; TREATMENT OF ENHANCEMENTS 

RUTGERS is under no obligation whatsoever to: (i) provide maintenance 
or support for the SOFTWARE; or (ii) to notify you of bug fixes, patches, 
or upgrades to the features, functionality or performance of the 
SOFTWARE ("Enhancements") (if any), whether developed by RUTGERS 
or third parties.  If, in its sole discretion, RUTGERS makes an 
Enhancement available to you and RUTGERS does not separately enter 
into a written license agreement with you relating to such bug fix, 
patch or upgrade, then it shall be deemed incorporated into the SOFTWARE 
and subject to this Agreement. You are under no obligation whatsoever 
to provide any Enhancements to RUTGERS or the public that you may 
develop over time; however, if you choose to provide your Enhancements 
to RUTGERS, or if you choose to otherwise publish or distribute your 
Enhancements, in source code form without contemporaneously requiring 
end users or RUTGERS to enter into a separate written license agreement 
for such Enhancements, then you hereby grant RUTGERS a non-exclusive,
royalty-free perpetual license to install, use, modify, prepare
derivative works, incorporate into the SOFTWARE or other computer
software, distribute, and sublicense your Enhancements or derivative
works thereof, in binary and source code form.

4. FEES.  There is no license fee for the SOFTWARE.  If Licensee
wishes to receive the SOFTWARE on media, there may be a small charge
for the media and for shipping and handling.  Licensee is
responsible for any and all taxes.

5. TERMINATION.  Without prejudice to any other rights, Licensor
may terminate this Agreement if Licensee breaches any of its terms
and conditions.  Upon termination, Licensee shall destroy all
copies of the SOFTWARE.

6. PROPRIETARY RIGHTS.  Title, ownership rights, and intellectual
property rights in the Product shall remain with RUTGERS.  Licensee 
acknowledges such ownership and intellectual property rights and will 
not take any action to jeopardize, limit or interfere in any manner 
with RUTGERS' ownership of or rights with respect to the SOFTWARE.  
The SOFTWARE is protected by copyright and other intellectual 
property laws and by international treaties.  Title and related 
rights in the content accessed through the SOFTWARE is the property 
of the applicable content owner and is protected by applicable law.  
The license granted under this Agreement gives Licensee no rights to such
content.

7. DISCLAIMER OF WARRANTY.  THE SOFTWARE IS PROVIDED FREE OF 
CHARGE, AND, THEREFORE, ON AN "AS IS" BASIS, WITHOUT WARRANTY OF 
ANY KIND, INCLUDING WITHOUT LIMITATION THE WARRANTIES THAT IT 
IS FREE OF DEFECTS, MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE 
OR NON-INFRINGING.  THE ENTIRE RISK AS TO THE QUALITY AND 
PERFORMANCE OF THE SOFTWARE IS BORNE BY LICENSEE.  SHOULD THE 
SOFTWARE PROVE DEFECTIVE IN ANY RESPECT, THE LICENSEE AND NOT 
LICENSOR ASSUMES THE ENTIRE COST OF ANY SERVICE AND REPAIR.  
THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF 
THIS AGREEMENT.  NO USE OF THE PRODUCT IS AUTHORIZED HEREUNDER 
EXCEPT UNDER THIS DISCLAIMER.

8. LIMITATION OF LIABILITY.  TO THE MAXIMUM EXTENT PERMITTED BY
APPLICABLE LAW,  IN NO EVENT WILL LICENSOR BE LIABLE FOR ANY 
INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
OUT OF THE USE OF OR INABILITY TO USE THE SOFTWARE, INCLUDING, 
WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK 
STOPPAGE, COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL 
OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF ADVISED OF THE
POSSIBILITY THEREOF. 
*/

#include "search-util.h"



char *get_attribute_value(CifFileObj *fobj, const char *blockId, const char *category, const char *attribute)
{
  ISTable *t=NULL;

  ReVarCifArray<CifString> *col=NULL;
  int nRow, iCol;
  char *string=NULL;

  ndb_log_message_text(NDB_MSG_DEBUG,"Searching for %s and %s",category,attribute);

  if (fobj->IsTablePresent(blockId,category)) {
    t=fobj->GetTablePtr(blockId,category);
    nRow = t->GetNumRows();
    ndb_log_message_text(NDB_MSG_DEBUG,"Got %d rows in %s",nRow,category);
    if (nRow > 0) {
      iCol = t->GetColumnIndex(attribute);
      ndb_log_message_text(NDB_MSG_DEBUG,"Attribute %s has index %d",attribute,iCol);
      if (iCol >= 0) {
	col=t->GetColumn(iCol);
	if (col && ((*col)[0].Length() > 0) ) {
	  string = new char[(*col)[0].Length() + 1];
	  strcpy(string,(*col)[0].Text());
	  ndb_log_message_text(NDB_MSG_DEBUG,"Attribute %s has value %s",attribute,string);
	}
      }
    }
  }
  return(string);
}


char **get_attribute_values(CifFileObj *fobj, const char *blockId, const char *category, const char *attribute,
			    int *nValues)
{
  ISTable *t=NULL;

  ReVarCifArray<CifString> *col=NULL;
  int nRow, iCol, i;
  char **strings=NULL;

  ndb_log_message_text(NDB_MSG_DEBUG,"Searching for %s and %s",category,attribute);
  *nValues=0;
  if (fobj->IsTablePresent(blockId,category)) {
    t=fobj->GetTablePtr(blockId,category);
    nRow = t->GetNumRows();
    ndb_log_message_text(NDB_MSG_DEBUG,"Got %d rows in %s",nRow,category);
    if (nRow > 0) {
      iCol = t->GetColumnIndex(attribute);
      ndb_log_message_text(NDB_MSG_DEBUG,"Attribute %s has index %d",attribute,iCol);
      if (iCol >= 0) {
	col=t->GetColumn(iCol);
	if (col && ((*col)[0].Length() > 0) ) {
          strings = new  char * [col->Length()+1]; 
	  *nValues = col->Length();
	  for (i = 0; i < col->Length(); i++) {
	    strings[i] = new char[(*col)[i].Length() + 1];
	    strcpy(strings[i],(*col)[i].Text());
	    ndb_log_message_text(NDB_MSG_DEBUG,"Attribute %s [%d] has value %s",attribute,i, strings[i]);
	  }
	}
      }
    }
  }
  return(strings);
}




void set_attribute_value_where(CifFileObj *fobj, const char *blockId, const char *category, 
			       const char *attributeA, const char *valA, 
			       const char *attributeB, const char *valB)
{
  ISTable *t=NULL;
  ReVarPCifArray<int>  *iResult=NULL;
  int i, nRow, iCol1, iCol2, errCode;
  CifString cs;


  ndb_log_message_text(NDB_MSG_DEBUG,"Setting  %s in %s to %s where %s = %s",
		       attributeA, category, valA, attributeB, valB);

  if ( !fobj  || !blockId || !category || !valA || !attributeA || !attributeB || !valB) return;

  t=fobj->GetTablePtr(blockId,category);

  if (t != NULL) {
    nRow = t->GetNumRows();
    ndb_log_message_text(NDB_MSG_DEBUG,"Got %d rows in %s",nRow,category);
    iCol1 = t->GetColumnIndex(attributeB);
    iCol2 = t->GetColumnIndex(attributeA);
    ndb_log_message_text(NDB_MSG_DEBUG,"iCol1 %d iCol2 %d",iCol1, iCol2);

    if (nRow > 0) {
#if 0
      if ( !t->isIndexed(iCol1) ) {     // make column searchable if required ... 
	ReVarCifArray<CifString> *col0 = t->GetColumn(iCol1);
	t->RemoveColumn(iCol1); t->InsertColumn(attributeB,iCol1,1);
	t->FillColumn(*col0,iCol1);
      }
#endif
      ndb_log_message_text(NDB_MSG_DEBUG,"iCol1 %d can be searched",iCol1);
      cs.Clear(); cs.Copy(valB);
      iResult = t->SearchColumn(cs, iCol1, errCode);
      if (!errCode && iResult && (iResult->Length() > 0)) {
	ndb_log_message_text(NDB_MSG_DEBUG,"Found %d values of %s matching %s",iResult->Length(),
			     attributeB, valB);
	CifString *csSet = new CifString(valA);
	for (i=0; i < iResult->Length(); i++) {
	  t->UpdateCell(*csSet, iCol2, (*iResult)[i]);
	  ndb_log_message_text(NDB_MSG_DEBUG,"Attribute %s in row %d column %d set to %s",
			       attributeA,(*iResult)[i],iCol2, csSet->Text());
	}
	fobj->WriteTable(t,blockId,category);
	  
      } else { // insert row . 
	ndb_log_message_text(NDB_MSG_DEBUG,"Search returned nothing - table length %d",t->GetNumRows());
	t->AddRow();
	int irow = t->GetNumRows() - 1;
	CifString *csSetA = new CifString(valA);
	CifString *csSetB = new CifString(valB);
	ndb_log_message_text(NDB_MSG_DEBUG,"Inserting row %d",irow);
	t->UpdateCell(*csSetA, iCol2, irow);
	t->UpdateCell(*csSetB, iCol1, irow);
	fobj->WriteTable(t,blockId,category);
      }
    } else { // insert row  .
      ndb_log_message_text(NDB_MSG_DEBUG,"Empty table");
      t->AddRow();
      int irow = t->GetNumRows() - 1;
      CifString *csSetA = new CifString(valA);
      CifString *csSetB = new CifString(valB);
      ndb_log_message_text(NDB_MSG_DEBUG,"Inserting row %d",irow);
      t->UpdateCell(*csSetA, iCol2, irow);
      t->UpdateCell(*csSetB, iCol1, irow);
      fobj->WriteTable(t,blockId,category);
    }
  }

}


char *get_attribute_value_where(CifFileObj *fobj, const char *blockId, const char *category, 
				const char *attributeA, const char *attributeB, const char *valB)
{
  ISTable *t=NULL;

  ReVarCifArray<CifString> *col=NULL;
  ReVarPCifArray<int>  *iResult=NULL;
  int nRow, iCol1, iCol2, errCode;
  char *string=NULL;
  CifString cs;

  ndb_log_message_text(NDB_MSG_DEBUG,"Searching for %s and %s where %s = %s",
		       category,attributeA, attributeB, valB);
  t=fobj->GetTablePtr(blockId,category);

  if (t != NULL) {
    nRow = t->GetNumRows();
    ndb_log_message_text(NDB_MSG_DEBUG,"Got %d rows in %s",nRow,category);
    if (nRow > 0) {
      iCol1 = t->GetColumnIndex(attributeB);
      iCol2 = t->GetColumnIndex(attributeA);
      ndb_log_message_text(NDB_MSG_DEBUG,"iCol1 %d iCol2 %d",iCol1, iCol2);
#if 0
      if ( !t->isIndexed(iCol1) ) {     // make column searchable if required ... 
	ReVarCifArray<CifString> *col0 = t->GetColumn(iCol1);
	t->RemoveColumn(iCol1); t->InsertColumn(attributeB,iCol1,1);
	t->FillColumn(*col0,iCol1);
      }
#endif
      ndb_log_message_text(NDB_MSG_DEBUG,"iCol1 %d can be searched",iCol1);
      cs.Clear(); cs.Copy(valB);
      iResult = t->SearchColumn(cs, iCol1, errCode);
      if (!errCode && iResult && (iResult->Length() > 0)) {
	ndb_log_message_text(NDB_MSG_DEBUG,"Found %d values of %s matching %s",iResult->Length(),
			     attributeB, valB);
	col = t->GetSubRow((*iResult)[0],iCol2, iCol2);
	string = new char[(*col)[0].Length() + 1];
	strcpy(string,(*col)[0].Text());
	ndb_log_message_text(NDB_MSG_DEBUG,"Attribute %s has value %s",attributeA,string);
      } else {
	ndb_log_message_text(NDB_MSG_DEBUG,"Search returned nothing");
      }
    }
  }
  return(string);
}



char **get_attribute_values_where(CifFileObj *fobj, const char *blockId, const char *category, 
				  const char *attributeA, const char *attributeB, const char *valB,
				  int *nValues)
{
  ISTable *t=NULL;

  ReVarCifArray<CifString> *col=NULL;
  ReVarPCifArray<int>  *iResult=NULL;
  int nRow, iCol1, iCol2, errCode;
  char **strings=NULL;
  CifString cs;

  ndb_log_message_text(NDB_MSG_DEBUG,"Searching for %s and %s where %s = %s",
		       category,attributeA, attributeB, valB);
  t=fobj->GetTablePtr(blockId,category);

  if (t != NULL) {
    nRow = t->GetNumRows();
    ndb_log_message_text(NDB_MSG_DEBUG,"Got %d rows in %s",nRow,category);
    if (nRow > 0) {
      iCol1 = t->GetColumnIndex(attributeB);
      iCol2 = t->GetColumnIndex(attributeA);
      ndb_log_message_text(NDB_MSG_DEBUG,"iCol1 %d iCol2 %d",iCol1, iCol2);
#if 0
      if ( !t->isIndexed(iCol1) ) {     // make column searchable if required ... 
	ReVarCifArray<CifString> *col0 = t->GetColumn(iCol1);
	t->RemoveColumn(iCol1); t->InsertColumn(attributeB,iCol1,1);
	t->FillColumn(*col0,iCol1);
      }
#endif
      ndb_log_message_text(NDB_MSG_DEBUG,"iCol1 %d can be searched",iCol1);
      cs.Clear(); cs.Copy(valB);
      iResult = t->SearchColumn(cs, iCol1, errCode);
      if (!errCode && iResult && (iResult->Length() > 0)) {
	ndb_log_message_text(NDB_MSG_DEBUG,"Found %d values of %s matching %s",iResult->Length(),
			     attributeB, valB);
	col = t->GetSubColumnByIndex(iCol2,iResult);
	//	col = t->GetSubRow((*iResult)[0],iCol2, iCol2);

	strings = new char *[ iResult->Length() + 1];
	for (int i=0;  i < iResult->Length(); i++) {
	  strings[i] = new char[(*col)[i].Length() + 1];
	  strcpy(strings[i],(*col)[i].Text());
	  ndb_log_message_text(NDB_MSG_DEBUG,"Attribute %s  %d  has value %s",attributeA,i,strings[i]);
	}
	*nValues = iResult->Length();
      } else {
	ndb_log_message_text(NDB_MSG_DEBUG,"Search returned nothing");
	*nValues = 0;
      }
    }
  }
  return(strings);
}




void Xupdate_attribute_value(CifFileObj *fobj, const char *blockId, const char *category, 
			    const char *attribute, const char *value)
{
  ISTable *t=NULL;
  CifString cs,us;
  int nRow, iCol;
  int i;

  ndb_log_message_text(NDB_MSG_DEBUG,"Updating block %s category %s item  %s with %s",
		       blockId,category,attribute,value);
  if ((blockId == NULL) || (category == NULL) || (attribute == NULL) || (value == NULL) ) return;
  
  //  if (fobj->IsTablePresent(blockId,category)) {
  if ((t=fobj->GetTablePtr(blockId,category)) != NULL) {
    nRow = t->GetNumRows();
    ndb_log_message_text(NDB_MSG_DEBUG,"Found %s with %d rows",category,nRow);
    if (nRow > 0) {
      iCol = t->GetColumnIndex(attribute);
      ndb_log_message_text(NDB_MSG_DEBUG,"Found %s at index %d",attribute,iCol);
      if (iCol >= 0) {
	us.Clear(); us.Copy(value);
	for (i=0; i< nRow; i++) {
	  cs.Clear();
	  t->GetCell(cs,iCol,i);
	  t->UpdateCell(us,iCol,i);
	  ndb_log_message_text(NDB_MSG_DEBUG,"At row %d - Replacing %s with %s",i,cs.Text(),us.Text());
	}
	//	fobj->WriteTable(t,blockId,category);
      }
    }
  }
}




void  update_entry_ids(CifFileObj *fobj, const char *blockId, const char *id)
{
    char idName[100];
    char *ndbID=NULL;
    char *pdbID=NULL;
    strncpy(idName,id,99);
    ndb_set_string_to_upper(idName);
    update_attribute_value(fobj,blockId,"entry","id",idName);    
    update_attribute_value(fobj,blockId,"struct_keywords","entry_id",idName);    

    update_attribute_value(fobj,blockId,"database","entry_id",idName);    
    update_attribute_value(fobj,blockId,"database_PDB_matrix","entry_id",idName);    

    update_attribute_value(fobj,blockId,"exptl","entry_id",idName);    

    update_attribute_value(fobj,blockId,"struct","entry_id",idName);    
    update_attribute_value(fobj,blockId,"ndb_na_struct_keywds","entry_id",idName);    
    update_attribute_value(fobj,blockId,"ndb_database_status","entry_id",idName);    
    update_attribute_value(fobj,blockId,"ndb_database_proc","entry_id",idName);    
    update_attribute_value(fobj,blockId,"ndb_coord","entry_id",idName);    
    update_attribute_value(fobj,blockId,"reflns","entry_id",idName);    
    update_attribute_value(fobj,blockId,"computing","entry_id",idName);    
    update_attribute_value(fobj,blockId,"symmetry","entry_id",idName);    
    update_attribute_value(fobj,blockId,"cell","entry_id",idName);    
    update_attribute_value(fobj,blockId,"refine","entry_id",idName);    
    update_attribute_value(fobj,blockId,"refine_analyze","entry_id",idName);    
    update_attribute_value(fobj,blockId,"ndb_refine","entry_id",idName);    
    update_attribute_value(fobj,blockId,"ndb_struct_conf_na","entry_id",idName);    
    // update_attribute_value(fobj,blockId,"struct_ref","db_code",idName);    
    // update_attribute_value(fobj,blockId,"struct_ref_seq","ndb_db_accession",idName);    

    // Added Nov 2 jdw...
    update_attribute_value(fobj,blockId,"atom_sites","entry_id",idName);    
    update_attribute_value(fobj,blockId,"ndb_database_message","entry_id",idName);    
    update_attribute_value(fobj,blockId,"rcsb_nmr_computing","entry_id",idName);    
    update_attribute_value(fobj,blockId,"rcsb_nmr_constraints","entry_id",idName);    
    update_attribute_value(fobj,blockId,"rcsb_nmr_details","entry_id",idName);    
    update_attribute_value(fobj,blockId,"rcsb_nmr_ensemble","entry_id",idName);    
    update_attribute_value(fobj,blockId,"rcsb_nmr_ensemble_rms","entry_id",idName);    
    update_attribute_value(fobj,blockId,"rcsb_nmr_refine","entry_id",idName);    
    update_attribute_value(fobj,blockId,"rcsb_nmr_representative","entry_id",idName);    
    update_attribute_value(fobj,blockId,"rcsb_post_process_details","entry_id",idName);    
    update_attribute_value(fobj,blockId,"rcsb_post_process_status","entry_id",idName);    

    // Removed Sep 1... All db codes set at deposition time... Do not resent here... jdw
    //    set_attribute_value_where(fobj, blockId, "database_2",
    //			      "database_code", idName, 
    //			      "database_id", "RCSB");

    // Make db codes consistent
    ndbID  = get_attribute_value_where(fobj, blockId, "database_2", "database_code", 
				      "database_id", "NDB");
    if (ndbID != NULL)     update_attribute_value(fobj,blockId,"database","ndb_code_NDB",ndbID);

    pdbID  = get_attribute_value_where(fobj, blockId, "database_2", "database_code", 
				      "database_id", "PDB");
    if (pdbID != NULL)     update_attribute_value(fobj,blockId,"database","ndb_code_PDB",pdbID);

#define DCDEMO 1
#if DCDEMO
    update_attribute_value(fobj,blockId,"rcsb_data_processing_cell","entry_id",idName);    
    update_attribute_value(fobj,blockId,"rcsb_data_processing_reflns","entry_id",idName);    
    update_attribute_value(fobj,blockId,"rcsb_data_processing_detector","entry_id",idName);    
#endif 

    // remove 
#if 0
    update_attribute_value(fobj,blockId,"database","ndb_code_NDB",idName);    
    set_attribute_value_where(fobj, blockId, "database_2", 
			      "database_code", idName, 
			      "database_id", "NDB");
#endif
    
}


void update_attribute_value(CifFileObj *fobj, const char *blockId, const char *category, 
			    const char *attribute, const char *value)
{
  ISTable *t=NULL;

  CifString cs;
  ReVarCifArray<CifString> uv;
  int nRow, iCol;
  int i;

  ndb_log_message_text(NDB_MSG_DEBUG,"Updating block %s category %s item  %s with %s",  
		       blockId,category,attribute,value);
  if ((blockId == NULL) || (category == NULL) || (attribute == NULL) || (value == NULL) ) return;

  cs.Clear(); cs.Copy(value);

  if ((t=fobj->GetTablePtr(blockId,category)) != NULL) {
    nRow = t->GetNumRows();
    ndb_log_message_text(NDB_MSG_DEBUG,"Found %s with %d rows",category,nRow);
    if (nRow > 0) {
      iCol = t->GetColumnIndex(attribute);
      ndb_log_message_text(NDB_MSG_DEBUG,"Found %s at index %d",attribute,iCol);
      if (iCol >= 0) {
        uv.Clear();
	for (i=0; i< nRow; i++) {
	  uv.Add(cs);
	}
	t->DeleteColumn(iCol);
	t->FillColumn(uv,iCol);
	fobj->WriteTable(t,blockId,category);
      }
    }
  }
}


void update_attribute_values(CifFileObj *fobj, const char *blockId, const char *category, 
			     const char *attribute, const char **values, int nValues)
  // Replaces a column ... **bookkeeping is responsibility of the application.
{
  ISTable *t=NULL;

  CifString cs;
  ReVarCifArray<CifString> uv;
  int nRow, iCol;
  int i;


  if ((blockId == NULL) || (category == NULL) || (attribute == NULL) || (values == NULL) ||
    (nValues <= 0) ) return;

  ndb_log_message_text(NDB_MSG_DEBUG,"Updating block %s category %s item  %s with %d values",  
		       blockId,category,attribute,nValues);

  if ((t=fobj->GetTablePtr(blockId,category)) != NULL) {
    nRow = t->GetNumRows();
    ndb_log_message_text(NDB_MSG_DEBUG,"Found %s with %d rows",category,nRow);
    if (nRow > 0) {
      iCol = t->GetColumnIndex(attribute);
      ndb_log_message_text(NDB_MSG_DEBUG,"Found %s at index %d",attribute,iCol);
      if (iCol >= 0) {
	uv.Clear();
	for (i=0; i < nValues; i++) {
	  cs.Clear(); cs.Copy(values[i]);
	  uv.Add(cs);
	}
	t->DeleteColumn(iCol);
	t->FillColumn(uv,iCol);
	fobj->WriteTable(t,blockId,category);
      }
    }
  }
}




void update_attribute_value_if_null(CifFileObj *fobj, const char *blockId, const char *category, 
				     const char *attribute, const char *value)
{
  ISTable *t=NULL;

  CifString cs;
  ReVarCifArray<CifString> uv;
  ReVarCifArray<CifString> *col=NULL;
  int nRow, iCol;
  int i;

  ndb_log_message_text(NDB_MSG_DEBUG,"Updating block %s category %s item  %s with %s",  
		       blockId,category,attribute,value);
  if ((blockId == NULL) || (category == NULL) || (attribute == NULL) || (value == NULL) ) return;

  cs.Clear(); cs.Copy(value);

  if ((t=fobj->GetTablePtr(blockId,category)) != NULL) {
    nRow = t->GetNumRows();
    ndb_log_message_text(NDB_MSG_DEBUG,"Found %s with %d rows",category,nRow);
    if (nRow > 0) {
      iCol = t->GetColumnIndex(attribute);
      ndb_log_message_text(NDB_MSG_DEBUG,"Found %s at index %d",attribute,iCol);
      if (iCol >= 0) {
	col = t->GetColumn(iCol);
	if (col == NULL || ((*col)[0].Length() == 0 ) || 
	    !strcmp((*col)[0].Text(),"?") || !strcmp((*col)[0].Text(),".") ) {
	  uv.Clear();
	  for (i=0; i< nRow; i++) {
	    uv.Add(cs);
	  }
	  t->DeleteColumn(iCol);
	  t->FillColumn(uv,iCol);
	  fobj->WriteTable(t,blockId,category);
	}
      }
    }
  }
}


int is_attribute_value_defined(CifFileObj *fobj, const char *blockId, const char *category, 
			       const char *attribute)
{
  ISTable *t=NULL;
  
  ReVarCifArray<CifString> *col=NULL;
  int nRow, iCol;

  ndb_log_message_text(NDB_MSG_DEBUG,"Checking block %s category %s item  %s",  
		       blockId,category,attribute);
  if ((blockId == NULL) || (category == NULL) || (attribute == NULL) ) return(0);
  
  if ((t=fobj->GetTablePtr(blockId,category)) != NULL) {
    nRow = t->GetNumRows();
    ndb_log_message_text(NDB_MSG_DEBUG,"Found %s with %d rows",category,nRow);
    if (nRow > 0) {
      iCol = t->GetColumnIndex(attribute);
      ndb_log_message_text(NDB_MSG_DEBUG,"Found %s at index %d",attribute,iCol);
      if (iCol >= 0) {
	col = t->GetColumn(iCol);
	if (col == NULL || ((*col)[0].Length() == 0 ) || 
	    !strcmp((*col)[0].Text(),"?") || !strcmp((*col)[0].Text(),".") ) {
	  return(0);
	} else 
	  return(1);
      }
    } else 
      return(0);
    
  } else 
    return(0);

  return(0);
}

#if 0
void  set_deposit_db_status(CifFileObj *fobj, const char *blockId) {
  time_t curr_time;
  struct tm *tmptr;
  char *curt;
  char timeString[31];

  time(&curr_time);
  tmptr = localtime(&curr_time);
  curt = ctime(&curr_time);
  ndb_clean_string(curt);
  if (curt[strlen(curt)-1] == '\n') curt[strlen(curt)-1] = '\0';
  strftime(timeString,30,"%Y-%m-%d", tmptr);

  update_attribute_value_if_null(fobj,blockId,"ndb_database_status","deposit_site","RCSB");    
  update_attribute_value_if_null(fobj,blockId,"ndb_database_status","process_site","RCSB");    

  update_attribute_value_if_null(fobj,blockId,"ndb_database_status","date_submitted",timeString);    
  update_attribute_value_if_null(fobj,blockId,"ndb_database_status","recvd_initial_deposition_date",timeString);    

  update_attribute_value_if_null(fobj,blockId,"ndb_database_status","status_code","PROC");    
  update_attribute_value_if_null(fobj,blockId,"ndb_database_status","recvd_manuscript","N");    

  if (iDF) {
    update_attribute_value_if_null(fobj,blockId,"ndb_database_status","date_deposition_form",timeString);    
    update_attribute_value_if_null(fobj,blockId,"ndb_database_status","recvd_deposit_form","Y");    
  } else {
    update_attribute_value_if_null(fobj,blockId,"ndb_database_status","recvd_deposit_form","N");    
  }

  if (iNC) {
    update_attribute_value_if_null(fobj,blockId,"ndb_database_status","date_nmr_constraints",timeString);    
    update_attribute_value_if_null(fobj,blockId,"ndb_database_status","recvd_nmr_constraints","Y");    
    update_attribute_value_if_null(fobj,blockId,"ndb_database_status","dep_release_code_nmr_constraints","RELEASE NOW");
  } else {
    update_attribute_value_if_null(fobj,blockId,"ndb_database_status","recvd_nmr_constraints","N");    
  }

  if (iXYZ) {
    update_attribute_value_if_null(fobj,blockId,"ndb_database_status","dep_release_code_coordinates","RELEASE NOW");    
    update_attribute_value_if_null(fobj,blockId,"ndb_database_status","date_coordinates",timeString);    
    update_attribute_value_if_null(fobj,blockId,"ndb_database_status","recvd_coordinates","Y");    
    update_attribute_value_if_null(fobj,blockId,"ndb_database_status","status_coordinates_in_NDB","Y");    
  } else {
    update_attribute_value_if_null(fobj,blockId,"ndb_database_status","recvd_coordinates","N");    
    update_attribute_value_if_null(fobj,blockId,"ndb_database_status","status_coordinates_in_NDB","N");    
  }
  if (iSF) {
    update_attribute_value_if_null(fobj,blockId,"ndb_database_status","dep_release_code_struct_fact","RELEASE NOW");    
    update_attribute_value_if_null(fobj,blockId,"ndb_database_status","date_struct_fact",timeString);    
    update_attribute_value_if_null(fobj,blockId,"ndb_database_status","recvd_struct_fact","Y");    
  } else {
    update_attribute_value_if_null(fobj,blockId,"ndb_database_status","recvd_struct_fact","N");    
  }
}

void  Oset_initial_db_status(CifFileObj *fobj, const char *blockId,  int iXYZ, int iSF, int iDF, int iNC) {
  time_t curr_time;
  struct tm *tmptr;
  char *curt;
  char timeString[31];

  time(&curr_time);
  tmptr = localtime(&curr_time);
  curt = ctime(&curr_time);
  ndb_clean_string(curt);
  if (curt[strlen(curt)-1] == '\n') curt[strlen(curt)-1] = '\0';
  strftime(timeString,30,"%Y-%m-%d", tmptr);

  update_attribute_value(fobj,blockId,"ndb_database_status","deposit_site","RCSB");    
  update_attribute_value(fobj,blockId,"ndb_database_status","process_site","RCSB");    

  update_attribute_value(fobj,blockId,"ndb_database_status","date_submitted",timeString);    
  update_attribute_value(fobj,blockId,"ndb_database_status","recvd_initial_deposition_date",timeString);    

  update_attribute_value(fobj,blockId,"ndb_database_status","status_code","PROC");    
  update_attribute_value(fobj,blockId,"ndb_database_status","recvd_manuscript","N");    

  if (iDF) {
    update_attribute_value(fobj,blockId,"ndb_database_status","date_deposition_form",timeString);    
    update_attribute_value(fobj,blockId,"ndb_database_status","recvd_deposit_form","Y");    
  } else {
    update_attribute_value(fobj,blockId,"ndb_database_status","recvd_deposit_form","N");    
  }

  if (iNC) {
    update_attribute_value(fobj,blockId,"ndb_database_status","date_nmr_constraints",timeString);    
    update_attribute_value(fobj,blockId,"ndb_database_status","recvd_nmr_constraints","Y");    
  } else {
    update_attribute_value(fobj,blockId,"ndb_database_status","recvd_nmr_constraints","N");    
  }

  if (iXYZ) {
    update_attribute_value(fobj,blockId,"ndb_database_status","date_coordinates",timeString);    
    update_attribute_value(fobj,blockId,"ndb_database_status","recvd_coordinates","Y");    
    update_attribute_value(fobj,blockId,"ndb_database_status","status_coordinates_in_NDB","Y");    
  } else {
    update_attribute_value(fobj,blockId,"ndb_database_status","recvd_coordinates","N");    
    update_attribute_value(fobj,blockId,"ndb_database_status","status_coordinates_in_NDB","N");    
  }
  if (iSF) {
    update_attribute_value(fobj,blockId,"ndb_database_status","date_struct_fact",timeString);    
    update_attribute_value(fobj,blockId,"ndb_database_status","recvd_struct_fact","Y");    
  } else {
    update_attribute_value(fobj,blockId,"ndb_database_status","recvd_struct_fact","N");    
  }
}
#endif
