/*
FILE:     start-session.C
*/
/*
VERSION:  0.904-beta
*/
/*
DATE:     8/14/2003
*/
/*
  Comments and Questions to: sw-help@rcsb.rutgers.edu
*/
/*
COPYRIGHT 1999-2003 Rutgers - The State University of New Jersey

This software is provided WITHOUT WARRANTY OF MERCHANTABILITY OR
FITNESS FOR A PARTICULAR PURPOSE OR ANY OTHER WARRANTY, EXPRESS OR
IMPLIED.  RUTGERS MAKE NO REPRESENTATION OR WARRANTY THAT THE
SOFTWARE WILL NOT INFRINGE ANY PATENT, COPYRIGHT OR OTHER
PROPRIETARY RIGHT.

The user of this software shall indemnify, hold harmless and defend
Rutgers, its governors, trustees, officers, employees, students,
agents and the authors against any and all claims, suits,
losses, liabilities, damages, costs, fees, and expenses including
reasonable attorneys' fees resulting from or arising out of the
use of this software.  This indemnification shall include, but is
not limited to, any and all claims alleging products liability.
*/
/*
               PDB SOFTWARE LICENSE AGREEMENT

BY CLICKING THE ACCEPTANCE BUTTON OR INSTALLING OR USING 
THIS "SOFTWARE, THE INDIVIDUAL OR ENTITY LICENSING THE  
SOFTWARE ("LICENSEE") IS CONSENTING TO BE BOUND BY AND IS 
BECOMING A PARTY TO THIS AGREEMENT.  IF LICENSEE DOES NOT 
AGREE TO ALL OF THE TERMS OF THIS AGREEMENT
THE LICENSEE MUST NOT INSTALL OR USE THE SOFTWARE.

1. LICENSE AGREEMENT

This is a license between you ("Licensee") and the Protein Data Bank (PDB) 
at Rutgers, The State University of New Jersey (hereafter referred to 
as "RUTGERS").   The software is owned by RUTGERS and protected by 
copyright laws, and some elements are protected by laws governing 
trademarks, trade dress and trade secrets, and may be protected by 
patent laws. 

2. LICENSE GRANT

RUTGERS grants you, and you hereby accept, non-exclusive, royalty-free 
perpetual license to install, use, modify, prepare derivative works, 
incorporate into other computer software, and distribute in binary 
and source code format, or any derivative work thereof, together with 
any associated media, printed materials, and on-line or electronic 
documentation (if any) provided by RUTGERS (collectively, the "SOFTWARE"), 
subject to the following terms and conditions: (i) any distribution 
of the SOFTWARE shall bind the receiver to the terms and conditions 
of this Agreement; (ii) any distribution of the SOFTWARE in modified 
form shall clearly state that the SOFTWARE has been modified from 
the version originally obtained from RUTGERS.  

2. COPYRIGHT; RETENTION OF RIGHTS.  

The above license grant is conditioned on the following: (i) you must 
reproduce all copyright notices and other proprietary notices on any 
copies of the SOFTWARE and you must not remove such notices; (ii) in 
the event you compile the SOFTWARE, you will include the copyright 
notice with the binary in such a manner as to allow it to be easily 
viewable; (iii) if you incorporate the SOFTWARE into other code, you 
must provide notice that the code contains the SOFTWARE and include 
a copy of the copyright notices and other proprietary notices.  All 
copies of the SOFTWARE shall be subject to the terms of this Agreement.  

3. NO MAINTENANCE OR SUPPORT; TREATMENT OF ENHANCEMENTS 

RUTGERS is under no obligation whatsoever to: (i) provide maintenance 
or support for the SOFTWARE; or (ii) to notify you of bug fixes, patches, 
or upgrades to the features, functionality or performance of the 
SOFTWARE ("Enhancements") (if any), whether developed by RUTGERS 
or third parties.  If, in its sole discretion, RUTGERS makes an 
Enhancement available to you and RUTGERS does not separately enter 
into a written license agreement with you relating to such bug fix, 
patch or upgrade, then it shall be deemed incorporated into the SOFTWARE 
and subject to this Agreement. You are under no obligation whatsoever 
to provide any Enhancements to RUTGERS or the public that you may 
develop over time; however, if you choose to provide your Enhancements 
to RUTGERS, or if you choose to otherwise publish or distribute your 
Enhancements, in source code form without contemporaneously requiring 
end users or RUTGERS to enter into a separate written license agreement 
for such Enhancements, then you hereby grant RUTGERS a non-exclusive,
royalty-free perpetual license to install, use, modify, prepare
derivative works, incorporate into the SOFTWARE or other computer
software, distribute, and sublicense your Enhancements or derivative
works thereof, in binary and source code form.

4. FEES.  There is no license fee for the SOFTWARE.  If Licensee
wishes to receive the SOFTWARE on media, there may be a small charge
for the media and for shipping and handling.  Licensee is
responsible for any and all taxes.

5. TERMINATION.  Without prejudice to any other rights, Licensor
may terminate this Agreement if Licensee breaches any of its terms
and conditions.  Upon termination, Licensee shall destroy all
copies of the SOFTWARE.

6. PROPRIETARY RIGHTS.  Title, ownership rights, and intellectual
property rights in the Product shall remain with RUTGERS.  Licensee 
acknowledges such ownership and intellectual property rights and will 
not take any action to jeopardize, limit or interfere in any manner 
with RUTGERS' ownership of or rights with respect to the SOFTWARE.  
The SOFTWARE is protected by copyright and other intellectual 
property laws and by international treaties.  Title and related 
rights in the content accessed through the SOFTWARE is the property 
of the applicable content owner and is protected by applicable law.  
The license granted under this Agreement gives Licensee no rights to such
content.

7. DISCLAIMER OF WARRANTY.  THE SOFTWARE IS PROVIDED FREE OF 
CHARGE, AND, THEREFORE, ON AN "AS IS" BASIS, WITHOUT WARRANTY OF 
ANY KIND, INCLUDING WITHOUT LIMITATION THE WARRANTIES THAT IT 
IS FREE OF DEFECTS, MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE 
OR NON-INFRINGING.  THE ENTIRE RISK AS TO THE QUALITY AND 
PERFORMANCE OF THE SOFTWARE IS BORNE BY LICENSEE.  SHOULD THE 
SOFTWARE PROVE DEFECTIVE IN ANY RESPECT, THE LICENSEE AND NOT 
LICENSOR ASSUMES THE ENTIRE COST OF ANY SERVICE AND REPAIR.  
THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF 
THIS AGREEMENT.  NO USE OF THE PRODUCT IS AUTHORIZED HEREUNDER 
EXCEPT UNDER THIS DISCLAIMER.

8. LIMITATION OF LIABILITY.  TO THE MAXIMUM EXTENT PERMITTED BY
APPLICABLE LAW,  IN NO EVENT WILL LICENSOR BE LIABLE FOR ANY 
INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
OUT OF THE USE OF OR INABILITY TO USE THE SOFTWARE, INCLUDING, 
WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK 
STOPPAGE, COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL 
OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF ADVISED OF THE
POSSIBILITY THEREOF. 
*/
/*
  PURPOSE:      Start or restart a session.
  
                If starting a new session:

                This module creates a unique identifier, makes a temporary
                directory for the session, imports a view to the temporary
                area and calls mk-top-interface-screen to build the 
                interface screen.

                start-session?view=<view_file_name>
                              ufid=<unique identifier>
			      context=<annotate|deposit|bmcd|assemble>

                If restarting,  only <ufid> need be specified to restart 
                an existing session.

*/    


#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "ndb_htmllib.h"
#include "ndb_misclib.h"

#include "CifFileObj.h"
#include "CifViewObj.h"
#include "CifStateObj.h"


#define HTML_ENV_GLOBAL_INIT  
#include "interface-defs.h"
#include "view-obj-util.h"

int main(int argc, char *argv[]) {
    entry entries[HT_MAX_ENTRIES];
    int  nentries;
    char  script[2048];
    char *prog_name, *uname=NULL, *viewFile=NULL;

    char dirPath[HT_MAX_NAME_LENGTH];
    char tmpDirPath[HT_MAX_NAME_LENGTH];

    char dirPath1[HT_MAX_NAME_LENGTH];
    char tmpDirPath1[HT_MAX_NAME_LENGTH];
    char iViewPath[HT_MAX_NAME_LENGTH];
    char fobjPath[HT_MAX_NAME_LENGTH];
    char vobjPath[HT_MAX_NAME_LENGTH];
    CifStateObj * sobj = NULL;
    char *stateString = NULL;
    char statePath[HT_MAX_NAME_LENGTH];
    //
    char category[CIF_MAX_NAME_LENGTH];
    char categoryView[CIF_MAX_NAME_LENGTH];
    char blockId[CIF_MAX_NAME_LENGTH];
    int iRestart=1;
    char *projDir=NULL, *dbname=NULL;
    char *expmeth=NULL, *moltype=NULL, *context=NULL;
    char *uname1=NULL;

    prog_name = argv[0];
    nentries  = ht_decode_get_query(entries);
    ht_set_httpd_environment(entries,nentries);

    ht_set_server_url(HT_SERVER_NAME);
    ht_set_server_cgi_path(HT_CGI_PATH);

    ndb_open_log(prog_name,HT_LOGFILE_NAME);
    ndb_set_debug_level(NDB_GLOBAL_DEBUG_LEVEL);
    ndb_log_message(NDB_MSG_INFO,"Starting session ...");

    viewFile  = ht_get_keyword_value(HT_ATTRIBUTE_VIEW_ID,entries,nentries);
    uname     = ht_get_keyword_value(HT_ATTRIBUTE_UNIQUE_ID,entries,nentries);
    expmeth   = ht_get_keyword_value(HT_ATTRIBUTE_EXP_METH,entries,nentries);
    moltype   = ht_get_keyword_value(HT_ATTRIBUTE_MOL_TYPE,entries,nentries);
    context   = ht_get_keyword_value(HT_ATTRIBUTE_CONTEXT,entries,nentries);
  
    if (uname == NULL || strlen(uname) ==0) {
      uname = ht_get_unique_name();
      //      iRestart= 0;
    } 

    ndb_log_message_text(NDB_MSG_INFO,"Starting session %s",uname);

    strcpy(dirPath,HT_SESSION_ROOT_PATH);
    strcat(dirPath,uname);
    strcat(dirPath,"/");

    strcpy(tmpDirPath,HT_SESSION_TMP_ROOT_PATH);
    strcat(tmpDirPath,uname);
    strcat(tmpDirPath,"/");

    ndb_log_message_text(NDB_MSG_INFO,"Using session directory %s",dirPath);
    ndb_log_message_text(NDB_MSG_INFO,"Using session temporary directory %s",tmpDirPath);

    if (!viewFile) {
      viewFile = ht_get_view_id(HT_MOL_TYPE, HT_EXP_METHOD, HT_VIEW_TYPE);
    }
    ndb_log_message_text(NDB_MSG_INFO,"Got viewid %s",viewFile);

    if (!iRestart) {
      mkdir(dirPath,0770);
      mkdir(tmpDirPath,0770);

      //
      //  Read the input view and construct the view object.  Initialize the table store
      //  and store the view object.
      //

      strcpy(vobjPath,dirPath);
      strcat(vobjPath,HT_VIEW_OBJECT_FILE);
      
      strcpy(iViewPath,HT_CONFIG_PATH);
      strcat(iViewPath,viewFile);
      ndb_log_message_text(NDB_MSG_DEBUG,"Opening view object store %s",vobjPath);
      ndb_log_message_text(NDB_MSG_DEBUG,"Getting view from CIF file %s",iViewPath);
      
      CifViewObj *vobj = new CifViewObj(iViewPath,vobjPath,HT_VIEW_NAME);
      ndb_log_message_text(NDB_MSG_DEBUG,"Diagnostics reading view %s",
			   vobj->GetViewDiagnostics());
      
      strcpy(fobjPath,dirPath);
      strcat(fobjPath,HT_TABLE_OBJECT_FILE);
      CifFileObj *fobj = new CifFileObj(fobjPath,WRITE_MODE);
      
      strcpy(blockId,NDB_DEFAULT_BLOCK_ID);

      sync_categories_to_view(vobj, fobj, blockId);

      strcpy(statePath,dirPath);
      strcat(statePath,HT_STATE_FILE);
      stateString=set_category_state_string(vobj, fobj, blockId);


      sobj = new CifStateObj(statePath, "state", WRITE_MODE, 0);
      sobj->SetSessionStateString(stateString);
      sobj->SetSessionId(uname);
      sobj->SetSessionExpMethod(HT_EXP_METHOD);
      sobj->SetSessionMolType(HT_MOL_TYPE);
      sobj->SetSessionOperation("edit");
      sobj->SetSessionRestartable("true");
      sobj->Close();
      fobj->CloseFile();
      vobj->Close();

    } else if (iRestart && HT_APPL_CONTEXT_DEMO)  { // demo mode - make copy of the restart area...

      ndb_log_message_text(NDB_MSG_INFO,"Duplicating demo session %s",uname);

      strcpy(dirPath1,dirPath);
      strcpy(tmpDirPath1,tmpDirPath);
      if (html_advise_missing_directory(stdout, dirPath1, HT_HOME_PATH)) exit(0);

      uname1 = ht_get_unique_name();
      strcpy(dirPath,HT_SESSION_ROOT_PATH);
      strcat(dirPath,uname1);
      strcat(dirPath,"/");
      
      strcpy(tmpDirPath,HT_SESSION_TMP_ROOT_PATH);
      strcat(tmpDirPath,uname1);
      strcat(tmpDirPath,"/");
      ndb_log_message_text(NDB_MSG_DEBUG,"New demo session is %s",uname1);

      mkdir(dirPath,0770);
      mkdir(tmpDirPath,0770);
      sprintf(script,"cd %s; cp * %s; ", dirPath1, dirPath);
      ndb_log_message_text(NDB_MSG_DEBUG,"Copy script: %s",script);
      wsystem(script);
      if (uname) delete uname;
      uname = new char[strlen(uname1) + 1];
      strcpy(uname,uname1);

      if (HT_APPL_CONTEXT_TYPE == HT_CONTEXT_DEPOSIT) {
	html_display_redeposit_block_message(stdout, uname, dirPath, RCSB_STYLE_DEFAULT);
      }

      strcpy(statePath,dirPath);
      strcat(statePath,HT_STATE_FILE);

      sobj = new CifStateObj(statePath, "state", APPEND_MODE, 0);
      if (sobj && !expmeth) {
	expmeth=sobj->GetSessionExpMethod();
	ndb_log_message_text(NDB_MSG_DEBUG,"Got expmeth from SOBJ: %s",expmeth);
	strcpy(HT_EXP_METHOD,expmeth);
      }
      if (sobj && !moltype) {
	moltype=sobj->GetSessionMolType();
	ndb_log_message_text(NDB_MSG_DEBUG,"Got moltype from SOBJ: %s",moltype);
	strcpy(HT_MOL_TYPE,moltype);
      }
      //
      // Project directory depends on: moltype and expmeth
      //
      projDir = ht_get_projdir(moltype,expmeth);
      if (projDir) strcpy(HT_ARCHIVE_DIRECTORY,projDir);
      
      //
      // DB also depends on: moltype and expmeth
      //
      if (!dbname) {
	dbname = ht_get_dbname(moltype,expmeth);
	// set the global database name ... 
	if (dbname) strcpy(HT_TARGET_DBNAME,dbname);
      }
      
      ndb_log_message_text(NDB_MSG_DEBUG,"Got uname   : %s",uname);
      ndb_log_message_text(NDB_MSG_DEBUG,"Got context : %s",context);
      ndb_log_message_text(NDB_MSG_DEBUG,"Got projDir : %s",projDir);
      ndb_log_message_text(NDB_MSG_DEBUG,"Got dbname  : %s",dbname);
      ndb_log_message_text(NDB_MSG_DEBUG,"Got expmeth : %s",expmeth);
      ndb_log_message_text(NDB_MSG_DEBUG,"Got moltype : %s",moltype);
      
      get_current_view_address(dirPath,blockId,category,categoryView);      
      ndb_log_message_text(NDB_MSG_DEBUG,"Restarting blockId %s category %s categoryView %s",
			   blockId, category, categoryView);

    } else { // pick up where we left off if restarting...
      if (html_advise_missing_directory(stdout, dirPath, HT_HOME_PATH)) exit(0);
      if (!is_directory(tmpDirPath))       mkdir(tmpDirPath,0770);

      //
      //jdw 12-apr-99 block restart of deposit sessions if deposited.
      //
      if (HT_APPL_CONTEXT_TYPE == HT_CONTEXT_DEPOSIT) {
	html_display_redeposit_block_message(stdout, uname, dirPath, RCSB_STYLE_DEFAULT);
      }

      strcpy(statePath,dirPath);
      strcat(statePath,HT_STATE_FILE);

      sobj = new CifStateObj(statePath, "state", APPEND_MODE, 0);
      if (sobj && !expmeth) {
	expmeth=sobj->GetSessionExpMethod();
	ndb_log_message_text(NDB_MSG_DEBUG,"Got expmeth from SOBJ: %s",expmeth);
	strcpy(HT_EXP_METHOD,expmeth);
      }
      if (sobj && !moltype) {
	moltype=sobj->GetSessionMolType();
	ndb_log_message_text(NDB_MSG_DEBUG,"Got moltype from SOBJ: %s",moltype);
	strcpy(HT_MOL_TYPE,moltype);
      }
      //
      // Project directory depends on: moltype and expmeth
      //
      projDir = ht_get_projdir(moltype,expmeth);
      if (projDir) strcpy(HT_ARCHIVE_DIRECTORY,projDir);
      
      //
      // DB also depends on: moltype and expmeth
      //
      if (!dbname) {
	dbname = ht_get_dbname(moltype,expmeth);
	// set the global database name ... 
	if (dbname) strcpy(HT_TARGET_DBNAME,dbname);
      }
      
      ndb_log_message_text(NDB_MSG_DEBUG,"Got uname   : %s",uname);
      ndb_log_message_text(NDB_MSG_DEBUG,"Got context : %s",context);
      ndb_log_message_text(NDB_MSG_DEBUG,"Got projDir : %s",projDir);
      ndb_log_message_text(NDB_MSG_DEBUG,"Got dbname  : %s",dbname);
      ndb_log_message_text(NDB_MSG_DEBUG,"Got expmeth : %s",expmeth);
      ndb_log_message_text(NDB_MSG_DEBUG,"Got moltype : %s",moltype);
      

      get_current_view_address(dirPath,blockId,category,categoryView);      
      ndb_log_message_text(NDB_MSG_DEBUG,"Restarting blockId %s category %s categoryView %s",
			   blockId, category, categoryView);
    }

    strcpy(script,"");
    sprintf(script,"%s?%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s",
            HT_INTERFACE_TOP_MENU_VIEW,
            HT_ATTRIBUTE_UNIQUE_ID,
            uname,
	    HT_ATTRIBUTE_DATABLOCK_ID,
	    blockId,
            HT_ATTRIBUTE_CONTEXT,
	    HT_APPL_CONTEXT,
	    HT_ATTRIBUTE_DBNAME,
	    HT_TARGET_DBNAME,
	    HT_ATTRIBUTE_EMAIL,
	    HT_ANNOTATOR_EMAIL,
	    HT_ATTRIBUTE_PROJDIR,
	    HT_ARCHIVE_DIRECTORY,
	    HT_ATTRIBUTE_MOL_TYPE, 
	    HT_MOL_TYPE,
	    HT_ATTRIBUTE_VIEW_TYPE, 
	    HT_VIEW_TYPE,
	    HT_ATTRIBUTE_EXP_METH, 
	    HT_EXP_METHOD);

    ndb_log_message(NDB_MSG_INFO,"Calling interface screen with: %s",script);
    ht_put_location_type_script(stdout,script);

    //    if (vobj) delete vobj;
    //    if (fobj) delete fobj;
    ndb_close_log();
    exit(0);
}








