/*
FILE:     view-obj-util.C
*/
/*
VERSION:  0.904-beta
*/
/*
DATE:     8/14/2003
*/
/*
  Comments and Questions to: sw-help@rcsb.rutgers.edu
*/
/*
COPYRIGHT 1999-2003 Rutgers - The State University of New Jersey

This software is provided WITHOUT WARRANTY OF MERCHANTABILITY OR
FITNESS FOR A PARTICULAR PURPOSE OR ANY OTHER WARRANTY, EXPRESS OR
IMPLIED.  RUTGERS MAKE NO REPRESENTATION OR WARRANTY THAT THE
SOFTWARE WILL NOT INFRINGE ANY PATENT, COPYRIGHT OR OTHER
PROPRIETARY RIGHT.

The user of this software shall indemnify, hold harmless and defend
Rutgers, its governors, trustees, officers, employees, students,
agents and the authors against any and all claims, suits,
losses, liabilities, damages, costs, fees, and expenses including
reasonable attorneys' fees resulting from or arising out of the
use of this software.  This indemnification shall include, but is
not limited to, any and all claims alleging products liability.
*/
/*
               PDB SOFTWARE LICENSE AGREEMENT

BY CLICKING THE ACCEPTANCE BUTTON OR INSTALLING OR USING 
THIS "SOFTWARE, THE INDIVIDUAL OR ENTITY LICENSING THE  
SOFTWARE ("LICENSEE") IS CONSENTING TO BE BOUND BY AND IS 
BECOMING A PARTY TO THIS AGREEMENT.  IF LICENSEE DOES NOT 
AGREE TO ALL OF THE TERMS OF THIS AGREEMENT
THE LICENSEE MUST NOT INSTALL OR USE THE SOFTWARE.

1. LICENSE AGREEMENT

This is a license between you ("Licensee") and the Protein Data Bank (PDB) 
at Rutgers, The State University of New Jersey (hereafter referred to 
as "RUTGERS").   The software is owned by RUTGERS and protected by 
copyright laws, and some elements are protected by laws governing 
trademarks, trade dress and trade secrets, and may be protected by 
patent laws. 

2. LICENSE GRANT

RUTGERS grants you, and you hereby accept, non-exclusive, royalty-free 
perpetual license to install, use, modify, prepare derivative works, 
incorporate into other computer software, and distribute in binary 
and source code format, or any derivative work thereof, together with 
any associated media, printed materials, and on-line or electronic 
documentation (if any) provided by RUTGERS (collectively, the "SOFTWARE"), 
subject to the following terms and conditions: (i) any distribution 
of the SOFTWARE shall bind the receiver to the terms and conditions 
of this Agreement; (ii) any distribution of the SOFTWARE in modified 
form shall clearly state that the SOFTWARE has been modified from 
the version originally obtained from RUTGERS.  

2. COPYRIGHT; RETENTION OF RIGHTS.  

The above license grant is conditioned on the following: (i) you must 
reproduce all copyright notices and other proprietary notices on any 
copies of the SOFTWARE and you must not remove such notices; (ii) in 
the event you compile the SOFTWARE, you will include the copyright 
notice with the binary in such a manner as to allow it to be easily 
viewable; (iii) if you incorporate the SOFTWARE into other code, you 
must provide notice that the code contains the SOFTWARE and include 
a copy of the copyright notices and other proprietary notices.  All 
copies of the SOFTWARE shall be subject to the terms of this Agreement.  

3. NO MAINTENANCE OR SUPPORT; TREATMENT OF ENHANCEMENTS 

RUTGERS is under no obligation whatsoever to: (i) provide maintenance 
or support for the SOFTWARE; or (ii) to notify you of bug fixes, patches, 
or upgrades to the features, functionality or performance of the 
SOFTWARE ("Enhancements") (if any), whether developed by RUTGERS 
or third parties.  If, in its sole discretion, RUTGERS makes an 
Enhancement available to you and RUTGERS does not separately enter 
into a written license agreement with you relating to such bug fix, 
patch or upgrade, then it shall be deemed incorporated into the SOFTWARE 
and subject to this Agreement. You are under no obligation whatsoever 
to provide any Enhancements to RUTGERS or the public that you may 
develop over time; however, if you choose to provide your Enhancements 
to RUTGERS, or if you choose to otherwise publish or distribute your 
Enhancements, in source code form without contemporaneously requiring 
end users or RUTGERS to enter into a separate written license agreement 
for such Enhancements, then you hereby grant RUTGERS a non-exclusive,
royalty-free perpetual license to install, use, modify, prepare
derivative works, incorporate into the SOFTWARE or other computer
software, distribute, and sublicense your Enhancements or derivative
works thereof, in binary and source code form.

4. FEES.  There is no license fee for the SOFTWARE.  If Licensee
wishes to receive the SOFTWARE on media, there may be a small charge
for the media and for shipping and handling.  Licensee is
responsible for any and all taxes.

5. TERMINATION.  Without prejudice to any other rights, Licensor
may terminate this Agreement if Licensee breaches any of its terms
and conditions.  Upon termination, Licensee shall destroy all
copies of the SOFTWARE.

6. PROPRIETARY RIGHTS.  Title, ownership rights, and intellectual
property rights in the Product shall remain with RUTGERS.  Licensee 
acknowledges such ownership and intellectual property rights and will 
not take any action to jeopardize, limit or interfere in any manner 
with RUTGERS' ownership of or rights with respect to the SOFTWARE.  
The SOFTWARE is protected by copyright and other intellectual 
property laws and by international treaties.  Title and related 
rights in the content accessed through the SOFTWARE is the property 
of the applicable content owner and is protected by applicable law.  
The license granted under this Agreement gives Licensee no rights to such
content.

7. DISCLAIMER OF WARRANTY.  THE SOFTWARE IS PROVIDED FREE OF 
CHARGE, AND, THEREFORE, ON AN "AS IS" BASIS, WITHOUT WARRANTY OF 
ANY KIND, INCLUDING WITHOUT LIMITATION THE WARRANTIES THAT IT 
IS FREE OF DEFECTS, MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE 
OR NON-INFRINGING.  THE ENTIRE RISK AS TO THE QUALITY AND 
PERFORMANCE OF THE SOFTWARE IS BORNE BY LICENSEE.  SHOULD THE 
SOFTWARE PROVE DEFECTIVE IN ANY RESPECT, THE LICENSEE AND NOT 
LICENSOR ASSUMES THE ENTIRE COST OF ANY SERVICE AND REPAIR.  
THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF 
THIS AGREEMENT.  NO USE OF THE PRODUCT IS AUTHORIZED HEREUNDER 
EXCEPT UNDER THIS DISCLAIMER.

8. LIMITATION OF LIABILITY.  TO THE MAXIMUM EXTENT PERMITTED BY
APPLICABLE LAW,  IN NO EVENT WILL LICENSOR BE LIABLE FOR ANY 
INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
OUT OF THE USE OF OR INABILITY TO USE THE SOFTWARE, INCLUDING, 
WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK 
STOPPAGE, COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL 
OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF ADVISED OF THE
POSSIBILITY THEREOF. 
*/
#include <sys/types.h>
#include <sys/stat.h>


#include "view-obj-util.h"
#include "CifSchemaMap.h"
#include "regex.h"
#include "interface-lang.h"

#if DCDEMO
static void adit_merge_data_collection_data(CifFileObj *fobj, const char *blockId, const char *dirPath);
#endif

static void set_sequence_prerelease_status(CifFileObj *fobj, const char *blockId);
static void update_source_data(CifFileObj *fobj, const char *blockId);
static void compute_hold_date_string(char hold_date[31], const char *hold_code);

static void  fraction_to_percent(CifFileObj *fobj, const char *blockId, 
				 const char *category, const char *attribute);

static int reformat_summary_letter(const char *path, const char *id, const char *sourceLetter,
				   const char *idName, CifFileObj *fobj, const char *blockId);


//static void formatStringData(FILE *fp, const char *cs, int maxWidth);
static void formatStringData2(FILE *fp, char *text, int maxWidth);

static char vlet_header_en[]="\
Dear Depositor, \n\
 \n\
    Thank you for depositing your structure with the Protein Data Bank (PDB).   \n\
 \n\
    Your entry has been assigned PDB ID %s and RCSB ID %s.\n\
A copy of your processed file in PDB format is attached to this message. \n\
 \n\
";

static char vlet_header_ndb_en[]="\
Dear Depositor, \n\
 \n\
    Thank you for depositing your structure with the Nucleic Acid Database (NDB).   \n\
 \n\
    Your entry has been assigned PDB ID %s, NDB ID %s and RCSB ID %s.\n\
A copy of your processed file in PDB format is attached to this message. \n\
 \n\
";


static char vlet_trailer_en[]="\
    It would be helpful if you could inform us when your primary \n\
citation has been published.  In your notification, please send the \n\
complete citation including volume, page numbers, and any changes in \n\
the citation title or the author list. \n\
 \n\
    Thank you for your time and effort in responding to these points.  \n\
 \n\
                                     Sincerely yours, \n\
 \n\
                                     PDB Data Processing Staff \n\
 \n\
Please direct all correspondence to: \n\
 \n\
The Protein Data Bank                           E-mail: \n\
Rutgers, The State University of New Jersey     deposit@rcsb.rutgers.edu \n\
Wright and Rieman Labs                          Fax:   (732) 445-4320 \n\
Department of Chemistry                         Phone: (732) 445-0103 \n\
610 Taylor Rd \n\
Piscataway, NJ 08854-8087 \n\
\n\
\n\
--------------------------------------------------------------------------------\n\
\n\
";


static char vlet_trailer_ndb_en[]="\
    It would be helpful if you could inform us when your primary \n\
citation has been published.  In your notification, please send the \n\
complete citation including volume, page numbers, and any changes in \n\
the citation title or the author list. \n\
 \n\
    Thank you for your time and effort in responding to these points.  \n\
 \n\
                                     Sincerely yours, \n\
 \n\
                                     NDB Data Processing Staff \n\
 \n\
Please direct all correspondence to: \n\
 \n\
The Nucleic Acid Database                       E-mail: \n\
Rutgers, The State University of New Jersey     deposit@ndbserver.rutgers.edu \n\
Wright and Rieman Labs                          Fax:   (732) 445-4320 \n\
Department of Chemistry                         Phone: (732) 445-0103 \n\
610 Taylor Rd \n\
Piscataway, NJ 08854-8087 \n\
\n\
\n\
--------------------------------------------------------------------------------\n\
\n\
";


static char blet_en[]="\
    As part of converting your file to final format, we have performed some \n\
additional checking and have encountered some issues requiring your input \n\
before we can release the file. \n\
 \n\
 < ---   ANNOTATOR INSERTS COMMENTS HERE....   --->  \n\
 \n\
    Further results of our checking are given at the conclusion of this \n\
letter. If you wish to view further validation tests of your structure  \n\
please go to our validation site located at http://pdb.rutgers.edu/validate. \n\
 \n\
    After reviewing the entry, please respond to the above issues \n\
as soon as possible so that we may complete and release your file. \n\
 \n\
";

static char glet_en[]="\
    As part of converting your file to final format, we have performed some \n\
checking.    Some points you may wish to consider are given \n\
at the conclusion of this letter.  If you wish to view further validation \n\
tests of your structure please go to our validation site located at \n\
http://pdb.rutgers.edu/validate. \n\
 \n\
    After reviewing the entry, please alert us immediately to any changes \n\
you may have.  If we do not hear from you within THREE days, we will consider \n\
this entry to have been approved by you.  The entry will then be released \n\
according to the following release/hold instructions you have provided: \n\
 \n\
";



#if 0
CifFileObj *update_cif_context(CifFileObj *fobj, CifViewObj *vobj, const char *dirPath,
			       const char *idName)
{
  char script[512];
  char  filePath[HT_MAX_NAME_LENGTH];    
  char  fobjPath[HT_MAX_NAME_LENGTH];    
  char *blockId=NULL;
  char *diagnostics=NULL;
  ndb_log_message_text(NDB_MSG_DEBUG,"UPDATE_CIF_CONTEXT for ID %s", idName);
  strcpy(filePath,dirPath);
  strcat(filePath,idName);
  strcat(filePath,".cif");
  fobj->Write(filePath,1);
  fobj->CloseFile();


  sprintf(script,"cd %s; %s -o 8 -i %s >& %s; mv -f %s.cif %s",
	  dirPath, HT_TRANSLATOR_NAME,filePath, "TRANSLATOR.ERR",filePath,filePath);
  ndb_log_message_text(NDB_MSG_DEBUG,"CIF->CIF script %s",script);
  wsystem(script);
  
  strcpy(fobjPath,dirPath);
  strcat(fobjPath,HT_TABLE_OBJECT_FILE);
  CifFileObj *fobj1 = new CifFileObj(fobjPath,WRITE_MODE);
  ndb_log_message_text(NDB_MSG_DEBUG,"Reading CIF file %s",filePath);
  fobj1->Read(filePath, diagnostics);
  ndb_log_message_text(NDB_MSG_DEBUG,"Diagnostics reading data %s",diagnostics);
  blockId = fobj1->GetBlockName(0);
  sync_categories_to_view(vobj, fobj1, blockId );
  ndb_log_message_text(NDB_MSG_DEBUG,"Sync complete for block %s", blockId);
  fobj1->CloseFile();
  fobj1 = CifFileObj(fobjPath,APPEND_MODE);
  
  return(fobj1);
}
#endif



void update_cif_context(CifFileObj *&fobj, CifViewObj *vobj, const char *dirPath,
			       const char *idName)
{
  char script[512];
  char filePath[HT_MAX_NAME_LENGTH];    
  char fobjPath[HT_MAX_NAME_LENGTH];    
  char *blockId=NULL;
  char *diagnostics=NULL;


  ndb_log_message_text(NDB_MSG_DEBUG,"UPDATE_CIF_CONTEXT for ID %s", idName);

  update_source_data(fobj,blockId);

  strcpy(filePath,dirPath);
  strcat(filePath,idName);
  strcat(filePath,".cif");
  fobj->Write(filePath,1);
  fobj->CloseFile();
  delete (fobj);

  sprintf(script,"cd %s; %s -o 8 -i %s >& %s; mv -f %s.cif %s",
	  dirPath, HT_TRANSLATOR_NAME,filePath, "TRANSLATOR.ERR",filePath,filePath);
  ndb_log_message_text(NDB_MSG_DEBUG,"CIF->CIF script %s",script);
  wsystem(script);
  
  strcpy(fobjPath,dirPath);
  strcat(fobjPath,HT_TABLE_OBJECT_FILE);
  fobj = new CifFileObj(fobjPath,WRITE_MODE);
  ndb_log_message_text(NDB_MSG_DEBUG,"Reading CIF file %s",filePath);
  fobj->Read(filePath, diagnostics);
  ndb_log_message_text(NDB_MSG_DEBUG,"Diagnostics reading data %s",diagnostics);
  blockId = fobj->GetBlockName(0);
  sync_categories_to_view(vobj, fobj, blockId );
  ndb_log_message_text(NDB_MSG_DEBUG,"Sync complete for block %s", blockId);
  //  set_sequence_prerelease_status(fobj, blockId);

  fobj->CloseFile();
  delete (fobj);
  ndb_log_message_text(NDB_MSG_DEBUG,"Deleted object db");

  fobj = new CifFileObj(fobjPath,APPEND_MODE);
  ndb_log_message_text(NDB_MSG_DEBUG,"Reading object file %s",fobjPath);
  blockId = fobj->GetBlockName(0);
  // 

  ndb_log_message_text(NDB_MSG_DEBUG,"Found block %s", blockId);
  fobj->Write(filePath);
  ndb_log_message_text(NDB_MSG_DEBUG,"Wrote updated CIF in %s", filePath);
}


void update_cif_context_exchange(CifFileObj *&fobj, CifViewObj *vobj, const char *dirPath,
				 const char *idName)
{
  char script[512];
  char filePath[HT_MAX_NAME_LENGTH];    
  char fobjPath[HT_MAX_NAME_LENGTH];    
  char *blockId=NULL;
  char *diagnostics=NULL;


  ndb_log_message_text(NDB_MSG_DEBUG,"UPDATE_CIF_CONTEXT for ID %s", idName);

  update_source_data(fobj,blockId);

  ndb_log_message_text(NDB_MSG_DEBUG,"UPDATE_CIF_CONTEXT dirPath %s", dirPath);  
  strcpy(filePath,dirPath);
  strcat(filePath,idName);
  strcat(filePath,".cif");
  ndb_log_message_text(NDB_MSG_DEBUG,"UPDATE_CIF_CONTEXT filePath %s", filePath);  
  fobj->Write(filePath,1);
  fobj->CloseFile();
  delete (fobj);


  sprintf(script,"cd %s; %s -o 8 -i %s >& %s; mv -f %s.cif %s",
	  dirPath, HT_TRANSLATOR_NAME,filePath, "TRANSLATOR.ERR",filePath,filePath);
  ndb_log_message_text(NDB_MSG_DEBUG,"CIF->CIF script %s",script);
  wsystem(script);
  
  strcpy(fobjPath,dirPath);
  strcat(fobjPath,HT_TABLE_OBJECT_FILE);
  fobj = new CifFileObj(fobjPath,WRITE_MODE);
  ndb_log_message_text(NDB_MSG_DEBUG,"Reading CIF file %s",filePath);
  fobj->Read(filePath, diagnostics);
  ndb_log_message_text(NDB_MSG_DEBUG,"Diagnostics reading data %s",diagnostics);
  blockId = fobj->GetBlockName(0);
  sync_categories_to_view(vobj, fobj, blockId );
  ndb_log_message_text(NDB_MSG_DEBUG,"Sync complete for block %s", blockId);
  //  set_sequence_prerelease_status(fobj, blockId);

  fobj->CloseFile();
  delete (fobj);
  ndb_log_message_text(NDB_MSG_DEBUG,"Deleted object db");

  fobj = new CifFileObj(fobjPath,APPEND_MODE);
  ndb_log_message_text(NDB_MSG_DEBUG,"Reading object file %s",fobjPath);
  blockId = fobj->GetBlockName(0);
  // 

  ndb_log_message_text(NDB_MSG_DEBUG,"Found block %s", blockId);
  fobj->Write(filePath);
  ndb_log_message_text(NDB_MSG_DEBUG,"Wrote updated CIF in %s", filePath);

  sprintf(script,"cd %s; %s %s",dirPath, HT_CIFEXCH_OUT_TRANSLATOR, filePath);
  ndb_log_message_text(NDB_MSG_DEBUG,"CIF->CIFexchange script %s",script);
  wsystem(script);
}


void  auto_update_on_input(UPD_HANDLE *upd, CifFileObj *fobj, const char *blockId) {
  scale_down_R_factor_values(fobj, blockId);

  ndb_log_message_text(NDB_MSG_DEBUG,"Launch editor ...");
  if ( (upd->context == HT_CONTEXT_DEPOSIT) || (upd->context == HT_CONTEXT_ASSEMBLE) ) { 
    set_initial_db_status(fobj, blockId, upd->iXYZ, upd->iSF, upd->iDF, upd->iNC);
  } 
  fix_misc_items(fobj,blockId);
}



#if 0
void  fix_misc_items(CifFileObj *fobj, const char *blockId) {

  char *oldNDBTitle=NULL;
  char *oldDescriptor=NULL;
  char *oldTitle=NULL;

  oldNDBTitle=  get_attribute_value(fobj,blockId, "struct", "ndb_title_text");
  oldTitle=     get_attribute_value(fobj,blockId, "struct", "title");
  oldDescriptor=get_attribute_value(fobj,blockId, "struct", "ndb_descriptor");
  
  ndb_log_message_text(NDB_MSG_DEBUG,"OldNDBTitle   %s ",oldNDBTitle);
  ndb_log_message_text(NDB_MSG_DEBUG,"OldTitle      %s ",oldTitle);
  ndb_log_message_text(NDB_MSG_DEBUG,"OldDescriptor %s ",oldDescriptor);


  if ((oldDescriptor==NULL || !strlen(oldDescriptor) || oldDescriptor[0] == '?' ) && 
      (oldTitle!=NULL && strlen(oldTitle))) {
    update_attribute_value(fobj,blockId,"struct","ndb_descriptor",oldTitle);    
    if (oldNDBTitle != NULL  && strlen(oldNDBTitle) && 
	(oldNDBTitle[0] != '?' ) && (oldNDBTitle[0] != '.' )) {
      update_attribute_value_if_null(fobj,blockId,"struct","title",oldNDBTitle);    
      update_attribute_value(fobj,blockId,"struct","ndb_title_text","?");    
    }
  }

  if (oldNDBTitle)   delete(oldNDBTitle);
  if (oldTitle)      delete(oldTitle);
  if (oldDescriptor) delete(oldDescriptor);

  /*
   *
   */
  char *idate1 = NULL, *idate2=NULL;
  idate1=get_attribute_value(fobj,blockId,"ndb_database_status","recvd_initial_deposition_date");    
  idate2=get_attribute_value(fobj,blockId,"database_PDB_rev","date_original");    
  if (((idate2 == NULL) || !strlen(idate2) || (idate2[0] == '?')) && 
      ((idate1 !=NULL) && strlen(idate1))) {
    update_attribute_value(fobj,blockId,"database_PDB_rev","date_original",idate1);    
  }
  if (idate1) delete(idate1);
  if (idate2) delete(idate2);

  char *dp1=NULL;
  dp1=get_attribute_value(fobj,blockId,"diffrn_radiation","rcsb_diffrn_protocol");
  if ((dp1 == NULL) || !strlen(dp1) || (dp1[0] == '?') ) {
    update_attribute_value(fobj,blockId,"diffrn_radiation","rcsb_diffrn_protocol","SINGLE WAVELENGTH");
    update_attribute_value(fobj,blockId,"diffrn_radiation","ndb_monochromatic_or_laue_m_l","M");
  } else if (!strcmp(dp1,"SINGLE WAVELENGTH") ) {
    update_attribute_value(fobj,blockId,"diffrn_radiation","ndb_monochromatic_or_laue_m_l","M");
  } else if (!strcmp(dp1,"MAD") ) {
    update_attribute_value(fobj,blockId,"diffrn_radiation","ndb_monochromatic_or_laue_m_l","M");
  } else if (!strcmp(dp1,"LAUE") ) {
    update_attribute_value(fobj,blockId,"diffrn_radiation","ndb_monochromatic_or_laue_m_l","L");
  }
  if (dp1) delete(dp1);
  //
  //  Fill audit_author.name if empty.
  //
  if (!is_attribute_value_defined(fobj,blockId,"audit_author","name")) {
    int nNames;
    char **names=NULL;
    ndb_log_message_text(NDB_MSG_DEBUG,"fix_misc_items(): Filling audit_author");
    names=get_attribute_values_where(fobj, blockId, "citation_author", "name", 
				    "citation_id", "primary", &nNames);
    update_attribute_values(fobj, blockId, "audit_author", "name", 
			    (const char **) names, nNames);
  }

#if 0  
  //  _exptl.method    
  if (!strcmp(HT_EXP_METHOD,"NMR")) {
    update_attribute_value_if_null(fobj,blockId,"exptl","method","NMR");
  } else   if (!strcmp(HT_EXP_METHOD,"X-ray")) {
    update_attribute_value_if_null(fobj,blockId,"exptl","method","X-RAY DIFFRACTION");
  }
#else 
  //  _exptl.method   JDW changed Sep 1, 2000 to force setting experiment type
  //                              Sep 12 only change method to NMR if it is NULL or X-ray. 

  if (HT_APPL_CONTEXT_TYPE == HT_CONTEXT_DEPOSIT) {
    
    if (!strcmp(HT_EXP_METHOD,"NMR")) {
      char *meth = NULL;
      meth=get_attribute_value(fobj,blockId,"exptl","method");
      if ( meth == NULL || !strncasecmp(meth,"X-RAY",5)) {
	update_attribute_value(fobj,blockId,"exptl","method","NMR");
      }
      if (meth) delete(meth);
      
    } else   if (!strcmp(HT_EXP_METHOD,"X-ray")) {
      update_attribute_value(fobj,blockId,"exptl","method","X-RAY DIFFRACTION");
    }
  }
    
#endif

  update_source_data(fobj,blockId);
}
#else

void  fix_misc_items(CifFileObj *fobj, const char *blockId) {

  /*
   *
   */
  //  _exptl.method   JDW changed Sep 1, 2000 to force setting experiment type
  //                              Sep 12 only change method to NMR if it is NULL or X-ray. 

  if (HT_APPL_CONTEXT_TYPE == HT_CONTEXT_DEPOSIT) {
    
    if (!strcmp(HT_EXP_METHOD,"NMR")) {
      char *meth = NULL;
      meth=get_attribute_value(fobj,blockId,"exptl","method");
      if ( meth == NULL || !strncasecmp(meth,"X-RAY",5)) {
	update_attribute_value(fobj,blockId,"exptl","method","NMR");
      }
      if (meth) delete(meth);
      
    } else   if (!strcmp(HT_EXP_METHOD,"X-ray")) {
      update_attribute_value(fobj,blockId,"exptl","method","X-RAY DIFFRACTION");
    }
  }
    

  //  update_source_data(fobj,blockId);
}

#endif

static void update_source_data(CifFileObj *fobj, const char *blockId)
{

  char *cNames[] ={"bovine",
		   "Chinese hamster",
		   "bacteria",
		   "chicken",
		   "human",
                   "archaea",
		   "mouse",
		   "rabbit",
		   "rat",
		   "yeast",
		   "pig",
		   "bacteria",
                   "fall armyworm",
                   ""};
  
  char *sNames[] = {"Bos taurus",
		    "Cricetulus griseus",
		    "Escherichia coli",
		    "Gallus gallus",
		    "Homo sapiens",
		    "Methanococcus thermolithotrophicus",
		    "Mus musculus",
		    "Oryctolagus cuniculus",
		    "Rattus norvegicus",
		    "Saccharomyces cerevisiae",
		    "Sus scrofa",
		    "Thermus thermophilus",
                    "Spodoptera frugiperda",
                    ""};


  char **systemN =NULL, **commonN = NULL;
  int i, j, nCNames=0, nSNames=0, iChange=0;

  systemN=get_attribute_values(fobj,blockId,"entity_src_nat","ndb_organism_scientific",&nSNames);
  commonN=get_attribute_values(fobj,blockId,"entity_src_nat","common_name",&nCNames);

  if (systemN != NULL && commonN != NULL && (nSNames == nCNames)) {
    iChange = 0;
    for (j=0; j < nSNames; j++) {
      if (systemN[j] != NULL && strlen(systemN[j]) ) {
	i=0;
	while (strlen(sNames[i])) {
	  if (!strcasecmp(systemN[j],sNames[i])) {
	    if (commonN[j] != NULL) delete (commonN[j]);
	    commonN[j] = new char[strlen(cNames[i]) + 1];
	    strcpy(commonN[j],cNames[i]);
	    iChange++;
	    break;
	  }
	  i++;
	}
	
      }
    }
    if (iChange) update_attribute_values(fobj,blockId,"entity_src_nat","common_name",
					 (const char **)commonN, nSNames);

    for (i = 0; i < nSNames; i++) {
      if (systemN[i] != NULL) delete (systemN[i]);
      if (commonN[i] != NULL) delete (commonN[i]);
    }
    delete (commonN);
    delete (systemN);
    nSNames = nCNames = 0;
  }

  //
  //
  systemN=get_attribute_values(fobj,blockId,"entity_src_gen","ndb_gene_src_scientific_name",&nSNames);
  commonN=get_attribute_values(fobj,blockId,"entity_src_gen","gene_src_common_name",&nCNames);

  if (systemN != NULL && commonN != NULL && (nSNames == nCNames)) {
    iChange = 0;
    for (j=0; j < nSNames; j++) {
      if (systemN[j] != NULL && strlen(systemN[j]) ) {
	i=0;
	while (strlen(sNames[i])) {
	  if (!strcasecmp(systemN[j],sNames[i])) {
	    if (commonN[j] != NULL) delete (commonN[j]);
	    commonN[j] = new char[strlen(cNames[i]) + 1];
	    strcpy(commonN[j],cNames[i]);
	    iChange++;
	    break;
	  }
	  i++;
	}
	
      }
    }
    if (iChange) update_attribute_values(fobj,blockId,"entity_src_gen","gene_src_common_name",
					 (const char **)commonN, nSNames);

    for (i = 0; i < nSNames; i++) {
      if (systemN[i] != NULL) delete (systemN[i]);
      if (commonN[i] != NULL) delete (commonN[i]);
    }
    delete (commonN);
    delete (systemN);
    nSNames = nCNames = 0;
  }

  //
  //
  systemN=get_attribute_values(fobj,blockId,"entity_src_gen","ndb_host_org_scientific_name",&nSNames);
  commonN=get_attribute_values(fobj,blockId,"entity_src_gen","host_org_common_name",&nCNames);

  if (systemN != NULL && commonN != NULL && (nSNames == nCNames)) {
    iChange = 0;
    for (j=0; j < nSNames; j++) {
      if (systemN[j] != NULL && strlen(systemN[j]) ) {
	i=0;
	while (strlen(sNames[i])) {
	  if (!strcasecmp(systemN[j],sNames[i])) {
	    if (commonN[j] != NULL) delete (commonN[j]);
	    commonN[j] = new char[strlen(cNames[i]) + 1];
	    strcpy(commonN[j],cNames[i]);
	    iChange++;
	    break;
	  }
	  i++;
	}
	
      }
    }
    if (iChange) update_attribute_values(fobj,blockId,"entity_src_gen","host_org_common_name",
					 (const char **)commonN, nSNames);

    for (i = 0; i < nSNames; i++) {
      if (systemN[i] != NULL) delete (systemN[i]);
      if (commonN[i] != NULL) delete (commonN[i]);
    }
    delete (commonN);
    delete (systemN);
    nSNames = nCNames = 0;
  }

}


#if 0
static void update_source_data(CifFileObj *fobj, const char *blockId)
{

  char *cNames[] ={"bovine",
		   "Chinese hamster",
		   "bacteria",
		   "chicken",
		   "human",
		   "mouse",
		   "rabbit",
		   "rat",
		   "yeast",
		   "pig",
		   "archaea",
                   ""};
  
  char *sNames[] = {"Bos taurus",
		    "Cricetulus griseus",
		    "Escherichia coli",
		    "Gallus gallus",
		    "Homo sapiens",
		    "Methanococcus thermolithotrophicus",
		    "Mus musculus",
		    "Oryctolagus cuniculus",
		    "Rattus norvegicus",
		    "Saccharomyces cerevisiae",
		    "Sus scrofa",
		    "Thermus thermophilus",
                    ""};


  char *commonN = NULL;
  int i;

  commonN=get_attribute_value(fobj,blockId,"entity_src_nat","common_name");
  if (commonN != NULL && strlen(commonN)) {
    i = 0;
    while (strlen(cNames[i])) {
      if (!strcasecmp(commonN,cNames[i])) {
	update_attribute_value(fobj,blockId,"entity_src_nat","ndb_organism_scientific",sNames[i]);
	break;
      }
      i++;
    }
    delete(commonN);
  }
  
  commonN=get_attribute_value(fobj,blockId,"entity_src_gen","gene_src_common_name");
  if (commonN != NULL && strlen(commonN)) {
    i = 0;
    while (strlen(cNames[i])) {
      if (!strcasecmp(commonN,cNames[i])) {
	update_attribute_value(fobj,blockId,"entity_src_gen","ndb_gene_src_scientific_name",sNames[i]);
	break;
      }
      i++;
    }
    delete(commonN);
  }
  
  commonN=get_attribute_value(fobj,blockId,"entity_src_gen","host_org_common_name");
  if (commonN != NULL && strlen(commonN)) {
    i = 0;
    while (strlen(cNames[i])) {
      if (!strcasecmp(commonN,cNames[i])) {
	update_attribute_value(fobj,blockId,"entity_src_gen","ndb_host_org_scientific_name",sNames[i]);
	break;
      }
      i++;
    }
    delete(commonN);
  }
}
#endif

static void compute_hold_date_string(char hold_date[31], const char *hold_code)
{
  time_t curr_time;
  struct tm *tmptr;
  //  char *curt;
  //  char timeString[31];
  char day[5], month[5], year[5];
  int imonth, iyear;

  if (!hold_code || (strlen(hold_code) < 2) || !strcmp(hold_code,"RELEASE_NOW")) {
    strcpy(hold_date,"");
    return;
  }

  time(&curr_time);
  tmptr = localtime(&curr_time);
  //  curt = ctime(&curr_time);
  //  ndb_clean_string(curt);
  //  if (curt[strlen(curt)-1] == '\n') curt[strlen(curt)-1] = '\0';
  //  strftime(timeString,30,"%Y-%m-%d", tmptr);

  strftime(day,  5,"%d", tmptr);
  strftime(month,5,"%m", tmptr);
  strftime(year, 5,"%Y", tmptr);
  iyear  = atoi(year);
  imonth = atoi(month);

  //

  if (!strcmp(hold_code,"HOLD FOR 6 MONTHS")) {
    imonth = imonth + 6;
    if (imonth > 12) {
      imonth = imonth - 12;
      iyear = iyear + 1;
    }
    
  } else if (!strcmp(hold_code,"HOLD FOR 1 YEAR")) {
    iyear = iyear + 1;
  } else if (!strcmp(hold_code,"HOLD FOR 2 YEARS")) {
    iyear = iyear + 2;
  } else if (!strcmp(hold_code,"HOLD FOR 3 YEARS")) {
    iyear = iyear + 3;
  } else if (!strcmp(hold_code,"HOLD FOR 4 YEARS")) {
    iyear = iyear + 4;
  } else {
    strcpy(hold_date,"");
    return;
  }

  if (imonth < 10) 
    sprintf(month,"0%1d",imonth);
  else
    sprintf(month,"%2d",imonth);

  sprintf(year,"%4d",iyear);
  sprintf(hold_date,"%4s-%2s-%2s",year,month,day);
  return;
}



void  set_initial_deposit_status(CifFileObj *fobj, const char *blockId) {
  time_t curr_time;
  struct tm *tmptr;
  //  char *curt;
  char timeString[31];

  time(&curr_time);
  tmptr = localtime(&curr_time);
  //  curt = ctime(&curr_time);
  //  ndb_clean_string(curt);
  //   if (curt[strlen(curt)-1] == '\n') curt[strlen(curt)-1] = '\0';
  strftime(timeString,30,"%Y-%m-%d", tmptr);
  update_attribute_value(fobj,blockId,"pdbx_database_status","date_submitted",               timeString);    
  update_attribute_value(fobj,blockId,"pdbx_database_status","recvd_initial_deposition_date",timeString);   
  update_attribute_value(fobj,blockId,"database_PDB_rev",   "date_original",                timeString);    

#if 1
  //
  // -------------------------------------------------------------------------------
  //  Compute dates and status from dep_release_code_* items set at deposition time.
  //

 char *RStatusCoords = NULL, *RStatusSF = NULL, *RStatusNC = NULL;
 char HDateCoords[31], HDateSF[31], HDateNC[31];
 char *RecvdSF = NULL,       *RecvdNC=NULL;

 // Coordinates  ....

 RStatusCoords = get_attribute_value(fobj,blockId,"pdbx_database_status","dep_release_code_coordinates");
 if (RStatusCoords  == NULL) {  
   RStatusCoords = new char[12];
   strcpy(RStatusCoords,"");
 } else {
   if (!strcmp(RStatusCoords,"HOLD FOR PUBLICATION")) {
     update_attribute_value(fobj,blockId,"pdbx_database_status","author_release_status_code","HPUB");    
     update_attribute_value(fobj,blockId,"pdbx_database_status","hold_for_publication","Y");    
   } else if (!strcmp(RStatusCoords,"RELEASE NOW")) {
     update_attribute_value(fobj,blockId,"pdbx_database_status","author_release_status_code","REL");    
     update_attribute_value(fobj,blockId,"pdbx_database_status","hold_for_publication","N");    
   } else if (!strncmp(RStatusCoords,"HOLD FOR ",9)) {
     update_attribute_value(fobj,blockId,"pdbx_database_status","author_release_status_code","HOLD");    
     update_attribute_value(fobj,blockId,"pdbx_database_status","hold_for_publication","N");    
     compute_hold_date_string(HDateCoords, RStatusCoords);
     update_attribute_value(fobj,blockId,"pdbx_database_status","date_hold_coordinates",HDateCoords);    
   }
 }

 //  SF ... 

 RecvdSF = get_attribute_value(fobj,blockId,"pdbx_database_status","recvd_struct_fact");
 if (RecvdSF && !strcmp(RecvdSF,"Y")) {
   RStatusSF = get_attribute_value(fobj,blockId,"pdbx_database_status","dep_release_code_struct_fact");
   if (RStatusSF  == NULL) {
     RStatusSF = new char[12];
     strcpy(RStatusSF,"");
   } else {
     if (!strcmp(RStatusSF,"HOLD FOR PUBLICATION")) {
       // 
     } else if (!strcmp(RStatusSF,"RELEASE NOW")) {
       //
     } else if (!strncmp(RStatusSF,"HOLD FOR ",9)) {
       compute_hold_date_string(HDateSF, RStatusSF);
       update_attribute_value(fobj,blockId,"pdbx_database_status","date_hold_struct_fact",HDateSF);    
     }
   }
 }   

 // NC ... 

 RecvdNC = get_attribute_value(fobj,blockId,"pdbx_database_status","recvd_nmr_constraints");
 if (RecvdNC && !strcmp(RecvdNC,"Y")) {
   RStatusNC = get_attribute_value(fobj,blockId,"pdbx_database_status","dep_release_code_nmr_constraints");
   if (RStatusNC  == NULL) {
     RStatusNC = new char[12];
     strcpy(RStatusNC,"");
   } else {
     if (!strcmp(RStatusNC,"HOLD FOR PUBLICATION")) {
       // 
     } else if (!strcmp(RStatusNC,"RELEASE NOW")) {
       //
     } else if (!strncmp(RStatusNC,"HOLD FOR ",9)) {
       compute_hold_date_string(HDateNC, RStatusNC);
       update_attribute_value(fobj,blockId,"pdbx_database_status","date_hold_nmr_constraints",HDateNC);    
     }
   }
 }   


#endif

}



void  set_initial_db_status(CifFileObj *fobj, const char *blockId,  int iXYZ, int iSF, int iDF, int iNC) {
  time_t curr_time;
  struct tm *tmptr;
  char *curt;
  char timeString[31];

  time(&curr_time);
  tmptr = localtime(&curr_time);
  curt = ctime(&curr_time);
  ndb_clean_string(curt);
  if (curt[strlen(curt)-1] == '\n') curt[strlen(curt)-1] = '\0';
  strftime(timeString,30,"%Y-%m-%d", tmptr);

  if (HT_OSAKA_SITE_FLAG==1) {
     update_attribute_value(fobj,blockId,"pdbx_database_status","deposit_site","OSAKA");    
     update_attribute_value(fobj,blockId,"pdbx_database_status","process_site","OSAKA");    
 } else {
     update_attribute_value(fobj,blockId,"pdbx_database_status","deposit_site","RCSB");    
     update_attribute_value(fobj,blockId,"pdbx_database_status","process_site","RCSB");    
 }


  update_attribute_value_if_null(fobj,blockId,"pdbx_database_status","date_submitted",timeString);    
  update_attribute_value_if_null(fobj,blockId,"pdbx_database_status","recvd_initial_deposition_date",timeString);    

  update_attribute_value_if_null(fobj,blockId,"pdbx_database_status","status_code","PROC");    
  update_attribute_value_if_null(fobj,blockId,"pdbx_database_status","recvd_manuscript","N");    

  if (iDF) {
    update_attribute_value_if_null(fobj,blockId,"pdbx_database_status","date_deposition_form",timeString);    
    update_attribute_value_if_null(fobj,blockId,"pdbx_database_status","recvd_deposit_form","Y");    
  } else {
    update_attribute_value_if_null(fobj,blockId,"pdbx_database_status","recvd_deposit_form","N");    
  }

  if (iNC) {
    if ( HT_APPL_CONTEXT_TYPE == HT_CONTEXT_ASSEMBLE)
      update_attribute_value_if_null(fobj,blockId,"pdbx_database_status","dep_release_code_nmr_constraints","?");    
    else
      update_attribute_value_if_null(fobj,blockId,"pdbx_database_status","dep_release_code_nmr_constraints","RELEASE NOW");
    update_attribute_value_if_null(fobj,blockId,"pdbx_database_status","date_nmr_constraints",timeString);    
    update_attribute_value_if_null(fobj,blockId,"pdbx_database_status","recvd_nmr_constraints","Y");    
  } else {
    update_attribute_value_if_null(fobj,blockId,"pdbx_database_status","recvd_nmr_constraints","N");    
  }

  if (iXYZ) {
    if ( HT_APPL_CONTEXT_TYPE == HT_CONTEXT_ASSEMBLE)
      update_attribute_value_if_null(fobj,blockId,"pdbx_database_status","dep_release_code_coordinates","?");    
    else
      update_attribute_value_if_null(fobj,blockId,"pdbx_database_status","dep_release_code_coordinates","RELEASE NOW");    
    update_attribute_value_if_null(fobj,blockId,"pdbx_database_status","date_coordinates",timeString);    
    update_attribute_value_if_null(fobj,blockId,"pdbx_database_status","recvd_coordinates","Y");    
    update_attribute_value_if_null(fobj,blockId,"pdbx_database_status","status_coordinates_in_NDB","Y");    
  } else {
    update_attribute_value_if_null(fobj,blockId,"pdbx_database_status","recvd_coordinates","N");    
    update_attribute_value_if_null(fobj,blockId,"pdbx_database_status","status_coordinates_in_NDB","N");    
  }
  if (iSF) {
    if ( HT_APPL_CONTEXT_TYPE == HT_CONTEXT_ASSEMBLE)
      update_attribute_value_if_null(fobj,blockId,"pdbx_database_status","dep_release_code_struct_fact","?");    
    else
      update_attribute_value_if_null(fobj,blockId,"pdbx_database_status","dep_release_code_struct_fact","RELEASE NOW");    
    update_attribute_value_if_null(fobj,blockId,"pdbx_database_status","date_struct_fact",timeString);    
    update_attribute_value_if_null(fobj,blockId,"pdbx_database_status","recvd_struct_fact","Y");    
  } else {
    update_attribute_value_if_null(fobj,blockId,"pdbx_database_status","recvd_struct_fact","N");    
  }
}




static void  set_sequence_prerelease_status(CifFileObj *fobj, const char *blockId) {

  ndb_log_message_text(NDB_MSG_DEBUG,"set_sequence_prerelease_status:begin");
  if ( HT_APPL_CONTEXT_TYPE != HT_CONTEXT_ANNOTATE)  return;
  if (!fobj || !blockId || !strlen(blockId)) return;

  ndb_log_message_text(NDB_MSG_DEBUG,"set_sequence_prerelease_status:begin");
  char *curStatus = NULL, *seqStatus=NULL;
  curStatus = get_attribute_value(fobj,blockId,"pdbx_database_status","status_code");
  seqStatus = get_attribute_value(fobj,blockId,"pdbx_database_status","dep_release_code_sequence");
  if ((seqStatus && !strcmp(seqStatus,"RELEASE NOW")) && 
      (curStatus  && (!strcmp(curStatus,"HOLD") || !strcmp(curStatus,"HPUB")) )) {
    int i, nRows=0, iCol1, iCol2, oCol1, oCol2;
    ISTable *ePoly = NULL, *pSeq = NULL;
    ndb_log_message_text(NDB_MSG_DEBUG,"set_sequence_prerelease_status:A");
    if ((pSeq=fobj->GetTablePtr(blockId,"rcsb_prerelease_seq")) != NULL) {
      ndb_log_message_text(NDB_MSG_DEBUG,"set_sequence_prerelease_status:B");
      nRows=pSeq->GetNumRows();
      for (i=0; i < nRows; i++) {
	pSeq->DeleteRow(i);
      }
      if ((ePoly=fobj->GetTablePtr(blockId,"entity_poly")) != NULL) {
	ndb_log_message_text(NDB_MSG_DEBUG,"set_sequence_prerelease_status:C");
	iCol1 = ePoly->GetColumnIndex("entity_id");
	iCol2 = ePoly->GetColumnIndex("ndb_seq_one_letter_code");
	nRows = ePoly->GetNumRows();
	oCol1 = pSeq->GetColumnIndex("entity_id");
	oCol2 = pSeq->GetColumnIndex("seq_one_letter_code");
	if (iCol1 < 0 || iCol2 < 0 || oCol1 < 0 || oCol2 < 0) return;
	ndb_log_message_text(NDB_MSG_DEBUG,"set_sequence_prerelease_status:D rows = %d", nRows);
	ReVarCifArray<CifString> *iRow=NULL;
	for (i=0; i < nRows; i++) {
	  iRow = ePoly->GetRow(i);
	  pSeq->AddRow();
	  pSeq->UpdateCell((*iRow)[iCol1], oCol1, i);
	  pSeq->UpdateCell((*iRow)[iCol2], oCol2, i);
	  if (iRow) delete iRow;
	}
      }
      fobj->WriteTable(pSeq,blockId,"rcsb_prerelease_seq");
    }
  }
}



static int reformat_summary_letter(const char *path, const char *id, const char *sourceLetter,
				   const char *idName, CifFileObj *fobj, const char *blockId)
{
  // blet_en and glet_en defined in interface-lang.h
  // returns 1 if letters are created... 

 char l1[HT_MAX_NAME_LENGTH];
 FILE *fp=NULL;
 time_t curr_time;
 struct tm *tmptr;
 char *curt;
 char timeString[31];
 char status1[150];
 char status2[150];
 char status3[150];

 if ((path==NULL) || (strlen(path)==0 )) return(0);
 if ((id==NULL) || (strlen(id)==0 )) return(0);
 if ((sourceLetter==NULL) || (strlen(sourceLetter)==0 )) return(0);

 time(&curr_time);
 tmptr = localtime(&curr_time);
 curt = ctime(&curr_time);
 ndb_clean_string(curt);
 if (curt[strlen(curt)-1] == '\n') curt[strlen(curt)-1] = '\0';
 strftime(timeString,30,"%Y-%m-%d", tmptr);


 char *PDBID = NULL, *NDBID = NULL;
 char *RStatusCoords = NULL, *RStatusSF = NULL, *RStatusNC = NULL, *RStatusSeq = NULL;
 char *HDateCoords = NULL, *HDateSF = NULL, *HDateNC = NULL;
 char *RecvdSF = NULL, *RecvdNC=NULL;



 PDBID=get_attribute_value_where(fobj, blockId, "database_2", "database_code", 
				 "database_id", "PDB");

 NDBID=get_attribute_value_where(fobj, blockId, "database_2", "database_code", 
				 "database_id", "NDB");
 if (PDBID == NULL) {
   PDBID = new char[10];
   strcpy(PDBID,"UNASSIGNED");
 }

 if ( NDBID == NULL && !strcmp(HT_MOL_TYPE,"na")) {
   NDBID = new char[10];
   strcpy(NDBID,"UNASSIGNED");
 }

 ndb_log_message_text(NDB_MSG_DEBUG,"reformat_summary_letter PDB ID %s RCSB ID %s",PDBID,idName);

 //
 // -------------------------------------------------------------------------------
 //
 RStatusCoords = get_attribute_value(fobj,blockId,"pdbx_database_status","author_release_status_code");
 if (RStatusCoords  == NULL) {
   RStatusCoords = new char[12];
   strcpy(RStatusCoords,"NONE");
 }

 HDateCoords = get_attribute_value(fobj,blockId,"pdbx_database_status","date_hold_coordinates");
 if (HDateCoords == NULL) {
   HDateCoords = new char[12];
   strcpy(HDateCoords,"UNASSIGNED");
 }

 RStatusSF = get_attribute_value(fobj,blockId,"pdbx_database_status","dep_release_code_struct_fact");
 if (RStatusSF  == NULL) {
   RStatusSF = new char[12];
   strcpy(RStatusSF,"UNASSIGNED");
 }

 RecvdSF = get_attribute_value(fobj,blockId,"pdbx_database_status","recvd_struct_fact");
 HDateSF = get_attribute_value(fobj,blockId,"pdbx_database_status","date_hold_struct_fact");
 if (HDateSF  == NULL) {
   HDateSF = new char[12];
   strcpy(HDateSF,"UNASSIGNED");
 }

 RecvdNC = get_attribute_value(fobj,blockId,"pdbx_database_status","recvd_nmr_constraints");
 RStatusNC = get_attribute_value(fobj,blockId,"pdbx_database_status","dep_release_code_nmr_constraints");
 if (RStatusNC  == NULL) {
   RStatusNC = new char[12];
   strcpy(RStatusNC,"UNASSIGNED");
 }

 HDateNC = get_attribute_value(fobj,blockId,"pdbx_database_status","date_hold_nmr_constraints");
 if (HDateNC  == NULL) {
   HDateNC = new char[12];
   strcpy(HDateNC,"UNASSIGNED");
 }

 RStatusSeq = get_attribute_value(fobj,blockId,"pdbx_database_status","dep_release_code_sequence");
 if (RStatusSeq  == NULL) {
   RStatusSeq = new char[20];
   strcpy(RStatusSeq,"HOLD FOR RELEASE");
 }

 ndb_log_message_text(NDB_MSG_DEBUG,"reformat_summary_letter Status Coords %s HData %s",RStatusCoords, 
		      HDateCoords);
 ndb_log_message_text(NDB_MSG_DEBUG,"reformat_summary_letter Status SF     %s HData %s",RStatusSF, HDateSF);
 ndb_log_message_text(NDB_MSG_DEBUG,"reformat_summary_letter Status NC     %s HData %s",RStatusNC, HDateNC);
 ndb_log_message_text(NDB_MSG_DEBUG,"reformat_summary_letter Status SEQ    %s",RStatusSeq);


 memset(status1,0,150);
 memset(status2,0,150);
 memset(status3,0,150);

 if ( !strcasecmp(RStatusCoords,"HPUB") ) {
   sprintf(status1,"\n    ** Your coordinates will be held until the publication of your \n         primary citation.\n");

 } else  if (!strcasecmp(RStatusCoords,"HOLD") ) {
   sprintf(status1,"\n    ** Your coordinates will be held until: %s\n", HDateCoords);
   ndb_log_message_text(NDB_MSG_DEBUG,"reformat_summary_letter Status Coords %s msg %s",RStatusCoords, status1);

 } else  if ( !strcasecmp(RStatusCoords,"REL") ) {
   sprintf(status1,"\n    ** Your coordinates will be released immediately.\n");
   ndb_log_message_text(NDB_MSG_DEBUG,"reformat_summary_letter Status Coords %s msg %s",RStatusCoords, status1);

 } else {
   sprintf(status1,"\n    ** No release information has been provided for your coordinates.\n");
   ndb_log_message_text(NDB_MSG_DEBUG,"reformat_summary_letter Status Coords %s msg %s",RStatusCoords, status1);
 }

 
 if (!strcasecmp(HT_EXP_METHOD,"NMR")) {
   if ( RecvdNC && !strcasecmp(RecvdNC,"Y") ) {
     if (!strcasecmp(RStatusNC,"HOLD FOR PUBLICATION") ) {
       sprintf(status2,"    ** Your constraint data will be held until the publication of your \n         primary citation.\n");
     } else  if ( !strncasecmp(RStatusNC,"HOLD",4) ) {
       sprintf(status2,"    ** Your constraint data will be held until: %s\n", HDateNC);
     } else  if (!strcasecmp(RStatusNC,"RELEASE NOW") ) {
       sprintf(status2,"    ** Your constraint data will be released immediately.\n");
     } else  if (HDateNC && (strlen(HDateNC) > 1)) {
       sprintf(status2,"    ** Your constraint data will be held until: %s\n", HDateNC);
     } else {
       sprintf(status2,"    ** No release information has been provided for your constraint data.\n");
     }
   }
 } else {
   if ( RecvdSF && !strcasecmp(RecvdSF,"Y") ) {
     if ( !strcasecmp(RStatusSF,"HOLD FOR PUBLICATION") ) {
       sprintf(status2,"    ** Your structure factor data will be held until the publication of your \n         primary citation.\n");
     } else  if ( !strncasecmp(RStatusSF,"HOLD",4) ) {
       sprintf(status2,"    ** Your structure factor data will be held until: %s\n", HDateSF);
     } else  if ( !strcasecmp(RStatusSF,"RELEASE NOW") ) {
       sprintf(status2,"    ** Your structure factor data will be released immediately.\n");
     } else  if (HDateSF && (strlen(HDateSF) > 1)) {
       sprintf(status2,"    ** Your structure factore data will be held until: %s\n", HDateSF);
     } else {
       sprintf(status2,"    ** No release information has been provided for your structure factor data.\n");
     }
   }
 }

 //
 if ( !strcasecmp(RStatusSeq,"RELEASE NOW") ) {
   sprintf(status3,"\n    ** The macromolecular sequence(s) for your entry will be displayed \n         in PDB status reports.\n");

 } else  if (!strcasecmp(RStatusSeq,"HOLD FOR RELEASE") ) {
   sprintf(status3,"\n    ** The macromolecular sequence(s) for your entry will not be displayed \n         in PDB status reports.\n");
 } else {
   sprintf(status3,"\n    ** The macromolecular sequence(s) for your entry will not be displayed \n         in PDB status reports.\n");
 }



 char *xTitle=NULL;
 // char *xDescriptor=NULL;
 int i,wLen = 76;
 xTitle=     get_attribute_value(fobj,blockId, "struct", "title");
 // xDescriptor=get_attribute_value(fobj,blockId, "struct", "ndb_descriptor");


 //
 //   Letter 1
 // 

 strcpy(l1, path);
 strcat(l1, id);
 strcat(l1, ".let1");
 fp = fopen(l1,"w");
 if (fp != NULL) {

   fprintf(fp,"\n                                                            %s\n",timeString);

   if (!strcmp(HT_MOL_TYPE,"na") &&  strcasecmp(HT_EXP_METHOD,"NMR") ) {
     fprintf(fp,vlet_header_ndb_en, PDBID, NDBID, idName);
   } else {
     fprintf(fp,vlet_header_en, PDBID, idName);

   }

   
   // Title etc...
   for (i=0; i < wLen/2-4; i++) {
     fprintf(fp,"=");
   }
   fprintf(fp," TITLE ");
   for (i=0; i < wLen/2-4; i++) {
     fprintf(fp,"=");
   }
   fprintf(fp,"\n");
   //     fprintf(fp,"Descriptor:  ");
   //     formatStringData(fp, xDescriptor, wLen-12);
   //     formatStringData(fp, xTitle, wLen-2);
   formatStringData2(fp, xTitle, wLen-2);
   for (i=0; i < wLen-1; i++) {
     fprintf(fp,"=");
   }
   fprintf(fp,"\n\n");
   
   fprintf(fp,glet_en);
   fprintf(fp,status1);
   // 
   if (strlen(status2) < 1) {
     if (!strcasecmp(HT_EXP_METHOD,"NMR") ) {
       fprintf(fp,"\n   We encourage the deposition of NMR constraint data.  If you would like \n");
       fprintf(fp,"to deposit this data, please attach your constraint data file when you \n");
       fprintf(fp,"respond to this message.\n");
     } else {
       fprintf(fp,"\n   We encourage the deposition of structure factor data.  If you would like \n");
       fprintf(fp,"to deposit this data, please attach your structure factor file when you \n");
       fprintf(fp,"respond to this message. \n");
     }
   } else {
     fprintf(fp,status2);
   }

   fprintf(fp,status3);

   fprintf(fp,"\n");

   if (!strcmp(HT_MOL_TYPE,"na") &&  strcasecmp(HT_EXP_METHOD,"NMR") ) {
     fprintf(fp,vlet_trailer_ndb_en);
   } else {
     fprintf(fp,vlet_trailer_en);
   }
   display_raw_file(fp,sourceLetter);
 } else {
   return(0);
 }
 //
 //   Letter 2
 // 
 strcpy(l1, path);
 strcat(l1, id);
 strcat(l1, ".let2");
 fp = fopen(l1,"w");
 if (fp != NULL) {
   fprintf(fp,"\n                                                            %s\n",timeString);  
   if (!strcmp(HT_MOL_TYPE,"na") &&  strcasecmp(HT_EXP_METHOD,"NMR") ) {
     fprintf(fp,vlet_header_en, PDBID, NDBID, idName);
   } else {
     fprintf(fp,vlet_header_en, PDBID, idName);
   }

   // Title etc...
   for (i=0; i < wLen/2-4; i++) {
     fprintf(fp,"=");
   }
   fprintf(fp," TITLE ");
   for (i=0; i < wLen/2-4; i++) {
     fprintf(fp,"=");
   }
   fprintf(fp,"\n");
   //     fprintf(fp,"Descriptor:  ");
   //     formatStringData(fp, xDescriptor, wLen-12);
   //     formatStringData(fp, xTitle, wLen-2);
   formatStringData2(fp, xTitle, wLen-2);
   for (i=0; i < wLen-1; i++) {
     fprintf(fp,"=");
   }
   fprintf(fp,"\n\n");
   
   fprintf(fp,blet_en);
   //     fprintf(fp,status1);
   //     fprintf(fp,status2);
   //     fprintf(fp,"\n");
   if ( !strcmp(HT_MOL_TYPE,"na") &&  strcasecmp(HT_EXP_METHOD,"NMR") ) {
     fprintf(fp,vlet_trailer_ndb_en);
   } else {
     fprintf(fp,vlet_trailer_en);
   }
   display_raw_file(fp,sourceLetter);
 } else {
   return(0);
 }
 return(1);
}


void mk_validation_report(FILE *io, const char *ufid, const char *idName, 
			  const char *fName, const char *fType,
			  const char *fNameSF, const char *fTypeSF, 
			  int preCheckFlag, CifFileObj *fobj, const char *blockId) {
  //
  // Beware of case assumptions with file names...
  // 
  int s1,s2,s3,s4,s5,s6;
  int p1, ilen, iSF=0, iPDB=0;
  char tFile[HT_MAX_NAME_LENGTH], sFile[HT_MAX_NAME_LENGTH], pFile[HT_MAX_NAME_LENGTH];
  char psFile[HT_MAX_NAME_LENGTH];
  char  valDirPath[HT_MAX_NAME_LENGTH];  
  char  dirPath[HT_MAX_NAME_LENGTH];  
  char urlNeighbors[255];
  char  idNamelc[20];
  char  htmlPath[HT_MAX_NAME_LENGTH];    
  char  script[2048];
  char  preCheckOpt[10];
  char line[151];
  struct stat statbuf;
  int nModel;
  FILE *fp=NULL;

  ndb_log_message_text(NDB_MSG_DEBUG,"mk_validation_report: idName %s fName %s fType %s preCheck %d",
		       idName,fName, fType, preCheckFlag);


  // dirPath is where the data files live....
  //
  strcpy(dirPath,HT_SESSION_ROOT_PATH);
  strcat(dirPath,ufid);
  strcat(dirPath,"/");

  //
  // valDirPath - points to the temporary session directory...
  //

  strcpy(valDirPath,HT_SESSION_TMP_ROOT_PATH);
  strcat(valDirPath,ufid);
  strcat(valDirPath,"/");
  strcat(valDirPath,"validation/");


  // preCheckOpt determines the type of letter that is produced by validation.

  strcpy(preCheckOpt,"");
  if (preCheckFlag)   
    strcpy(preCheckOpt," -p ");
  else {

    //    if (!strcmp(HT_MOL_TYPE,"na")) 
    //  strcpy(preCheckOpt," ");
    //else
    //  strcpy(preCheckOpt," -adit ");

    strcpy(preCheckOpt," -adit ");

  }

  //  strcpy(idNamelc,idName);

  strcpy(idNamelc,fName);


  translate_coordinate_format(fType, "cif", dirPath ,fName);
  sprintf(script,"rm -rf %s; mkdir %s; cd %s; %s  -s %s -o 2 -filename %s -f %s%s.cif >& %s;  ",
	  valDirPath, valDirPath, valDirPath, HT_VALIDATOR_NAME, preCheckOpt, fName, dirPath, 
	  fName, "VALIDATION1.ERR");


  strcat(script,HT_RCSBAPPS_PATH);
  strcat(script,"/bin/ps2gifdir1 >& /dev/null; ");
  ndb_log_message_text(NDB_MSG_DEBUG,"Geometric validation script %s",script);
  wsystem(script);

  //
  //  Make a PDB format file.
  //
  iPDB=translate_coordinate_format(fType, "pdb", dirPath ,fName);

  if (iPDB && !strcmp(HT_EXP_METHOD,"NMR")) { // count the number of models in the ensemble
    nModel=0;
    sprintf(sFile,"%s%s.%s",dirPath,fName,"pdb");
    fp = fopen(sFile,"r");
    if (fp != NULL) {
      fgets(line,150,fp);
      
      while ( !feof(fp) ) {
	if (!strncmp(line,"MODEL ",6)) nModel++;
	fgets(line, 150, fp);       
      }
      fclose(fp);
    }
    ndb_log_message_text(NDB_MSG_DEBUG,"Found %d models in coordinate file",nModel);
  }

  if (iPDB) {
    //
    sprintf(script,"cd %s; rm -rf sfcheck splus; mkdir sfcheck splus; ", dirPath);
    wsystem(script);
    

    //
    //    Run SFCHECK if we find a structure factor file.
    //

    iSF = 0;
    sprintf(sFile,"%s%s.%s",dirPath,fNameSF,"cif");
    ndb_log_message_text(NDB_MSG_DEBUG,"Search for SF file %s",sFile);

    if (stat(sFile,&statbuf) == 0) {
      sprintf(tFile,"%s%s",dirPath,"sfcheck/SF-SCRIPT.csh");
      fp = fopen(tFile,"w");
      if (fp != NULL) {
	fprintf(fp,"#!/bin/csh\n");
	fprintf(fp,"#\n");
	//	fprintf(fp,"%s/sfcheck.4.5/bin/sfcheck <<EOF\n",HT_RCSBAPPS_PATH);
	fprintf(fp,"%s/sfcheck/bin/sfcheck <<EOF\n",HT_RCSBAPPS_PATH);
	fprintf(fp,"Y\n");
	fprintf(fp,"%s%s.pdb\n",dirPath,fName);
	fprintf(fp,"%s%s.%s\n",dirPath,fNameSF,"cif");
	fprintf(fp,"\nEOF\n#\n");
	fclose(fp);
	sprintf(script,"cd %s%s; chmod 755 SF-SCRIPT.csh; ./SF-SCRIPT.csh >& SF.LOG; cp -f *.ps %s/sfcheck.ps >& /dev/null; ",
		dirPath, "sfcheck", valDirPath);
	ndb_log_message_text(NDB_MSG_DEBUG,"SFCHECK script %s",script);
	wsystem(script);
	iSF = 1;
	ndb_log_message_text(NDB_MSG_DEBUG,"SFCHECK completed ");
      }
    }
  }



  // SPLUS ---

  if (HT_APPL_CONTEXT_TYPE != HT_CONTEXT_DEPOSIT && iPDB) {
    sprintf(sFile,"%s",HT_SPLUS_INTERPRETER);
    ndb_log_message_text(NDB_MSG_DEBUG,"MK-VAL-REPORT: Check for SPLUS interpreter %s",sFile);
    if (stat(sFile,&statbuf) == 0) {

      sprintf(tFile,"%s%s",dirPath,"splus/SP.csh");
      ndb_log_message_text(NDB_MSG_DEBUG,"MK-VAL-REPORT: Found SPLUS interpreter %s",sFile);
      ndb_log_message_text(NDB_MSG_DEBUG,"MK-VAL-REPORT: SPLUS script file is %s",tFile);
      fp = fopen(tFile,"w");
      if (fp != NULL) {
	fprintf(fp,"#!/bin/csh\n");
	fprintf(fp,"#\n");
        fprintf(fp,"set echo\n");
        fprintf(fp,"whoami\n");

	fprintf(fp,"setenv S_WORK %s%s\n setenv PKBHOME /splus/pkb\n", 
	      dirPath, "splus");
	fprintf(fp,"setenv S_MYSWORK %s%s\n setenv SHOME /splus\n", 
	      dirPath, "splus");

	fprintf(fp,"setenv HOME %s%s\n", 
	      dirPath, "splus");

        fprintf(fp,"printenv\n");
	fprintf(fp,"cd %s%s\n %s CHAPTER\n", 
		dirPath, "splus",  HT_SPLUS_INTERPRETER);

	fprintf(fp,"%s <<EOF >& SPLUS.OUT\n",HT_SPLUS_INTERPRETER);
        fprintf(fp,"options(memory=300000000,object.size=30000000,echo=1,free=T,reference=T)\n");
	fprintf(fp,"attach(\"$PKBHOME/Database/.Data\",pos=2)\n");
	fprintf(fp,"attach(\"$PKBHOME/Functions/.Data\",pos=3)\n");
	fprintf(fp,"attach(\"$PKBHOME/Subroutines/.Data\",pos=4)\n");

        fprintf(fp,"LOAD(\"../%s.pdb\")\n",fName);
	fprintf(fp,"EOF\n#\n");
	fclose(fp);

	sprintf(script,"cd %s%s; chmod 755 SP.csh; ./SP.csh >>& SP.LOG; cp -f SPLUS.OUT %s%s >& /dev/null; ",
		dirPath, "splus", valDirPath, "SPLUS.OUT");
	ndb_log_message_text(NDB_MSG_DEBUG,"SPLUS script %s",script);
	wsystem(script);
	ndb_log_message_text(NDB_MSG_DEBUG,"SPLUS completed");      
      }
    }
  }
    

#if 0
  if (!preCheckFlag) {
    //
    // Structure neighbors 
    //
    sprintf(script,"cd %s; %s/bin/structure-neighbors -f %s.pdb >& SN.ERR; ",
	    dirPath, HT_RCSBAPPS_PATH,fName);
    ndb_log_message_text(NDB_MSG_DEBUG,"Structure neighbors script %s",script);
    wsystem(script);
    
    
    //
    // Sequence neighbors 
    //
    sprintf(script,"cd %s; %s/bin/blast-search-1 -f SEQUENCE.DAT >& SEQN.ERR; ",
	    valDirPath,HT_RCSBAPPS_PATH);
    ndb_log_message_text(NDB_MSG_DEBUG,"Sequence neighbors script %s",script);
    wsystem(script);
  }
#endif

  // ------------------------------------------------------------------
  //  Build HTML report of validation processing ... 
  //

  strcpy(htmlPath,HT_SESSION_REL_TMP_ROOT_PATH);
  strcat(htmlPath,ufid);
  strcat(htmlPath,"/validation/");



  // ***************
  s1=s2=s3=s4=s5=s6=0;
  sprintf(tFile,"%s%s.html",htmlPath,idNamelc);  // atlas page
  sprintf(sFile,"%s%s.html",valDirPath,idNamelc);
  
  ndb_log_message_text(NDB_MSG_DEBUG,"Search for %s",sFile);
  if (stat(sFile,&statbuf) == 0) s1=1;
  ndb_log_message_text(NDB_MSG_DEBUG,"Stat returns %d",s1);

  if (s1) {  // Summary   
    fprintf(io,"<h2>Structure Summary</h2>\n");
    sprintf(tFile,"%s%s.html",htmlPath,idNamelc);
    sprintf(sFile,"%s%s.html",valDirPath,idNamelc);

    //    if (stat(sFile,&statbuf) == 0)
    // fprintf(io,"<li><h3>Structure Summary Report</h3>\n");

    fprintf(io,"<ul>\n");
    fprintf(io,"<li><font size=4><a href=\"%s\">Atlas summary</a> </font>\n",tFile);


#if 0
    sprintf(tFile,"%s%s.%s.letter",htmlPath,idNamelc,fType);
    sprintf(sFile,"%s%s.%s.letter",valDirPath,idNamelc,fType);
#endif


    sprintf(tFile,"%s%s.letter",htmlPath,fName);
    sprintf(sFile,"%s%s.letter",valDirPath,fName);

    ndb_log_message_text(NDB_MSG_DEBUG,"MK-VAL-REPORT: Search for %s",sFile);

    if (stat(sFile,&statbuf) == 0) {
      if (preCheckFlag) {
	fprintf(io,"<li><font size=4>Validation <a href=\"%s\">summary letter</a> </font>\n",tFile);
      } else {

#if 0
	if ( reformat_summary_letter(valDirPath, fName, sFile, idName,fobj, blockId)) {
	  sprintf(tFile,"%s%s.let1",htmlPath,fName);
	  fprintf(io,"<li><font size=4>Validation <a href=\"%s\">summary letter template</a> (no problems)</font>\n",
		  tFile);
	  sprintf(tFile,"%s%s.let2",htmlPath,fName);
	  fprintf(io,"<li><font size=4>Validation <a href=\"%s\">summary letter template</a> (problems)</font>\n",
		  tFile);
	}
#else
	fprintf(io,"<li><font size=4>Validation <a href=\"%s\">summary letter</a> </font>\n",tFile);
#endif
      }
    }
    if (HT_APPL_CONTEXT_TYPE != HT_CONTEXT_DEPOSIT) {
      sprintf(tFile,"%s%s",htmlPath,"check.err");
      sprintf(sFile,"%s%s",valDirPath,"check.err");
      ndb_log_message_text(NDB_MSG_DEBUG,"MK-VAL-REPORT: Search for %s",sFile);
      if (stat(sFile,&statbuf) == 0) {
	fprintf(io,"<li><font size=4>Detailed <a href=\"%s\">validation checks</a></font>\n",tFile);
      }

      sprintf(tFile,"%s%s",htmlPath,"SPLUS.OUT");
      sprintf(sFile,"%s%s",valDirPath,"SPLUS.OUT");
      ndb_log_message_text(NDB_MSG_DEBUG,"MK-VAL-REPORT: Search for %s",sFile);
      if (stat(sFile,&statbuf) == 0) {
	fprintf(io,"<li><font size=4>NCBI <a href=\"%s\">validation checks</a></font>\n",tFile);
      }
    }



    // Neighbors  ...
    //
    sprintf(sFile,"%s%s",dirPath,"sn.url");
    fp = fopen(sFile,"r");
    if (fp != NULL) {
      fgets(urlNeighbors,255,fp);
      ilen = strlen(urlNeighbors);
      if (ilen) {
	if (urlNeighbors[ilen-1] == '\n') urlNeighbors[ilen-1] = '\0';
	fprintf(io,"<li><font size=4>Structure neighbors <a href=\"%s\">summary report</a> </font>\n",
		urlNeighbors);
      }
      fclose(fp);
    }

    fflush(io);

    sprintf(sFile,"%s%s",valDirPath,"sequence-neighbors.html");
    if (stat(sFile,&statbuf) == 0) {
      sprintf(tFile,"%s%s",htmlPath,"sequence-neighbors.html");
      fprintf(io,"<li><font size=4>Sequence neighbors <a href=\"%s\">summary report</a></font>\n",tFile);
    }

    if (iSF) {
      sprintf(pFile,"%s%s",htmlPath,"sfcheck.ps");
      fprintf(io,"<li><font size=4>SFCHECK <a href=\"%s\">summary report</a></font>\n",pFile);
    }

    fprintf(io,"</ul>\n");
  }

  fflush(io);

  // ***************  PROCHECK and PROCHECK NMR  *****************************
  // jdw - care here as the PROCHECK NMR output depends on the number of models.

  if (!strcmp(HT_EXP_METHOD,"NMR") && nModel > 1) {
    p1=0;  
    sprintf(tFile,"%s%s_01.gif",htmlPath,idNamelc);
    sprintf(psFile,"%s%s_01.ps",htmlPath,idNamelc);
    sprintf(sFile,"%s%s_01.gif",valDirPath,idNamelc);
    if (stat(sFile,&statbuf) == 0) p1=1;
    
    if (p1) { // Protein presentation ...
      fprintf(io,"<h2>NMR Protein Validation Report</h2><br>\n");
      fprintf(io,"<ul>\n");
      if (p1) 
	fprintf(io,"<li><font size=4>Ramachandran plot <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",
		tFile,psFile);
      sprintf(tFile,"%s%s_02.gif",htmlPath,idNamelc);
      sprintf(psFile,"%s%s_02.ps",htmlPath,idNamelc);
      sprintf(sFile,"%s%s_02.gif",valDirPath,idNamelc);
      if (stat(sFile,&statbuf) == 0)
	fprintf(io,"<li><font size=4>Ramachandran plots for all residue types <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",tFile,psFile);
      
      sprintf(tFile,"%s%s_03.gif",htmlPath,idNamelc);
      sprintf(psFile,"%s%s_03.ps",htmlPath,idNamelc);
      sprintf(sFile,"%s%s_03.gif",valDirPath,idNamelc);
      if (stat(sFile,&statbuf) == 0)
	fprintf(io,"<li><font size=4>Chi1-Chi2 plots <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",
		tFile,psFile);
      
      sprintf(tFile,"%s%s_04.gif",htmlPath,idNamelc);
      sprintf(psFile,"%s%s_04.ps",htmlPath,idNamelc);
      sprintf(sFile,"%s%s_04.gif",valDirPath,idNamelc);
      if (stat(sFile,&statbuf) == 0)
	fprintf(io,"<li><font size=4>Chi1 frequency distributions <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",tFile,psFile);
      
      sprintf(tFile,"%s%s_05.gif",htmlPath,idNamelc);
      sprintf(psFile,"%s%s_05.ps",htmlPath,idNamelc);
      sprintf(sFile,"%s%s_05.gif",valDirPath,idNamelc);
      if (stat(sFile,&statbuf) == 0)
	fprintf(io,"<li><font size=4>Chi2 frequency distributions <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",tFile,psFile);
      
      sprintf(tFile,"%s%s_06.gif",htmlPath,idNamelc);
      sprintf(psFile,"%s%s_06.ps",htmlPath,idNamelc);
      sprintf(sFile,"%s%s_06.gif",valDirPath,idNamelc);
      if (stat(sFile,&statbuf) == 0)
	fprintf(io,"<li><font size=4>Ensemble Ramachandran plots <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",tFile,psFile);
      
      sprintf(tFile,"%s%s_07.gif",htmlPath,idNamelc);
      sprintf(psFile,"%s%s_07.ps",htmlPath,idNamelc);
      sprintf(sFile,"%s%s_07.gif",valDirPath,idNamelc);
      if (stat(sFile,&statbuf) == 0)
	fprintf(io,"<li><font size=4>Residue properties <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",
		tFile,psFile);
      
      sprintf(tFile,"%s%s_08.gif",htmlPath,idNamelc);
      sprintf(psFile,"%s%s_08.ps",htmlPath,idNamelc);
      sprintf(sFile,"%s%s_08.gif",valDirPath,idNamelc);
      if (stat(sFile,&statbuf) == 0)
	fprintf(io,"<li><font size=4>Equivalent resolution <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",
		tFile,psFile);
      
      sprintf(tFile,"%s%s_09.gif",htmlPath,idNamelc);
      sprintf(psFile,"%s%s_09.ps",htmlPath,idNamelc);
      sprintf(sFile,"%s%s_09.gif",valDirPath,idNamelc);
      if (stat(sFile,&statbuf) == 0)
	fprintf(io,"<li><font size=4>Model secondary structures <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",tFile,psFile);
      
      
      //Procheck Citation.
      fprintf(io,"<br><br><b> The above figures were obtained using PROCHECK V 3.4.4 </b><br>\n");
      fprintf(io,"<font size=2> \n");
      fprintf(io," Laskowski R A, MacArthur M W, Moss D S & Thornton J M (1993). <br>\n");
      fprintf(io,"PROCHECK: a program to check the stereochemical quality of protein structures. <br>\n");
      fprintf(io,"<i>J. Appl. Cryst.</i>, <b>26</b>, 283-291. <br>\n");
      fprintf(io,"<br>\n");
      fprintf(io,"Morris A L, MacArthur M W, Hutchinson E G & Thornton J M (1992). <br>\n");
      fprintf(io,"Stereochemical quality of protein structure coordinates. <br>\n");
      fprintf(io,"<i>Proteins</i>, <b>12</b>, 345-364. <br>\n");
      fprintf(io,"</font>\n");
      
      fprintf(io,"</ul>\n");
    } 
    fflush(io);
  } else if (!strcmp(HT_EXP_METHOD,"NMR") && nModel <= 1) {
    p1=0;  
    sprintf(tFile,"%s%s_01.gif",htmlPath,idNamelc);
    sprintf(psFile,"%s%s_01.ps",htmlPath,idNamelc);
    sprintf(sFile,"%s%s_01.gif",valDirPath,idNamelc);
    if (stat(sFile,&statbuf) == 0) p1=1;
    
    if (p1) { // Protein presentation ...
      fprintf(io,"<h2>NMR Protein Validation Report</h2><br>\n");
      fprintf(io,"<ul>\n");
      if (p1) 
	fprintf(io,"<li><font size=4>Ramachandran plot <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",
		tFile,psFile);
      sprintf(tFile,"%s%s_02.gif",htmlPath,idNamelc);
      sprintf(psFile,"%s%s_02.ps",htmlPath,idNamelc);
      sprintf(sFile,"%s%s_02.gif",valDirPath,idNamelc);
      if (stat(sFile,&statbuf) == 0)
	fprintf(io,"<li><font size=4>Ramachandran plots for all residue types <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",tFile,psFile);
      
      sprintf(tFile,"%s%s_03.gif",htmlPath,idNamelc);
      sprintf(psFile,"%s%s_03.ps",htmlPath,idNamelc);
      sprintf(sFile,"%s%s_03.gif",valDirPath,idNamelc);
      if (stat(sFile,&statbuf) == 0)
	fprintf(io,"<li><font size=4>Chi1-Chi2 plots <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",
		tFile,psFile);
      
      sprintf(tFile,"%s%s_04.gif",htmlPath,idNamelc);
      sprintf(psFile,"%s%s_04.ps",htmlPath,idNamelc);
      sprintf(sFile,"%s%s_04.gif",valDirPath,idNamelc);
      if (stat(sFile,&statbuf) == 0)
	fprintf(io,"<li><font size=4>Main-chain parameters <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",tFile,psFile);
      
      sprintf(tFile,"%s%s_05.gif",htmlPath,idNamelc);
      sprintf(psFile,"%s%s_05.ps",htmlPath,idNamelc);
      sprintf(sFile,"%s%s_05.gif",valDirPath,idNamelc);
      if (stat(sFile,&statbuf) == 0)
	fprintf(io,"<li><font size=4>Side-chain parameters <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",tFile,psFile);
      
      sprintf(tFile,"%s%s_06.gif",htmlPath,idNamelc);
      sprintf(psFile,"%s%s_06.ps",htmlPath,idNamelc);
      sprintf(sFile,"%s%s_06.gif",valDirPath,idNamelc);
      if (stat(sFile,&statbuf) == 0)
	fprintf(io,"<li><font size=4>Residue properties <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",tFile,psFile);
      
      sprintf(tFile,"%s%s_07.gif",htmlPath,idNamelc);
      sprintf(psFile,"%s%s_07.ps",htmlPath,idNamelc);
      sprintf(sFile,"%s%s_07.gif",valDirPath,idNamelc);
      if (stat(sFile,&statbuf) == 0)
	fprintf(io,"<li><font size=4>Main-chain bond lengths <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",
		tFile,psFile);
      
      sprintf(tFile,"%s%s_08.gif",htmlPath,idNamelc);
      sprintf(psFile,"%s%s_08.ps",htmlPath,idNamelc);
      sprintf(sFile,"%s%s_08.gif",valDirPath,idNamelc);
      if (stat(sFile,&statbuf) == 0)
	fprintf(io,"<li><font size=4>Main-chain bond angles <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",
		tFile,psFile);
      
      sprintf(tFile,"%s%s_09.gif",htmlPath,idNamelc);
      sprintf(psFile,"%s%s_09.ps",htmlPath,idNamelc);
      sprintf(sFile,"%s%s_09.gif",valDirPath,idNamelc);
      if (stat(sFile,&statbuf) == 0)
	fprintf(io,"<li><font size=4>RMS distances from planarity <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",tFile,psFile);
      
      
      //Procheck Citation.
      fprintf(io,"<br><br><b> The above figures were obtained using PROCHECK V 3.4.4 </b><br>\n");
      fprintf(io,"<font size=2> \n");
      fprintf(io," Laskowski R A, MacArthur M W, Moss D S & Thornton J M (1993). <br>\n");
      fprintf(io,"PROCHECK: a program to check the stereochemical quality of protein structures. <br>\n");
      fprintf(io,"<i>J. Appl. Cryst.</i>, <b>26</b>, 283-291. <br>\n");
      fprintf(io,"<br>\n");
      fprintf(io,"Morris A L, MacArthur M W, Hutchinson E G & Thornton J M (1992). <br>\n");
      fprintf(io,"Stereochemical quality of protein structure coordinates. <br>\n");
      fprintf(io,"<i>Proteins</i>, <b>12</b>, 345-364. <br>\n");
      fprintf(io,"</font>\n");
      
      fprintf(io,"</ul>\n");
    } 
    fflush(io);


  } else {  // X-ray case...
    p1=0;  
    sprintf(tFile,"%s%s_01.gif",htmlPath,idNamelc);
    sprintf(psFile,"%s%s_01.ps",htmlPath,idNamelc);
    sprintf(sFile,"%s%s_01.gif",valDirPath,idNamelc);
    if (stat(sFile,&statbuf) == 0) p1=1;
    
    if (p1) { // Protein presentation ...
      fprintf(io,"<h2>Protein Validation Report</h2><br>\n");
      fprintf(io,"<ul>\n");
      if (p1) 
	fprintf(io,"<li><font size=4>Ramachandran plot <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",
		tFile,psFile);
      sprintf(tFile,"%s%s_02.gif",htmlPath,idNamelc);
      sprintf(psFile,"%s%s_02.ps",htmlPath,idNamelc);
      sprintf(sFile,"%s%s_02.gif",valDirPath,idNamelc);
      if (stat(sFile,&statbuf) == 0)
	fprintf(io,"<li><font size=4>Ramachandran plots by residue <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",tFile,psFile);
      
      sprintf(tFile,"%s%s_03.gif",htmlPath,idNamelc);
      sprintf(psFile,"%s%s_03.ps",htmlPath,idNamelc);
      sprintf(sFile,"%s%s_03.gif",valDirPath,idNamelc);
      if (stat(sFile,&statbuf) == 0)
	fprintf(io,"<li><font size=4>Chi1-Chi2 plots <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",
		tFile,psFile);
      
      sprintf(tFile,"%s%s_04.gif",htmlPath,idNamelc);
      sprintf(psFile,"%s%s_04.ps",htmlPath,idNamelc);
      sprintf(sFile,"%s%s_04.gif",valDirPath,idNamelc);
      if (stat(sFile,&statbuf) == 0)
	fprintf(io,"<li><font size=4>Main-chain parameters <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",
		tFile,psFile);
      
      sprintf(tFile,"%s%s_05.gif",htmlPath,idNamelc);
      sprintf(psFile,"%s%s_05.ps",htmlPath,idNamelc);
      sprintf(sFile,"%s%s_05.gif",valDirPath,idNamelc);
      if (stat(sFile,&statbuf) == 0)
	fprintf(io,"<li><font size=4>Side-chain parameters <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",
		tFile,psFile);
      
      sprintf(tFile,"%s%s_06.gif",htmlPath,idNamelc);
      sprintf(psFile,"%s%s_06.ps",htmlPath,idNamelc);
      sprintf(sFile,"%s%s_06.gif",valDirPath,idNamelc);
      if (stat(sFile,&statbuf) == 0)
	fprintf(io,"<li><font size=4>Residue properties <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",
		tFile,psFile);
      
      sprintf(tFile,"%s%s_07.gif",htmlPath,idNamelc);
      sprintf(psFile,"%s%s_07.ps",htmlPath,idNamelc);
      sprintf(sFile,"%s%s_07.gif",valDirPath,idNamelc);
      if (stat(sFile,&statbuf) == 0)
	fprintf(io,"<li><font size=4>Main-chain bond distance comparisons <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",tFile,psFile);
      
      sprintf(tFile,"%s%s_08.gif",htmlPath,idNamelc);
      sprintf(psFile,"%s%s_08.ps",htmlPath,idNamelc);
      sprintf(sFile,"%s%s_08.gif",valDirPath,idNamelc);
      if (stat(sFile,&statbuf) == 0)
	fprintf(io,"<li><font size=4>Main-chain bond angle comparisons <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",tFile,psFile);
      
      sprintf(tFile,"%s%s_09.gif",htmlPath,idNamelc);
      sprintf(psFile,"%s%s_09.ps",htmlPath,idNamelc);
      sprintf(sFile,"%s%s_09.gif",valDirPath,idNamelc);
      if (stat(sFile,&statbuf) == 0)
	fprintf(io,"<li><font size=4>RMS deviations from planarity <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",tFile,psFile);
      
      sprintf(tFile,"%s%s_10.gif",htmlPath,idNamelc);
      sprintf(psFile,"%s%s_10.ps",htmlPath,idNamelc);
      sprintf(sFile,"%s%s_10.gif",valDirPath,idNamelc);
      if (stat(sFile,&statbuf) == 0)
	fprintf(io,"<li><font size=4>Summary of geometrical distortions <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",tFile,psFile);
      
      //Procheck Citation.
      fprintf(io,"<br><br><b> The above figures were obtained using PROCHECK V 3.4.4 </b><br>\n");
      fprintf(io,"<font size=2> \n");
      fprintf(io," Laskowski R A, MacArthur M W, Moss D S & Thornton J M (1993). <br>\n");
      fprintf(io,"PROCHECK: a program to check the stereochemical quality of protein structures. <br>\n");
      fprintf(io,"<i>J. Appl. Cryst.</i>, <b>26</b>, 283-291. <br>\n");
      fprintf(io,"<br>\n");
      fprintf(io,"Morris A L, MacArthur M W, Hutchinson E G & Thornton J M (1992). <br>\n");
      fprintf(io,"Stereochemical quality of protein structure coordinates. <br>\n");
      fprintf(io,"<i>Proteins</i>, <b>12</b>, 345-364. <br>\n");
      fprintf(io,"</font>\n");
      
      fprintf(io,"</ul>\n");
    } 
    fflush(io);
  } 
    
  s1=s2=s3=s4=s5=s6=0;
  sprintf(tFile,"%s%s-bond.gif",htmlPath,idNamelc);
  sprintf(psFile,"%s%s-bond.ps",htmlPath,idNamelc);
  sprintf(sFile,"%s%s-bond.gif",valDirPath,idNamelc);
  ndb_log_message_text(NDB_MSG_DEBUG,"Search for %s",sFile);
  if (stat(sFile,&statbuf) == 0) s1=1;
  ndb_log_message_text(NDB_MSG_DEBUG,"Stat returns %d",s1);

  if (s1) {  // NA presentation   
    fprintf(io,"<h2>Nucleic Acid Validation Report</h2>\n");


    fprintf(io,"<br><li><h3>Bond Distance and Angle Comparisons</h3>\n");
    fprintf(io,"<ul>\n");
    sprintf(tFile,"%s%s-bond.gif",htmlPath,idNamelc);
    sprintf(psFile,"%s%s-bond.ps",htmlPath,idNamelc);
    sprintf(sFile,"%s%s-bond.gif",valDirPath,idNamelc);
    if (stat(sFile,&statbuf) == 0) 
      fprintf(io,"<li><font size=4>Sugar and phosphate bond lengths <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",tFile,psFile);

    sprintf(tFile,"%s%s-angle.gif",htmlPath,idNamelc);
    sprintf(psFile,"%s%s-angle.ps",htmlPath,idNamelc);
    sprintf(sFile,"%s%s-angle.gif",valDirPath,idNamelc);
    if (stat(sFile,&statbuf) == 0) 
      fprintf(io,"<li><font size=4>Sugar and phosphate bond angles <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",tFile,psFile);

    sprintf(tFile,"%s%s-bond-A.gif",htmlPath,idNamelc);
    sprintf(psFile,"%s%s-bond-A.ps",htmlPath,idNamelc);
    sprintf(sFile,"%s%s-bond-A.gif",valDirPath,idNamelc);
    if (stat(sFile,&statbuf) == 0) 
      fprintf(io,"<li><font size=4>Adenine bond distances <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a></font>\n",
	      tFile,psFile);

    sprintf(tFile,"%s%s-angle-A.gif",htmlPath,idNamelc);
    sprintf(psFile,"%s%s-angle-A.ps",htmlPath,idNamelc);
    sprintf(sFile,"%s%s-angle-A.gif",valDirPath,idNamelc);
    if (stat(sFile,&statbuf) == 0) 
      fprintf(io,"<li><font size=4>Adenine bond angles <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a></font>\n",
	      tFile,psFile);
	 
    sprintf(tFile,"%s%s-bond-C.gif",htmlPath,idNamelc);
    sprintf(psFile,"%s%s-bond-C.ps",htmlPath,idNamelc);
    sprintf(sFile,"%s%s-bond-C.gif",valDirPath,idNamelc);
    if (stat(sFile,&statbuf) == 0)
      fprintf(io,"<li><font size=4>Cytosine bond distances <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a></font>\n",
	      tFile,psFile);

    sprintf(tFile,"%s%s-angle-C.gif",htmlPath,idNamelc);
    sprintf(psFile,"%s%s-angle-C.ps",htmlPath,idNamelc);
    sprintf(sFile,"%s%s-angle-C.gif",valDirPath,idNamelc);
    if (stat(sFile,&statbuf) == 0)
      fprintf(io,"<li><font size=4>Cystosine bond angles <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a></font>\n",tFile,psFile);

    sprintf(tFile,"%s%s-bond-G.gif",htmlPath,idNamelc);
    sprintf(psFile,"%s%s-bond-G.ps",htmlPath,idNamelc);
    sprintf(sFile,"%s%s-bond-G.gif",valDirPath,idNamelc);
    if (stat(sFile,&statbuf) == 0)
      fprintf(io,"<li><font size=4>Guanine bond distances <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a></font>\n",tFile,psFile);

    sprintf(tFile,"%s%s-angle-G.gif",htmlPath,idNamelc);
    sprintf(psFile,"%s%s-angle-G.ps",htmlPath,idNamelc);
    sprintf(sFile,"%s%s-angle-G.gif",valDirPath,idNamelc);
    if (stat(sFile,&statbuf) == 0)
      fprintf(io,"<li><font size=4>Guanine bond angles <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a></font>\n",
	      tFile,psFile);
	 
    sprintf(tFile,"%s%s-bond-T.gif",htmlPath,idNamelc);
    sprintf(psFile,"%s%s-bond-T.ps",htmlPath,idNamelc);
    sprintf(sFile,"%s%s-bond-T.gif",valDirPath,idNamelc);
    if (stat(sFile,&statbuf) == 0)
      fprintf(io,"<li><font size=4>Thymine bond distances <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a></font>\n",
	      tFile,psFile);

    sprintf(tFile,"%s%s-angle-T.gif",htmlPath,idNamelc);
    sprintf(psFile,"%s%s-angle-T.ps",htmlPath,idNamelc);
    sprintf(sFile,"%s%s-angle-T.gif",valDirPath,idNamelc);
    if (stat(sFile,&statbuf) == 0)
      fprintf(io,"<li><font size=4>Thymine bond angles <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a></font>\n",
	      tFile,psFile);
	 
    sprintf(tFile,"%s%s-bond-U.gif",htmlPath,idNamelc);
    sprintf(psFile,"%s%s-bond-U.ps",htmlPath,idNamelc);
    sprintf(sFile,"%s%s-bond-U.gif",valDirPath,idNamelc);
    if (stat(sFile,&statbuf) == 0)
      fprintf(io,"<li><font size=4>Uracil bond distances <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a></font>\n",
	      tFile,psFile);

    sprintf(tFile,"%s%s-angle-U.gif",htmlPath,idNamelc);
    sprintf(psFile,"%s%s-angle-U.ps",htmlPath,idNamelc);
    sprintf(sFile,"%s%s-angle-U.gif",valDirPath,idNamelc);
    if (stat(sFile,&statbuf) == 0)
      fprintf(io,"<li><font size=4>Uracil bond angles <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a></font>\n",
	      tFile,psFile);

    //
    fprintf(io,"<br><br><b> The above figures are created by NDB NUCHECK and refer to the dictionaries:</b><br>\n");
    fprintf(io,"<font size=2> \n");
    fprintf(io,"Anke Gelbin, Bohdan Schneider, Lester Clowney, Shu-Hsin Hsieh, Wilma K. Olson, and Helen M. Berman. (1996).<br>\n");
    fprintf(io,"Geometric Parameters in Nucleic Acids: Sugar and Phosphate Constituents.<br>\n");
    fprintf(io,"<i>J. Am. Chem. Soc.</i>, <b>118</b>, 519-529.<br>\n");
    fprintf(io,"Lester Clowney, Shri C. Jain, A. R. Srinivasan, John Westbrook, Wilma K. Olson, and Helen M. Berman. (1996)<br>\n");
    fprintf(io,"Geometric Parameters in Nucleic Acids: Nitrogenous Bases. <br>\n");
    fprintf(io,"<i>J. Am. Chem. Soc.</i>, <b>118</b>, 509-518.\n");
    fprintf(io,"</font>\n");

    //
    fprintf(io,"</ul><br>\n");
    fflush(io);
    s2=s3=0;
    sprintf(tFile,"%s%s-tor-scatt.gif",htmlPath,idNamelc);
    sprintf(psFile,"%s%s-tor-scatt.ps",htmlPath,idNamelc);
    sprintf(sFile,"%s%s-tor-scatt.gif",valDirPath,idNamelc);
    if (stat(sFile,&statbuf) == 0) s2 = 1;

    sprintf(tFile,"%s%s-tor-ring.gif",htmlPath,idNamelc);
    sprintf(psFile,"%s%s-tor-ring.ps",htmlPath,idNamelc);
    sprintf(sFile,"%s%s-tor-ring.gif",valDirPath,idNamelc);
    if (stat(sFile,&statbuf) == 0) s3 = 1;
    if (s2 || s3) {
      fprintf(io,"<li><h3>Torsion Angle Comparisons</h3>\n");
      fprintf(io,"<ul>\n");
      sprintf(tFile,"%s%s-tor-scatt.gif",htmlPath,idNamelc);
      sprintf(psFile,"%s%s-tor-scatt.ps",htmlPath,idNamelc);
      sprintf(sFile,"%s%s-tor-scatt.gif",valDirPath,idNamelc);
      if (stat(sFile,&statbuf) == 0) 
	fprintf(io,"<li><font size=4>Scattergrams of selected torsion angles <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",tFile,psFile);

      sprintf(tFile,"%s%s-tor-ring.gif",htmlPath,idNamelc);
      sprintf(psFile,"%s%s-tor-ring.ps",htmlPath,idNamelc);
      sprintf(sFile,"%s%s-tor-ring.gif",valDirPath,idNamelc);
      if (stat(sFile,&statbuf) == 0)
	fprintf(io,"<li><font size=4>Torsion rings for backbone torsion angles <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",tFile,psFile);

      fprintf(io,"<br><br>\n");
      fprintf(io,"<b> The above figures are created by NDB NUCHECK and refer to the conformational preferences described in:</b><br>\n");
      fprintf(io,"<font size=2> \n");
      fprintf(io,"Bohdan Schneider, Stephen Neidle and Helen M. Berman. (1997).<br>\n");
      fprintf(io,"Conformations of the Sugar-Phosphate Backbone in Helical DNA Crystal Structures<br>\n");
      fprintf(io,"<i>Biopoly.</i>, <b>42</b>, 113-124.<br>\n");
      fprintf(io,"</ul>\n");
    }

    fflush(io);
    s4=0;
    sprintf(tFile,"%s%s-mor.gif",htmlPath,idNamelc);
    sprintf(psFile,"%s%s-mor.ps",htmlPath,idNamelc);
    sprintf(sFile,"%s%s-mor.gif",valDirPath,idNamelc);
    if (stat(sFile,&statbuf) == 0) s4 = 1;
    if (s4) {
      fprintf(io,"<br><li><h3>Base Morphology Comparisons</h3>\n");
      fprintf(io,"<ul>\n");
      fprintf(io,"<li><font size=4>Parameter from CURVES 5.1 <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",
	      tFile,psFile);

      fprintf(io,"<br><br>\n");
      fprintf(io,"<b> The above figures were obtained using CURVES V5.1</b><br>\n");
      fprintf(io,"<font size=2> \n");
      fprintf(io,"R. Lavery and H. Sklenar (1988). <br>\n");
      fprintf(io,"The Definition of Generalised Helicoidal Parameters and of Axis Curvature\n");
      fprintf(io,"in Irregular Nucleic Acids. <br>\n");
      fprintf(io,"<i>Biomol. Struct. Dynam.</i>, <b>6</b>, 63-91. <br>\n");
      fprintf(io,"</ul>\n");
    }

    fflush(io);
    s5=s6=0;
    sprintf(tFile,"%s%s-biol-1.gif",htmlPath,idNamelc);
    sprintf(sFile,"%s%s-biol-1.gif",valDirPath,idNamelc);
    if (stat(sFile,&statbuf) == 0) s5 = 1;

    sprintf(tFile,"%s%s-packing.gif",htmlPath,idNamelc);
    sprintf(sFile,"%s%s-packing.gif",valDirPath,idNamelc);
    if (stat(sFile,&statbuf) == 0) s6 = 1;

    if (s5 || s6) {
      fprintf(io,"<br><li><h3>Molecular Visualization</h3>\n");
      fprintf(io,"<ul>\n");
      sprintf(tFile,"%s%s-biol-1.gif",htmlPath,idNamelc);
      sprintf(psFile,"%s%s-biol-1.ps",htmlPath,idNamelc);
      sprintf(sFile,"%s%s-biol-1.gif",valDirPath,idNamelc);
      if (stat(sFile,&statbuf) == 0)
	fprintf(io,"<li><font size=4>Four views of biological unit 1 <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",tFile,psFile);

      sprintf(tFile,"%s%s-biol-2.gif",htmlPath,idNamelc);
      sprintf(psFile,"%s%s-biol-2.ps",htmlPath,idNamelc);
      sprintf(sFile,"%s%s-biol-2.gif",valDirPath,idNamelc);
      if (stat(sFile,&statbuf) == 0)
	fprintf(io,"<li><font size=4>Four views of biological unit 2 <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",tFile,psFile);

      sprintf(tFile,"%s%s-biol-3.gif",htmlPath,idNamelc);
      sprintf(psFile,"%s%s-biol-3.ps",htmlPath,idNamelc);
      sprintf(sFile,"%s%s-biol-3.gif",valDirPath,idNamelc);
      if (stat(sFile,&statbuf) == 0)
	fprintf(io,"<li><font size=4>Four views of biological unit 3 <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",tFile,psFile);

      sprintf(tFile,"%s%s-biol-4.gif",htmlPath,idNamelc);
      sprintf(psFile,"%s%s-biol-4.ps",htmlPath,idNamelc);
      sprintf(sFile,"%s%s-biol-4.gif",valDirPath,idNamelc);
      if (stat(tFile,&statbuf) == 0)
	fprintf(io,"<li><font size=4>Four views of biological unit 4 <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",tFile,psFile);

      sprintf(tFile,"%s%s-packing.gif",htmlPath,idNamelc);
      sprintf(psFile,"%s%s-packing.ps",htmlPath,idNamelc);
      sprintf(sFile,"%s%s-packing.gif",valDirPath,idNamelc);
      if (stat(sFile,&statbuf) == 0)
	fprintf(io,"<li><font size=4>Crystal packing picture <a href=\"%s\">(GIF)</a> <a href=\"%s\">(PS)</a> </font>\n",tFile,psFile);
      fprintf(io,"</ul>\n");

      fprintf(io,"</ul>\n");

    }


  }

  if (HT_APPL_CONTEXT_TYPE != HT_CONTEXT_DEPOSIT) {
    fprintf(io,"<p><br><Center><FORM METHOD=\"POST\" ACTION=\"%s%s%s\">\n",
	    HT_SERVER_NAME,HT_CGI_PATH,HT_INTERFACE_PRINTER_OP);
    fprintf(io,"<INPUT TYPE=\"HIDDEN\" NAME=\"%s\" VALUE=\"%s\">\n",HT_ATTRIBUTE_UNIQUE_ID,ufid);
    fprintf(io,"<b>Enter printer Name: </b><INPUT NAME=\"%s\" type=text  size=\"5\">\n",HT_ATTRIBUTE_PRINTER_ID);
    fprintf(io,"<br><INPUT TYPE=\"SUBMIT\" VALUE=\"Print Report\">\n");
    fprintf(io,"</FORM></center>\n");
  }
}


char *update_db_status(CifFileObj *fobj, const char *dirPath, const char *blockId) {

  int verbose, logging;
  char  script[2048];
  char  loadPath[HT_MAX_NAME_LENGTH];    
  char  odbPath[HT_MAX_NAME_LENGTH];    
  char  errPath[HT_MAX_NAME_LENGTH];    
  char *diags=NULL;

  verbose = NDB_GLOBAL_DEBUG_LEVEL;
  ndb_log_message_text(NDB_MSG_DEBUG,"DB STATUS UPDATE:  dirPath %s blockId %s verbose %d",
		       dirPath,blockId,verbose);

  strcpy(loadPath,dirPath);
  strcat(loadPath,"db1/");
  mkdir(loadPath,0771);

  strcpy(odbPath,loadPath);
  strcat(odbPath,"schema.odb");

  if (verbose >  NDB_MSG_ERR) {
    logging = 1;
  } else {
    logging = 0;
  }

#if 0
  ndb_log_message_text(NDB_MSG_DEBUG,"DB STATUS UPDATE:  HT_SCHEMA_OBJ_PATH %s LoadPath %s",
		       HT_SCHEMA_OBJ_PATH, loadPath); 
  CifSchemaMap *sch = new CifSchemaMap(HT_SCHEMA_OBJ_PATH, logging, loadPath); 
#endif

  ndb_log_message_text(NDB_MSG_DEBUG,"DB STATUS UPDATE:  HT_SCHEMA_CIF_PATH %s LoadPath %s",
		       HT_SCHEMA_CIF_PATH, loadPath); 
  CifSchemaMap *sch = new CifSchemaMap(HT_SCHEMA_CIF_PATH, odbPath, loadPath, logging); 


  ndb_log_message_text(NDB_MSG_DEBUG,"DB STATUS UPDATE:  Loading file ...");
  sch->LoadFileObj(fobj, CifSchemaMap::MODE_SQL);
  char *comName  = sch->GetCommandPath();
  ndb_log_message_text(NDB_MSG_DEBUG,"DB COMMAND comName:    %s",comName);


  strcpy(script,"");
  sprintf(script,"setenv NDB_XDBUSER %s; setenv NDB_XDBPW %s; setenv SYBASE %s; setenv DSQUERY %s; cd %s; chmod u+x %s; %s >& %s%s; ",
	  HT_DBUSER, HT_DBPW, HT_SYBASE_PATH, HT_SYBASE_SERVER, loadPath,comName,comName,loadPath,"dbload.log");

  ndb_log_message_text(NDB_MSG_DEBUG,"Status load script %s",script);
  wsystem(script);


#if 1
  FILE *fp=NULL;
  struct stat statbuf;

  strcpy(errPath,loadPath);
  strcat(errPath,"dbload.log");

  if ( (fp=fopen(errPath,"r")) != NULL ) {
    stat(errPath,&statbuf);
    ndb_log_message(NDB_MSG_DEBUG,"Reading %s size %d",errPath,statbuf.st_size);
    diags = (char *) calloc(statbuf.st_size+1, sizeof(char));
    int len=fread(diags, sizeof(char), statbuf.st_size+1, fp);
    ndb_log_message(NDB_MSG_DEBUG,"Read %d bytes in %s ",len, errPath);
    fclose(fp);    
  }
#endif

  return(diags); 

}



int item_integrity_check(CifBuilder &cb, const char *item, char *value, int mCodeView)
  /*
   *  Return = 1  for OK
   *         = 0  missing optional
   *         = -1 missing mandatory
   *         < -1 other diags.
   */
{
  regex_t preg;
  regmatch_t pmatch[12];

  char *typeCode=NULL;

  char *typeCondition=NULL;
  int ret, icmp;
  uWord nMin, nMax;
  char **cmin=NULL, **cmax=NULL;
  int i, ival,iok, len;
  double  dval;

  ndb_log_message_text(NDB_MSG_DEBUG,"integrity check item %s value %s",item,value);

#if 0
  char *mandatoryCode=NULL;
  int   mandatory;
  mandatoryCode = cb.GetItemMandatoryCode(item);
  if (!strcmp(mandatoryCode,"no")) 
    mandatory = 0;
  else
    mandatory = 1;
#endif

  //
  //  Check for missing values
  //

  if ((value == NULL) || (strlen(value) <= 0) || !strcmp(value,"?") || !strcmp(value,".") ) {
    if (mCodeView==VIEW_ITEM_IS_MANDATORY ) return(-1);
    return (0);
  }


  typeCondition = cb.GetItemRegex(item);
  typeCode = cb.GetItemType(item);
  ndb_log_message_text(NDB_MSG_DEBUG,"typeCondition %s ",typeCondition);
  ndb_log_message_text(NDB_MSG_DEBUG,"typeCode %s ",typeCode);

  if (typeCondition != NULL) {

    icmp=regcomp(&preg, typeCondition, REG_EXTENDED  );

    if (icmp==0) {
      ret= regexec(&preg,value,10,pmatch, 0);
      if (ret != 0) {
	return(-2);
      } else {
	len = pmatch[0].rm_eo - pmatch[0].rm_so;
	if (len != strlen(value) && strcasecmp(typeCode,"text") )  {
	  return(-2);
	}
      }
      regfree(&preg);
    }
  }

  //
  // Range checks  ...
  //

  if (typeCode == NULL) {
    ndb_log_message_text(NDB_MSG_ERR,"Dictionary typeCode is null for %s",item);
    return(1);
  }

  nMin = nMax = 0;
  if (!strcmp(typeCode,"float")) {
    cmin = cb.GetItemRangeMin(item, nMin);
    cmax = cb.GetItemRangeMax(item, nMax);
    ndb_align_string(value);
    dval = atof(value);
    ndb_log_message_text(NDB_MSG_DEBUG,"integrity check item %s value %s",item,value);  

    if (nMin > 0 && nMin == nMax) {
      iok=0;
      for (i=0; i < nMin; i++) {
	if (cmin[i] == NULL && cmax[i] == NULL) {
	  continue;
	} else if (cmin[i] == NULL || (strlen(cmin[i]) == 0)) {
	  if ( dval < atof(cmax[i]) ) {
	    iok++;
	    break;
	  }

	} else if (cmax[i] == NULL || (strlen(cmax[i]) == 0)) {
	  if ( dval > atof(cmin[i]) ) {
	    iok++;
	    break;
	  }
	} else if (!strcmp(cmin[i],cmax[i])) {
	  if ( dval == atof(cmin[i]) ) {
	    iok++;
	    break;
	  }
	} else {
	  if ( dval < atof(cmax[i]) && dval > atof(cmin[i])) {
	    iok++;
	    break;
	  }
	}
      }

      if (iok == 0) {
	for (i=0; i < nMin; i++) {
	  if (cmin[i] == NULL && cmax[i] == NULL) {
	    continue;
	  } else if (cmin[i] == NULL || (strlen(cmin[i]) == 0)) {
	    if ( dval >= atof(cmax[i]) ) {
	      return(-3);
	    }
	    
	  } else if (cmax[i] == NULL || (strlen(cmax[i]) == 0)) {
	    if ( dval <= atof(cmin[i]) ) {
	      return(-3);
	    }
	  } else {
	    if ( dval >= atof(cmax[i]) && dval <= atof(cmin[i])) {
	      return(-3);
	    }
	  }
	  
	}
      }
    }
    
  } else if (!strcmp(typeCode,"int")) {
    cmin = cb.GetItemRangeMin(item, nMin);
    cmax = cb.GetItemRangeMax(item, nMax);
    ndb_align_string(value);
    ival = atoi(value);
    ndb_log_message_text(NDB_MSG_DEBUG,"integrity check item %s value %s",item,value);  

    if (nMin > 0 && nMin == nMax) {
      iok=0;
      for (i=0; i < nMin; i++) {
	if (cmin[i] == NULL && cmax[i] == NULL) {
	  continue;
	} else if (cmin[i] == NULL || (strlen(cmin[i]) == 0)) {
	  if ( ival < atof(cmax[i]) ) {
	    iok++;
	    break;
	  }

	} else if (cmax[i] == NULL || (strlen(cmax[i]) == 0)) {
	  if ( ival > atof(cmin[i]) ) {
	    iok++;
	    break;
	  }
	} else if (!strcmp(cmin[i],cmax[i])) {
	  if ( ival == atof(cmin[i]) ) {
	    iok++;
	    break;
	  }
	} else {
	  if ( ival < atof(cmax[i]) && dval > atof(cmin[i])) {
	    iok++;
	    break;
	  }
	}
	
      }
      if (iok == 0) {
	for (i=0; i < nMin; i++) {
	  if (cmin[i] == NULL && cmax[i] == NULL) {
	    continue;
	  } else if (cmin[i] == NULL || (strlen(cmin[i]) == 0)) {
	    if ( ival >= atoi(cmax[i]) ) {
	      return(-3);
	    }
	    
	  } else if (cmax[i] == NULL || (strlen(cmax[i]) == 0)) {
	    if ( ival <= atoi(cmin[i]) ) {
	      return(-3);
	    }
	  } else {
	    if ( ival >= atoi(cmax[i]) && dval <= atoi(cmin[i])) {
	      return(-3);
	    }
	  }
	  
	}
      }
    }
  }

  return(1);
}


UPD_HANDLE *get_file_upload_status(CifStateObj *sobj) {

    UPD_HANDLE *upd=NULL;

    upd = (UPD_HANDLE *) malloc(sizeof(UPD_HANDLE));
    upd->context = HT_APPL_CONTEXT_TYPE;
    
    char *st = NULL;
    st = sobj->GetSessionStatusCoords();
    upd->iXYZ = 0;
    if (!strcmp(st,"loaded"))  upd->iXYZ = 1;
    delete st;

    st = sobj->GetSessionStatusSF();
    upd->iSF = 0;
    if (!strcmp(st,"loaded"))  upd->iSF = 1;
    delete st;

    st = sobj->GetSessionStatusDF();
    upd->iDF = 0;
    if (!strcmp(st,"loaded"))  upd->iDF = 1;
    delete st;

    st = sobj->GetSessionStatusCC();
    upd->iCC = 0;
    if (!strcmp(st,"loaded"))  upd->iCC = 1;
    delete st;

    st = sobj->GetSessionStatusCS();
    upd->iCS = 0;
    if (!strcmp(st,"loaded"))  upd->iCS = 1;
    delete st;

    st = sobj->GetSessionStatusNC();
    upd->iNC = 0;
    if (!strcmp(st,"loaded"))  upd->iNC = 1;
    delete st;

    st = sobj->GetSessionStatusDC();
    upd->iDC = 0;
    if (!strcmp(st,"loaded"))  upd->iDC = 1;
    delete st;



    return(upd);
}


UPD_HANDLE *adit_file_import_drv(entry entries[], int nentries, const char *ufid, 
				 CifStateObj *sobj) {

    char *fileid=NULL, *formatType=NULL;
    char *fileidSF=NULL, *formatTypeSF=NULL;
    char *fileidNC=NULL, *formatTypeNC=NULL;
    char *fileidDC=NULL, *formatTypeDC=NULL;
    char *fileidDF=NULL, *sidCode=NULL;
    char *fileidCC=NULL, *fileidCS=NULL;
    char dirPath[HT_MAX_NAME_LENGTH];

    UPD_HANDLE *upd=NULL;

    char  archPath[HT_MAX_NAME_LENGTH];     
    char  cifPath[HT_MAX_NAME_LENGTH];     
    char  cifFile[HT_MAX_NAME_LENGTH];     
    char  script[HT_MAX_NAME_LENGTH]; 
    char  idlc[20];
    struct stat statbuf;


    strcpy(dirPath,HT_SESSION_ROOT_PATH);
    strcat(dirPath,ufid);
    strcat(dirPath,"/");

    fileid        = ht_get_keyword_file(HT_ATTRIBUTE_FILE_ID,entries,nentries);
    formatType    = ht_get_keyword_value(HT_ATTRIBUTE_FORMAT_TYPE,entries,nentries);

    fileidSF      = ht_get_keyword_file(HT_ATTRIBUTE_FILE_ID_SF,entries,nentries);
    formatTypeSF  = ht_get_keyword_value(HT_ATTRIBUTE_FORMAT_TYPE_SF,entries,nentries);

    fileidNC      = ht_get_keyword_file(HT_ATTRIBUTE_FILE_ID_NC,entries,nentries);
    formatTypeNC  = ht_get_keyword_value(HT_ATTRIBUTE_FORMAT_TYPE_NC,entries,nentries);

    fileidDC      = ht_get_keyword_file(HT_ATTRIBUTE_FILE_ID_DC,entries,nentries);
    formatTypeDC  = ht_get_keyword_value(HT_ATTRIBUTE_FORMAT_TYPE_DC,entries,nentries);

    fileidDF      = ht_get_keyword_file(HT_ATTRIBUTE_FILE_ID_DF,entries,nentries);

    fileidCC      = ht_get_keyword_file(HT_ATTRIBUTE_FILE_ID_CC,entries,nentries);
    fileidCS      = ht_get_keyword_file(HT_ATTRIBUTE_FILE_ID_CS,entries,nentries);

    sidCode       = ht_get_keyword_value(HT_ATTRIBUTE_SID_CODE,entries,nentries);

    ndb_log_message_text(NDB_MSG_DEBUG,"In adit_file_import_drv: *************");
    ndb_log_message_text(NDB_MSG_DEBUG,"fileid:       %s",fileid);
    ndb_log_message_text(NDB_MSG_DEBUG,"formatType:   %s",formatType);
    ndb_log_message_text(NDB_MSG_DEBUG,"fileidSF:     %s",fileidSF);
    ndb_log_message_text(NDB_MSG_DEBUG,"formatTypeSF: %s",formatTypeSF);
    ndb_log_message_text(NDB_MSG_DEBUG,"fileidNC:     %s",fileidNC);
    ndb_log_message_text(NDB_MSG_DEBUG,"formatTypeNC: %s",formatTypeNC);
    ndb_log_message_text(NDB_MSG_DEBUG,"fileidDF:     %s",fileidDF);    
    ndb_log_message_text(NDB_MSG_DEBUG,"fileidCC:     %s",fileidCC);    
    ndb_log_message_text(NDB_MSG_DEBUG,"fileidCS:     %s",fileidCS);    
    ndb_log_message_text(NDB_MSG_DEBUG,"fileidDC:     %s",fileidDC);
    ndb_log_message_text(NDB_MSG_DEBUG,"formatTypeDC: %s",formatTypeDC);
    ndb_log_message_text(NDB_MSG_DEBUG,"sidCode:      %s",sidCode);


    if (sidCode != NULL) {
      ndb_log_message_text(NDB_MSG_DEBUG,"In adit_file_input_drv: Initial processing *************");
      upd = (UPD_HANDLE *) malloc(sizeof(UPD_HANDLE));

      strcpy(archPath,HT_ARCHIVE_PATH);
      strcat(archPath,HT_ARCHIVE_DIRECTORY);
      strcat(archPath,"/");
      strcpy(idlc,sidCode);
      ndb_set_string_to_lower(idlc);
      strcat(archPath,idlc);
      strcat(archPath,"/");

      strcpy(cifPath,archPath);
      strcat(cifPath,idlc);
      strcat(cifPath,".cif");

      if (stat(cifPath,&statbuf) == 0) { 

	strcpy(cifFile,idlc);
	strcat(cifFile,".cif");
	sprintf(script,"cd %s; cp %s  ./%s.cif", dirPath, cifPath, NDB_DEF_ROOT_FILE_NAME_XYZ);
	ndb_log_message_text(NDB_MSG_DEBUG,"Coordinate file renamed by command: %s",script);      
	if (strlen(script)) wsystem(script);      
	upd->iXYZ=1;
	upd->iSF=0;
	upd->iNC=0;
	upd->iDF=0;
	upd->iCC=0;
	upd->iCS=0;
	upd->iDC=0;

	upd->context = HT_APPL_CONTEXT_TYPE;
	sobj->SetSessionFileCoords(cifFile);
	sobj->SetSessionFormatCoords("cif");
	sobj->SetSessionStatusCoords("loaded");
	sobj->SetSessionSIDCode(sidCode);


	// Pick up SF file if it exists ... 
	//
	strcpy(cifPath,archPath);
	strcat(cifPath,idlc);
	strcat(cifPath,"-sf.cif");

	if (stat(cifPath,&statbuf) == 0) { 
	  strcpy(cifFile,idlc);
	  strcat(cifFile,"-sf.cif");
	  sprintf(script,"cd %s; cp %s  ./%s.cif", dirPath, cifPath, NDB_DEF_ROOT_FILE_NAME_SF);
	  ndb_log_message_text(NDB_MSG_DEBUG,"SF file renamed by command: %s",script);      
	  if (strlen(script)) wsystem(script);      
	  upd->iSF=1;
	  sobj->SetSessionFileSF(cifFile);
	  sobj->SetSessionFormatSF("cif");
	  sobj->SetSessionStatusSF("loaded");
	}
	ndb_log_message_text(NDB_MSG_DEBUG,"adit_file_import_drv: Done1");
	return(upd);
      } else {
	if (html_advise_missing_file(stdout, cifPath, HT_HOME_PATH)) exit(0);
      }
    }
    

    //
    // Must have a coordinate file to continue here so
    // advise user if problem with coordinate input file and quit.
    //
    if (html_advise_missing_file(stdout, fileid, HT_HOME_PATH)) exit(0);

    // 
    // Collect the input files and standardize names by type.
    //
    upd = (UPD_HANDLE *) malloc(sizeof(UPD_HANDLE));
    upd->iXYZ=capture_input_data(fileid,  formatType,  dirPath, NDB_DEF_ROOT_FILE_NAME_XYZ);
    upd->iSF =capture_input_data(fileidSF,formatTypeSF,dirPath, NDB_DEF_ROOT_FILE_NAME_SF);
    //    upd->iNC =capture_input_data(fileidNC,formatTypeNC,dirPath, NDB_DEF_ROOT_FILE_NAME_NC);
    upd->iNC =capture_input_data(fileidNC,"mr",        dirPath, NDB_DEF_ROOT_FILE_NAME_NC);
    upd->iDF =capture_input_data(fileidDF,"txt",       dirPath, NDB_DEF_ROOT_FILE_NAME_FORM);
    upd->iCC =capture_input_data(fileidCC,"any",       dirPath, NDB_DEF_ROOT_FILE_NAME_CC);
    upd->iCS =capture_input_data(fileidCS,"any",       dirPath, NDB_DEF_ROOT_FILE_NAME_CS);
    upd->iDC =capture_input_data(fileidDC,formatTypeDC,dirPath, NDB_DEF_ROOT_FILE_NAME_DC);

    upd->context = HT_APPL_CONTEXT_TYPE;

    if (fileid)       {
      sobj->SetSessionFileCoords(fileid);
      if (formatType)   sobj->SetSessionFormatCoords(formatType);
    }

    if (fileidNC)  {
      sobj->SetSessionFileNC(fileidNC); 
      if (formatTypeNC) sobj->SetSessionFormatNC(formatTypeNC);  
    }

    if (fileidDC)  {
      sobj->SetSessionFileDC(fileidDC); 
      if (formatTypeDC) sobj->SetSessionFormatDC(formatTypeDC);  
    }

    if (fileidSF) {
      sobj->SetSessionFileSF(fileidSF); 
      if (formatTypeSF) sobj->SetSessionFormatSF(formatTypeSF);
    }

    if (fileidDF)     sobj->SetSessionFileDF(fileidDF); 
    if (fileidCC)     sobj->SetSessionFileCC(fileidCC); 
    if (fileidCS)     sobj->SetSessionFileCS(fileidCS); 

    if (upd->iXYZ)    sobj->SetSessionStatusCoords("loaded");
    if (upd->iSF)     sobj->SetSessionStatusSF("loaded");
    if (upd->iDF)     sobj->SetSessionStatusDF("loaded");
    if (upd->iNC)     sobj->SetSessionStatusNC("loaded");
    if (upd->iDC)     sobj->SetSessionStatusDC("loaded");
    if (upd->iCC)     sobj->SetSessionStatusCC("loaded");
    if (upd->iCS)     sobj->SetSessionStatusCS("loaded");

    ndb_log_message_text(NDB_MSG_DEBUG,"adit_file_import_drv: Done");
    return(upd);
}


void adit_data_check_drv(const char *ufid, UPD_HANDLE *upd, CifStateObj *sobj)

{
  
  char *formatType=NULL, *formatTypeSF=NULL;
  int iOpen;
  
  char dirPath[HT_MAX_NAME_LENGTH];
  
  strcpy(dirPath,HT_SESSION_ROOT_PATH);
  strcat(dirPath,ufid);
  strcat(dirPath,"/");
  
  if (!upd->iXYZ && !upd->iSF) {
    ht_put_content_type_target(stdout,"_top");
    html_display_open_page(stdout,"RCSB Data Syntax Check", RCSB_STYLE_DEFAULT);
    html_display_page_title_logo(stdout, "Data Format Check", RCSB_STYLE_DEFAULT);
    //
    html_display_return_to_session(stdout, ufid, "Continue");
    
    printf("<center><font size=8>Sorry, unable to up-load your input file  ...</font></center>");
    ndb_log_message_text(NDB_MSG_ERR,"Unable to open input file");
    html_display_close_page(stdout, RCSB_STYLE_DEFAULT);
    exit(0);
  }
  
  formatType   = sobj->GetSessionFormatCoords();
  formatTypeSF = sobj->GetSessionFormatSF();	

  ht_put_content_type_target(stdout,"_top");
  
  //
  // Check input data ...
  if (HT_APPL_CONTEXT_TYPE==HT_CONTEXT_VALIDATE) {
    iOpen = adit_check_input_data_syntax(stdout, ufid,
					 upd->iXYZ, formatType,   NDB_DEF_ROOT_FILE_NAME_XYZ,
					 upd->iSF,  formatTypeSF, NDB_DEF_ROOT_FILE_NAME_SF,
					 "Continue to Validation");
  } else {
    iOpen = adit_check_input_data_syntax(stdout, ufid,
					 upd->iXYZ, formatType,   NDB_DEF_ROOT_FILE_NAME_XYZ,
					 upd->iSF,  formatTypeSF, NDB_DEF_ROOT_FILE_NAME_SF,
					 "Continue to Validation or Deposition");
  }
    
  //
  // Go here if there are no diagnostics produced...
  //
  
  if (!iOpen) {
    iOpen=1;
    html_display_open_page(stdout,"RCSB Data Syntax Check", RCSB_STYLE_DEFAULT);

    html_display_page_title_logo(stdout, "Data Format Check", RCSB_STYLE_DEFAULT);

    if (HT_APPL_CONTEXT_TYPE==HT_CONTEXT_VALIDATE) 
      html_display_return_to_session(stdout, ufid, "Continue to Validation");
    else
      html_display_return_to_session(stdout, ufid, "Continue to Validation or Deposition");

    if (upd->iXYZ==1) fprintf(stdout,"<br><br><h2>Coordinate format is OK!</h2>\n");      
    if (upd->iSF==1)  fprintf(stdout,"<br><br><h2>Structure factor format is OK!</h2>\n");      
    html_display_close_page(stdout, RCSB_STYLE_DEFAULT);
  }
}


int adit_check_input_data_syntax(FILE *io, const char * ufid, 
				 int flagXYZ, const char *formatTypeXYZ, const char *rootNameXYZ,
				 int flagSF,  const char *formatTypeSF, const char *rootNameSF,
				 const char *returnText) {

  char dirPath[HT_MAX_NAME_LENGTH];
  char diagPath[HT_MAX_NAME_LENGTH];
  char seqPath[HT_MAX_NAME_LENGTH];
  int iOk, iOpen;
  struct stat statbuf;

  strcpy(dirPath,HT_SESSION_ROOT_PATH);
  strcat(dirPath,ufid);
  strcat(dirPath,"/");

  iOpen=0;
  if (flagXYZ ==1) {
    strcpy(diagPath,dirPath);
    strcpy(seqPath,dirPath);
    strcat(diagPath,"xyz.err");
    strcat(seqPath,"filter.out");
    iOk=coordinate_format_ok(formatTypeXYZ, dirPath, rootNameXYZ, diagPath);
    if (!iOpen) {
      html_display_open_page(io,"RCSB Data Syntax Check", RCSB_STYLE_DEFAULT);
      html_display_page_title_logo(io, "Data Syntax Check", RCSB_STYLE_DEFAULT);
      //**********
      html_display_return_to_session(io, ufid, returnText);

      iOpen=1;
    }
    if (!iOk) {
      fprintf(io,"<br><br><h2>Coordinate Syntax Diagnostics:</h2><ul>\n");
      display_raw_file(io,diagPath);
      fprintf(io,"</ul><br><br>");
    } else {
      fprintf(io,"<br><br><h2>Coordinate format is OK!</h2>\n");      

      if (stat(seqPath,&statbuf) == 0) { 
	fprintf(io,"<br><br><h2>Sequence information:</h2><br><pre>\n");      
	display_raw_file(io,seqPath);
	fprintf(io,"</pre>\n"); 
	fprintf(io,"<br><br><font size=4 color=\"#900000\"><b> If a sequence above has missing residues,\n");
	fprintf(io,"please add the missing <br> residues before entering the sequence into the deposition tool.</b></font>\n");      
      }
    }
  }
  
  if (flagSF == 1) {
    strcpy(diagPath,dirPath);
    strcat(diagPath,"sf.err");
    iOk=sf_format_ok(formatTypeSF, dirPath, rootNameSF, diagPath);
    if (!iOpen) {
      html_display_open_page(io,"RCSB Data Syntax Check", RCSB_STYLE_DEFAULT);
      html_display_page_title_logo(io, "Data Syntax Check", RCSB_STYLE_DEFAULT);
      //********
      html_display_return_to_session(io, ufid,returnText);
      iOpen=1;
    }

    if (!iOk) {
      fprintf(io,"<br><br><h2>Structure Factor Syntax Diagnostics:</h2><ul>");
      display_raw_file(io,diagPath);
      fprintf(io,"</ul><br><br>");
    } else {
      fprintf(io,"<br><br><h2>Structure Factor format is OK!</h2><br>\n");      
    }
  }
  if (iOpen) html_display_close_page(io, RCSB_STYLE_DEFAULT);
  return(iOpen);
}



void html_display_return_to_session(FILE *io, const char *ufid, const char *text ) 
{

  fprintf(io,"<center><A href=\"%s%s%s?%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s\" TARGET=\"_top\">",
	  
	  HT_SERVER_NAME,HT_CGI_PATH,HT_INTERFACE_SESSION_DRIVER,
	  HT_ATTRIBUTE_UNIQUE_ID,ufid,
	  HT_ATTRIBUTE_CONTEXT,HT_APPL_CONTEXT,
	  HT_ATTRIBUTE_DBNAME,HT_TARGET_DBNAME,
	  HT_ATTRIBUTE_EMAIL, HT_ANNOTATOR_EMAIL,
	  HT_ATTRIBUTE_VIEW_TYPE, HT_VIEW_TYPE,
	  HT_ATTRIBUTE_PROJDIR, HT_ARCHIVE_DIRECTORY);

  fprintf(io,"<FONT FACE=\"%s\" SIZE=4>%s</FONT></A></center>\n",
	  LANG_HTML_FONT_FACE,text);
}


void adit_validation_drv(const char *ufid, UPD_HANDLE *upd, CifStateObj *sobj, int syncheck) 
{
  char *formatType=NULL, *formatTypeSF=NULL;
  
  char rptPath[HT_MAX_NAME_LENGTH];
  char rptPathHT[HT_MAX_NAME_LENGTH];
  FILE *io=NULL;
  int iOpen;
  
  char dirPath[HT_MAX_NAME_LENGTH];
  
  strcpy(dirPath,HT_SESSION_ROOT_PATH);  // absolute path to data files
  strcat(dirPath,ufid);
  strcat(dirPath,"/");


  strcpy(rptPathHT,HT_SESSION_REL_TMP_ROOT_PATH);   // relative url to report files.
  strcat(rptPathHT,ufid);
  strcat(rptPathHT,"/");
  
  
  formatType   = sobj->GetSessionFormatCoords();
  formatTypeSF = sobj->GetSessionFormatSF();
  ndb_log_message_text(NDB_MSG_DEBUG,"adit-validation-drv: formatType %s formatTypeSF %s",
		       formatType,formatTypeSF);


  if (syncheck) {
    //
    // Check input data ...
    //
    iOpen = adit_check_input_data_syntax(stdout, ufid, 
				    upd->iXYZ, formatType,   NDB_DEF_ROOT_FILE_NAME_XYZ,
				    upd->iSF,  formatTypeSF, NDB_DEF_ROOT_FILE_NAME_SF,
				    "Continue");
    
    if (iOpen) { // exit here if format errors are detected...
      ndb_close_log();    
      exit(0);
    }
  } 
  
  // -----------------------------------------------------------------------------------
  //                      Create validation report ....
  //          (This is created in the session temporary directory...)
  
  //  strcpy(rptPath,dirPath);  

  strcpy(rptPath,HT_SESSION_TMP_ROOT_PATH);  
  strcat(rptPath,ufid);
  strcat(rptPath,"/");
  strcat(rptPath,"validation-report.html");

  strcat(rptPathHT,"validation-report.html");


  io=fopen(rptPath,"w");
  html_display_open_page_w_target(io,"ADIT Validation Report", RCSB_STYLE_DEFAULT, "_top");

  html_display_page_title_logo(io,"ADIT Validation Report", RCSB_STYLE_DEFAULT);

  html_display_return_to_session(io, ufid, "Continue");

  mk_validation_report(io, ufid, NDB_DEFAULT_BLOCK_ID,
		       NDB_DEF_ROOT_FILE_NAME_XYZ, formatType, 
		       NDB_DEF_ROOT_FILE_NAME_SF,  formatTypeSF, 1, NULL, NULL);

  html_display_close_page(io, RCSB_STYLE_DEFAULT);
  fclose(io);


  fprintf(stdout,"Location: %s%c%c",rptPathHT,10,10);
  fprintf(stdout,"Window-target: %s%c%c","_top",10,10);

}



void adit_editor_drv(const char *ufid, UPD_HANDLE *upd, CifStateObj *sobj) {

    char *formatType=NULL;
    char dirPath[HT_MAX_NAME_LENGTH];

    char  script[2048];
    char *viewFile=NULL;
    char *diagnostics=NULL;

    char iViewPath[HT_MAX_NAME_LENGTH];
    char iFilePath[HT_MAX_NAME_LENGTH];
    char fobjPath[HT_MAX_NAME_LENGTH];
    char vobjPath[HT_MAX_NAME_LENGTH];

    char *blockId=NULL, *stateString=NULL;


    strcpy(dirPath,HT_SESSION_ROOT_PATH);
    strcat(dirPath,ufid);
    strcat(dirPath,"/");


    //-----
    //
    // If the input file format is other than mmCIF then perform format translation.
    //
    // Note:  If the current context is DEPOSIT mode then we simply encapsulate 
    //        the coordinates during PDB->mmCIF translation; otherwise, an explicit
    //        translation is performed.

    strcpy(iFilePath,dirPath);
    strcat(iFilePath, NDB_DEF_ROOT_FILE_NAME_XYZ); 
    ndb_log_message_text(NDB_MSG_DEBUG,"adit-editor-drv: iFilePath %s",iFilePath);


    formatType = sobj->GetSessionFormatCoords();

    ndb_log_message_text(NDB_MSG_DEBUG,"adit-editor-drv: formatType %s context %d",
			 formatType, HT_APPL_CONTEXT_TYPE);

    if ( (HT_APPL_CONTEXT_TYPE == HT_CONTEXT_DEPOSIT) || (HT_APPL_CONTEXT_TYPE == HT_CONTEXT_ASSEMBLE) ) {
      if (!strcmp(formatType,"cif")) {
	translate_coordinate_format(formatType,"cif", dirPath, NDB_DEF_ROOT_FILE_NAME_XYZ); 
	strcat(iFilePath,".cif");
      } else if (!strcmp(formatType,"cifeps")) {
	translate_coordinate_format(formatType,"cif", dirPath, NDB_DEF_ROOT_FILE_NAME_XYZ); 
	strcat(iFilePath,".cif");
      } else {
	// Workstation version - go directly to CIF by-passing cifeps..
	//	translate_coordinate_format(formatType,"cifeps", dirPath, NDB_DEF_ROOT_FILE_NAME_XYZ); 
	//      strcat(iFilePath,".cifeps");
	translate_coordinate_format(formatType,"cif", dirPath, NDB_DEF_ROOT_FILE_NAME_XYZ); 
	strcat(iFilePath,".cif");
      }

    } else if (HT_APPL_CONTEXT_TYPE == HT_CONTEXT_ANNOTATE) {
      translate_coordinate_format(formatType,"cif", dirPath, NDB_DEF_ROOT_FILE_NAME_XYZ); 
      strcat(iFilePath,".cif");
    }

    //
    //  Read the input view and construct the view object.  Initialize the table store
    //  and store the view object.
    //
    // View name depends on: context, moltype, and expmeth.
    // Currently, derived by rules encoded in routine:  char *ht_get_view_id();

    if (!viewFile) {
      viewFile=ht_get_view_id(HT_MOL_TYPE,HT_EXP_METHOD, HT_VIEW_TYPE);
    }

    strcpy(vobjPath,dirPath);
    strcat(vobjPath,HT_VIEW_OBJECT_FILE);
    strcpy(iViewPath,HT_CONFIG_PATH);
    strcat(iViewPath,viewFile);

    ndb_log_message_text(NDB_MSG_DEBUG,"Opening view object store %s",vobjPath);
    ndb_log_message_text(NDB_MSG_DEBUG,"Reading view from CIF file %s",iViewPath);
    CifViewObj *vobj = new CifViewObj(iViewPath,vobjPath,HT_VIEW_NAME);
    ndb_log_message_text(NDB_MSG_DEBUG,"Diagnostics reading view %s",
			 vobj->GetViewDiagnostics());

    strcpy(fobjPath,dirPath);
    strcat(fobjPath,HT_TABLE_OBJECT_FILE);
    CifFileObj *fobj = new CifFileObj(fobjPath,WRITE_MODE);
    ndb_log_message_text(NDB_MSG_DEBUG,"Reading CIF file %s",iFilePath);
    fobj->Read(iFilePath, diagnostics);
    ndb_log_message_text(NDB_MSG_DEBUG,"Diagnostics reading data %s",diagnostics);

    blockId = fobj->GetBlockName(0);
    sync_categories_to_view(vobj, fobj, blockId );
    ndb_log_message_text(NDB_MSG_DEBUG,"Sync complete for block %s", blockId);

    stateString=set_category_state_string(vobj, fobj, blockId );
    ndb_log_message_text(NDB_MSG_DEBUG,"State string  %s", stateString);
    sobj->SetSessionStateString(stateString);

    sobj->SetSessionRestartable("true");
    if (stateString) delete stateString;

    auto_update_on_input(upd, fobj, blockId);

#if DCDEMO
    adit_merge_data_collection_data(fobj,blockId,dirPath);
#endif

    fobj->CloseFile();
    vobj->Close();

    ndb_log_message_text(NDB_MSG_DEBUG,"adit_editor_drv: Launch editor ...");

    memset(script,'\0',2048);
    sprintf(script,"%s?%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s",
	    HT_INTERFACE_TOP_MENU_VIEW,
	    HT_ATTRIBUTE_UNIQUE_ID,
	    ufid,
	    HT_ATTRIBUTE_DATABLOCK_ID,
	    blockId,
            HT_ATTRIBUTE_CONTEXT,
	    HT_APPL_CONTEXT,
	    HT_ATTRIBUTE_DBNAME,
	    HT_TARGET_DBNAME,
	    HT_ATTRIBUTE_EMAIL,
	    HT_ANNOTATOR_EMAIL,
	    HT_ATTRIBUTE_PROJDIR,
	    HT_ARCHIVE_DIRECTORY,
	    HT_ATTRIBUTE_EXP_METH, HT_EXP_METHOD,
	    HT_ATTRIBUTE_VIEW_TYPE, HT_VIEW_TYPE,
	    HT_ATTRIBUTE_MOL_TYPE, HT_MOL_TYPE);

    ndb_log_message_text(NDB_MSG_DEBUG,"Calling interface screen with: %s",script);
    ht_put_location_type_script(stdout,script);
    
    //    if (vobj) delete vobj;
    //    if (fobj) delete fobj;
}


#if DCDEMO
static void adit_merge_data_collection_data(CifFileObj *fobj, const char *blockId, const char *dirPath)
{
  char htmlPath[HT_MAX_NAME_LENGTH];
  char dataPath[HT_MAX_NAME_LENGTH];
  char  script[2048];
  char  buf[512];
  char  delimiters[2];
  char  *p1=NULL, *p2=NULL, *p3=NULL;
  int   iLines;
  
  char value[512];
  int i,j,k;

  FILE *fp = NULL;

  strcpy(htmlPath,dirPath);
  strcat(htmlPath,NDB_DEF_ROOT_FILE_NAME_DC); 
  strcat(htmlPath,".html");

  strcpy(dataPath,dirPath);
  strcat(dataPath,NDB_DEF_ROOT_FILE_NAME_DC); 
  strcat(dataPath,".data");

  sprintf(script,"%s %s  %s",
	  HT_HTML_TRANSLATOR_NAME,htmlPath,dataPath);
  ndb_log_message_text(NDB_MSG_DEBUG,"HTML to Text script %s",script);
  wsystem(script);

  delimiters[0] = '\t';
  delimiters[1] = '\0';

  fp = fopen(dataPath,"r");
  if (fp == NULL) {
    ndb_log_message_text(NDB_MSG_INFO,"HTML to Text translation failed for %s",dataPath);
    return;
  }
  iLines = 0;
  while (!feof(fp)) {
    iLines++;
    memset(buf,0,512);
    fgets(buf,511,fp);
    p1 = strtok(buf,delimiters);
    p2 = strtok(NULL,delimiters);
    p3 = strtok(NULL,delimiters);

    ndb_log_message_text(NDB_MSG_INFO,"Line %d p1 %s p2 %s p3 %s",iLines,p1,p2,p3);    

    if ((p1 == NULL) || (p2 == NULL) || (p3 ==NULL)) continue;
    
    rcsb_clean_string(p1);
    rcsb_clean_string(p2);
    rcsb_clean_string(p3);

    if (!strcmp(p1,"1")) {

      if (!strcmp(p2,"a")) {
	if (p3[0] != '&') {  // non null value
	  for (i=0; i < strlen(p3); i++) {
            if (p3[i] == '&') break;
	  }
	  j = i;
	  if (i > 0) {
	    memset(value,0,512);
	    for (i=0; i < j; i++) {
	      value[i] = p3[i];
	    }
	    rcsb_clean_string(value);
	    update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_cell","a",value);
	    update_attribute_value_if_null(fobj,blockId,"cell","length_a",value);
	  }
	  if (j+5 < strlen(p3)) {
	    memset(value,0,512);
	    k=0;
	    for (i=j+5; i < strlen(p3); i++) {
	      value[k] = p3[i]; k++;
	    }
	    rcsb_clean_string(value);
	    update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_cell","a_tolerance",value);
	  }
	}
      } else if (!strcmp(p2,"b")) {
	if (p3[0] != '&') {  // non null value
	  for (i=0; i < strlen(p3); i++) {
            if (p3[i] == '&') break;
	  }
	  j = i;
	  if (i > 0) {
	    memset(value,0,512);
	    for (i=0; i < j; i++) {
	      value[i] = p3[i];
	    }
	    rcsb_clean_string(value);
	    update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_cell","b",value);
	    update_attribute_value_if_null(fobj,blockId,"cell","length_b",value);
	  }
	  if (j+5 < strlen(p3)) {
	    memset(value,0,512);
	    k=0;
	    for (i=j+5; i < strlen(p3); i++) {
	      value[k] = p3[i]; k++;
	    }
	    rcsb_clean_string(value);
	    update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_cell","b_tolerance",value);
	  }
	  
	}
      } else if (!strcmp(p2,"c")) {
	if (p3[0] != '&') {  // non null value
	  for (i=0; i < strlen(p3); i++) {
            if (p3[i] == '&') break;
	  }
	  j = i;
	  if (i > 0) {
	    memset(value,0,512);
	    for (i=0; i < j; i++) {
	      value[i] = p3[i];
	    }
	    rcsb_clean_string(value);
	    update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_cell","c",value);
	    update_attribute_value_if_null(fobj,blockId,"cell","length_c",value);
	  }
	  if (j+5 < strlen(p3)) {
	    k=0;
	    memset(value,0,512);
	    for (i=j+5; i < strlen(p3); i++) {
	      value[k] = p3[i]; k++;
	    }
	    rcsb_clean_string(value);
	    update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_cell","c_tolerance",value);
	  }
	}

      } else if (!strcmp(p2,"alpha")) {
	if (p3[0] != '&') {  // non null value
	  for (i=0; i < strlen(p3); i++) {
            if (p3[i] == '&') break;
	  }
	  j = i;
	  if (i > 0) {
	    memset(value,0,512);
	    for (i=0; i < j; i++) {
	      value[i] = p3[i];
	    }
	    rcsb_clean_string(value);
	    update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_cell","alpha",value);
	    update_attribute_value_if_null(fobj,blockId,"cell","angle_alpha",value);
	  }
	  if (j+5 < strlen(p3)) {
	    memset(value,0,512);
	    k=0;
	    for (i=j+5; i < strlen(p3); i++) {
	      value[k] = p3[i]; k++;
	    }
	    rcsb_clean_string(value);
	    update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_cell","alpha_tolerance",value);
	  }
	}
      } else if (!strcmp(p2,"beta")) {
	if (p3[0] != '&') {  // non null value
	  for (i=0; i < strlen(p3); i++) {
            if (p3[i] == '&') break;
	  }
	  j = i;
	  if (i > 0) {
	    memset(value,0,512);
	    for (i=0; i < j; i++) {
	      value[i] = p3[i];
	    }
	    rcsb_clean_string(value);
	    update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_cell","beta",value);
	    update_attribute_value_if_null(fobj,blockId,"cell","angle_beta",value);
	  }
	  if (j+5 < strlen(p3)) {
	    memset(value,0,512);
	    k=0;
	    for (i=j+5; i < strlen(p3); i++) {
	      value[k] = p3[i];  k++;
	    }
	    rcsb_clean_string(value);
	    update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_cell","beta_tolerance",value);
	  }
	}

      } else if (!strcmp(p2,"gamma")) {
	if (p3[0] != '&') {  // non null value
	  for (i=0; i < strlen(p3); i++) {
            if (p3[i] == '&') break;
	  }
	  j = i;
	  if (i > 0) {
	    memset(value,0,512);
	    for (i=0; i < j; i++) {
	      value[i] = p3[i];
	    }
	    rcsb_clean_string(value);
	    update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_cell","gamma",value);
	    update_attribute_value_if_null(fobj,blockId,"cell","angle_gamma",value);
	  }
	  if (j+5 < strlen(p3)) {
	    k=0;
	    memset(value,0,512);
	    for (i=j+5; i < strlen(p3); i++) {
	      value[k] = p3[i]; k++;
	    }
	    rcsb_clean_string(value);
	    update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_cell","gamma_tolerance",value);
	  }
	}


      } else if (!strcmp(p2,"Resolution")) {
	update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_cell","resolution_range",p3);
      } else if (!strcmp(p2,"Mosaicity")) {
	update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_cell","mosaicity",p3);
      } else if (!strcmp(p1,"1") && !strcmp(p2,"Space Group")) {
	update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_cell","space_group",p3);
	    update_attribute_value_if_null(fobj,blockId,"symmetry","space_group_name_H-M",value);

      }

    }  else if (!strcmp(p1,"2")) {  
      if (!strcmp(p2,"Total Number of Reflections")) {
	update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_reflns","number_all",p3);
	update_attribute_value_if_null(fobj,blockId,"reflns","number_all",p3);
      } else if (!strcmp(p2,"Number of Reflections Marked for Rejection")) {
	update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_reflns","number_marked_reject",p3);
      } else if (!strcmp(p2,"Percentage of Reflections Marked for Rejection")) {
	update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_reflns","percent_marked_reject",p3);
      } else if (!strcmp(p2,"Percentage of Reflections Rejected")) {
	update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_reflns","percent_rejected",p3);
      } else if (!strcmp(p2,"Total Linear R-Factor")) {
	update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_reflns","R_factor_all_linear",p3);
      }

    }  else if (!strcmp(p1,"3")) {  
      if (!strcmp(p2,"Detector")) {
	update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_detector","name",p3);
	update_attribute_value_if_null(fobj,blockId,"diffrn_detector","diffrn_id","1");
	update_attribute_value_if_null(fobj,blockId,"diffrn_detector","type",p3);
	if (strstr(p3,"CCD") != NULL) 
	  update_attribute_value_if_null(fobj,blockId,"diffrn_detector","detector","CCD");
	else if (strstr(p3,"AREA DETECTOR") != NULL) 
	  update_attribute_value_if_null(fobj,blockId,"diffrn_detector","detector","AREA DETECTOR");
	else if (strstr(p3,"IMAGE PLATE") != NULL) 
	  update_attribute_value_if_null(fobj,blockId,"diffrn_detector","detector","IMAGE PLATE");
	else if (strstr(p3,"DIFFRACTOMETER") != NULL) 
	  update_attribute_value_if_null(fobj,blockId,"diffrn_detector","detector","DIFFRACTOMETER");



      } else if (!strcmp(p2,"Wavelength")) {
	update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_detector","wavelength",p3);
	update_attribute_value_if_null(fobj,blockId,"diffrn_source","rcsb_wavelength",p3);
	update_attribute_value_if_null(fobj,blockId,"diffrn_source","diffrn_id","1");

      } else if (!strcmp(p2,"Polarization")) {
	update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_detector","polarization",p3);
      } else if (!strcmp(p2,"X Beam Position")) {
	update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_detector","beam_position_x",p3);
      } else if (!strcmp(p2,"Y Beam Position")) {
	update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_detector","beam_position_y",p3);

      } else if (!strcmp(p2,"Cassette Rotation X")) {
	update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_detector","cassette_rot_x",p3);
      } else if (!strcmp(p2,"Cassette Rotation Y")) {
	update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_detector","cassette_rot_y",p3);
      } else if (!strcmp(p2,"Cassette Rotation Z")) {
	update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_detector","cassette_rot_z",p3);

      } else if (!strcmp(p2,"Yscale")) {
	update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_detector","scale_y",p3);

      } else if (!strcmp(p2,"Skew")) {
	update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_detector","skew",p3);

      } else if (!strcmp(p2,"Crossfire X")) {
	update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_detector","crossfire_x",p3);

      } else if (!strcmp(p2,"Crossfire Y")) {
	update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_detector","crossfire_y",p3);

      } else if (!strcmp(p2,"Crossfire XY")) {
	update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_detector","crossfire_xy",p3);

      } else if (!strcmp(p2,"Date")) {
	update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_detector","date",p3);
	update_attribute_value_if_null(fobj,blockId,"diffrn_detector","ndb_collection_date",p3);

      } else if (!strcmp(p2,"Experimentor")) {
	update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_detector","experimentor",p3);

      } else if (!strcmp(p2,"Crystal")) {
	update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_detector","crystal_data_id",p3);

      } else if (!strcmp(p2,"Processing directory")) {
	update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_detector","processing_path",p3);

      } else if (!strcmp(p2,"Data files")) {
	update_attribute_value_if_null(fobj,blockId,"rcsb_data_processing_detector","processing_files",p3);
      }
    }

  }  // end while
  fclose(fp);

}
#endif

void adit_editor_restart(const char *ufid) 
{
  char dirPath[HT_MAX_NAME_LENGTH];
  char  script[HT_MAX_NAME_LENGTH];
  char category[CIF_MAX_NAME_LENGTH];
  char categoryView[CIF_MAX_NAME_LENGTH];
  char blockId[CIF_MAX_NAME_LENGTH];
  
  
  strcpy(dirPath,HT_SESSION_ROOT_PATH);
  strcat(dirPath,ufid);
  strcat(dirPath,"/");
  
  get_current_view_address(dirPath,blockId,category,categoryView);      
  ndb_log_message_text(NDB_MSG_DEBUG,"Restarting with blockId %s category %s categoryView %s",
		       blockId, category, categoryView);
  
  ndb_log_message_text(NDB_MSG_DEBUG,"Using session directory %s",dirPath);
  ndb_log_message_text(NDB_MSG_DEBUG,"adit-editor-restart: Launch editor ...");
  
  memset(script,'\0',HT_MAX_NAME_LENGTH);
  sprintf(script,"%s?%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s",
	  HT_INTERFACE_TOP_MENU_VIEW,
	  HT_ATTRIBUTE_UNIQUE_ID,
	  ufid,
	  HT_ATTRIBUTE_DATABLOCK_ID,
	  blockId,
	  HT_ATTRIBUTE_CONTEXT,
	  HT_APPL_CONTEXT,
	  HT_ATTRIBUTE_DBNAME,
	  HT_TARGET_DBNAME,
	  HT_ATTRIBUTE_EMAIL,
	  HT_ANNOTATOR_EMAIL,
	  HT_ATTRIBUTE_PROJDIR,HT_ARCHIVE_DIRECTORY,
	  HT_ATTRIBUTE_EXP_METH, HT_EXP_METHOD,
	  HT_ATTRIBUTE_VIEW_TYPE, HT_VIEW_TYPE,
	  HT_ATTRIBUTE_MOL_TYPE, HT_MOL_TYPE);
  
  ndb_log_message_text(NDB_MSG_DEBUG,"adit-editor-restart: script: %s",script);
  ht_put_location_type_script(stdout,script);
  
}



void sync_categories_to_view(CifViewObj *vobj, CifFileObj *fobj, const char *blockId ) {

  int i, iAdd;
  ReVarCifArray<CifString> * cats=NULL;
  ISTable * t = NULL;  
  CifString cs;

  if ((blockId == NULL) || (vobj == NULL) || (fobj == NULL)) return;

  cs.Clear(); cs.Copy("?");
  cats = vobj->GetCategoriesViewable();

  // needed internally... 
  cats->Add("database_2"); 

  //JDW JDW 
  //  cats->Add("ndb_database_status"); 
  cats->Add("pdbx_database_status"); 

  for (i=0; i < cats->Length(); i++) {
    ndb_log_message_text(NDB_MSG_DEBUG,"Synchronizing table %s",(*cats)[i].Text());
    t = fobj->GetTablePtr(blockId, (*cats)[i].Text());
    if (!t)    t = new ISTable((*cats)[i].Text());
    
    if (iAdd=vobj->FixMissingColumns(t, cs)) {
      fobj->WriteTable(t, blockId, (*cats)[i].Text());
      ndb_log_message_text(NDB_MSG_DEBUG,"Wrote table %s adding %d columns",
			   (*cats)[i].Text(),iAdd);
    }
  }
  if (cats) delete cats;
}


char *set_category_state_string(CifViewObj *vobj, CifFileObj *fobj, const char *blockId ) {

  // 
  //  Set the initial completeness category state ...
  //
  ReVarCifArray<CifString> *viewItems=NULL, *aRow=NULL, *cats=NULL;
  ISTable * t = NULL;  
  char *stateString=NULL;
  int *ind=NULL, numCats=0, iCat;
  
  int i, j, numMissMan, numMissOpt;
  
  cats = vobj->GetCategoriesViewable();
  
  numCats = vobj->GetNumViewCategories();
  stateString = (char *) calloc(numCats+1,sizeof(char));
  memset(stateString,'0',numCats);

  
  for (i=0; i < cats->Length(); i++) {
    ndb_log_message_text(NDB_MSG_DEBUG,"Fetching table %s",(*cats)[i].Text());
    iCat = vobj->GetCategoryIndex((*cats)[i].Text());
    if ((iCat < 0) || (iCat > numCats)) continue;
    t = fobj->GetTablePtr(blockId, (*cats)[i].Text());
    viewItems = vobj->GetCategoryItemsViewable((*cats)[i].Text());
    if (t && viewItems && (viewItems->Length() > 0)) {
      numMissMan = numMissOpt = 0;
      ind = vobj->GetItemColumnIndices(t,viewItems);
      if (ind && (t->GetNumRows() > 0)) {
	aRow = t->GetRow(0); // perform tests on first row ...
	
	for (j=0; j < viewItems->Length(); j++) {
	  if ( ((*aRow)[ind[j]].Length() <= 0 ) || !strcmp( (*aRow)[ind[j]].Text(), "?") ) {
	    if (vobj->GetItemMandatoryCode((*viewItems)[j].Text()) == VIEW_ITEM_IS_MANDATORY ) {
	      numMissMan++;
	    } else {
	      numMissOpt++;
	    }
	  }
	}
	if (aRow) delete aRow;
      }
      if (ind) delete ind;
      if (numMissMan) {
	stateString[iCat] = '2';
      } else if (numMissOpt) {
	stateString[iCat] = '1';
      } else {
	stateString[iCat] = '0';
      }
    } else {
      stateString[iCat] = '0';  // no viewable items, so implicitly this category is ok.
    }
    if (viewItems) delete viewItems;
  }
  if (cats) delete cats;
  
  return (stateString);
  
}


void encode_table_param_name(char name[MAX_CIF_NAME_LENGTH], int row, int col, int altFlag) {
  CifString cs;
  cs.Clear();
  if (altFlag)
    cs.Copy("ALT_R"); 
  else
    cs.Copy("_R"); 
  cs += row; cs += "_C"; cs += col;
  strncpy(name,cs.Text(),MAX_CIF_NAME_LENGTH-1);
}

int decode_table_param_name(char *name, int &row, int &col, int altFlag) {
  char *r, *c;
  char crow[20], ccol[20];
  int il;

  if (altFlag) {
    if (strncmp(name,"ALT_R",5)) return 0;
    r = &name[5];
  } else {
    if (strncmp(name,"_R",2)) return 0;
    r = &name[2];
  }
  c = strstr(r,"C"); 
  if (c == NULL) return 0;

  il = (int) (c - r);
  strncpy(crow,r,il); row = atoi(crow);
  c++;
  strcpy(ccol,c);     col = atoi(ccol);
  return 1;
}


char *get_dictionary_item_name(CifBuilder &cb, const char *item)
{
  char *s=NULL;
  char **iList=NULL;
  int i;
  uWord nList=0;
  iList = cb.GetDictionaryItems(nList);
  for (i=0; i < nList; i++) {
    if (!strcasecmp(iList[i],item) ) {
      s = (char *)calloc(strlen(iList[i])+1,sizeof(char));
      strcpy(s,iList[i]);
      return(s);
    }
  }
  return NULL;

}

void write_category_status(const char *fName, const char *string)
{
  FILE *fp=NULL;
  if (!fName || !strlen(fName) || !string ||  !strlen(string)) return;

  fp = fopen(fName, "w");
  if (fp != NULL) {
    fprintf(fp,"%s\n",string);
    fclose(fp);
  }
}



void  mk_category_frame_from_view_obj(char *fname, char *ufid, CifViewObj *vobj)
{
  //
  // fname is relative to HT server root directory .
  //
    ReVarCifArray<CifString> *c0 = NULL;
    ReVarCifArray<CifString> *c3 = NULL;
    ReVarCifArray<CifString> *c4 = NULL;
    CifString *csp=NULL;

    FILE *fp;
    char  fn[HT_ENV_MAX_NAME_LEN];
    char  dirPath[HT_ENV_MAX_NAME_LEN];

    char  statePath[HT_ENV_MAX_NAME_LEN];
    char *cStatus = NULL;
    int  lenStatus=0;

    int i, j, iCat;
    char cstat;

    char group[HT_ENV_MAX_NAME_LEN];

    strcpy(dirPath,HT_SESSION_ROOT_PATH);
    strcat(dirPath,ufid);
    strcat(dirPath,"/");

    strcpy(fn,HT_SERVER_PATH);
    strcat(fn,fname);
    ndb_log_message_text(NDB_MSG_DEBUG,"Category frame filename is  %s\n",fn);

    strcpy(statePath,dirPath);
    strcat(statePath,HT_STATE_FILE);
    CifStateObj * sobj = NULL;
    sobj = new CifStateObj(statePath, "state", APPEND_MODE, 0);
    if (sobj) {
      cStatus = sobj->GetSessionStateString();
      sobj->Close();
    }
    if (cStatus) lenStatus = strlen(cStatus);

    ndb_log_message(NDB_MSG_DEBUG,"Updating category status with string (len=%d) is %s",
		    lenStatus,cStatus);


    fp=fopen(fn,"w");
    fprintf(fp,"<HTML>");
    fprintf(fp,"<BODY BGCOLOR=\"#ffffff\" TEXT=\"#000000\" LINK=\"#0000ff\" VLINK=\"#0000ff\">\n");

    fprintf(fp,"<table vspace=0>\n");


    c0= vobj->GetGroups();
    for (i=0; i < c0->Length(); i++) {

      c3 = vobj->GetGroupCategoriesViewable((*c0)[i].Text());
      if (c3 && c3->Length() > 0) {
	csp=vobj->GetGroupDescription((*c0)[i].Text());
	if (csp){
	  fprintf(fp,"<tr><td colspan=2 align=left><b>%s</b></td></tr>\n",csp->Text());
	  delete(csp); csp = NULL;
	} else {
	  fprintf(fp,"<tr><td colspan=2 align=left><b>%s</b></td></tr>\n",(*c0)[i].Text());
	}

	//	for (j=0; j < c3->Length(); j++) { // write a list of anchors for category names in this group.
	//          fprintf(fp,"<a name=\"%s\"></a>\n",(*c3)[j].Text());	  
	//       } 



	strcpy(group,(*c0)[i].Text());
	spacetoplus(group);

	c4=vobj->GetGroupCategoryViewNamesViewable((*c0)[i].Text());
	if (!c4) {
	  ndb_log_message_text(NDB_MSG_ERR,"No view categories in group %s",(*c0)[i].Text());
	} else if ( c4->Length() != c3->Length() ) {
	  ndb_log_message_text(NDB_MSG_ERR,"Group %s length mismatch in view c= %d .ne. v= %d",
			       (*c0)[i].Text(),c3->Length(),c4->Length());
	}

	// c3 has category names
        // c4 has category view names
        //
	for (j=0; j < c3->Length(); j++) {

	  iCat = vobj->GetViewCategoryIndex((*c4)[j].Text());

	  ndb_log_message_text(NDB_MSG_DEBUG,"index iCat %d",iCat);

	  //	  ndb_log_message_text(NDB_MSG_DEBUG,"Category %s view category %s status index ind %d len %d",
	  //		       (*c3)[j].Text(), (*c4)[j].Text(), iCat,lenStatus);

	  if (iCat >= 0 && iCat < lenStatus) 
	    cstat = cStatus[iCat];
	  else
	    cstat = '2';

	  if (HT_APPL_CONTEXT_TYPE == HT_CONTEXT_DEPOSIT) {
	    fprintf(fp,"<tr>");
	  } else {
	    if (cstat == '2') {
	      fprintf(fp,"<tr><td align=left><font size=3 color=\"#900000\"><b>>></b>&nbsp</font></td>\n");
	    } else if (cstat == '1') {
	      fprintf(fp,"<tr><td align=right><font size=3 color=\"#900000\"> <b>></b>&nbsp</font></td>\n");
	    } else 
	      fprintf(fp,"<tr><td align=left><font size=3 color=\"#900000\">&nbsp</font></td>\n");
	  }

	  fprintf(fp,"<td align=left><font size=3>\n");
	  if ( (*c4)[j].Text() && (*c4)[j].Length() )
	    fprintf(fp,"<A name=\"%s\" HREF=\"%s%s%s?%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s\" TARGET=\"item\"> %s </A>\n",
		    (*c3)[j].Text(),
		  HT_SERVER_NAME,HT_CGI_PATH,HT_INTERFACE_ITEM_FORM_VIEW,
		  HT_ATTRIBUTE_UNIQUE_ID,ufid,
		  HT_ATTRIBUTE_CATEGORY_ID, (*c3)[j].Text(),
		  HT_ATTRIBUTE_ROW_ID,"0",
		  HT_ATTRIBUTE_ITEM_FORM_ACTION,"new",
		  HT_ATTRIBUTE_CONTEXT,HT_APPL_CONTEXT,
		    HT_ATTRIBUTE_EXP_METH, HT_EXP_METHOD,
		  HT_ATTRIBUTE_PROJDIR,HT_ARCHIVE_DIRECTORY,
		  (*c4)[j].Text());
	  else
	    fprintf(fp,"<A name=\"%s\" HREF=\"%s%s%s?%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s&%s=%s\" TARGET=\"item\"> %s </A>\n",
		    (*c3)[j].Text(),
		  HT_SERVER_NAME,HT_CGI_PATH,HT_INTERFACE_ITEM_FORM_VIEW,
		  HT_ATTRIBUTE_UNIQUE_ID,ufid,
		  HT_ATTRIBUTE_CATEGORY_ID, (*c3)[j].Text(),
		  HT_ATTRIBUTE_ROW_ID,"0",
		  HT_ATTRIBUTE_ITEM_FORM_ACTION,"new",
		  HT_ATTRIBUTE_CONTEXT,HT_APPL_CONTEXT,
		    HT_ATTRIBUTE_EXP_METH, HT_EXP_METHOD,
		  HT_ATTRIBUTE_PROJDIR,HT_ARCHIVE_DIRECTORY,
		  (*c3)[j].Text());

	  fprintf(fp,"</font></td></tr>");
	}
	if (c4) delete c4;
	//	if (csp) delete csp;
      }
      if (c3) delete c3;
    }
    if (c0) delete c0;
    
    fprintf(fp,"</table>\n");
    fprintf(fp,"</BODY>");
    fprintf(fp,"</HTML>");
    fclose(fp);
}


ReVarCifArray<CifString> *mk_category_list_from_view_obj(CifViewObj *vobj)
{
  ReVarCifArray<CifString> *c0 = NULL;
  ReVarCifArray<CifString> *c3 = NULL;
  
  int i, j;

  ReVarCifArray<CifString> * ret = new ReVarCifArray<CifString>;

  c0= vobj->GetGroups();
  for (i=0; i < c0->Length(); i++) {  // over groups
    // c3 has category names
    c3 = vobj->GetGroupCategoriesViewable((*c0)[i].Text());
    if (c3 && c3->Length() > 0) {
      for (j=0; j < c3->Length(); j++) { // over categories in group
	ndb_log_message_text(NDB_MSG_DEBUG,"Group %s category %s",(*c0)[i].Text(),(*c3)[j].Text() );
	ret->Add((*c3)[j].Text());
      }
    }
    if (c3) delete c3;
  }
  if (c0) delete c0;
  return(ret);
}



CIFVIEWOBJITEM *build_item_obj_list(CifViewObj *vobj, const char *category, int *nItems)
{

  CIFVIEWOBJITEM *itemobj=NULL;
  ReVarCifArray<CifString> *items = NULL;

  int i;
  items = vobj->GetCategoryItems(category);
  itemobj = (CIFVIEWOBJITEM *) malloc(items->Length()*sizeof(CIFVIEWOBJITEM));
  //  ndb_log_message_text(NDB_MSG_DEBUG,"Vobj Category %s has %d items",category,items->Length());

  for (i=0; i < items->Length(); i++) {
    strcpy(itemobj[i].category, category);
    strcpy(itemobj[i].item,    (*items)[i].Text());
    vobj->GetItemInfo((*items)[i].Text(), itemobj[i].viewName, 
		      itemobj[i].mandatoryCode, itemobj[i].allowAlternateCode);
    itemobj[i].defaultValue = NULL;
    //    ndb_log_message_text(NDB_MSG_DEBUG,"Vobj Category %s",category);
    //    ndb_log_message_text(NDB_MSG_DEBUG,"Vobj item     %s",itemobj[i].item);
    //    ndb_log_message_text(NDB_MSG_DEBUG,"Vobj viewname %s",itemobj[i].viewName);
    //    ndb_log_message_text(NDB_MSG_DEBUG,"Vobj mcode    %d",itemobj[i].mandatoryCode);
    //    ndb_log_message_text(NDB_MSG_DEBUG,"Vobj acode    %d",itemobj[i].allowAlternateCode);
    //    ndb_log_message_text(NDB_MSG_DEBUG,"Vobj defval   %s",itemobj[i].defaultValue);
  }
  *nItems = items->Length();
  if (items) delete items;
  return(itemobj);
}


CIFVIEWOBJITEM *build_item_obj_list_w_table(CifViewObj *vobj, const char *category, int *nItems, ISTable *t)
{
  CIFVIEWOBJITEM *itemobj=NULL;
  ReVarCifArray<CifString> *items = NULL;
  ReVarCifArray<CifString> *colNames = NULL;
  char tmpS[MAX_VIEW_NAME_LENGTH];
  int i, j, k, iok, nCols, len;

  nCols=t->GetNumColumns();
  colNames = t->GetColumnNames();

  items = vobj->GetCategoryItems(category);
  len  = items->Length();
  if (nCols > len) {
    itemobj = (CIFVIEWOBJITEM *) malloc(nCols*sizeof(CIFVIEWOBJITEM));
  } else {
    itemobj = (CIFVIEWOBJITEM *) malloc(len*sizeof(CIFVIEWOBJITEM));
  }
  
  for (i=0; i < len; i++) {
    strcpy(itemobj[i].category, category);
    strcpy(itemobj[i].item,     (*items)[i].Text());
    itemobj[i].defaultValue = NULL;
    vobj->GetItemInfo((*items)[i].Text(), 
		      itemobj[i].viewName, 
		      itemobj[i].mandatoryCode, 
		      itemobj[i].allowAlternateCode);
  }
  // 
  //  Add items not in view as hidden items...
  //
  k=len;
  for (i=0; i < nCols; i++) {
    iok=0;
    strcpy(tmpS,"_"); strcat(tmpS,category); strcat(tmpS,"."); strcat(tmpS,(*colNames)[i].Text());
    for (j=0; j < len; j++) {
      if ( !strcmp( itemobj[j].item, tmpS )) {
	iok=1;
	break;
      }
    }
    if (!iok) {
      strcpy(itemobj[k].category, category);
      strcpy(itemobj[k].item,     tmpS);
      strcpy(itemobj[k].item,     tmpS);
      strcpy(itemobj[k].viewName, tmpS);
      itemobj[k].mandatoryCode=VIEW_ITEM_IS_HIDDEN;
      itemobj[k].allowAlternateCode=0;
      itemobj[k].defaultValue = NULL;
      k++;
    }
    
  }
  if (k != nCols) ndb_log_message_text(NDB_MSG_ERR,"Error in item count k %d  nCols %d",k,nCols);

  if (colNames) delete colNames;
  if (items) delete items;

  *nItems = nCols;
  return(itemobj);
}

static void  fraction_to_percent(CifFileObj *fobj, const char *blockId, 
				 const char *category, const char *attribute)
{
  // Express R factors in percent ..
  char value[20];
  double rval;
  char *curVal=NULL;

  curVal=get_attribute_value(fobj,blockId, category, attribute);
  ndb_log_message_text(NDB_MSG_DEBUG,"For %s got value %s",attribute,curVal);
  if (curVal != NULL && strlen(curVal) > 0 && curVal[0] != '?') {
    rval = atof(curVal);
    ndb_log_message_text(NDB_MSG_DEBUG,"Conversion of value %s is %f",curVal,rval);
    if (rval < 1.0001) {
      rval = rval*100.0;
      sprintf(value,"%5.2f",rval);
      ndb_align_string(value);
      update_attribute_value(fobj,blockId, category, attribute, value);    
    }
  }
}


static void  percent_to_fraction(CifFileObj *fobj, const char *blockId, 
				 const char *category, const char *attribute)
{
  // Express R factors as fractions ..
  char value[20];
  double rval;
  char *curVal=NULL;

  curVal=get_attribute_value(fobj,blockId, category, attribute);
  ndb_log_message_text(NDB_MSG_DEBUG,"For %s got value %s",attribute,curVal);
  if (curVal != NULL && strlen(curVal) > 0 && curVal[0] != '?') {
    rval = atof(curVal);
    ndb_log_message_text(NDB_MSG_DEBUG,"Conversion of value %s is %f",curVal,rval);
    if (rval > 1.0000) {
      rval = rval/100.0;
      sprintf(value,"%6.3f",rval);
      ndb_align_string(value);
      update_attribute_value(fobj,blockId, category, attribute, value);    
    }
  }
}


void scale_up_R_factor_values(CifFileObj *fobj, const char *blockId) {
  fraction_to_percent(fobj,blockId,"ndb_refine","R_factor_all_4sig_cutoff");
  fraction_to_percent(fobj,blockId,"ndb_refine","R_factor_all_no_cutoff");
  fraction_to_percent(fobj,blockId,"ndb_refine","R_factor_obs_4sig_cutoff");
  fraction_to_percent(fobj,blockId,"ndb_refine","R_factor_obs_no_cutoff");
  fraction_to_percent(fobj,blockId,"ndb_refine","free_R_factor_4sig_cutoff");
  fraction_to_percent(fobj,blockId,"ndb_refine","free_R_factor_no_cutoff");
  fraction_to_percent(fobj,blockId,"refine","ls_R_factor_R_free");
  //  fraction_to_percent(fobj,blockId,"refine","ls_R_factor_R_free_error");
  //  fraction_to_percent(fobj,blockId,"refine","ls_R_factor_R_free_error_details");
  fraction_to_percent(fobj,blockId,"refine","ls_R_factor_R_work");
  fraction_to_percent(fobj,blockId,"refine","ls_R_factor_all");
  fraction_to_percent(fobj,blockId,"refine","ls_R_factor_obs");
  fraction_to_percent(fobj,blockId,"refine_ls_shell","R_factor_R_free");
  //  fraction_to_percent(fobj,blockId,"refine_ls_shell","R_factor_R_free_error");
  fraction_to_percent(fobj,blockId,"refine_ls_shell","R_factor_R_work");
  fraction_to_percent(fobj,blockId,"reflns","ndb_Rmerge_I_obs");
  fraction_to_percent(fobj,blockId,"reflns_shell","Rmerge_I_obs");
}


void scale_down_R_factor_values(CifFileObj *fobj, const char *blockId) {
  percent_to_fraction(fobj,blockId,"ndb_refine","R_factor_all_4sig_cutoff");
  percent_to_fraction(fobj,blockId,"ndb_refine","R_factor_all_no_cutoff");
  percent_to_fraction(fobj,blockId,"ndb_refine","R_factor_obs_4sig_cutoff");
  percent_to_fraction(fobj,blockId,"ndb_refine","R_factor_obs_no_cutoff");
  percent_to_fraction(fobj,blockId,"ndb_refine","free_R_factor_4sig_cutoff");
  percent_to_fraction(fobj,blockId,"ndb_refine","free_R_factor_no_cutoff");
  percent_to_fraction(fobj,blockId,"refine","ls_R_factor_R_free");
  //  percent_to_fraction(fobj,blockId,"refine","ls_R_factor_R_free_error");
  //  percent_to_fraction(fobj,blockId,"refine","ls_R_factor_R_free_error_details");
  percent_to_fraction(fobj,blockId,"refine","ls_R_factor_R_work");
  percent_to_fraction(fobj,blockId,"refine","ls_R_factor_all");
  percent_to_fraction(fobj,blockId,"refine","ls_R_factor_obs");
  percent_to_fraction(fobj,blockId,"refine_ls_shell","R_factor_R_free");
  //  percent_to_fraction(fobj,blockId,"refine_ls_shell","R_factor_R_free_error");
  percent_to_fraction(fobj,blockId,"refine_ls_shell","R_factor_R_work");
  percent_to_fraction(fobj,blockId,"reflns","ndb_Rmerge_I_obs");
  percent_to_fraction(fobj,blockId,"reflns_shell","Rmerge_I_obs");
}




void print_table_attribute_name(FILE *fp, const char *string)
{
   char *p;
   if (string == NULL || (strlen(string) == 0)) return;
   p=strstr((char*)string,".");
   if ( (string[0] == '_') && (p != NULL)) {
     p++;
     fprintf(fp,"%s",p);
   } else {
     fprintf(fp,"%s",string);
   }
}


int display_view_category_as_table(FILE *fp, CifFileObj *fobj, CifViewObj *vobj,  
				   const char *blockId, const char *category ) {
  int j,k, nRow=0, nColsView=0;
  ReVarCifArray<CifString> * rowData = NULL, * colNamesView = NULL, *colLabels = NULL;
  int *ind=NULL, dCode, iBlank, iColBlank=1;
  ISTable * t=NULL;

  CIFVIEWOBJITEM *itemList=NULL;
  itemList = build_item_obj_list(vobj, category, &nColsView);    


  if ((t=fobj->GetTablePtr(blockId,category)) != NULL) {

    nRow=t->GetNumRows();

    CifString * categoryViewName = vobj->GetViewCategory(category);
    
    if (colNamesView) delete colNamesView;
    colNamesView = vobj->GetCategoryItems(category);

    if (colLabels) delete colLabels;
    colLabels = vobj->GetCategoryViewItems(category);

    ind = vobj->GetItemColumnIndices(t,colNamesView);
    dCode = vobj->GetCategoryDisplayCode(category);
    
    fprintf(fp,"<FONT SIZE=5>%s</FONT>",categoryViewName->Text());
    fprintf(fp,"<TABLE BORDER CELLPADDING=4>\n");

    if (nRow > 1) { // Format table with multiple rows ...

      fprintf(fp,"<TR>\n");
      for (j=0; j < colNamesView->Length(); j++) {
	if (itemList[j].mandatoryCode == VIEW_ITEM_IS_HIDDEN) continue;
	fprintf(fp,"<TH ALIGN=CENTER VALIGN=MIDDLE><FONT SIZE=3>");
	print_table_attribute_name(fp, (*colLabels)[j].Text() );
	fprintf(fp,"</FONT></TH>\n");
      }
      fprintf(fp,"</TR>\n");
      
      for (j=0; j < nRow; j++) {
	if (rowData) delete rowData;
 	ndb_log_message_text(NDB_MSG_DEBUG,"View category as table  - row %d",j);
	rowData = t->GetRow(j);
	iBlank=0;
	if (dCode == 7) {
	  iBlank = 1;
	  for (k=iColBlank; k < colNamesView->Length(); k++) {
	    if (((*rowData)[ind[k]].Length() > 0) && strcmp((*rowData)[ind[k]].Text(),"?") &&
		strcmp((*rowData)[ind[k]].Text(),".") ) {
	      iBlank = 0;
	      break;
	    }
	  }
	}
	if (iBlank) continue;
	fprintf(fp,"<TR>\n");	
	for (k =0; k < colNamesView->Length(); k++) {
	if (itemList[k].mandatoryCode == VIEW_ITEM_IS_HIDDEN) continue;
	  fprintf(fp,"<TD ALIGN=LEFT VALIGN=MIDDLE>");
	  if ((*rowData)[ind[k]].Length() > 0)
	    fprintf(fp,"<FONT SIZE=3> %s </FONT>",(*rowData)[ind[k]].Text());
	  else 
	    fprintf(fp,"<FONT SIZE=3> %s </FONT>","&nbsp");
	  fprintf(fp,"</TD>\n");
	}
	fprintf(fp,"</TR>\n");
      }
    } else { // Format table with single row ...
      if (rowData) delete rowData;
      ndb_log_message_text(NDB_MSG_DEBUG,"View category as table  - row 0");
      rowData = t->GetRow(0);
      for (j=0; j < colNamesView->Length(); j++) {
	if (itemList[j].mandatoryCode == VIEW_ITEM_IS_HIDDEN) continue;

	fprintf(fp,"<TR>\n");
	fprintf(fp,"<TD ALIGN=CENTER VALIGN=MIDDLE><FONT SIZE=3><b>");
	print_table_attribute_name(fp, (*colLabels)[j].Text() );
	fprintf(fp,"</b></FONT></TD>\n");
	
	fprintf(fp,"<TD ALIGN=LEFT VALIGN=MIDDLE>");
	if ((*rowData)[ind[j]].Length() > 0)
	  fprintf(fp,"<FONT SIZE=3> %s </FONT>",(*rowData)[ind[j]].Text());
	else 
	  fprintf(fp,"<FONT SIZE=3> %s </FONT>","&nbsp");
	fprintf(fp,"</TD>\n");
	fprintf(fp,"</TR>\n");

      }
    }
    fprintf(fp,"</TABLE><BR><BR>\n");
    
  } else {
    return (0);
  }
  

  if (colNamesView) delete colNamesView;
  if (colLabels) delete colLabels;
  if (rowData) delete rowData;
  if (ind) delete ind;
  return(nRow);
}


int display_view_category_as_table_w_check(FILE *fp, CifFileObj *fobj, CifViewObj *vobj,  CifBuilder &cb,
					   const char *blockId, const char *category ) {
  int j,k, nRow=0, nColsView=0;
  ReVarCifArray<CifString> * rowData = NULL, * colNamesView = NULL, *colLabels = NULL;
  int *ind=NULL;
  ISTable * t=NULL;
  CIFVIEWOBJITEM *itemList=NULL;
  int iCheck, dCode, iBlank, iColBlank=1;
  itemList = build_item_obj_list(vobj, category, &nColsView);    

  if ((t=fobj->GetTablePtr(blockId,category)) != NULL) {
    nRow=t->GetNumRows();
    ndb_log_message_text(NDB_MSG_DEBUG,"Preview -  category %s rows %d",category,nRow);

    CifString * categoryViewName = vobj->GetViewCategory(category);
    
    if (colNamesView) delete colNamesView;
    colNamesView = vobj->GetCategoryItems(category);

    if (colLabels) delete colLabels;
    colLabels = vobj->GetCategoryViewItems(category);

    ind = vobj->GetItemColumnIndices(t,colNamesView);
    dCode = vobj->GetCategoryDisplayCode(category);
    
    fprintf(fp,"<FONT SIZE=5>%s</FONT>",categoryViewName->Text());
    fprintf(fp,"<TABLE BORDER CELLPADDING=4>\n");
    ndb_log_message_text(NDB_MSG_DEBUG,"Preview -  category %s got view info", category);

    if (nRow > 1) { // Format table with multiple rows ............................................

      fprintf(fp,"<TR>\n");
      for (j=0; j < colNamesView->Length(); j++) {
	if (itemList[j].mandatoryCode == VIEW_ITEM_IS_HIDDEN) continue;
	fprintf(fp,"<TH ALIGN=CENTER VALIGN=MIDDLE><FONT SIZE=3>");
	print_table_attribute_name(fp, (*colLabels)[j].Text() );
	fprintf(fp,"</FONT></TH>\n");
      }
      fprintf(fp,"</TR>\n");
      
      for (j=0; j < nRow; j++) {
	ndb_log_message_text(NDB_MSG_DEBUG,"Preview -  category %s got row %d",category,j);
	if (rowData) delete rowData;
	rowData = t->GetRow(j);
#if 1
	iBlank=0;
	if (dCode == 7) {
	  iBlank = 1;
	  for (k=iColBlank; k < colNamesView->Length(); k++) {
	    if (((*rowData)[ind[k]].Length() > 0) && strcmp((*rowData)[ind[k]].Text(),"?") &&
		strcmp((*rowData)[ind[k]].Text(),".") ) {
	      iBlank = 0;
	      break;
	    }
	  }
	}
	if (iBlank) continue;
#endif
	fprintf(fp,"<TR>\n");	
	for (k =0; k < colNamesView->Length(); k++) {
	if (itemList[k].mandatoryCode == VIEW_ITEM_IS_HIDDEN) continue;
	  fprintf(fp,"<TD ALIGN=LEFT VALIGN=MIDDLE>");
	  ndb_log_message_text(NDB_MSG_DEBUG,"Preview -  category %s checking item ",category,j);
	  iCheck = item_integrity_check(cb, itemList[k].item, 
					(*rowData)[ind[k]].Text(), 
					itemList[k].mandatoryCode);

	  if ((*rowData)[ind[k]].Length() > 0) {
	    if (iCheck >= 0) 
	      fprintf(fp,"<FONT SIZE=3> %s </FONT>",(*rowData)[ind[k]].Text());
	    else
	      fprintf(fp,"<FONT SIZE=3 color=red> &gt&gt %s &lt&lt</FONT>",
		      (*rowData)[ind[k]].Text());
	  } else { 
	    if (itemList[k].mandatoryCode == VIEW_ITEM_IS_MANDATORY) 
	      fprintf(fp,"<FONT SIZE=3 color=red> &gt&gt?&lt&lt </FONT>");
	    else
	      fprintf(fp,"<FONT SIZE=3> %s </FONT>","&nbsp");
	  }
	  fprintf(fp,"</TD>\n");
	}
	fprintf(fp,"</TR>\n");
      }
    } else { // Format table with single row ........................................

      if (rowData) delete rowData;
      rowData = t->GetRow(0);

      for (j=0; j < colNamesView->Length(); j++) {
	if (itemList[j].mandatoryCode == VIEW_ITEM_IS_HIDDEN) continue;
	fprintf(fp,"<TR>\n");
	fprintf(fp,"<TD ALIGN=CENTER VALIGN=MIDDLE><FONT SIZE=3><b>");
	print_table_attribute_name(fp, (*colLabels)[j].Text() );
	fprintf(fp,"</b></FONT></TD>\n");
	fprintf(fp,"<TD ALIGN=LEFT VALIGN=MIDDLE>");

	iCheck = item_integrity_check(cb, itemList[j].item, 
				      (*rowData)[ind[j]].Text(), 
				      itemList[j].mandatoryCode);

	if ((*rowData)[ind[j]].Length() > 0) {
	  if (iCheck >= 0) 
	    fprintf(fp,"<FONT SIZE=3> %s </FONT>", (*rowData)[ind[j]].Text());
	  else
	    fprintf(fp,"<FONT SIZE=3 color=red> &gt&gt %s &lt&lt </FONT>", 
		    (*rowData)[ind[j]].Text());
	} else { 
	  if (itemList[j].mandatoryCode == VIEW_ITEM_IS_MANDATORY) 
	    fprintf(fp,"<FONT SIZE=3 color=red> &gt&gt?&lt&lt </FONT>");
	  else
	    fprintf(fp,"<FONT SIZE=3>%s</FONT>","&nbsp");
	}

	fprintf(fp,"</TD>\n");
	fprintf(fp,"</TR>\n");

      }
    }
    fprintf(fp,"</TABLE><BR><BR>\n");
    
  } else {
    return (0);
  }
  
  if (colNamesView) delete colNamesView;
  if (colLabels) delete colLabels;
  if (rowData) delete rowData;
  if (ind) delete ind;
  return(nRow);
}


//
// JDW  - revised metric of completeness...
//
double compute_view_category_completeness_1(FILE *fp, CifFileObj *fobj, CifViewObj *vobj,  
					  const char *blockId, const char *category, 
					  int &numMandatory, int &numMandatoryTotal) {
  int j,k, nRow=0, nColsView=0;
  ReVarCifArray<CifString> * rowData = NULL, * colNamesView = NULL;
  int *ind=NULL;
  ISTable * t=NULL;
  CIFVIEWOBJITEM *itemList=NULL;
  double fract;
  int nMandatory, nOptional, nMissing;
  int nMandatoryTotal, nOptionalTotal;

  itemList = build_item_obj_list(vobj, category, &nColsView);    

  nMandatory = nOptional = nMissing = 0;
  nMandatoryTotal = nOptionalTotal = 0;

  if ((t=fobj->GetTablePtr(blockId,category)) != NULL) {
    nRow=t->GetNumRows();
    
    if (colNamesView) delete colNamesView;
    colNamesView = vobj->GetCategoryItems(category);
    ind = vobj->GetItemColumnIndices(t,colNamesView);

    for (j=0; j < nRow; j++) {
      if (rowData) delete rowData;
      rowData = t->GetRow(j);
      for (k =0; k < colNamesView->Length(); k++) {
	if (itemList[k].mandatoryCode == VIEW_ITEM_IS_HIDDEN) continue;
	// exists
	if (itemList[k].mandatoryCode == 1) 
	  nMandatoryTotal++;
	else 
	  nOptionalTotal++;
	
	if ( ((*rowData)[ind[k]].Length() == 0) || 
	     ( ((*rowData)[ind[k]].Length() == 1) && 
	       (((*rowData)[ind[k]].Text()[0] == '?') || ((*rowData)[ind[k]].Text()[0] == '.')   ))) {
	  // missing
	  nMissing++;
	} else {
	  if (itemList[k].mandatoryCode == 1) 
	    nMandatory++;
	  else 
	    nOptional++;
	}
      }
    }

    if (nMandatoryTotal > 0) 
      fract = (double) nMandatory / (double) nMandatoryTotal;
    else
      fract = 1.0;
    if (fract > 1.0) fract = 1.0;

    ndb_log_message_text(NDB_MSG_DEBUG,"Completeness %s rows %d fract %f",
			 category,nRow, fract);

    ndb_log_message_text(NDB_MSG_DEBUG,"Total Mandatory %d total optional %d input mand %d input opt %d",
			 nMandatoryTotal, nOptionalTotal, nMandatory, nOptional);

    if (colNamesView) delete colNamesView;
    if (rowData) delete rowData;
    if (ind) delete ind;
    
  } else {
    fract = 0.0;
  }
  numMandatory  = nMandatory;
  numMandatoryTotal =  nMandatoryTotal;

  return(fract);
}


double compute_view_category_completeness(FILE *fp, CifFileObj *fobj, CifViewObj *vobj,  
					  const char *blockId, const char *category ) {
  int j,k, nRow=0, nColsView=0;
  ReVarCifArray<CifString> * rowData = NULL, * colNamesView = NULL;
  int *ind=NULL;
  ISTable * t=NULL;
  CIFVIEWOBJITEM *itemList=NULL;
  double fract;
  int nMandatory, nOptional, nMissing;
  int nMandatoryTotal, nOptionalTotal;

  itemList = build_item_obj_list(vobj, category, &nColsView);    


  if ((t=fobj->GetTablePtr(blockId,category)) != NULL) {
    nRow=t->GetNumRows();

    
    if (colNamesView) delete colNamesView;
    colNamesView = vobj->GetCategoryItems(category);
    ind = vobj->GetItemColumnIndices(t,colNamesView);

    nMandatory = nOptional = nMissing = 0;
    nMandatoryTotal = nOptionalTotal = 0;
    for (j=0; j < nRow; j++) {
      if (rowData) delete rowData;
      rowData = t->GetRow(j);
      for (k =0; k < colNamesView->Length(); k++) {
	if (itemList[k].mandatoryCode == VIEW_ITEM_IS_HIDDEN) continue;
	// exists
	if (itemList[k].mandatoryCode == 1) 
	  nMandatoryTotal++;
	else 
	  nOptionalTotal++;
	
	if ( ((*rowData)[ind[k]].Length() == 0) || 
	     ( ((*rowData)[ind[k]].Length() == 1) && 
	       (((*rowData)[ind[k]].Text()[0] == '?') || ((*rowData)[ind[k]].Text()[0] == '.')   ))) {
	  // missing
	  nMissing++;
	} else {
	  if (itemList[k].mandatoryCode == 1) 
	    nMandatory++;
	  else 
	    nOptional++;
	}
      }
    }

    if (nMandatoryTotal > 0) 
      fract = (double) nMandatory / (double) nMandatoryTotal;
    else
      fract = 1.0;
    if (fract > 1.0) fract = 1.0;

    ndb_log_message_text(NDB_MSG_DEBUG,"Completeness %s rows %d fract %f",
			 category,nRow, fract);

    ndb_log_message_text(NDB_MSG_DEBUG,"Total Mandatory %d total optional %d input mand %d input opt %d",
			 nMandatoryTotal, nOptionalTotal, nMandatory, nOptional);

    if (colNamesView) delete colNamesView;
    if (rowData) delete rowData;
    if (ind) delete ind;
    
  } else {
    fract = 0.0;
  }

  return(fract);
}


#if 0 

void set_category_status(const char *fName, int iCat, char status)
{
  FILE *fp=NULL;
  char string[1024];
  int ilen=0;

  if (!fName || !strlen(fName) || (iCat < 0)) return;

  
  fp = fopen(fName, "r");
  if (fp == NULL)  return;

  strcpy(string,"");
  fgets(string,1023,fp);
  fclose(fp);
  ilen = strlen(string);
  if (ilen > 1 && iCat < ilen) {
    if (string[ilen-1] == '\n')  string[ilen-1] = '\0';
    string[iCat] = status;
  } else {
    return;
  }

  fp = fopen(fName, "w");
  if (fp != NULL) {
    fprintf(fp,"%s",string);
    fclose(fp);
  }
}


char get_category_status(const char *fName, int iCat)
{
  FILE *fp=NULL;
  char string[1024];
  int ilen=0;
  char ret = '0';

  if (!fName || !strlen(fName) || (iCat < 0)) return (ret);

   fp = fopen(fName, "r");
  if (fp == NULL)  return (ret);

  strcpy(string,"");
  fgets(string,1023,fp);
  fclose(fp);
  ilen = strlen(string);
  if (ilen > 1 && iCat < ilen) {
    ret = string[iCat];
  }
  return (ret);
}

char *get_category_status_string(const char *fName)
{
  FILE *fp=NULL;
  char string[1024];
  int ilen=0;
  char *ret=NULL;

  if (!fName || !strlen(fName)) return (ret);

  ndb_log_message(NDB_MSG_DEBUG,"Status file name is %s",fName);
  fp = fopen(fName, "r");
  if (fp == NULL)  return (ret);

  strcpy(string,"");
  fgets(string,1023,fp);
  ndb_log_message(NDB_MSG_DEBUG,"Status string is  %s",string);
  fclose(fp);
  ilen = strlen(string);
  if (ilen > 0) {
    if (ilen > 1 && (string[ilen-1] == '\n')) string[ilen-1] = '\0';
    ret = (char *) calloc(strlen(string) + 1, sizeof(char));
    strcpy(ret,string);
  }
  return (ret);
}

#endif

#if 0
static void formatStringData(FILE *fp, const char *cs, int maxWidth) 
{
  int len, icount;
  if (cs == NULL || (strlen(cs) == 0) || 
      ((strlen(cs) == 1) && (cs[0] =='?' || cs[0] =='.'))) return;
  len = strlen(cs);

  //  if (len > maxWidth) len = maxWidth;

  icount = 0;
  for (int i=0; i < len; i++) {
    
    if (isspace(cs[i]) && i==0)
      continue;
    else  if (isspace(cs[i]))
      fprintf(fp," ");
    else  if (cs[i] == '"') // string double quotes
      fprintf(fp," ");
    else
      fprintf(fp,"%c",cs[i]);
    
    +icount++;
    if (icount > maxWidth) {
      fprintf(fp,"\n");
      icount=0;
    }
  }

  fprintf(fp,"\n");

}
#endif


static void formatStringData2(FILE *fp, char *text, int maxWidth)
{
  int i, ibeg, iend, iend1, ilen;

  if (text == NULL || (strlen(text) == 0) || 
      ((strlen(text) == 1) && (text[0] =='?' || text[0] =='.'))) return;

  ilen = strlen(text);

  if (ilen < maxWidth) {
    for (i=0; i < ilen; i++ ) {
      fprintf(fp,"%c", text[i]);
    }
    fprintf(fp,"\n");
    return;
  }    

  // Remove newlines... 
  //
  for (i=0; i < ilen; i++) {
    if (text[i] == '\n') text[i] = ' ';
  }


  ibeg = 0;
  while (ibeg < ilen-1) {
    iend = ibeg + maxWidth; 
    if (iend >= ilen - 1) {
      iend = ilen -1;

    } else {
      iend1 = iend;
      while ((iend >= ibeg ) && text[iend] != ' ' && text[iend] != ',' ) {
	--iend;
      }
      if (iend ==ibeg) iend = iend1;
    }
    // jdw 5-sep
    while ((ibeg < iend) && (text[ibeg] <= 32)) {
      ibeg++;
    }

    //   fprintf(fp,"   ");
    for (i=ibeg; i <= iend; i++) {
      fprintf(fp,"%c",text[i]);
    }
    fprintf(fp,"\n");
    ibeg = iend;
  }
  fprintf(fp,"\n");
}


